@@ -1,9 +1,11 @@
 #pragma once
 #ifdef WIN32
 #include "1WireSystem.h"
-#include "../../json/json.h"
 
-#include <boost/asio.hpp>
+namespace Json
+{
+	class Value;
+};
 
 class C1WireForWindows : public I_1WireSystem
 {
@@ -36,10 +38,10 @@ protected:
    static bool StartService(){return false;}
 
    // Socket
-   std::string SendAndReceive(std::string requestToSend) const;
+   std::string SendAndReceive(const std::string &requestToSend) const;
 
    SOCKET m_Socket;
-   boost::mutex m_SocketMutex;
+   std::mutex m_SocketMutex;
 };
 
 class C1WireForWindowsReadException : public std::exception
