@@ -1,44 +1,21 @@
 #include "stdafx.h"
 #include "Comm5Serial.h"
-#include "../main/Logger.h"
 #include "../main/Helper.h"
 #include "../main/localtime_r.h"
-#include "../main/mainworker.h"
-#include "../httpclient/HTTPClient.h"
-#include "../httpclient/UrlEncode.h"
+#include "../main/Logger.h"
+#include "../main/RFXtrx.h"
 
-#include <iostream>
+/*
+	This driver allows Domoticz to control any I/O module from the MA-4xxx Family
 
-#include <boost/lexical_cast.hpp>
-
-#define RETRY_DELAY 30
-#define Max_Comm5_MA_Relais 16
-
-static inline std::string &rtrim(std::string &s) {
-	s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
-	return s;
-}
-
-static inline std::vector<std::string> tokenize(const std::string &s) {
-	std::vector<std::string> tokens;
-	std::istringstream iss(s);
-	std::copy(std::istream_iterator<std::string>(iss),
-		std::istream_iterator<std::string>(),
-		std::back_inserter(tokens));
-	return tokens;
-}
-
-static inline bool startsWith(const std::string &haystack, const std::string &needle) {
-	return needle.length() <= haystack.length()
-		&& std::equal(needle.begin(), needle.end(), haystack.begin());
-}
+	These modules provide relays and digital sensors in the range of 5-30V DC.
+*/
 
 Comm5Serial::Comm5Serial(const int ID, const std::string& devname, unsigned int baudRate /*= 115200*/) :
 	m_szSerialPort(devname),
 	m_baudRate(baudRate)
 {
 	m_HwdID=ID;
-	m_stoprequested=false;
 	lastKnownSensorState = 0;
 	initSensorData = true;
 	reqState = Idle;
@@ -49,7 +26,8 @@ Comm5Serial::Comm5Serial(const int ID, const std::string& devname, unsigned int
 	frameCRC = 0;
 }
 
-bool Comm5Serial::WriteToHardware(const char * pdata, const unsigned char length) {
+bool Comm5Serial::WriteToHardware(const char * pdata, const unsigned char /*length*/)
+{
 	const tRBUF *pSen = reinterpret_cast<const tRBUF*>(pdata);
 
 	unsigned char packettype = pSen->ICMND.packettype;
@@ -82,13 +60,14 @@ bool Comm5Serial::WriteToHardware(const char * pdata, const unsigned char length
 
 bool Comm5Serial::StartHardware()
 {
-	m_stoprequested = false;
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&Comm5Serial::Do_Work, this)));
+	RequestStart();
+
+	m_thread = std::make_shared<std::thread>(&Comm5Serial::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 
 	//Try to open the Serial Port
 	try
 	{
-		_log.Log(LOG_STATUS,"Comm5 MA-4200: Using serial port: %s", m_szSerialPort.c_str());
 		open(
 			m_szSerialPort,
 			m_baudRate,
@@ -99,9 +78,9 @@ bool Comm5Serial::StartHardware()
 	}
 	catch (boost::exception & e)
 	{
-		_log.Log(LOG_ERROR,"Comm5 MA-4200: Error opening serial port!");
+		Log(LOG_ERROR,"Error opening serial port!");
 #ifdef _DEBUG
-		_log.Log(LOG_ERROR,"-----------------\n%s\n-----------------",boost::diagnostic_information(e).c_str());
+		Log(LOG_ERROR,"-----------------\n%s\n-----------------",boost::diagnostic_information(e).c_str());
 #else
 		(void)e;
 #endif
@@ -109,25 +88,23 @@ bool Comm5Serial::StartHardware()
 	}
 	catch ( ... )
 	{
-		_log.Log(LOG_ERROR,"Comm5 MA-4200: Error opening serial port!!!");
+		_log.Log(LOG_ERROR,"Error opening serial port!!!");
 		return false;
 	}
 	m_bIsStarted=true;
 	setReadCallback(boost::bind(&Comm5Serial::readCallBack, this, _1, _2));
-	
+
 	sOnConnected(this);
 	return true;
 }
 
 bool Comm5Serial::StopHardware()
 {
-	terminate();
-	m_stoprequested = true;
 	if (m_thread)
 	{
+		RequestStop();
 		m_thread->join();
-		// Wait a while. The read thread might be reading. Adding this prevents a pointer error in the async serial class.
-		sleep_milliseconds(10);
+		m_thread.reset();
 	}
 	m_bIsStarted = false;
 	return true;
@@ -135,41 +112,40 @@ bool Comm5Serial::StopHardware()
 
 void Comm5Serial::Do_Work()
 {
-	int sec_counter = 0;
+	//int sec_counter = 0;
 	int msec_counter = 0;
-	
+
 	queryRelayState();
 	querySensorState();
 	enableNotifications();
 
-	while (!m_stoprequested)
-	{
-		sleep_milliseconds(200);
-		if (m_stoprequested)
-			break;
-		msec_counter++;
-		if (msec_counter == 5)
-		{
-			msec_counter = 0;
+	Log(LOG_STATUS, "Worker started...");
 
-			sec_counter++;
-			if (sec_counter % 12 == 0) {
-				m_LastHeartbeat = mytime(NULL);
-			}
+
+	while (!IsStopRequested(100))
+	{
+		m_LastHeartbeat = mytime(NULL);
+		if (msec_counter++ >= 40) 
+		{
+			//every 4 seconds ?
+			msec_counter = 0;
+			querySensorState();
 		}
 	}
-	_log.Log(LOG_STATUS, "Comm5 MA-42XX: Serial Worker stopped...");
+	terminate();
+
+	Log(LOG_STATUS, "Worker stopped...");
 }
 
-void Comm5Serial::requestDigitalInputResponseHandler(const std::string & frame)
+void Comm5Serial::requestDigitalInputResponseHandler(const std::string & mframe)
 {
-	uint8_t mask = frame[5];
-	uint8_t sensorStatus = frame[6];
+	//uint8_t mask = mframe[5];
+	uint8_t sensorStatus = mframe[6];
 
 	for (int i = 0; i < 8; ++i) {
 		bool on = (sensorStatus & (1 << i)) != 0 ? true : false;
 		if (((lastKnownSensorState & (1 << i)) ^ (sensorStatus & (1 << i))) || initSensorData) {
-			SendSwitch((i + 1) << 8, 1, 255, on, 0, "Sensor " + boost::lexical_cast<std::string>(i + 1));
+			SendSwitchUnchecked((i + 1) << 8, 1, 255, on, 0, "Sensor " + std::to_string(i + 1));
 		}
 	}
 	lastKnownSensorState = sensorStatus;
@@ -177,37 +153,34 @@ void Comm5Serial::requestDigitalInputResponseHandler(const std::string & frame)
 	reqState = Idle;
 }
 
-void Comm5Serial::requestDigitalOutputResponseHandler(const std::string & frame)
+void Comm5Serial::requestDigitalOutputResponseHandler(const std::string & mframe)
 {
 	//  0     1     2       3       4        5        6
 	// 0x05 0x64 <size> <control> <id> <operation> <value>
 
-	relayStatus = frame[6];
+	relayStatus = mframe[6];
 	for (int i = 0; i < 8; ++i) {
 		bool on = (relayStatus & (1 << i)) != 0 ? true : false;
-		SendSwitch(i + 1, 1, 255, on, 0, "Relay " + boost::lexical_cast<std::string>(i + 1));
+		SendSwitch(i + 1, 1, 255, on, 0, "Relay " + std::to_string(i + 1));
 	}
 }
 
-void Comm5Serial::enableNotificationResponseHandler(const std::string & frame)
+void Comm5Serial::enableNotificationResponseHandler(const std::string & /*mframe*/)
 {
-	uint8_t operation = frame[5];
-	uint8_t operationType = frame[6];
-	uint8_t mask = frame[7];
-
+	//uint8_t operation = mframe[5];
+	//uint8_t operationType = mframe[6];
+	//uint8_t mask = mframe[7];
 }
 
 void Comm5Serial::readCallBack(const char * data, size_t len)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
-
 	if (!m_bEnableReceive)
 		return; //receiving not enabled
 
 	ParseData((const unsigned char*)data, static_cast<int>(len));
 }
 
-static uint16_t crc16_update(uint16_t crc, uint8_t data)
+uint16_t Comm5Serial::crc16_update(uint16_t crc, const uint8_t data)
 {
 #define POLYNOME 0x13D65
 	int i;
@@ -216,7 +189,7 @@ static uint16_t crc16_update(uint16_t crc, uint8_t data)
 	for (i = 0; i<8; i++)
 	{
 		if (crc & 0x8000)
-			crc = (crc << 1) ^ POLYNOME;
+			crc = (uint16_t)((crc << 1) ^ POLYNOME);
 		else
 			crc <<= 1;
 	}
@@ -240,7 +213,7 @@ void Comm5Serial::ParseData(const unsigned char* data, const size_t len)
 
 			}
 			else {
-				_log.Log(LOG_ERROR, "Comm5 MA-4200: Framing error");
+				Log(LOG_ERROR, "Framing error");
 				currentState = STSTART_OCTET1;
 			}
 			break;
@@ -252,7 +225,7 @@ void Comm5Serial::ParseData(const unsigned char* data, const size_t len)
 				currentState = STFRAME_SIZE;
 			}
 			else {
-				_log.Log(LOG_ERROR, "Comm5 MA-4200: Framing error");
+				Log(LOG_ERROR, "Framing error");
 				currentState = STSTART_OCTET1;
 			}
 			break;
@@ -273,11 +246,11 @@ void Comm5Serial::ParseData(const unsigned char* data, const size_t len)
 		case STFRAME_DATA:
 			frame.push_back(data[i]);
 			frameCRC = crc16_update(frameCRC, data[i]);
-			
+
 			if ( frame.size() >= frameSize )
 				currentState = STFRAME_CRC1;
 			break;
-		
+
 		case STFRAME_CRC1:
 			frame.push_back(data[i]);
 			frameCRC = crc16_update(frameCRC, 0);
@@ -298,17 +271,17 @@ void Comm5Serial::ParseData(const unsigned char* data, const size_t len)
 	}
 }
 
-void Comm5Serial::parseFrame(std::string & frame)
+void Comm5Serial::parseFrame(const std::string & mframe)
 {
-	switch (frame.at(4)) {
+	switch (mframe.at(4)) {
 	case 0x01:
-		requestDigitalInputResponseHandler(frame);
+		requestDigitalInputResponseHandler(mframe);
 		break;
 	case 0x02:
-		requestDigitalOutputResponseHandler(frame);
+		requestDigitalOutputResponseHandler(mframe);
 		break;
 	case 0x04:
-		enableNotificationResponseHandler(frame);
+		enableNotificationResponseHandler(mframe);
 		break;
 	}
 }
@@ -316,19 +289,19 @@ void Comm5Serial::parseFrame(std::string & frame)
 bool Comm5Serial::writeFrame(const std::string & data)
 {
 	char length = (uint8_t)data.size();
-	std::string frame("\x05\x64", 2);
-	frame.push_back(length);
-	frame.push_back(0x00);
-	frame.append(data);
+	std::string mframe("\x05\x64", 2);
+	mframe.push_back(length);
+	mframe.push_back(0x00);
+	mframe.append(data);
 	uint16_t crc = 0;
-	for (size_t i = 0; i < frame.size(); ++i)
-		crc = crc16_update(crc, frame.at(i));
+	for (size_t i = 0; i < mframe.size(); ++i)
+		crc = crc16_update(crc, mframe.at(i));
 	crc = crc16_update(crc, 0);
 	crc = crc16_update(crc, 0);
 
-	frame.append(1, crc >> 8);
-	frame.append(1, crc & 0xFF);
-	write(frame);
+	mframe.append(1, crc >> 8);
+	mframe.append(1, crc & 0xFF);
+	write(mframe);
 	return true;
 }
 
@@ -353,15 +326,9 @@ void Comm5Serial::enableNotifications()
 	writeFrame(data);
 }
 
-void Comm5Serial::OnData(const unsigned char *pData, size_t length)
-{
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
-	ParseData(pData, length);
-}
-
 void Comm5Serial::OnError(const std::exception e)
 {
-	_log.Log(LOG_ERROR, "Comm5 MA-4200: Error: %s", e.what());
+	Log(LOG_ERROR, "Error: %s", e.what());
 }
 
 
