@@ -1,25 +1,28 @@
 #include "stdafx.h"
 #include "Comm5TCP.h"
-#include "../main/Logger.h"
 #include "../main/Helper.h"
 #include "../main/localtime_r.h"
-#include "../main/mainworker.h"
-#include "../httpclient/HTTPClient.h"
-#include "../httpclient/UrlEncode.h"
-
-#include <iostream>
-
-#include <boost/lexical_cast.hpp>
+#include "../main/Logger.h"
+#include "../main/RFXtrx.h"
 
 #define RETRY_DELAY 30
 #define Max_Comm5_MA_Relais 16
 
-static inline std::string &rtrim(std::string &s) {
+/*
+   This driver allows Domoticz to control any I/O module from the MA-5xxx-2 Family, including the fiber-optics (FX)
+   variations.
+   https://www.comm5.com.br/en/products/io-modules/
+   These modules provide relays and digital sensors in the range of 5-30V DC or 70-240V AC selectable by dip switches
+   for each individual input.
+
+*/
+
+static inline std::string &c5_rtrim(std::string &s) {
 	s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
 	return s;
 }
 
-static inline std::vector<std::string> tokenize(const std::string &s) {
+static inline std::vector<std::string> c5_tokenize(const std::string &s) {
 	std::vector<std::string> tokens;
 	std::istringstream iss(s);
 	std::copy(std::istream_iterator<std::string>(iss),
@@ -28,7 +31,7 @@ static inline std::vector<std::string> tokenize(const std::string &s) {
 	return tokens;
 }
 
-static inline bool startsWith(const std::string &haystack, const std::string &needle) {
+static inline bool c5_startsWith(const std::string &haystack, const std::string &needle) {
 	return needle.length() <= haystack.length()
 		&& std::equal(needle.begin(), needle.end(), haystack.begin());
 }
@@ -37,7 +40,6 @@ Comm5TCP::Comm5TCP(const int ID, const std::string &IPAddress, const unsigned sh
 m_szIPAddress(IPAddress)
 {
 	m_HwdID=ID;
-	m_stoprequested=false;
 	m_usIPPort=usIPPort;
 	lastKnownSensorState = 0;
 	initSensorData = true;
@@ -47,28 +49,29 @@ m_szIPAddress(IPAddress)
 
 bool Comm5TCP::StartHardware()
 {
-	m_stoprequested = false;
+	RequestStart();
+
 	m_bReceiverStarted = false;
 
 	//force connect the next first time
 	m_bIsStarted = true;
-	m_rxbufferpos = 0;
 
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&Comm5TCP::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&Comm5TCP::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 
-	_log.Log(LOG_STATUS, "Comm5 MA-5XXX: Started");
+	Log(LOG_STATUS, "Started");
 
-	return (m_thread != NULL);
+	return (m_thread != nullptr);
 }
 
 bool Comm5TCP::StopHardware()
 {
-	if (m_thread != NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
 	m_bIsStarted = false;
 	return true;
@@ -76,7 +79,7 @@ bool Comm5TCP::StopHardware()
 
 void Comm5TCP::OnConnect()
 {
-	_log.Log(LOG_STATUS, "Comm5 MA-5XXX: connected to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+	Log(LOG_STATUS, "connected to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 	m_bIsStarted = true;
 	notificationEnabled = false;
 
@@ -88,41 +91,32 @@ void Comm5TCP::OnConnect()
 
 void Comm5TCP::OnDisconnect()
 {
-	_log.Log(LOG_ERROR, "Comm5 MA-5XXX: disconected");
+	Log(LOG_ERROR, "disconected");
 }
 
 void Comm5TCP::Do_Work()
 {
-	bool bFirstTime = true;
-	int count = 0;
-	while (!m_stoprequested)
+	int sec_counter = 0;
+	connect(m_szIPAddress, m_usIPPort);
+	while (!IsStopRequested(1000))
 	{
-		m_LastHeartbeat = mytime(NULL);
-		if (bFirstTime)
-		{
-			bFirstTime = false;
-			if (!mIsConnected)
-			{
-				m_rxbufferpos = 0;
-				connect(m_szIPAddress, m_usIPPort);
-			}
+		sec_counter++;
+
+		if (sec_counter % 12 == 0) {
+			m_LastHeartbeat = mytime(NULL);
 		}
-		else
-		{
-			sleep_milliseconds(40);
-			update();
-			if (count++ >= 100) {
-				count = 0;
-				querySensorState();
-			}
+		if (sec_counter % 4 == 0) {
+			querySensorState();
 		}
 	}
-	_log.Log(LOG_STATUS, "Comm5 MA-5XXX: TCP/IP Worker stopped...");
+	terminate();
+
+	Log(LOG_STATUS, "TCP/IP Worker stopped...");
 }
 
-void Comm5TCP::processSensorData(const std::string& line) 
+void Comm5TCP::processSensorData(const std::string& line)
 {
-	std::vector<std::string> tokens = tokenize(line);
+	std::vector<std::string> tokens = c5_tokenize(line);
 	if (tokens.size() < 2)
 		return;
 
@@ -130,7 +124,7 @@ void Comm5TCP::processSensorData(const std::string& line)
 	for (int i = 0; i < 16; ++i) {
 		bool on = (sensorbitfield & (1 << i)) != 0 ? true : false;
 		if (((lastKnownSensorState & (1 << i)) ^ (sensorbitfield & (1 << i))) || initSensorData) {
-			SendSwitch((i + 1) << 8, 1, 255, on, 0, "Sensor " + boost::lexical_cast<std::string>(i + 1));
+			SendSwitchUnchecked((i + 1) << 8, 1, 255, on, 0, "Sensor " + std::to_string(i + 1));
 		}
 	}
 	lastKnownSensorState = sensorbitfield;
@@ -145,21 +139,21 @@ void Comm5TCP::ParseData(const unsigned char* data, const size_t len)
 	std::string line;
 
 	while (std::getline(stream, line, '\n')) {
-		line = rtrim(line);
-		if (startsWith(line, "211")) {
-			std::vector<std::string> tokens = tokenize(line);
+		line = c5_rtrim(line);
+		if (c5_startsWith(line, "211")) {
+			std::vector<std::string> tokens = c5_tokenize(line);
 			if (tokens.size() < 2)
 				break;
 
 			unsigned int relaybitfield = ::strtol(tokens[1].c_str(), 0, 16);
 			for (int i = 0; i < 16; ++i) {
 				bool on = (relaybitfield & (1 << i)) != 0 ? true : false;
-				SendSwitch(i + 1, 1, 255, on, 0, "Relay " + boost::lexical_cast<std::string>(i + 1));
+				SendSwitch(i + 1, 1, 255, on, 0, "Relay " + std::to_string(i + 1));
 			}
 		}
-		else if (startsWith(line, "210") && (!startsWith(line, "210 OK"))) {
+		else if (c5_startsWith(line, "210") && (!c5_startsWith(line, "210 OK"))) {
 			processSensorData(line);
-		} 
+		}
 	}
 
 	// Trim consumed bytes.
@@ -182,14 +176,14 @@ void Comm5TCP::enableNotifications()
 	notificationEnabled = true;
 }
 
-bool Comm5TCP::WriteToHardware(const char *pdata, const unsigned char length)
+bool Comm5TCP::WriteToHardware(const char *pdata, const unsigned char /*length*/)
 {
 	const tRBUF *pSen = reinterpret_cast<const tRBUF*>(pdata);
 
 	unsigned char packettype = pSen->ICMND.packettype;
 	//unsigned char subtype = pSen->ICMND.subtype;
 
-	if (!mIsConnected)
+	if (!isConnected())
 		return false;
 
 	if (packettype == pTypeLighting2 && pSen->LIGHTING2.id3 == 0)
@@ -201,9 +195,9 @@ bool Comm5TCP::WriteToHardware(const char *pdata, const unsigned char length)
 			return false;
 
 		if (pSen->LIGHTING2.cmnd == light2_sOff)
-			write("RESET " + boost::lexical_cast<std::string>(Relay) + "\n");
+			write("RESET " + std::to_string(Relay) + '\n');
 		else
-			write("SET " + boost::lexical_cast<std::string>(Relay) + "\n");
+			write("SET " + std::to_string(Relay) + '\n');
 
 		return true;
 	}
@@ -212,15 +206,9 @@ bool Comm5TCP::WriteToHardware(const char *pdata, const unsigned char length)
 
 void Comm5TCP::OnData(const unsigned char *pData, size_t length)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
 	ParseData(pData, length);
 }
 
-void Comm5TCP::OnError(const std::exception e)
-{
-	_log.Log(LOG_ERROR, "Comm5 MA-5XXX: Error: %s", e.what());
-}
-
 void Comm5TCP::OnError(const boost::system::error_code& error)
 {
 	switch (error.value())
@@ -230,13 +218,13 @@ void Comm5TCP::OnError(const boost::system::error_code& error)
 	case boost::asio::error::access_denied:
 	case boost::asio::error::host_unreachable:
 	case boost::asio::error::timed_out:
-		_log.Log(LOG_ERROR, "Comm5 MA-5XXX: Can not connect to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+		Log(LOG_ERROR, "Can not connect to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 		break;
 	case boost::asio::error::eof:
 	case boost::asio::error::connection_reset:
-		_log.Log(LOG_ERROR, "Comm5 MA-5XXX: Connection reset!");
+		Log(LOG_ERROR, "Connection reset!");
 		break;
 	default:
-		_log.Log(LOG_ERROR, "Comm5 MA-5XXX: %s", error.message().c_str());
+		Log(LOG_ERROR, "%s", error.message().c_str());
 	}
 }
