@@ -1,7 +1,5 @@
 #pragma once
 
-#include <deque>
-#include <iostream>
 #include "CurrentCostMeterBase.h"
 
 class CurrentCostMeterTCP: public CurrentCostMeterBase
@@ -10,11 +8,11 @@ public:
 	CurrentCostMeterTCP(const int ID, const std::string &IPAddress, const unsigned short usIPPort);
 	virtual ~CurrentCostMeterTCP(void);
 
-	virtual bool WriteToHardware(const char *pdata, const unsigned char length);
+	virtual bool WriteToHardware(const char *pdata, const unsigned char length) override;
 
-protected:	
-	virtual bool StartHardware();
-	virtual bool StopHardware();
+protected:
+	virtual bool StartHardware() override;
+	virtual bool StopHardware() override;
 
 private:
 	void write(const char *data, size_t size);
@@ -26,9 +24,8 @@ private:
 	int m_retrycntr;
 	std::string m_szIPAddress;
 	unsigned short m_usIPPort;
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
-	sockaddr_in m_addr;
+	std::shared_ptr<std::thread> m_thread;
+		sockaddr_in m_addr;
 	int m_socket;
 };
 
