@@ -9,16 +9,21 @@
 #include "../main/SQLHelper.h"
 #include <sstream>
 
-#define DenkoviSmartdenLan_POLL_INTERVAL 60
+#define MAX_POLL_INTERVAL 30*1000
 
-CDenkoviSmartdenLan::CDenkoviSmartdenLan(const int ID, const std::string &IPAddress, const unsigned short usIPPort, const std::string &password) :
+CDenkoviSmartdenLan::CDenkoviSmartdenLan(const int ID, const std::string &IPAddress, const unsigned short usIPPort, const std::string &password, const int pollInterval) :
 m_szIPAddress(IPAddress),
-m_Password(CURLEncode::URLEncode(password))
+m_Password(CURLEncode::URLEncode(password)),
+m_pollInterval(pollInterval)
 {
 	m_HwdID=ID;
 	m_usIPPort=usIPPort;
 	m_stoprequested=false;
 	m_bOutputLog = false;
+	if (m_pollInterval < 500)
+		m_pollInterval = 500;
+	else if (m_pollInterval > MAX_POLL_INTERVAL)
+		m_pollInterval = MAX_POLL_INTERVAL;
 	Init();
 }
 
@@ -34,20 +39,21 @@ bool CDenkoviSmartdenLan::StartHardware()
 {
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CDenkoviSmartdenLan::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CDenkoviSmartdenLan::Do_Work, this);
+	SetThreadName(m_thread->native_handle(), "DenkoviSmartdenLAN");
 	m_bIsStarted=true;
 	sOnConnected(this);
 	_log.Log(LOG_STATUS, "Denkovi: Started");
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CDenkoviSmartdenLan::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
 		m_stoprequested = true;
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -55,18 +61,19 @@ bool CDenkoviSmartdenLan::StopHardware()
 
 void CDenkoviSmartdenLan::Do_Work()
 {
-	int sec_counter = DenkoviSmartdenLan_POLL_INTERVAL - 2;
+	int poll_interval = m_pollInterval / 100;
+	int poll_counter = poll_interval - 2;
 
 	while (!m_stoprequested)
 	{
-		sleep_seconds(1);
-		sec_counter++;
+		sleep_milliseconds(100);
+		poll_counter++;
 
-		if (sec_counter % 12 == 0) {
-			m_LastHeartbeat=mytime(NULL);
+		if (poll_counter % 12 * 10 == 0) { //10 steps = 1 second (10 * 100)
+			m_LastHeartbeat = mytime(NULL);
 		}
 
-		if (sec_counter % DenkoviSmartdenLan_POLL_INTERVAL == 0)
+		if (poll_counter % poll_interval == 0)
 		{
 			GetMeterDetails();
 		}
@@ -74,7 +81,7 @@ void CDenkoviSmartdenLan::Do_Work()
 	_log.Log(LOG_STATUS,"Denkovi: Worker stopped...");
 }
 
-bool CDenkoviSmartdenLan::WriteToHardware(const char *pdata, const unsigned char length)
+bool CDenkoviSmartdenLan::WriteToHardware(const char *pdata, const unsigned char /*length*/)
 {
 	const tRBUF *pSen = reinterpret_cast<const tRBUF*>(pdata);
 
@@ -157,7 +164,7 @@ void CDenkoviSmartdenLan::UpdateSwitch(const unsigned char Idx, const int SubUni
 	lcmd.LIGHTING2.id2 = 0;
 	lcmd.LIGHTING2.id3 = 0;
 	lcmd.LIGHTING2.id4 = Idx;
-	lcmd.LIGHTING2.unitcode = SubUnit;
+	lcmd.LIGHTING2.unitcode = (uint8_t)SubUnit;
 	if (!bOn)
 	{
 		lcmd.LIGHTING2.cmnd = light2_sOff;
@@ -166,7 +173,7 @@ void CDenkoviSmartdenLan::UpdateSwitch(const unsigned char Idx, const int SubUni
 	{
 		lcmd.LIGHTING2.cmnd = light2_sOn;
 	}
-	lcmd.LIGHTING2.level = level;
+	lcmd.LIGHTING2.level = (uint8_t)level;
 	lcmd.LIGHTING2.filler = 0;
 	lcmd.LIGHTING2.rssi = 12;
 	sDecodeRXMessage(this, (const unsigned char *)&lcmd.LIGHTING2, defaultname.c_str(), 255);
@@ -202,13 +209,12 @@ void CDenkoviSmartdenLan::GetMeterDetails()
 	}
 	size_t ii;
 	std::string tmpstr;
-	int pos1;
 	int Idx = -1;
 
 	for (ii = 1; ii < results.size(); ii++)
 	{
 		tmpstr = results[ii];
-		pos1 = tmpstr.find("<Relay");
+		size_t pos1 = tmpstr.find("<Relay");
 		if (pos1 != std::string::npos)
 		{
 			tmpstr = tmpstr.substr(pos1 + strlen("<Relay"));
