@@ -1,17 +1,16 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 
 class CDummy : public CDomoticzHardwareBase
 {
 public:
 	explicit CDummy(const int ID);
 	~CDummy(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
 	void Init();
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 };
 
