@@ -1,7 +1,5 @@
 #pragma once
 
-#include <deque>
-#include <iostream>
 #include "ASyncTCP.h"
 #include "DomoticzHardware.h"
 
@@ -36,30 +34,24 @@ class Ec3kMeterTCP : public CDomoticzHardwareBase, ASyncTCP
 public:
 	Ec3kMeterTCP(const int ID, const std::string &IPAddress, const unsigned short usIPPort);
 	~Ec3kMeterTCP(void);
-	bool isConnected(){ return mIsConnected; };
-	bool WriteToHardware(const char *pdata, const unsigned char length);
-public:
-	// signals
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 	boost::signals2::signal<void()>	sDisconnected;
+private:
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+	void ParseData(const unsigned char *pData, int Len);
+
+	void OnConnect() override;
+	void OnDisconnect() override;
+	void OnData(const unsigned char *pData, size_t length) override;
+	void OnError(const boost::system::error_code& error) override;
 private:
 	int m_retrycntr;
-	bool StartHardware();
-	bool StopHardware();
 	Ec3kLimiter *m_limiter;
-protected:
 	std::string m_szIPAddress;
 	unsigned short m_usIPPort;
-	bool m_bDoRestart;
 
-	void Do_Work();
-	void OnConnect();
-	void OnDisconnect();
-	void OnData(const unsigned char *pData, size_t length);
-	void OnError(const std::exception e);
-	void OnError(const boost::system::error_code& error);
-
-	void ParseData(const unsigned char *pData, int Len);
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
+	std::shared_ptr<std::thread> m_thread;
 };
 
