@@ -33,9 +33,10 @@ Version history
 #include "hardwaretypes.h"
 #include "../main/localtime_r.h"
 #include "../httpclient/HTTPClient.h"
-#include <../tinyxpath/xpath_static.h>
+#include "../tinyxpath/tinyxml.h"
+#include "../tinyxpath/xpath_static.h"
 #include "../webserver/Base64.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 #include <sstream>
 
 #ifdef _DEBUG
@@ -52,27 +53,25 @@ Version history
 #define MINOR_RT2 0
 #define RELEASE_RT2 29
 
-CEcoDevices::CEcoDevices(const int ID, const std::string &IPAddress, const unsigned short usIPPort, 
-	const std::string &username, const std::string &password, const int datatimeout, const int model, const int ratelimit)
-
+CEcoDevices::CEcoDevices(const int ID, const std::string &IPAddress, const unsigned short usIPPort,
+	const std::string &username, const std::string &password, const int datatimeout, const int model, const int ratelimit) :
+	m_szIPAddress(IPAddress),
+	m_username(username),
+	m_password(password)
 {
 	m_HwdID = ID;
-	m_szIPAddress = IPAddress;
 	m_usIPPort = usIPPort;
-	m_username = username;
-	m_password = password;
-	m_stoprequested = false;
 	m_iModel = model;
-        
-        // Updates must be at least every 5mn in order to keep consistent historical data
+
+	// Updates must be at least every 5mn in order to keep consistent historical data
 	m_iDataTimeout = (datatimeout >= 300 || datatimeout == 0) ? 300 : datatimeout;
-	
- 	// system seems unstable if going too fast
+
+	// system seems unstable if going too fast
 	m_iRateLimit = (ratelimit < 2) ? 2 : ratelimit;
 
-        // RateLimit > DataTimeout is an inconsistent setting. In that case, decrease RateLimit (which increases update rate) 
-	// down to Timeout in order to avoir watchdog errors due to this user configuration mistake.
-        if (m_iRateLimit > m_iDataTimeout)   m_iRateLimit = m_iDataTimeout;
+	// RateLimit > DataTimeout is an inconsistent setting. In that case, decrease RateLimit (which increases update rate)
+// down to Timeout in order to avoir watchdog errors due to this user configuration mistake.
+	if (m_iRateLimit > m_iDataTimeout)   m_iRateLimit = m_iDataTimeout;
 
 	Init();
 }
@@ -85,37 +84,39 @@ CEcoDevices::~CEcoDevices(void)
 
 void CEcoDevices::Init()
 {
-	m_stoprequested = false;
 	m_bFirstRun = true;
 
 	// Is the device we poll password protected?
 	m_ssURL.str("");
 	if ((m_username.size() > 0) && (m_password.size() > 0))
-        	m_ssURL << "http://" << m_username << ":" << m_password << "@";
-        else
-                m_ssURL <<"http://";
+		m_ssURL << "http://" << m_username << ":" << m_password << "@";
+	else
+		m_ssURL << "http://";
 
-        m_ssURL << m_szIPAddress << ":" << m_usIPPort;
+	m_ssURL << m_szIPAddress << ":" << m_usIPPort;
 
 }
 
 bool CEcoDevices::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CEcoDevices::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CEcoDevices::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted = true;
 	sOnConnected(this);
-	return (m_thread != NULL);
+	return (m_thread != nullptr);
 }
 
 bool CEcoDevices::StopHardware()
 {
-	if (m_thread != NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
 	m_bIsStarted = false;
 	return true;
@@ -124,10 +125,9 @@ bool CEcoDevices::StopHardware()
 void CEcoDevices::Do_Work()
 {
 	int sec_counter = m_iRateLimit - 2; // Make sure we update once soon after restart
-	_log.Log(LOG_STATUS, "(%s): Worker started...", Name.c_str());
-	while (!m_stoprequested)
+	_log.Log(LOG_STATUS, "(%s): Worker started...", m_Name.c_str());
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 		mytime(&m_LastHeartbeat);
 		if (sec_counter >= m_iRateLimit)
@@ -139,10 +139,10 @@ void CEcoDevices::Do_Work()
 				GetMeterRT2Details();
 		}
 	}
-	_log.Log(LOG_STATUS, "(%s): Worker stoped...", Name.c_str());
+	_log.Log(LOG_STATUS, "(%s): Worker stopped...", m_Name.c_str());
 }
 
-bool CEcoDevices::WriteToHardware(const char *pdata, const unsigned char length)
+bool CEcoDevices::WriteToHardware(const char* /*pdata*/, const unsigned char /*length*/)
 {
 	return true;
 }
@@ -209,15 +209,15 @@ void CEcoDevices::GetMeterDetails()
 
 	// Check EcoDevices firmware version and process pulse counters
 	sstr << m_ssURL.str() << "/status.xml";
-	
+
 	if (m_status.hostname.empty()) m_status.hostname = m_szIPAddress;
 	if (HTTPClient::GET(sstr.str(), ExtraHeaders, sResult))
 	{
-		_log.Log(LOG_NORM, "(%s) Fetching counters and status data", Name.c_str());
+		_log.Log(LOG_NORM, "(%s) Fetching counters and status data", m_Name.c_str());
 	}
 	else
 	{
-		_log.Log(LOG_ERROR, "(%s) Error getting data from: %s", Name.c_str(), sstr.str().c_str());
+		_log.Log(LOG_ERROR, "(%s) Error getting data from: %s", m_Name.c_str(), sstr.str().c_str());
 		return;
 	}
 
@@ -225,7 +225,7 @@ void CEcoDevices::GetMeterDetails()
 	XMLdoc.Parse(sResult.c_str(), 0, TIXML_ENCODING_UTF8);
 	if (XMLdoc.Error())
 	{
-		_log.Log(LOG_ERROR, "(%s) Error parsing XML at /status.xml: %s", Name.c_str(), XMLdoc.ErrorDesc());
+		_log.Log(LOG_ERROR, "(%s) Error parsing XML at /status.xml: %s", m_Name.c_str(), XMLdoc.ErrorDesc());
 		return;
 	}
 
@@ -286,9 +286,9 @@ void CEcoDevices::GetMeterDetails()
 	else
 	{
 		message = "EcoDevices firmware needs to be at least version ";
-		message = message + static_cast<std::ostringstream*>(&(std::ostringstream() << min_major << "." << min_minor << "." << min_release))->str();
+		message = message + std::to_string(min_major) + "." + std::to_string(min_minor) + "." + std::to_string(min_release);
 		message = message + ", current version is " + m_status.version;
-		_log.Log(LOG_ERROR, "(%s) %s", Name.c_str(), message.c_str());
+		_log.Log(LOG_ERROR, "(%s) %s", m_Name.c_str(), message.c_str());
 		return;
 	}
 
@@ -300,10 +300,10 @@ void CEcoDevices::GetMeterDetails()
 		sstr.str("");
 		sstr << m_ssURL.str() << "/protect/settings/teleinfo1.xml";
 
-		_log.Log(LOG_NORM, "(%s) Fetching Teleinfo 1 data", Name.c_str());
+		_log.Log(LOG_NORM, "(%s) Fetching Teleinfo 1 data", m_Name.c_str());
 		if (!HTTPClient::GET(sstr.str(), ExtraHeaders, sResult))
 		{
-			_log.Log(LOG_ERROR, "(%s) Error getting teleinfo1.xml from EcoDevices!", Name.c_str());
+			_log.Log(LOG_ERROR, "(%s) Error getting teleinfo1.xml from EcoDevices!", m_Name.c_str());
 			return;
 		}
 #ifdef DEBUG_EcoDevices
@@ -324,11 +324,11 @@ void CEcoDevices::GetMeterDetails()
 	{
 		sstr.str("");
 		sstr << m_ssURL.str() << "/protect/settings/teleinfo2.xml";
-		
-		_log.Log(LOG_NORM, "(%s) Fetching Teleinfo 2 data", Name.c_str());
+
+		_log.Log(LOG_NORM, "(%s) Fetching Teleinfo 2 data", m_Name.c_str());
 		if (!HTTPClient::GET(sstr.str(), ExtraHeaders, sResult))
 		{
-			_log.Log(LOG_ERROR, "(%s) Error getting teleinfo2.xml!", Name.c_str());
+			_log.Log(LOG_ERROR, "(%s) Error getting teleinfo2.xml!", m_Name.c_str());
 			return;
 		}
 #ifdef DEBUG_EcoDevices
@@ -361,7 +361,7 @@ void CEcoDevices::GetMeterRT2Details()
 	std::stringstream sstr;
 	std::map<std::string, std::string> XMLmap;
 	TiXmlDocument XMLdoc("Teleinfo.xml");
-	time_t atime = mytime(NULL);
+	//time_t atime = mytime(NULL);
 	int   i, major, minor, release;
 	int min_major = MAJOR_RT2, min_minor = MINOR_RT2, min_release = RELEASE_RT2;
 
@@ -372,21 +372,20 @@ void CEcoDevices::GetMeterRT2Details()
 	std::string sURL = sstr.str();
 	if (!HTTPClient::GET(sURL, ExtraHeaders, sResult))
 	{
-		_log.Log(LOG_ERROR, "(%s) Error getting system information from: %s", Name.c_str(), sstr.str().c_str());
+		_log.Log(LOG_ERROR, "(%s) Error getting system information from: %s", m_Name.c_str(), sstr.str().c_str());
 		return;
 	}
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if ((!bRet) || (!root.isObject()))
 	{
-		_log.Log(LOG_ERROR, "(%s) Invalid JSON data received from /admin/system.json", Name.c_str());
+		_log.Log(LOG_ERROR, "(%s) Invalid JSON data received from /admin/system.json", m_Name.c_str());
 		return;
 	}
 	if (root["confighostname"].empty() == true)
 	{
-		_log.Log(LOG_ERROR, "(%s) Invalid JSON data received from /admin/system.json, hostname missing", Name.c_str());
+		_log.Log(LOG_ERROR, "(%s) Invalid JSON data received from /admin/system.json, hostname missing", m_Name.c_str());
 	}
 	else
 	{
@@ -406,11 +405,11 @@ void CEcoDevices::GetMeterRT2Details()
 
 	if (HTTPClient::GET(sstr.str(), ExtraHeaders, sResult))
 	{
-		_log.Log(LOG_NORM, "(%s) Fetching data from /admin/status.xml", Name.c_str());
+		_log.Log(LOG_NORM, "(%s) Fetching data from /admin/status.xml", m_Name.c_str());
 	}
 	else
 	{
-		_log.Log(LOG_ERROR, "(%s) Error getting status.xml!", Name.c_str());
+		_log.Log(LOG_ERROR, "(%s) Error getting status.xml!", m_Name.c_str());
 		return;
 	}
 
@@ -418,7 +417,7 @@ void CEcoDevices::GetMeterRT2Details()
 	XMLdoc.Parse(sResult.c_str(), 0, TIXML_ENCODING_UTF8);
 	if (XMLdoc.Error())
 	{
-		_log.Log(LOG_ERROR, "(%s) Error parsing XML at /admin/status.xml: %s", Name.c_str(), XMLdoc.ErrorDesc());
+		_log.Log(LOG_ERROR, "(%s) Error parsing XML at /admin/status.xml: %s", m_Name.c_str(), XMLdoc.ErrorDesc());
 		return;
 	}
 
@@ -427,7 +426,7 @@ void CEcoDevices::GetMeterRT2Details()
 	std::string product = S_xpath_string(XMLdoc.RootElement(), "/response/product/text()").c_str();
 	if (product != "ECODEVICES RT 2")
 	{
-		_log.Log(LOG_ERROR, "(%s) Product information found in XML file is not 'ECODEVICES RT 2' as expected, but '%s'", Name.c_str(), product.c_str());
+		_log.Log(LOG_ERROR, "(%s) Product information found in XML file is not 'ECODEVICES RT 2' as expected, but '%s'", m_Name.c_str(), product.c_str());
 		return;
 	}
 	m_status.version = S_xpath_string(XMLdoc.RootElement(), "/response/infofirm/text()").c_str();
@@ -447,13 +446,13 @@ void CEcoDevices::GetMeterRT2Details()
 	if (!((major > min_major) || ((major == min_major) && (minor > min_minor)) || ((major == min_major) && (minor == min_minor) && (release >= min_release))))
 	{
 		message = "EcoDevices RT2 firmware needs to be at least version ";
-		message = message + static_cast<std::ostringstream*>(&(std::ostringstream() << min_major << "." << min_minor << "." << min_release))->str();
+		message = message + std::to_string(min_major) + "." + std::to_string(min_minor) + "." + std::to_string(min_release);
 		message = message + ", current version is " + m_status.version;
-		_log.Log(LOG_ERROR, "(%s) %s", Name.c_str(), message.c_str());
+		_log.Log(LOG_ERROR, "(%s) %s", m_Name.c_str(), message.c_str());
 		return;
 	}
 
-	//Measured voltage on power supply 
+	//Measured voltage on power supply
 	m_status.voltage = i_xpath_int(XMLdoc.RootElement(), "/response/vmesure/text()");
 	SendVoltageSensor(m_HwdID, 1, 255, (float)m_status.voltage, "EcoDevice RT2");
 
@@ -513,7 +512,7 @@ void CEcoDevices::GetMeterRT2Details()
 			fvalue1 = (float)atof(splitresults[0].c_str());
 			if (fvalue1 > 0) SendMeterSensor(m_HwdID, i, 255, fvalue1 / 1000, m_status.hostname + " " + label);
 			fvalue2 = (float)atof(splitresults[1].c_str());
-			if (fvalue2 > 0) SendWaterflowSensor(m_HwdID, i, 255, fvalue2, m_status.hostname + " " + label);
+			if (fvalue2 > 0) SendWaterflowSensor(m_HwdID, (uint8_t)i, 255, fvalue2, m_status.hostname + " " + label);
 		}
 	}
 }
