@@ -1,6 +1,5 @@
 #pragma once
 
-#include <vector>
 #include "ASyncSerial.h"
 #include "DomoticzHardware.h"
 
@@ -17,38 +16,26 @@ class CEnOceanESP2: public AsyncSerial, public CDomoticzHardwareBase
 		ERS_CHECKSUM
 	};
 public:
-    /**
-    * Opens a serial device.
-    * \param devname serial device name, example "/dev/ttyS0" or "COM1"
-    * \param baud_rate serial baud rate
-    * \param opt_parity serial parity, default even
-    * \param opt_csize serial character size, default 7bit
-    * \param opt_flow serial flow control, default none
-    * \param opt_stop serial stop bits, default 1
-    * \throws boost::system::system_error if cannot open the
-    * serial device
-    */
 	CEnOceanESP2(const int ID, const std::string& devname, const int type);
-
     ~CEnOceanESP2();
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 	void SendDimmerTeachIn(const char *pdata, const unsigned char length);
 	unsigned long m_id_base;
 private:
 	void Init();
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	bool OpenSerialDevice();
 	void Do_Work();
 	bool ParseData();
 	void Add2SendQueue(const char* pData, const size_t length);
 	float GetValueRange(const float InValue, const float ScaleMax, const float ScaleMin=0, const float RangeMax=255, const float RangeMin=0);
-
+	void readCallback(const char *data, size_t len);
+private:
 	_eEnOcean_Receive_State m_receivestate;
 	int m_wantedlength;
 
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
+	std::shared_ptr<std::thread> m_thread;
     int m_Type;
 	std::string m_szSerialPort;
 
@@ -57,13 +44,7 @@ private:
 	int m_bufferpos;
 	int m_retrycntr;
 
-	boost::mutex m_sendMutex;
+	std::mutex m_sendMutex;
 	std::vector<std::string> m_sendqueue;
-
-	/**
-     * Read callback, stores data in the buffer
-     */
-    void readCallback(const char *data, size_t len);
-
 };
 
