@@ -36,11 +36,9 @@ datum;DISCONNECT;ConnectionID;dauerInSekunden;
 */
 
 FritzboxTCP::FritzboxTCP(const int ID, const std::string &IPAddress, const unsigned short usIPPort) :
-m_szIPAddress(IPAddress)
+	m_szIPAddress(IPAddress)
 {
 	m_HwdID=ID;
-	m_bDoRestart=false;
-	m_stoprequested=false;
 	m_usIPPort=usIPPort;
 	m_retrycntr = RETRY_DELAY;
 	m_bufferpos = 0;
@@ -52,49 +50,33 @@ FritzboxTCP::~FritzboxTCP(void)
 
 bool FritzboxTCP::StartHardware()
 {
-	m_stoprequested=false;
-	m_bDoRestart=false;
+	RequestStart();
 
 	//force connect the next first time
 	m_retrycntr=RETRY_DELAY;
 	m_bIsStarted=true;
 
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&FritzboxTCP::Do_Work, this)));
-	return (m_thread!=NULL);
+	m_thread = std::make_shared<std::thread>(&FritzboxTCP::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
+	return (m_thread != nullptr);
 }
 
 bool FritzboxTCP::StopHardware()
 {
-	m_stoprequested=true;
-	try {
-		if (m_thread)
-		{
-			m_thread->join();
-		}
-	}
-	catch (...)
+	if (m_thread)
 	{
-		//Don't throw from a Stop command
+		RequestStop();
+		m_thread->join();
+		m_thread.reset();
 	}
-	if (isConnected())
-	{
-		try {
-			disconnect();
-		} catch(...)
-		{
-			//Don't throw from a Stop command
-		}
-	}
-
 	m_bIsStarted=false;
 	return true;
 }
 
 void FritzboxTCP::OnConnect()
 {
-	_log.Log(LOG_STATUS,"Fritzbox: connected to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
-	m_bDoRestart=false;
+	_log.Log(LOG_STATUS,"Fritzbox: connected to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 	m_bIsStarted=true;
 	m_bufferpos=0;
 
@@ -108,45 +90,26 @@ void FritzboxTCP::OnDisconnect()
 
 void FritzboxTCP::Do_Work()
 {
-	bool bFirstTime=true;
 	int sec_counter = 0;
-	while (!m_stoprequested)
+	connect(m_szIPAddress,m_usIPPort);
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 
 		if (sec_counter  % 12 == 0) {
-			m_LastHeartbeat=mytime(NULL);
-		}
-
-		if (bFirstTime)
-		{
-			bFirstTime=false;
-			connect(m_szIPAddress,m_usIPPort);
-		}
-		else
-		{
-			if ((m_bDoRestart) && (sec_counter % 30 == 0))
-			{
-				connect(m_szIPAddress,m_usIPPort);
-			}
-			update();
+			m_LastHeartbeat = mytime(NULL);
 		}
 	}
+	terminate();
+
 	_log.Log(LOG_STATUS,"Fritzbox: TCP/IP Worker stopped...");
-} 
+}
 
 void FritzboxTCP::OnData(const unsigned char *pData, size_t length)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
 	ParseData(pData,length);
 }
 
-void FritzboxTCP::OnError(const std::exception e)
-{
-	_log.Log(LOG_ERROR,"Fritzbox: Error: %s",e.what());
-}
-
 void FritzboxTCP::OnError(const boost::system::error_code& error)
 {
 	if (
@@ -157,7 +120,7 @@ void FritzboxTCP::OnError(const boost::system::error_code& error)
 		(error == boost::asio::error::timed_out)
 		)
 	{
-		_log.Log(LOG_ERROR, "Fritzbox: Can not connect to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+		_log.Log(LOG_ERROR, "Fritzbox: Can not connect to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 	}
 	else if (
 		(error == boost::asio::error::eof) ||
@@ -172,7 +135,7 @@ void FritzboxTCP::OnError(const boost::system::error_code& error)
 
 bool FritzboxTCP::WriteToHardware(const char *pdata, const unsigned char length)
 {
-	if (!mIsConnected)
+	if (!isConnected())
 	{
 		return false;
 	}
@@ -182,7 +145,7 @@ bool FritzboxTCP::WriteToHardware(const char *pdata, const unsigned char length)
 
 void FritzboxTCP::WriteInt(const std::string &sendStr)
 {
-	if (!mIsConnected)
+	if (!isConnected())
 	{
 		return;
 	}
@@ -217,10 +180,10 @@ void FritzboxTCP::ParseData(const unsigned char *pData, int Len)
 	}
 }
 
-void FritzboxTCP::UpdateSwitch(const unsigned char Idx, const int SubUnit, const bool bOn, const double Level, const std::string &defaultname)
+void FritzboxTCP::UpdateSwitch(const unsigned char Idx, const uint8_t SubUnit, const bool bOn, const double Level, const std::string &defaultname)
 {
 	double rlevel = (15.0 / 100)*Level;
-	int level = int(rlevel);
+	uint8_t level = (uint8_t)(rlevel);
 
 	char szIdx[10];
 	sprintf(szIdx, "%X%02X%02X%02X", 0, 0, 0, Idx);
@@ -289,7 +252,7 @@ void FritzboxTCP::ParseLine()
 		if (results.size() < 6)
 			return;
 		sstr << "Call From: " << results[4] << " to: " << results[5];
-		m_sql.UpdateValue(m_HwdID, "1", 1, pTypeGeneral, sTypeTextStatus, 12, 255, 0, sstr.str().c_str(), devname);
+		SendTextSensor(1, 1, 255, sstr.str(), devname);
 	}
 	else if (Cmd == "RING")
 	{
@@ -298,7 +261,7 @@ void FritzboxTCP::ParseLine()
 		if (results.size() < 5)
 			return;
 		sstr << "Received From: " << results[3] << " to: " << results[4];
-		m_sql.UpdateValue(m_HwdID, "1", 1, pTypeGeneral, sTypeTextStatus, 12, 255, 0, sstr.str().c_str(), devname);
+		SendTextSensor(1, 1, 255, sstr.str(), devname);
 	}
 	else if (Cmd == "CONNECT")
 	{
@@ -310,13 +273,7 @@ void FritzboxTCP::ParseLine()
 		UpdateSwitch(1, 1, true, 100, "Call");
 
 		sstr << "Connected ID: " << results[2] << " Number: " << results[4];
-		m_sql.UpdateValue(m_HwdID, "1", 1, pTypeGeneral, sTypeTextStatus, 12, 255, 0, sstr.str().c_str(), devname);
-		//result = m_sql.safe_query("SELECT ID FROM DeviceStatus WHERE (HardwareID==%d) AND (Type==%d) AND (Subtype==%d)", m_HwdID, int(pTypeGeneral), int(sTypeTextStatus));
-		//if (!result.empty())
-		//{
-		//	std::string idx = result[0][0];
-		//	m_sql.safe_query("INSERT INTO LightingLog (DeviceRowID, sValue) VALUES ('%q', '%q')", idx.c_str(), sstr.str().c_str());
-		//}
+		SendTextSensor(1, 1, 255, sstr.str(), devname);
 	}
 	else if (Cmd == "DISCONNECT")
 	{
@@ -328,12 +285,6 @@ void FritzboxTCP::ParseLine()
 		UpdateSwitch(1, 1, false, 100, "Call");
 
 		sstr << "Disconnect ID: " << results[2] << " Duration: " << results[3] << " seconds";
-		m_sql.UpdateValue(m_HwdID, "1", 1, pTypeGeneral, sTypeTextStatus, 12, 255, 0, sstr.str().c_str(), devname);
-		//result = m_sql.safe_query("SELECT ID FROM DeviceStatus WHERE (HardwareID==%d) AND (Type==%d) AND (Subtype==%d)", m_HwdID, int(pTypeGeneral), int(sTypeTextStatus));
-		//if (!result.empty())
-		//{
-		//	std::string idx = result[0][0];
-		//	m_sql.safe_query("INSERT INTO LightingLog (DeviceRowID, sValue) VALUES ('%q', '%q')", idx.c_str(), sstr.str().c_str());
-		//}
+		SendTextSensor(1, 1, 255, sstr.str(), devname);
 	}
 }
