@@ -26,10 +26,7 @@
 	#include <fstream>
 	#include <string>
 	#include <limits>
-	#include <sys/time.h>
 	#include <unistd.h>
-	#include <vector>
-	#include <map>
 
 	struct _tDUsageStruct
 	{
@@ -72,19 +69,21 @@ extern std::string szInternalVoltageCommand;
 extern bool bHasInternalCurrent;
 extern std::string szInternalCurrentCommand;
 
-
 #define round(a) ( int ) ( a + .5 )
 
 CHardwareMonitor::CHardwareMonitor(const int ID)
 {
 	m_HwdID = ID;
-	m_stoprequested=false;
 	m_bOutputLog = false;
+	m_lastquerytime = 0;
+#if defined (__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__OpenBSD__)
+	m_totcpu = 0;
+	m_lastloadcpu = 0;
+#endif
 #ifdef WIN32
 	m_pLocator = NULL;
 	m_pServicesOHM = NULL;
 	m_pServicesSystem = NULL;
-	m_lastquerytime = 0;
 	//	CoInitializeEx(0, COINIT_MULTITHREADED);
 //	CoInitializeSecurity(NULL, -1, NULL, NULL, RPC_C_AUTHN_LEVEL_DEFAULT, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE, NULL);
 #endif
@@ -107,12 +106,14 @@ bool CHardwareMonitor::StartHardware()
 #endif
 	StopHardware();
 
+	RequestStart();
+
 #ifdef WIN32
 	InitWMI();
 #endif
-	m_stoprequested = false;
 	m_lastquerytime = 0;
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CHardwareMonitor::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CHardwareMonitor::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted = true;
 	sOnConnected(this);
 #if defined(__linux__) || defined(__CYGWIN32__) || defined(__FreeBSD__) || defined(__OpenBSD__)
@@ -128,9 +129,9 @@ bool CHardwareMonitor::StartHardware()
 
 bool CHardwareMonitor::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
 		m_thread.reset();
 	}
@@ -148,9 +149,8 @@ void CHardwareMonitor::Do_Work()
 
 	int msec_counter = 0;
 	int64_t sec_counter = POLL_INTERVAL_CPU - 5;
-	while (!m_stoprequested)
+	while (!IsStopRequested(500))
 	{
-		sleep_milliseconds(500);
 		msec_counter++;
 		if (msec_counter == 2)
 		{
@@ -167,7 +167,7 @@ void CHardwareMonitor::Do_Work()
 				}
 				catch (...)
 				{
-					_log.Log(LOG_STATUS, "Hardware Monitor: Error occurred while Fetching motherboard sensors!...");
+					_log.Log(LOG_ERROR, "Hardware Monitor: Error occurred while Fetching motherboard sensors!...");
 				}
 			}
 
@@ -180,7 +180,7 @@ void CHardwareMonitor::Do_Work()
 				}
 				catch (...)
 				{
-					_log.Log(LOG_STATUS, "Hardware Monitor: Error occurred while Fetching CPU data!...");
+					_log.Log(LOG_ERROR, "Hardware Monitor: Error occurred while Fetching CPU data!...");
 				}
 			}
 
@@ -192,7 +192,7 @@ void CHardwareMonitor::Do_Work()
 				}
 				catch (...)
 				{
-					_log.Log(LOG_STATUS, "Hardware Monitor: Error occurred while Fetching memory data!...");
+					_log.Log(LOG_ERROR, "Hardware Monitor: Error occurred while Fetching memory data!...");
 				}
 			}
 
@@ -204,7 +204,7 @@ void CHardwareMonitor::Do_Work()
 				}
 				catch (...)
 				{
-					_log.Log(LOG_STATUS, "Hardware Monitor: Error occurred while Fetching disk data!...");
+					_log.Log(LOG_ERROR, "Hardware Monitor: Error occurred while Fetching disk data!...");
 				}
 			}
 #endif
@@ -223,16 +223,6 @@ void CHardwareMonitor::SendCurrent(const unsigned long Idx, const float Curr, co
 	sDecodeRXMessage(this, (const unsigned char *)&gDevice, defaultname.c_str(), 255);
 }
 
-void CHardwareMonitor::SendFanSensor(const int Idx, const int FanSpeed, const std::string &defaultname)
-{
-	_tGeneralDevice gDevice;
-	gDevice.subtype = sTypeFan;
-	gDevice.id = 1;
-	gDevice.intval1 = static_cast<int>(Idx);
-	gDevice.intval2 = FanSpeed;
-	sDecodeRXMessage(this, (const unsigned char *)&gDevice, defaultname.c_str(), 255);
-}
-
 void CHardwareMonitor::GetInternalTemperature()
 {
 	int returncode = 0;
@@ -353,13 +343,13 @@ void CHardwareMonitor::UpdateSystemSensor(const std::string& qType, const int di
 	{
 		doffset = 1200;
 		int fanspeed = atoi(devValue.c_str());
-		SendFanSensor(doffset + dindex, fanspeed, devName);
+		SendFanSensor(doffset + dindex, 255, fanspeed, devName);
 	}
 	else if (qType == "Voltage")
 	{
 		doffset = 1300;
 		float volt = static_cast<float>(atof(devValue.c_str()));
-		SendVoltageSensor(0, doffset + dindex, 255, volt, devName);
+		SendVoltageSensor(0, (uint32_t)(doffset + dindex), 255, volt, devName);
 	}
 	else if (qType == "Current")
 	{
@@ -367,6 +357,14 @@ void CHardwareMonitor::UpdateSystemSensor(const std::string& qType, const int di
 		float curr = static_cast<float>(atof(devValue.c_str()));
 		SendCurrent(doffset + dindex, curr, devName);
 	}
+#if defined (__linux__)
+	else if (qType == "Process")
+	{
+		doffset = 1500;
+		float usage = static_cast<float>(atof(devValue.c_str()));
+		SendCustomSensor(0, doffset + dindex, 255, usage, devName, "MB");
+	}
+#endif
 	return;
 }
 
@@ -459,7 +457,6 @@ void CHardwareMonitor::RunWMIQuery(const char* qTable, const std::string &qType)
 	if ((m_pServicesOHM == NULL) || (m_pServicesSystem == NULL))
 		return;
 	HRESULT hr;
-	int dindex = 0;
 	std::string query = "SELECT * FROM ";
 	query.append(qTable);
 	query.append(" WHERE SensorType = '");
@@ -469,12 +466,14 @@ void CHardwareMonitor::RunWMIQuery(const char* qTable, const std::string &qType)
 	hr = m_pServicesOHM->ExecQuery(L"WQL", bstr_t(query.c_str()), WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, NULL, &pEnumerator);
 	if (!FAILED(hr))
 	{
+		int dindex = 0;
+
 		// Get the data from the query
 		IWbemClassObject *pclsObj = NULL;
 		while (pEnumerator)
 		{
 			ULONG uReturn = 0;
-			HRESULT hr = pEnumerator->Next(WBEM_INFINITE, 1, &pclsObj, &uReturn);
+			hr = pEnumerator->Next(WBEM_INFINITE, 1, &pclsObj, &uReturn);
 			if (FAILED(hr) || (0 == uReturn))
 			{
 				break;
@@ -532,6 +531,32 @@ void CHardwareMonitor::RunWMIQuery(const char* qTable, const std::string &qType)
 			(((double) tp.tv_usec) * 0.000001 );
 	}
 
+#if defined(__linux__)
+	float CHardwareMonitor::GetProcessMemUsage()
+	{
+		pid_t pid = getpid();
+		std::stringstream ssPidfile;
+		ssPidfile << "/proc/" << pid << "/status";
+		std::ifstream mfile(ssPidfile.str().c_str());
+		if (!mfile.is_open())
+			return -1;
+		uint32_t VmRSS = -1;
+		uint32_t VmSwap = -1;
+		std::string token;
+		while (mfile >> token)
+		{
+			if (token == "VmRSS:")
+				mfile >> VmRSS;
+			else if (token == "VmSwap:")
+				mfile >> VmSwap;
+
+			// ignore rest of the line
+			mfile.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
+		}
+		return (VmRSS + VmSwap) / 1000.f;
+	}
+#endif
+
 	float GetMemUsageLinux()
 	{
 #if defined(__FreeBSD__)
@@ -629,6 +654,14 @@ void CHardwareMonitor::RunWMIQuery(const char* qTable, const std::string &qType)
 #endif
 		sprintf(szTmp,"%.2f",memusedpercentage);
 		UpdateSystemSensor("Load", 0, "Memory Usage", szTmp);
+#ifdef __linux__
+		float memProcess = GetProcessMemUsage();
+		if (memProcess != -1)
+		{
+			sprintf(szTmp, "%.2f", memProcess);
+			UpdateSystemSensor("Process", 0, "Process Usage", szTmp);
+		}
+#endif
 	}
 
 	void CHardwareMonitor::FetchUnixCPU()
@@ -746,21 +779,19 @@ void CHardwareMonitor::RunWMIQuery(const char* qTable, const std::string &qType)
 	void CHardwareMonitor::FetchUnixDisk()
 	{
 		//Disk Usage
-		char szTmp[300];
 		std::map<std::string, _tDUsageStruct> _disks;
 		std::map<std::string, std::string> _dmounts_;
 		int returncode = 0;
 		std::vector<std::string> _rlines=ExecuteCommandAndReturn(m_dfcommand, returncode);
 		if (!_rlines.empty())
 		{
-			std::vector<std::string>::const_iterator ittDF;
-			for (ittDF = _rlines.begin(); ittDF != _rlines.end(); ++ittDF)
+			for (const auto & ittDF : _rlines)
 			{
 				char dname[200];
 				char suse[30];
 				char smountpoint[300];
 				long numblock, usedblocks, availblocks;
-				int ret = sscanf((*ittDF).c_str(), "%s\t%ld\t%ld\t%ld\t%s\t%s\n", dname, &numblock, &usedblocks, &availblocks, suse, smountpoint);
+				int ret = sscanf(ittDF.c_str(), "%s\t%ld\t%ld\t%ld\t%s\t%s\n", dname, &numblock, &usedblocks, &availblocks, suse, smountpoint);
 				if (ret == 6)
 				{
 					std::map<std::string, std::string>::iterator it = _dmounts_.find(dname);
@@ -788,14 +819,14 @@ void CHardwareMonitor::RunWMIQuery(const char* qTable, const std::string &qType)
 				}
 			}
 			int dindex = 0;
-			std::map<std::string, _tDUsageStruct>::const_iterator ittDisks;
-			for (ittDisks = _disks.begin(); ittDisks != _disks.end(); ++ittDisks)
+			for (const auto & ittDisks : _disks)
 			{
-				_tDUsageStruct dusage = (*ittDisks).second;
+				_tDUsageStruct dusage = ittDisks.second;
 				if (dusage.TotalBlocks > 0)
 				{
 					double UsagedPercentage = (100 / double(dusage.TotalBlocks))*double(dusage.UsedBlocks);
-					//std::cout << "Disk: " << (*ittDisks).first << ", Mount: " << dusage.MountPoint << ", Used: " << UsagedPercentage << std::endl;
+					//std::cout << "Disk: " << ittDisks.first << ", Mount: " << dusage.MountPoint << ", Used: " << UsagedPercentage << std::endl;
+					char szTmp[300];
 					sprintf(szTmp, "%.2f", UsagedPercentage);
 					std::string hddname = "HDD " + dusage.MountPoint;
 					UpdateSystemSensor("Load", 2 + dindex, hddname, szTmp);
