@@ -12,21 +12,22 @@ class CHardwareMonitor : public CDomoticzHardwareBase
 public:
 	explicit CHardwareMonitor(const int ID);
 	~CHardwareMonitor(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length) { return false; };
+	bool WriteToHardware(const char* /*pdata*/, const unsigned char /*length*/) override { return false; };
+#if defined (__linux__)
+	float GetProcessMemUsage();
+#endif
 private:
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	double m_lastquerytime;
 	void Do_Work();
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
+	std::shared_ptr<std::thread> m_thread;
 	void FetchData();
 	void GetInternalTemperature();
 	void GetInternalVoltage();
 	void GetInternalCurrent();
 	void UpdateSystemSensor(const std::string& qType, const int dindex, const std::string& devName, const std::string& devValue);
 	void SendCurrent(const unsigned long Idx, const float Curr, const std::string &defaultname);
-	void SendFanSensor(const int Idx, const int FanSpeed, const std::string &defaultname);
 #ifdef WIN32
 	bool InitWMI();
 	void ExitWMI();
