@@ -1,11 +1,8 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 #include "hardwaretypes.h"
 
-#define BUFFER_SIZE								2*1024*1024
-
 class csocket;
 
 class Action
@@ -77,45 +74,81 @@ public:
 
 class CHarmonyHub : public CDomoticzHardwareBase
 {
+	enum _eConnectionStatus
+	{
+		DISCONNECTED=0,
+		CONNECTED,
+		INITIALIZED,
+		AUTHENTICATED,
+		BOUND
+	};
 public:
 	CHarmonyHub(const int ID, const std::string &IPAddress, const unsigned int port);
 	~CHarmonyHub(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
-	bool Login();
-	void Logout();
-	bool SetupCommandSocket();
-	bool UpdateActivities();
-	bool UpdateCurrentActivity();
-	void CheckSetActivity(const std::string &activityID, const bool on);
-	void UpdateSwitch(const unsigned char idx, const char * szIdx, const bool bOn, const std::string &defaultname);
-	
-	void Init();
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	void Do_Work();
 
-	bool ConnectToHarmony(const std::string &strHarmonyIPAddress, const int harmonyPortNumber, csocket* harmonyCommunicationcsocket);
-	bool StartCommunication(csocket* communicationcsocket, const std::string &strUserName, const std::string &strPassword);
-	bool GetAuthorizationToken(csocket* authorizationcsocket);
-	bool SubmitCommand(const std::string &strCommand, const std::string &strCommandParameterPrimary, const std::string &strCommandParameterSecondary);
-	bool CheckIfChanging(const std::string& strData);
-	bool SendPing();
-	bool ParseAction(const std::string& strAction, std::vector<Action>& vecDeviceActions, const std::string& strDeviceID);
-	//bool ParseFunction(const std::string& strFunction, std::vector<Function>& vecDeviceFunctions, const std::string& strDeviceID);
+	// Init and cleanup
+	void Init();
+	void Logout();
 
-	std::string m_harmonyAddress;
-	unsigned short m_usIPPort;
+	// Helper functions for changing switch status in Domoticz
+	void CheckSetActivity(const std::string &activityID, const bool on);
+	void UpdateSwitch(const unsigned char idx, const char * szIdx, const bool bOn, const std::string &defaultname);
+
+	// Raw socket functions
+	bool ConnectToHarmony(const std::string &szHarmonyIPAddress, const int HarmonyPortNumber, csocket* connection);
+	bool SetupCommunicationSocket();
+	void ResetCommunicationSocket();
+	int WriteToSocket(std::string *szReq);
+	std::string ReadFromSocket(csocket *connection);
+	std::string ReadFromSocket(csocket *connection, float waitTime);
+
+	// XMPP Stream initialization
+	int StartStream(csocket *connection);
+	int SendAuth(csocket *connection, const std::string &szUserName, const std::string &szPassword);
+	int SendPairRequest(csocket *connection);
+	int CloseStream(csocket *connection);
+
+	// XMPP commands
+	int SendPing();
+	int SubmitCommand(const std::string &szCommand);
+	int SubmitCommand(const std::string &szCommand, const std::string &szActivityId);
+
+	// XMPP reading
+	bool CheckForHarmonyData();
+	void ParseHarmonyTransmission(std::string *szHarmonyData);
+	void ProcessHarmonyConnect(std::string *szHarmonyData);
+	void ProcessHarmonyMessage(std::string *szMessageBlock);
+	void ProcessQueryResponse(std::string *szQueryResponse);
+
+	// Helper function for XMPP reading
+	bool IsTransmissionComplete(std::string *szHarmonyData);
+private:
+	// hardware parameters
+	std::string m_szHarmonyAddress;
+	unsigned short m_usHarmonyPort;
+
+	std::shared_ptr<std::thread> m_thread;
+	std::mutex m_mutex;
+
+	csocket * m_connection;
+	_eConnectionStatus m_connectionstatus;
+
+	bool m_bNeedMoreData;
+	bool m_bWantAnswer;
+	bool m_bNeedEcho;
+	bool m_bReceivedMessage;
+	bool m_bLoginNow;
+	bool m_bIsChangingActivity;
+	bool m_bShowConnectError;
+
+	std::string m_szHubSwVersion;
+	std::string m_szHarmonyData;
 	std::string m_szAuthorizationToken;
 	std::string m_szCurActivityID;
-	boost::mutex m_mutex;
-
-	csocket * m_commandcsocket;
-	volatile bool m_stoprequested;
-	bool m_bDoLogin;
-	bool m_bIsChangingActivity;
-	std::string m_hubSwVersion;
-	boost::shared_ptr<boost::thread> m_thread;
-	char m_databuffer[BUFFER_SIZE];
-	std::string m_szResultString;
+	std::map<std::string, std::string> m_mapActivities;
 };
