@@ -7,7 +7,7 @@
 #include "../main/RFXtrx.h"
 #include "hardwaretypes.h"
 #include "../httpclient/HTTPClient.h"
-#include "../json/json.h"
+#include <json/json.h>
 #include "../webserver/Base64.h"
 #include "../main/WebServer.h"
 #include "../main/LuaHandler.h"
@@ -23,11 +23,10 @@ m_refresh(refresh)
 	// extract the data
 	std::vector<std::string> strextra;
 	StringSplit(extradata, "|", strextra);
-	std::string script;
 	if (strextra.size() == 3 || strextra.size() == 4 || strextra.size() == 5)
 	{
 		m_script = base64_decode(strextra[0]);
-		m_method = atoi(base64_decode(strextra[1]).c_str());
+		m_method = (unsigned short)atoi(base64_decode(strextra[1]).c_str());
 		m_contenttype = base64_decode(strextra[2]);
 		if (strextra.size() >= 4)
 		{
@@ -40,8 +39,6 @@ m_refresh(refresh)
 	}
 
 	m_HwdID=ID;
-
-	m_stoprequested=false;
 	Init();
 }
 
@@ -53,27 +50,31 @@ void CHttpPoller::Init()
 {
 }
 
-bool CHttpPoller::WriteToHardware(const char *pdata, const unsigned char length)
+bool CHttpPoller::WriteToHardware(const char* /*pdata*/, const unsigned char /*length*/)
 {
 	return false;
 }
 
 bool CHttpPoller::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CHttpPoller::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CHttpPoller::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CHttpPoller::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -83,11 +84,8 @@ void CHttpPoller::Do_Work()
 {
 	int sec_counter = 300 - 5;
 	_log.Log(LOG_STATUS, "Http: Worker started...");
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
-		if (m_stoprequested)
-			break;
 		sec_counter++;
 		if (sec_counter % 12 == 0) {
 			m_LastHeartbeat = mytime(NULL);
@@ -131,7 +129,7 @@ void CHttpPoller::GetScript()
 		{
 			auth += m_password;
 		}
-		std::string encodedAuth = base64_encode((const unsigned char *)auth.c_str(), auth.length());
+		std::string encodedAuth = base64_encode(auth);
 		ExtraHeaders.push_back("Authorization:Basic " + encodedAuth);
 	}
 
@@ -139,14 +137,14 @@ void CHttpPoller::GetScript()
 		if (!HTTPClient::GET(sURL, ExtraHeaders, sResult))
 		{
 			std::string err = "Http: Error getting data from url \"" + sURL + "\"";
-			_log.Log(LOG_ERROR, err.c_str());
+			_log.Log(LOG_ERROR, err);
 			return;
 		}
 	}
 	if (m_method == 1) {
 		if (!HTTPClient::POST(sURL, m_postdata, ExtraHeaders, sResult)) {
 			std::string err = "Http: Error getting data from url \"" + sURL + "\"";
-			_log.Log(LOG_ERROR, err.c_str());
+			_log.Log(LOG_ERROR, err);
 			return;
 		}
 	}
