@@ -2,6 +2,12 @@
 
 #include "DomoticzHardware.h"
 
+typedef union _i2c_data
+{
+	uint8_t  byte[2];
+	uint16_t word;
+} i2c_data;
+
 class I2C : public CDomoticzHardwareBase
 {
 public:
@@ -12,15 +18,15 @@ public:
 		I2CTYPE_HTU21D,
 		I2CTYPE_TSL2561,
 		I2CTYPE_PCF8574,
-		I2CTYPE_BME280
+		I2CTYPE_BME280,
+		I2CTYPE_MCP23017
 	};
-
-	explicit I2C(const int ID, const _eI2CType DevType, const int Port);
+	explicit I2C(const int ID, const _eI2CType DevType, const std::string &Address, const std::string &SerialPort, const int Mode1);
 	~I2C();
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	void HTU21D_ReadSensorDetails();
 	void bmp_Read_BMP_SensorDetails();
 	void bmp_Read_BME_SensorDetails();
@@ -29,23 +35,27 @@ private:
 	bool readBME280All(const int fd, float &temp, float &pressure, int &humidity);
 
 	void Do_Work();
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
-
-	std::string m_ActI2CBus;
-	_eI2CType m_dev_type;
-
 
 	bool i2c_test(const char *I2CBusName);
 	int i2c_Open(const char *I2CBusName);
 
 	int ReadInt(int fd, uint8_t *devValues, uint8_t startReg, uint8_t bytesToRead);
 	int WriteCmd(int fd, uint8_t devAction);
+	int WriteCmdAddr(const int fd, const uint8_t CmdAddr, const uint8_t devAction);
+
+private:
+	std::shared_ptr<std::thread> m_thread;
+
+	_eI2CType m_dev_type;
+	uint8_t m_i2c_addr;
+	std::string m_ActI2CBus;
+	bool m_invert_data;
+
 
 	// BMP085 stuff
 	//Forecast
-	int bmp_CalculateForecast(const float pressure);
-	int CalculateForcast(const float pressure);
+	uint8_t bmp_CalculateForecast(const float pressure);
+	uint8_t CalculateForecast(const float pressure);
 	float m_LastPressure;
 	int m_LastMinute;
 	float m_pressureSamples[180];
@@ -85,15 +95,18 @@ private:
 	// TSL2561 stuff
 	void TSL2561_ReadSensorDetails();
 	void TSL2561_Init();
-	
+
 	// PCF8574
-	unsigned char	i2c_addr;
 	void			PCF8574_ReadChipDetails();
-	char			PCF8574_get_pin_number_from_Unit(unsigned char unit);
-	char			PCF8574_get_i2c_addr_from_Unit(unsigned char unit);
-	int				PCF8574_create_DeviceID(unsigned char i2c_address,unsigned char pin_mask);
-	unsigned char	PCF8574_create_Unit(unsigned char i2c_address, char pin);
-	char			PCF8574_WritePin(char pin_number,char  value);
-	char 			readByteI2C(int fd, char *byte, char i2c_addr);
-	char 			writeByteI2C(int fd, char byte, char i2c_addr);
+	int			PCF8574_WritePin(uint8_t pin_number, uint8_t  value);
+	int 			readByteI2C(int fd, uint8_t *byte, uint8_t i2c_addr);
+	int 			writeByteI2C(int fd, uint8_t byte, uint8_t i2c_addr);
+
+	// MCP23017
+	void 			MCP23017_Init();
+	void			MCP23017_ReadChipDetails();
+	int				MCP23017_WritePin(uint8_t pin_number, uint8_t  value);
+	int 			I2CWriteReg16(int fd, uint8_t reg, uint16_t value);
+	int		 		I2CReadReg16(int fd, unsigned char reg, i2c_data *data);
 };
+
