@@ -8,7 +8,7 @@
 #include "../main/SQLHelper.h"
 #include "../httpclient/HTTPClient.h"
 #include "../main/mainworker.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 
 #define round(a) ( int ) ( a + .5 )
 
@@ -29,7 +29,6 @@ m_UserName(Username),
 m_Password(Password)
 {
 	m_HwdID=ID;
-	m_stoprequested=false;
 	m_companymode = CMODE_UNKNOWN;
 	Init();
 }
@@ -46,21 +45,24 @@ void CICYThermostat::Init()
 
 bool CICYThermostat::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CICYThermostat::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CICYThermostat::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CICYThermostat::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -72,11 +74,9 @@ void CICYThermostat::Do_Work()
 {
 	int sec_counter = ICY_POLL_INTERVAL-5;
 	_log.Log(LOG_STATUS,"ICYThermostat: Worker started...");
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
-
 		if (sec_counter % 12 == 0)
 		{
 			m_LastHeartbeat = mytime(NULL);
@@ -89,7 +89,7 @@ void CICYThermostat::Do_Work()
 	_log.Log(LOG_STATUS,"ICYThermostat: Worker stopped...");
 }
 
-bool CICYThermostat::WriteToHardware(const char *pdata, const unsigned char length)
+bool CICYThermostat::WriteToHardware(const char* /*pdata*/, const unsigned char /*length*/)
 {
 	return false;
 }
@@ -176,8 +176,7 @@ bool CICYThermostat::GetSerialAndToken()
 
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "ICYThermostat: Invalid data received, or invalid username/password!");
@@ -242,8 +241,7 @@ void CICYThermostat::GetMeterDetails()
 
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "ICYThermostat: Invalid data received!");
@@ -293,7 +291,7 @@ void CICYThermostat::SetSetpoint(const int idx, const float temp)
 			_log.Log(LOG_ERROR,"ICYThermostat: Error setting SetPoint temperature!");
 		}
 		else {
-			_log.Log(LOG_STATUS,"ICYThermostat: Setting Room SetPoint to: %.1f",temp);
+			_log.Debug(DEBUG_HARDWARE, "ICYThermostat: Setting Room SetPoint to: %.1f", temp);
 		}
 	}
 }
