@@ -1,7 +1,6 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 
 class CICYThermostat : public CDomoticzHardwareBase
 {
@@ -14,26 +13,22 @@ class CICYThermostat : public CDomoticzHardwareBase
 public:
 	CICYThermostat(const int ID, const std::string &Username, const std::string &Password);
 	~CICYThermostat(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 	void SetSetpoint(const int idx, const float temp);
 private:
+	void Init();
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+	void GetMeterDetails();
 	void SendSetPointSensor(const unsigned char Idx, const float Temp, const std::string &defaultname);
-
 	bool GetSerialAndToken();
-
+private:
 	std::string m_UserName;
 	std::string m_Password;
 	std::string m_SerialNumber;
 	std::string m_Token;
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
+	std::shared_ptr<std::thread> m_thread;
 	_eICYCompanyMode m_companymode;
-
-	void Init();
-	bool StartHardware();
-	bool StopHardware();
-	void Do_Work();
-	void GetMeterDetails();
 };
 
