@@ -9,15 +9,14 @@ public:
 	KMTronic433(const int ID, const std::string& devname);
 	~KMTronic433();
 private:
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 
 	void GetRelayStates();
 	int m_iQueryState;
 	bool m_bHaveReceived;
 
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
+	std::shared_ptr<std::thread> m_thread;
 	int m_retrycntr;
 	void Do_Work();
 	bool OpenSerialDevice();
@@ -26,6 +25,6 @@ private:
 	* Read callback, stores data in the buffer
 	*/
 	void readCallback(const char *data, size_t len);
-	bool WriteInt(const unsigned char *data, const size_t len, const bool bWaitForReturn);
+	bool WriteInt(const unsigned char *data, const size_t len, const bool bWaitForReturn) override;
 };
 
