@@ -1,25 +1,32 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 
 class CLimitLess : public CDomoticzHardwareBase
 {
+public:
 	enum _eLimitlessBridgeType
 	{
 		LBTYPE_V4 = 0,
 		LBTYPE_V5,
 		LBTYPE_V6
 	};
-public:
 	CLimitLess(const int ID, const int LedType, const int BridgeType, const std::string &IPAddress, const unsigned short usIPPort);
 	~CLimitLess(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
 	bool AddSwitchIfNotExits(const unsigned char Unit, const std::string& devname);
 	bool GetV6BridgeID();
 	bool SendV6Command(const uint8_t *pCmd);
+	void Send_V6_RGBWW_On(const uint8_t dunit, const long delay);
+	void Send_V6_RGBW_On(const uint8_t dunit, const long delay);
+	void Send_V4V5_RGBW_On(const uint8_t dunit, const long delay);
 	bool IsDataAvailable(const SOCKET sock);
+	void Init();
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+private:
 	_eLimitlessBridgeType m_BridgeType;
 	unsigned char m_LEDType;
 
@@ -33,12 +40,6 @@ private:
 	SOCKET	m_RemoteSocket;
 	sockaddr_in m_stRemoteDestAddr;
 
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
-	void Init();
-	bool StartHardware();
-	bool StopHardware();
-	void Do_Work();
+	std::shared_ptr<std::thread> m_thread;
 };
 
