@@ -26,12 +26,20 @@
 //
 //Class Meteostick
 //
-Meteostick::Meteostick(const int ID, const std::string& devname, const unsigned int baud_rate)
+Meteostick::Meteostick(const int ID, const std::string& devname, const unsigned int baud_rate):
+	m_szSerialPort(devname)
 {
 	m_HwdID=ID;
-	m_szSerialPort=devname;
 	m_iBaudRate=baud_rate;
-	m_stoprequestedpoller = false;
+	m_state = MSTATE_INIT;
+	m_bufferpos = 0;
+	for (int ii = 0; ii < MAX_IDS; ii++)
+	{
+		m_LastOutsideTemp[ii] = 12345;
+		m_LastOutsideHum[ii] = 0;
+		m_ActRainCounter[ii] = -1;
+		m_LastRainValue[ii] = -1;
+	}
 }
 
 Meteostick::~Meteostick()
@@ -41,32 +49,26 @@ Meteostick::~Meteostick()
 
 bool Meteostick::StartHardware()
 {
+	RequestStart();
+
 	m_retrycntr = RETRY_DELAY; //will force reconnect first thing
-	StartPollerThread();
+
+	m_thread = std::make_shared<std::thread>(&Meteostick::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
+
 	return true;
 }
 
 bool Meteostick::StopHardware()
 {
-	m_bIsStarted = false;
-	terminate();
-	StopPollerThread();
-	return true;
-}
-
-void Meteostick::StartPollerThread()
-{
-	m_pollerthread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&Meteostick::Do_PollWork, this)));
-}
-
-void Meteostick::StopPollerThread()
-{
-	if (m_pollerthread != NULL)
+	if (m_thread)
 	{
-		assert(m_pollerthread);
-		m_stoprequestedpoller = true;
-		m_pollerthread->join();
+		RequestStop();
+		m_thread->join();
+		m_thread.reset();
 	}
+	m_bIsStarted = false;
+	return true;
 }
 
 bool Meteostick::OpenSerialDevice()
@@ -101,15 +103,12 @@ bool Meteostick::OpenSerialDevice()
 	m_bIsStarted = true;
 	m_bufferpos = 0;
 
-	int ii;
-
-	for (ii = 0; ii < MAX_IDS; ii++)
+	for (int ii = 0; ii < MAX_IDS; ii++)
 	{
 		m_LastOutsideTemp[ii]	= 12345;
 		m_LastOutsideHum[ii]	= 0;
 		m_ActRainCounter[ii]	= -1;
 		m_LastRainValue[ii]		= -1;
-
 	}
 	setReadCallback(boost::bind(&Meteostick::readCallback, this, _1, _2));
 	sOnConnected(this);
@@ -117,13 +116,11 @@ bool Meteostick::OpenSerialDevice()
 }
 
 
-void Meteostick::Do_PollWork()
+void Meteostick::Do_Work()
 {
-	bool bFirstTime = true;
 	int sec_counter = 0;
-	while (!m_stoprequestedpoller)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 
 		if (sec_counter % 12 == 0) {
@@ -140,18 +137,18 @@ void Meteostick::Do_PollWork()
 			if (m_retrycntr >= RETRY_DELAY)
 			{
 				m_retrycntr = 0;
-				if (OpenSerialDevice())
-					bFirstTime = true;
+				OpenSerialDevice();
 			}
 		}
 	}
+	terminate();
+
 	_log.Log(LOG_STATUS, "Meteostick: Worker stopped...");
 }
 
 
 void Meteostick::readCallback(const char *data, size_t len)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
 	if (!m_bIsStarted)
 		return;
 
@@ -161,7 +158,7 @@ void Meteostick::readCallback(const char *data, size_t len)
 	ParseData((const unsigned char*)data, static_cast<int>(len));
 }
 
-bool Meteostick::WriteToHardware(const char *pdata, const unsigned char length)
+bool Meteostick::WriteToHardware(const char* /*pdata*/, const unsigned char /*length*/)
 {
 	return false;
 }
@@ -236,13 +233,9 @@ void Meteostick::SendWindSensor(const unsigned char Idx, const float Temp, const
 	tsen.WIND.gusth = 0;
 	tsen.WIND.gustl = 0;
 
-	//this is not correct, why no wind temperature? and only chill?
 	tsen.WIND.chillh = 0;
 	tsen.WIND.chilll = 0;
-	//tsen.WIND.temperatureh = 0;
-	//tsen.WIND.temperaturel = 0;
-	//tsen.WIND.tempsign = (Temp >= 0) ? 0 : 1;
-	
+
 	float dWindSpeed = Speed * 3.6f;
 	float dWindChill = Temp;
 	if (dWindSpeed > 5 && Temp < 10)
@@ -254,10 +247,8 @@ void Meteostick::SendWindSensor(const unsigned char Idx, const float Temp, const
 	}
 	dWindChill*=10.0f;
 	tsen.WIND.chillsign = (dWindChill >= 0) ? 0 : 1;
-	//tsen.WIND.temperatureh = (BYTE)(dWindChill / 256);
 	tsen.WIND.chillh = (BYTE)(dWindChill / 256);
 	dWindChill -= (tsen.WIND.chillh * 256);
-	//tsen.WIND.temperaturel = (BYTE)(dWindChill);
 	tsen.WIND.chilll = (BYTE)(dWindChill);
 
 	sDecodeRXMessage(this, (const unsigned char *)&tsen.WIND, defaultname.c_str(), 255);
@@ -269,13 +260,12 @@ void Meteostick::SendLeafWetnessRainSensor(const unsigned char Idx, const unsign
 	_tGeneralDevice gdevice;
 	gdevice.subtype = sTypeLeafWetness;
 	gdevice.intval1 = Wetness;
-	gdevice.id = finalID;
+	gdevice.id = (uint8_t)finalID;
 	sDecodeRXMessage(this, (const unsigned char *)&gdevice, defaultname.c_str(), 255);
 }
 
 void Meteostick::SendSoilMoistureSensor(const unsigned char Idx, const unsigned char Channel, const int Moisture, const std::string &defaultname)
 {
-	bool bDeviceExits = true;
 	int finalID = (Idx * 10) + Channel;
 	SendMoistureSensor(finalID,255, Moisture, defaultname);
 }
@@ -304,7 +294,7 @@ void Meteostick::ParseLine()
 	{
 	case MSTATE_INIT:
 		if (sLine.find("# MeteoStick Version") == 0) {
-			_log.Log(LOG_STATUS, sLine.c_str());
+			_log.Log(LOG_STATUS, sLine);
 			return;
 		}
 		if (results[0] == "?")
@@ -342,7 +332,7 @@ void Meteostick::ParseLine()
 		return;
 
 //#ifdef _DEBUG
-	_log.Log(LOG_NORM, sLine.c_str());
+	_log.Log(LOG_NORM, sLine);
 //#endif
 
 	switch (rCode)
