@@ -1,7 +1,6 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 #include "hardwaretypes.h"
 
 class CNefitEasy : public CDomoticzHardwareBase
@@ -9,8 +8,22 @@ class CNefitEasy : public CDomoticzHardwareBase
 public:
 	CNefitEasy(const int ID, const std::string &IPAddress, const unsigned short usIPPort);
 	~CNefitEasy(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 	void SetSetpoint(const int idx, const float temp);
+private:
+	void Init();
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+
+	bool GetStatusDetails();
+	bool GetOutdoorTemp();
+	bool GetFlowTemp();
+	bool GetPressure();
+	bool GetDisplayCode();
+	bool GetGasUsage();
+	void SetUserMode(bool bSetUserModeClock);
+	void SetHotWaterMode(bool bTurnOn);
 private:
 	std::string m_AccessKey;
 	std::string m_SerialNumber;
@@ -28,30 +41,9 @@ private:
 	std::string m_LastBoilerStatus;
 	bool m_bClockMode;
 
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
-	void Init();
-	bool StartHardware();
-	bool StopHardware();
-	void Do_Work();
-
-	bool GetStatusDetails();
-	bool GetOutdoorTemp();
-	bool GetFlowTemp();
-	bool GetPressure();
-	bool GetDisplayCode();
-	bool GetGasUsage();
+	std::shared_ptr<std::thread> m_thread;
 
 	uint32_t m_lastgasusage;
 	P1Gas	m_p1gas;
-
-	void SetUserMode(bool bSetUserModeClock);
-	void SetHotWaterMode(bool bTurnOn);
-
-	//XMPP stuff
-	bool ConnectToXMPP(const std::string &IPAddress, const int PortNumber);
-	void Logout();
-	bool m_bDoLogin;
 };
 
