@@ -8,7 +8,7 @@
 #include "../main/SQLHelper.h"
 #include "../httpclient/HTTPClient.h"
 #include "../main/mainworker.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 
 #define round(a) ( int ) ( a + .5 )
 
@@ -18,6 +18,8 @@ const std::string NEST_SET_SHARED = "/v2/put/shared.";
 const std::string NEST_SET_STRUCTURE = "/v2/put/structure.";
 const std::string NEST_SET_DEVICE = "/v2/put/device.";
 
+#define NEST_USER_AGENT_STRING "User-Agent: Nest/3.0.1.15"
+
 #ifdef _DEBUG
 	//#define DEBUG_NextThermostatR
 	//#define DEBUG_NextThermostatW
@@ -69,43 +71,42 @@ void CNest::Init()
 {
 	m_AccessToken = "";
 	m_UserID = "";
-	m_stoprequested = false;
 	m_bDoLogin = true;
 }
 
 bool CNest::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CNest::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CNest::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CNest::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
-	if (!m_bDoLogin)
-		Logout();
     return true;
 }
 
-#define NEST_POLL_INTERVAL 30
+#define NEST_POLL_INTERVAL 60
 
 void CNest::Do_Work()
 {
 	_log.Log(LOG_STATUS,"Nest: Worker started...");
 	int sec_counter = NEST_POLL_INTERVAL-5;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 		if (sec_counter % 12 == 0)
 		{
@@ -118,6 +119,7 @@ void CNest::Do_Work()
 		}
 
 	}
+	Logout();
 	_log.Log(LOG_STATUS,"Nest: Worker stopped...");
 }
 
@@ -140,7 +142,6 @@ void CNest::SendSetPointSensor(const unsigned char Idx, const float Temp, const
 // Creates and updates switch used to log Heating and/or Colling.
 void CNest::UpdateSwitch(const unsigned char Idx, const bool bOn, const std::string &defaultname)
 {
-	bool bDeviceExits = true;
 	char szIdx[10];
 	sprintf(szIdx, "%X%02X%02X%02X", 0, 0, 0, Idx);
 	std::vector<std::vector<std::string> > result;
@@ -178,7 +179,7 @@ void CNest::UpdateSwitch(const unsigned char Idx, const bool bOn, const std::str
 		level = 15;
 		lcmd.LIGHTING2.cmnd = light2_sOn;
 	}
-	lcmd.LIGHTING2.level = level;
+	lcmd.LIGHTING2.level = (BYTE)level;
 	lcmd.LIGHTING2.filler = 0;
 	lcmd.LIGHTING2.rssi = 12;
 	sDecodeRXMessage(this, (const unsigned char *)&lcmd.LIGHTING2, defaultname.c_str(), 255);
@@ -197,7 +198,7 @@ bool CNest::Login()
 	sstr << "username=" << m_UserName << "&password=" << m_Password;
 	std::string szPostdata=sstr.str();
 	std::vector<std::string> ExtraHeaders;
-	ExtraHeaders.push_back("user-agent:Nest/1.1.0.10 CFNetwork/548.0.4");
+	ExtraHeaders.push_back(NEST_USER_AGENT_STRING);
 	std::string sResult;
 
 	std::string sURL = NEST_LOGIN_PATH;
@@ -208,8 +209,7 @@ bool CNest::Login()
 	}
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if ((!bRet) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "Nest: Invalid data received, or invalid username/password!");
@@ -233,7 +233,7 @@ bool CNest::Login()
 		return false;
 	}
 	m_AccessToken = root["access_token"].asString();
-	
+
 	if (root["userid"].empty())
 	{
 		_log.Log(LOG_ERROR, "Nest: Invalid data received, or invalid username/password!");
@@ -255,7 +255,7 @@ void CNest::Logout()
 }
 
 
-bool CNest::WriteToHardware(const char *pdata, const unsigned char length)
+bool CNest::WriteToHardware(const char *pdata, const unsigned char /*length*/)
 {
 	if (m_UserName.size() == 0)
 		return false;
@@ -273,13 +273,13 @@ bool CNest::WriteToHardware(const char *pdata, const unsigned char length)
 	if (node_id % 3 == 0)
 	{
 		//Away
-		return SetAway(node_id, bIsOn);
+		return SetAway((uint8_t)node_id, bIsOn);
 	}
 
 	if (node_id % 4 == 0)
 	{
 		//Manual Eco Mode
-		return SetManualEcoMode(node_id, bIsOn);
+		return SetManualEcoMode((uint8_t)node_id, bIsOn);
 	}
 
 	return false;
@@ -292,7 +292,7 @@ void CNest::UpdateSmokeSensor(const unsigned char Idx, const bool bOn, const std
 	sprintf(szIdx, "%X%02X%02X%02X", 0, 0, Idx, 0);
 	std::vector<std::vector<std::string> > result;
 	result = m_sql.safe_query("SELECT Name,nValue,sValue FROM DeviceStatus WHERE (HardwareID==%d) AND (DeviceID=='%q')", m_HwdID, szIdx);
-	if (result.size() < 1)
+	if (result.empty())
 	{
 		bDeviceExits = false;
 	}
@@ -342,7 +342,7 @@ void CNest::UpdateSmokeSensor(const unsigned char Idx, const bool bOn, const std
 		level = 15;
 		lcmd.LIGHTING2.cmnd = light2_sOn;
 	}
-	lcmd.LIGHTING2.level = level;
+	lcmd.LIGHTING2.level = (uint8_t)level;
 	lcmd.LIGHTING2.filler = 0;
 	lcmd.LIGHTING2.rssi = 12;
 
@@ -379,7 +379,7 @@ void CNest::GetMeterDetails()
 	}
 	std::vector<std::string> ExtraHeaders;
 
-	ExtraHeaders.push_back("user-agent:Nest/1.1.0.10 CFNetwork/548.0.4");
+	ExtraHeaders.push_back(NEST_USER_AGENT_STRING);
 	ExtraHeaders.push_back("Authorization:Basic " + m_AccessToken);
 	ExtraHeaders.push_back("X-nl-user-id:" + m_UserID);
 	ExtraHeaders.push_back("X-nl-protocol-version:1");
@@ -399,8 +399,7 @@ void CNest::GetMeterDetails()
 #endif
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if ((!bRet) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "Nest: Invalid data received!");
@@ -464,7 +463,7 @@ void CNest::GetMeterDetails()
 							}
 						}
 					}
-					
+
 				}
 			}
 			bool bIAlarm = false;
@@ -481,24 +480,28 @@ void CNest::GetMeterDetails()
 				if (!bBool)
 					bIAlarm = true;
 			}
+/*
 			if (!device["component_heat_test_passed"].empty())
 			{
 				bBool = device["component_heat_test_passed"].asBool();
 				if (!bBool)
 					bIAlarm = true;
 			}
+*/
 			if (!device["component_buzzer_test_passed"].empty())
 			{
 				bBool = device["component_buzzer_test_passed"].asBool();
 				if (!bBool)
 					bIAlarm = true;
 			}
+/*
 			if (!device["component_us_test_passed"].empty())
 			{
 				bBool = device["component_us_test_passed"].asBool();
 				if (!bBool)
 					bIAlarm = true;
 			}
+*/
 			if (!device["component_temp_test_passed"].empty())
 			{
 				bBool = device["component_temp_test_passed"].asBool();
@@ -529,7 +532,7 @@ void CNest::GetMeterDetails()
 				if (!bBool)
 					bIAlarm = true;
 			}
-			UpdateSmokeSensor(SwitchIndex, bIAlarm, devName);
+			UpdateSmokeSensor((uint8_t)SwitchIndex, bIAlarm, devName);
 			SwitchIndex++;
 		}
 	}
@@ -670,7 +673,7 @@ void CNest::SetSetpoint(const int idx, const float temp)
 
 	std::vector<std::string> ExtraHeaders;
 
-	ExtraHeaders.push_back("user-agent:Nest/1.1.0.10 CFNetwork/548.0.4");
+	ExtraHeaders.push_back(NEST_USER_AGENT_STRING);
 	ExtraHeaders.push_back("Authorization:Basic " + m_AccessToken);
 	ExtraHeaders.push_back("X-nl-protocol-version:1");
 
@@ -716,7 +719,7 @@ bool CNest::SetAway(const unsigned char Idx, const bool bIsAway)
 
 	std::vector<std::string> ExtraHeaders;
 
-	ExtraHeaders.push_back("user-agent:Nest/1.1.0.10 CFNetwork/548.0.4");
+	ExtraHeaders.push_back(NEST_USER_AGENT_STRING);
 	ExtraHeaders.push_back("Authorization:Basic " + m_AccessToken);
 	ExtraHeaders.push_back("X-nl-protocol-version:1");
 
@@ -756,7 +759,7 @@ bool CNest::SetManualEcoMode(const unsigned char Idx, const bool bIsManualEcoMod
 
 	std::vector<std::string> ExtraHeaders;
 
-	ExtraHeaders.push_back("user-agent:Nest/1.1.0.10 CFNetwork/548.0.4");
+	ExtraHeaders.push_back(NEST_USER_AGENT_STRING);
 	ExtraHeaders.push_back("Authorization:Basic " + m_AccessToken);
 	ExtraHeaders.push_back("X-nl-protocol-version:1");
 
@@ -782,7 +785,7 @@ bool CNest::SetManualEcoMode(const unsigned char Idx, const bool bIsManualEcoMod
 	return true;
 }
 
-void CNest::SetProgramState(const int newState)
+void CNest::SetProgramState(const int /*newState*/)
 {
 	if (m_UserName.size() == 0)
 		return;
