@@ -1,9 +1,7 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 #include "hardwaretypes.h"
-#include <map>
 
 class CNest : public CDomoticzHardwareBase
 {
@@ -16,10 +14,15 @@ class CNest : public CDomoticzHardwareBase
 public:
 	CNest(const int ID, const std::string &Username, const std::string &Password);
 	~CNest(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 	void SetSetpoint(const int idx, const float temp);
 	void SetProgramState(const int newState);
 private:
+	void Init();
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+	void GetMeterDetails();
 	void SendSetPointSensor(const unsigned char Idx, const float Temp, const std::string &defaultname);
 	void UpdateSwitch(const unsigned char Idx, const bool bOn, const std::string &defaultname);
 	void UpdateSmokeSensor(const unsigned char Idx, const bool bOn, const std::string &defaultname);
@@ -27,7 +30,7 @@ private:
 	bool SetManualEcoMode(const unsigned char Idx, const bool bIsManualEcoMode);
 	bool Login();
 	void Logout();
-
+private:
 	std::string m_UserName;
 	std::string m_Password;
 	std::string m_TransportURL;
@@ -35,15 +38,8 @@ private:
 	std::string m_UserID;
 //	std::string m_Serial;
 	//std::string m_StructureID;
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
+	std::shared_ptr<std::thread> m_thread;
 	std::map<int, _tNestThemostat> m_thermostats;
 	bool m_bDoLogin;
-
-	void Init();
-	bool StartHardware();
-	bool StopHardware();
-	void Do_Work();
-	void GetMeterDetails();
 };
 
