@@ -6,7 +6,7 @@
 #include "hardwaretypes.h"
 #include "../main/localtime_r.h"
 #include "../httpclient/HTTPClient.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 #include "../main/RFXtrx.h"
 #include "../main/mainworker.h"
 #include "../main/SQLHelper.h"
@@ -55,7 +55,6 @@ COpenWeatherMap::COpenWeatherMap(const int ID, const std::string &APIKey, const
 	m_Language("en")
 {
 	m_HwdID=ID;
-	m_stoprequested=false;
 
 	m_bHaveGPSCoordinated = (Location.find("lat=") != std::string::npos);
 
@@ -73,20 +72,23 @@ COpenWeatherMap::~COpenWeatherMap(void)
 
 bool COpenWeatherMap::StartHardware()
 {
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&COpenWeatherMap::Do_Work, this)));
+	RequestStart();
+
+	m_thread = std::make_shared<std::thread>(&COpenWeatherMap::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
 	_log.Log(LOG_STATUS, "OpenWeatherMap: Started");
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool COpenWeatherMap::StopHardware()
 {
-	m_stoprequested = true;
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
 	return true;
@@ -95,9 +97,8 @@ bool COpenWeatherMap::StopHardware()
 void COpenWeatherMap::Do_Work()
 {
 	int sec_counter = 595;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 		if (sec_counter % 12 == 0) {
 			m_LastHeartbeat = mytime(NULL);
@@ -117,7 +118,7 @@ void COpenWeatherMap::Do_Work()
 	_log.Log(LOG_STATUS,"OpenWeatherMap: Worker stopped...");
 }
 
-bool COpenWeatherMap::WriteToHardware(const char *pdata, const unsigned char length)
+bool COpenWeatherMap::WriteToHardware(const char* /*pdata*/, const unsigned char /*length*/)
 {
 	return false;
 }
@@ -143,10 +144,7 @@ void COpenWeatherMap::GetMeterDetails()
 
 	try
 	{
-		bool bret;
-		std::string szURL = sURL.str();
-		bret = HTTPClient::GET(szURL, sResult);
-		if (!bret)
+		if (!HTTPClient::GET(sURL.str(), sResult))
 		{
 			_log.Log(LOG_ERROR, "OpenWeatherMap: Error getting http data!");
 			return;
@@ -164,8 +162,7 @@ void COpenWeatherMap::GetMeterDetails()
 
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret=jReader.parse(sResult,root);
+	bool ret= ParseJSon(sResult,root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR,"OpenWeatherMap: Invalid data received!");
@@ -189,13 +186,12 @@ void COpenWeatherMap::GetMeterDetails()
 		return;
 	}
 
-	float temp =- 999.9f;
-	int humidity = 0;
-	int barometric = 0;
-	int barometric_forecast = baroForecastNoInfo;
-
+	float temp = -999.9f;
 	if (!root["main"].empty())
 	{
+		int humidity = 0;
+		int barometric = 0;
+		int barometric_forecast = baroForecastNoInfo;
 		if (!root["main"]["temp"].empty())
 		{
 			temp = root["main"]["temp"].asFloat();
@@ -268,7 +264,11 @@ void COpenWeatherMap::GetMeterDetails()
 			windspeed_ms = root["wind"]["speed"].asFloat();
 		}
 		if ((wind_degrees != -1) && (windspeed_ms != -1))
-			SendWind(1, 255, wind_degrees, windspeed_ms, 0, 0, 0, false, "Wind");
+		{
+			bool bHaveTemp = (temp != -999.9f);
+			float rTemp = (temp != -999.9f) ? temp : 0;
+			SendWind(1, 255, wind_degrees, windspeed_ms, 0, rTemp, 0, bHaveTemp, false, "Wind");
+		}
 	}
 
 	//Rain
@@ -319,6 +319,16 @@ void COpenWeatherMap::GetMeterDetails()
 		}
 	}
 
+	//clouds
+	if (!root["clouds"].empty())
+	{
+		if (!root["clouds"]["all"].empty())
+		{
+			float clouds = root["clouds"]["all"].asFloat();
+			SendPercentageSensor(1, 0, 255, clouds, "Clouds %");
+		}
+	}
+
 	//Forecast URL
 	if (!root["id"].empty())
 	{
