@@ -9,17 +9,6 @@ class P1MeterSerial: public AsyncSerial, public P1MeterBase
 {
 public:
 	P1MeterSerial(const int ID, const std::string& devname, const unsigned int baud_rate, const bool disable_crc, const int ratelimit);
-    /**
-    * Opens a serial device.
-    * \param devname serial device name, example "/dev/ttyS0" or "COM1"
-    * \param baud_rate serial baud rate
-    * \param opt_parity serial parity, default even
-    * \param opt_csize serial character size, default 7bit
-    * \param opt_flow serial flow control, default none
-    * \param opt_stop serial stop bits, default 1
-    * \throws boost::system::system_error if cannot open the
-    * serial device
-    */
     P1MeterSerial(const std::string& devname, unsigned int baud_rate,
         boost::asio::serial_port_base::parity opt_parity=
             boost::asio::serial_port_base::parity(
@@ -32,23 +21,17 @@ public:
         boost::asio::serial_port_base::stop_bits opt_stop=
             boost::asio::serial_port_base::stop_bits(
                 boost::asio::serial_port_base::stop_bits::one));
-
     ~P1MeterSerial();
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
+private:
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+	void readCallback(const char *data, size_t len);
+private:
 	std::string m_szSerialPort;
 	unsigned int m_iBaudRate;
-	bool WriteToHardware(const char *pdata, const unsigned char length);
-private:
-	bool StartHardware();
-	bool StopHardware();
-
-	void Do_Work();
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
-
-    /**
-     * Read callback, stores data in the buffer
-     */
-    void readCallback(const char *data, size_t len);
+	std::shared_ptr<std::thread> m_thread;
 };
 
 #endif //BUFFEREDP1ASYNCSERIAL_H
