@@ -1,24 +1,23 @@
 #pragma once
 
 #include "DomoticzHardware.h"
-#include <iostream>
 
 class CPVOutputInput : public CDomoticzHardwareBase
 {
 public:
 	CPVOutputInput(const int ID, const std::string &SID, const std::string &Key);
 	~CPVOutputInput(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
+private:
+	void Init();
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+	void GetMeterDetails();
 private:
 	std::string m_SID;
 	std::string m_KEY;
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
-	void Init();
-	bool StartHardware();
-	bool StopHardware();
-	void Do_Work();
-	void GetMeterDetails();
+	std::shared_ptr<std::thread> m_thread;
+	bool m_bHadConsumption = false;
 };
 
