@@ -4,29 +4,29 @@
  Date: 01-10-2013
  Written by: Robin Stevenhagen
  License: Public domain
- 
+
  Date: 18-06-2015
  GizMoCuz: Added messages to a send queue
- 
+
  Date: 08-04-2016
  BobKersten: Added multiple counter types
- 
+
  SPI code is based on code from:
  The WiringPi project
  *    Copyright (c) 2012 Gordon Henderson
  ***********************************************************************
  * See    https://projects.drogon.net/raspberry-pi/wiringpi/
  ***********************************************************************
- 
+
  Code uses SPI dev 0.0, as this is the interface were the PiFace resides.
  PiFace has 8 Input (4 buttons) and 8 Output pins, of which output 0 and 1 are relay outputs.
  Driver autodetects all available PiFace boards on startup, and scans them.
- 
+
  Default config (without config):
  Input 0..3 enabled with rising edge detection
  Input 4..7 enabled for pulse counting
  Output 0..7 enbled with level feedback reporting.
- 
+
  ************************************************************************/
 
 #include "stdafx.h"
@@ -74,10 +74,9 @@ const std::string CPiFace::ParameterCountTypeValueNames[CONFIG_NR_OF_PARAMETER_C
 
 CPiFace::CPiFace(const int ID)
 {
-    m_stoprequested=false;
     m_HwdID=ID;
     m_bIsStarted=false;
-    
+
     for (int i = 0; i < 4; i++)
     {
         m_Inputs[i].Callback_pntr = (void*)this; // Set callback pointer, to enable events from IO class
@@ -90,7 +89,7 @@ CPiFace::CPiFace(const int ID)
 
 CPiFace::~CPiFace()
 {
-    
+
 }
 
 
@@ -121,12 +120,12 @@ std::string & CPiFace::preprocess(std::string &s)
 {
     std::string temp;
     std::string tempstripped;
-    
+
     temp.resize(s.size());
     std::transform(s.begin(),s.end(),temp.begin(),::tolower);
-    
+
     tempstripped = trim(temp);
-    
+
     s=tempstripped;
     return s;
 }
@@ -135,7 +134,7 @@ int CPiFace::LocateValueInParameterArray(std::string Parametername,const std::st
 {
     int NameFound=-1; //assume not found
     int Parameter_Index;
-    
+
     for (Parameter_Index=0;(Parameter_Index < Items) && (NameFound==-1);Parameter_Index++)
     {
         if (Parametername.compare(ParameterArray[Parameter_Index])==0)
@@ -151,7 +150,7 @@ int CPiFace::GetParameterString(std::string TargetString,const char * SearchStr,
     int EndPos=-1;
     std::string Substring;
     std::string SearchString(SearchStr);
-    
+
     EndPos=TargetString.find(SearchString,StartPos);
     if (EndPos >= 0)
     {
@@ -166,19 +165,19 @@ int CPiFace::GetParameterString(std::string TargetString,const char * SearchStr,
 int CPiFace::LoadConfig(void)
 {
     int result=-1;
-    
+
     int StartPos=-1;
     int EndPos=-1;
-    
+
     std::string input;
-    
+
     std::string Line;
     std::string Parameter;
     std::string Porttype;
     std::string Pinnumber;
     std::string Parametername;
     std::string Parametervalue;
-    
+
     unsigned char Address=0;
     unsigned char PinNumber=0;
     unsigned char PortType=0;
@@ -186,11 +185,11 @@ int CPiFace::LoadConfig(void)
     int ValueFound=0;
     CIOPort *IOport=NULL;
     bool Regenerate_Config=false;
-    
+
     std::string configfile=szUserDataFolder + "piface.conf";
-    
-    fstream ConfigFile(configfile.c_str(), ios::in);
-    
+
+    std::fstream ConfigFile(configfile.c_str(), std::ios::in);
+
     if (!ConfigFile.is_open())
     {
         //try to create one for the default board 0
@@ -199,10 +198,10 @@ int CPiFace::LoadConfig(void)
         _log.Log(LOG_ERROR,"PiFace: Auto creating for board 0");
         LoadDefaultConfig();
         AutoCreate_piface_config();
-        fstream ConfigFile(configfile.c_str(), ios::in);
+        std::fstream ConfigFile(configfile.c_str(), std::ios::in);
     }
-    
-    
+
+
     if (ConfigFile.is_open())
     {
         while (ConfigFile.good()==true)
@@ -213,7 +212,7 @@ int CPiFace::LoadConfig(void)
                 EndPos=-1;
                 getline(ConfigFile,input,'\n');    //get line from config file
                 Line=preprocess(input);
-                
+
                 //find any comments
                 StartPos=Line.find("//",0);
                 if (StartPos>=0)
@@ -222,7 +221,7 @@ int CPiFace::LoadConfig(void)
                     Line=Line.substr(0,StartPos);
                     StartPos=-1; // reset marker
                 }
-                
+
                 //find linesync
                 EndPos=GetParameterString(Line,".",0,Parameter);
                 if (EndPos>=0)
@@ -234,7 +233,7 @@ int CPiFace::LoadConfig(void)
                 }
             }
             while ((StartPos<0) && (ConfigFile.good()==true));
-            
+
             if (StartPos > 0)
             {
                 //first lets get PiFace Address
@@ -244,7 +243,7 @@ int CPiFace::LoadConfig(void)
                     Address=(unsigned char)(strtol(Parameter.c_str(),NULL,0) & 0xFF); //data can be restricted further but as we check later on keep it wide for future use.
                     StartPos=EndPos;
                 }
-                
+
                 //Now lets get Port Type
                 EndPos=GetParameterString(Line,".",StartPos,Parameter);
                 if (EndPos>=0)
@@ -259,10 +258,10 @@ int CPiFace::LoadConfig(void)
                         // we have an output
                         PortType='O';
                     }
-                    
+
                     StartPos=EndPos;
                 }
-                
+
                 //Now lets get Pinnumber
                 EndPos=GetParameterString(Line,".",StartPos,Parameter);
                 if (EndPos>=0)
@@ -270,7 +269,7 @@ int CPiFace::LoadConfig(void)
                     PinNumber=(unsigned char)(strtol(Parameter.c_str(),NULL,0) & 0xFF); //data can be restricted further but as we check later on keep it wide for future use.
                     StartPos=EndPos;
                 }
-                
+
                 //Now lets get parameter name
                 EndPos=GetParameterString(Line,"=",StartPos,Parameter);
                 if (EndPos>=0)
@@ -278,16 +277,16 @@ int CPiFace::LoadConfig(void)
                     Parametername=Parameter;
                     StartPos=EndPos;
                 }
-                
+
                 //finaly lets get parameter value
                 Parametervalue=Line.substr(StartPos,Line.length()-StartPos);
                 Parametervalue=preprocess(Parametervalue);
-                
+
                 if ((Address <= 3) && (PinNumber <= 7) && ((PortType=='I') || (PortType=='O')) && (Parametername.length() >0 ) && (Parametervalue.length() >0 ))
                 {
                     _log.Log(LOG_STATUS,"PiFace: config file: Valid address: %d , Pin: %d and Port %c Parameter: %s , Value %s",Address,PinNumber,PortType,Parametername.c_str(),Parametervalue.c_str());
                     NameFound=LocateValueInParameterArray(Parametername,ParameterNames,CONFIG_NR_OF_PARAMETER_TYPES);
-                    
+
                     if (PortType=='I')
                     {
                         IOport=&m_Inputs[Address];
@@ -297,9 +296,9 @@ int CPiFace::LoadConfig(void)
                         IOport=&m_Outputs[Address];
                         m_Outputs[Address].Pin[PinNumber].Direction = 'O';
                     }
-                    
+
                     result=0;
-                    
+
                     switch (NameFound)
                     {
                         default:
@@ -322,7 +321,7 @@ int CPiFace::LoadConfig(void)
                             }
                             else _log.Log(LOG_ERROR,"PiFace: Error config file: unknown value %s found", Parametervalue.c_str() );
                             break;
-                            
+
                         case 2:
                             //found pintype
                             ValueFound=LocateValueInParameterArray(Parametervalue,ParameterPinTypeValueNames,CONFIG_NR_OF_PARAMETER_PIN_TYPES);
@@ -332,7 +331,7 @@ int CPiFace::LoadConfig(void)
                                     _log.Log(LOG_ERROR,"PiFace: Error config file: unknown value %s found =>setting default level %d", Parametervalue.c_str(),ValueFound );
                                     IOport->Pin[PinNumber].Type=LEVEL;
                                     break;
-                                    
+
                                 case 0:
                                     IOport->Pin[PinNumber].Type=LEVEL;
                                     break;
@@ -348,12 +347,12 @@ int CPiFace::LoadConfig(void)
                             }
                             result++;
                             break;
-                            
+
                         case 3:
                         case 4:
                             //found count_enable(d)
                             ValueFound=LocateValueInParameterArray(Parametervalue,ParameterBooleanValueNames,CONFIG_NR_OF_PARAMETER_BOOL_TYPES);
-                            
+
                             if (ValueFound >=0)
                             {
                                 if ((ValueFound == 0) || (ValueFound == 1))
@@ -367,13 +366,13 @@ int CPiFace::LoadConfig(void)
                             }
                             else _log.Log(LOG_ERROR,"PiFace: Error config file: unknown value %s found", Parametervalue.c_str() );
                             break;
-                            
+
                         case 5:
                         case 6:
                         case 7:
                             //count_update_interval(_s)(ec)
                             unsigned long UpdateInterval;
-                            
+
                             UpdateInterval=strtol(Parametervalue.c_str(),NULL,0);
                             IOport->Pin[PinNumber].Count.SetUpdateInterval(UpdateInterval*1000);
                             result++;
@@ -382,7 +381,7 @@ int CPiFace::LoadConfig(void)
                         case 8:
                             //count_update_interval_diff_perc
                             unsigned long UpdateIntervalPerc;
-                            
+
                             UpdateIntervalPerc=strtol(Parametervalue.c_str(),NULL,0);
                             if ( UpdateIntervalPerc < 1 || UpdateIntervalPerc > 1000 )
                             {
@@ -391,13 +390,13 @@ int CPiFace::LoadConfig(void)
                             }
                             IOport->Pin[PinNumber].Count.SetUpdateIntervalPerc(UpdateIntervalPerc);
                             result++;
-                            
+
 #ifndef DISABLE_NEW_FUNCTIONS
                             /*  disabled until code part is completed and tested */
                         case 9:
                             //count_initial_value
                             unsigned long StartValue;
-                            
+
                             StartValue=strtol(Parametervalue.c_str(),NULL,0);
                             IOport->Pin[PinNumber].Count.SetTotal(StartValue);
                             result++;
@@ -407,12 +406,12 @@ int CPiFace::LoadConfig(void)
                         case 10:
                             //count_minimum_pulse_period_msec
                             unsigned long Min_Pulse_Period;
-                            
+
                             Min_Pulse_Period=strtol(Parametervalue.c_str(),NULL,0); // results in 0 if str is invalid
                             IOport->Pin[PinNumber].Count.SetRateLimit(Min_Pulse_Period);
                             result++;
                             break;
-                        
+
                         case 11:
                             //count_type
                             ValueFound=LocateValueInParameterArray(Parametervalue,ParameterCountTypeValueNames,CONFIG_NR_OF_PARAMETER_COUNT_TYPES);
@@ -421,7 +420,7 @@ int CPiFace::LoadConfig(void)
                                 default:
                                     _log.Log(LOG_ERROR,"PiFace: Error config file: unknown value %s found", Parametervalue.c_str());
                                     break;
-                                    
+
                                 case 0: // generic
                                     IOport->Pin[PinNumber].Count.Type = COUNT_TYPE_GENERIC;
                                     break;
@@ -434,7 +433,7 @@ int CPiFace::LoadConfig(void)
                             }
                             result++;
                             break;
-                        
+
                         case 12:
                             //count_divider
                             IOport->Pin[PinNumber].Count.SetDivider(strtol(Parametervalue.c_str(), NULL, 0));
@@ -477,7 +476,7 @@ void CPiFace::LoadDefaultConfig(void)
             m_Inputs[i].Pin[PinNr].Type=TOGGLE_RISING;
             m_Inputs[i].Pin[PinNr].Count.SetUpdateInterval(10000);
             m_Inputs[i].Pin[PinNr].Direction = 'I';
-            
+
             if (PinNr >= 4)
             {
                 m_Inputs[i].ConfigureCounter(PinNr,true);
@@ -569,10 +568,10 @@ void CPiFace::AutoCreate_piface_config(void)
     std::string PortType;
     int Value;
     CIOPort *IOport;
-    
+
     std::string configfile=szUserDataFolder + "piface.conf";
-    fstream ConfigFile(configfile.c_str(), ios::out);
-    
+    std::fstream ConfigFile(configfile.c_str(), std::ios::out);
+
 	int total_explanations = sizeof(explanation);
 
     if (ConfigFile.is_open())
@@ -583,7 +582,7 @@ void CPiFace::AutoCreate_piface_config(void)
             ConfigFile.write(explanation[i],strlen(explanation[i]));
 			i++;
         }
-        
+
         for (int BoardNr=0; BoardNr< 1 ;BoardNr++) // Note there could be 4 boards, but this will create a lot of entires in the configfile
         {
             for (int PinNr=0; PinNr<=7 ;PinNr++)
@@ -599,17 +598,17 @@ void CPiFace::AutoCreate_piface_config(void)
                         PortType="input";
                         IOport=&m_Inputs[BoardNr];
                     }
-                    
+
                     if (IOport->Pin[PinNr].Enabled)
                         ValueText="true";
                     else ValueText="false";
                     sprintf(configline,"piface.%d.%s.%d.enabled=%s\r\n",BoardNr,PortType.c_str(),PinNr,ValueText.c_str());
                     ConfigFile.write(configline,strlen(configline));
-                    
+
                     ValueText=ParameterPinTypeValueNames[IOport->Pin[PinNr].Type];
                     sprintf(configline,"piface.%d.%s.%d.pin_type=%s\r\n",BoardNr,PortType.c_str(),PinNr,ValueText.c_str());
                     ConfigFile.write(configline,strlen(configline));
-                    
+
                     if (IOport->Pin[PinNr].Count.Enabled)
                         ValueText="true";
                     else ValueText="false";
@@ -628,7 +627,7 @@ void CPiFace::AutoCreate_piface_config(void)
                         sprintf(configline,"piface.%d.%s.%d.count_divider=%d\r\n",BoardNr,PortType.c_str(),PinNr,Value);
                         ConfigFile.write(configline,strlen(configline));
                     }
-                    
+
                     Value=IOport->Pin[PinNr].Count.GetUpdateInterval()/1000;
                     sprintf(configline,"piface.%d.%s.%d.count_update_interval_sec=%d\r\n",BoardNr,PortType.c_str(),PinNr,Value);
                     ConfigFile.write(configline,strlen(configline));
@@ -638,19 +637,19 @@ void CPiFace::AutoCreate_piface_config(void)
                         sprintf(configline,"piface.%d.%s.%d.count_update_interval_diff_perc=%d\r\n",BoardNr,PortType.c_str(),PinNr,Value);
                         ConfigFile.write(configline,strlen(configline));
                     }
-                    
+
                     Value=IOport->Pin[PinNr].Count.GetRateLimit();
                     if ( Value > 0 ) {
                         sprintf(configline,"piface.%d.%s.%d.count_minimum_pulse_period_msec=%d\r\n",BoardNr,PortType.c_str(),PinNr,Value);
                         ConfigFile.write(configline,strlen(configline));
                     }
-                    
+
                     sprintf(configline,"\r\n");
                     ConfigFile.write(configline,strlen(configline));
                 }
             }
         }
-        
+
     }
     ConfigFile.close();
 }
@@ -661,7 +660,7 @@ void CPiFace::CallBackSendEvent(const unsigned char *pEventPacket, const unsigne
 {
     std::string sendData;
     sendData.insert(sendData.begin(), pEventPacket, pEventPacket + PacketLength);
-    boost::lock_guard<boost::mutex> l(m_queue_mutex);
+    std::lock_guard<std::mutex> l(m_queue_mutex);
     if (m_send_queue.size() < 100)
         m_send_queue.push_back(sendData);
     else
@@ -675,17 +674,17 @@ void CPiFace::CallBackSetPinInterruptMode(unsigned char devId,unsigned char pinI
     //make sure that we clear the interrupts states before we update the port
     Read_MCP23S17_Register (devId,MCP23x17_INTFB);
     Read_MCP23S17_Register (devId,MCP23x17_INTCAPB);
-    
+
     pinmask=1;
     pinmask<<=pinID;
     Cur_Int_Enable_State=Read_MCP23S17_Register (devId,MCP23x17_GPINTENB);
-    
+
     if (Interrupt_Enable)
         Cur_Int_Enable_State|=pinmask; //enable pin interrupt
     else Cur_Int_Enable_State&=~pinmask; //disable pin interrupt
-    
+
     Write_MCP23S17_Register (devId, MCP23x17_GPINTENB, Cur_Int_Enable_State);
-    
+
     //_log.Log(LOG_NORM,"PiFace: SetPin Interrupt mode: devid: %d, Pinnr: %d, Enable %d-- Prev 0x%02X Cur 0x%02X",devId,pinID,Interrupt_Enable,Prev_Int_Enable_State,Cur_Int_Enable_State);
 }
 
@@ -693,64 +692,74 @@ void CPiFace::CallBackSetPinInterruptMode(unsigned char devId,unsigned char pinI
 bool CPiFace::StartHardware()
 {
     StopHardware();
+
+	RequestStart();
+	m_TaskQueue.RequestStart();
+
     m_InputSample_waitcntr=(PIFACE_INPUT_PROCESS_INTERVAL)*20;
     m_CounterEdgeSample_waitcntr=(PIFACE_COUNTER_COUNTER_INTERVAL)*20;
-    
-    m_stoprequested=false;
+
     memset(m_DetectedHardware,0,sizeof(m_DetectedHardware));
-    
+
 #ifndef DISABLE_NEW_FUNCTIONS
     LoadConfig();
 #endif
     //Start worker thread
-    if (Init_SPI_Device(1) > 0)
-    {
-        if (Detect_PiFace_Hardware() > 0)
-        {
-            //we have hardware, so lets use it
-            for (int devId=0;  devId<4 ;devId++)
-            {
-                if (m_DetectedHardware[devId]==true)
-                    Init_Hardware(devId);
-            }
+	if (Init_SPI_Device(1) > 0)
+	{
+		if (Detect_PiFace_Hardware() > 0)
+		{
+			//we have hardware, so lets use it
+			for (int devId = 0; devId < 4; devId++)
+			{
+				if (m_DetectedHardware[devId] == true)
+					Init_Hardware(devId);
+			}
 #ifdef DISABLE_NEW_FUNCTIONS
-            LoadConfig();
+			LoadConfig();
 #endif
 
-            for (int devId=0;  devId<4 ;devId++)
-            {
-                if (m_DetectedHardware[devId]==true)
-                    GetAndSetInitialDeviceState(devId);
-            }
-            
-            m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CPiFace::Do_Work, this)));
-            m_queue_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CPiFace::Do_Work_Queue, this)));
-        }
-        else m_stoprequested=true;
-    }
-    else m_stoprequested=true;
+			for (int devId = 0; devId < 4; devId++)
+			{
+				if (m_DetectedHardware[devId] == true)
+					GetAndSetInitialDeviceState(devId);
+			}
+
+			m_thread = std::make_shared<std::thread>(&CPiFace::Do_Work, this);
+			SetThreadNameInt(m_thread->native_handle());
+			m_queue_thread = std::make_shared<std::thread>(&CPiFace::Do_Work_Queue, this);
+			SetThreadName(m_queue_thread->native_handle(), "PiFaceQueue");
+		}
+		else
+		{
+			return false;
+		}
+	}
+	else
+	{
+		return false;
+	}
     m_bIsStarted=true;
     sOnConnected(this);
-    return (m_thread!=NULL);
+    return (m_thread != nullptr);
 }
 
 bool CPiFace::StopHardware()
 {
-    m_stoprequested=true;
-    if (m_thread!=NULL)
-    {
-        m_stoprequested = true;
-        if (m_thread != NULL)
-        {
-            m_thread->join();
-            m_thread.reset();
-        }
-        if (m_queue_thread != NULL)
-        {
-            m_queue_thread->join();
-            m_queue_thread.reset();
-        }
-    }
+	RequestStop();
+	m_TaskQueue.RequestStop();
+
+	if (m_thread)
+	{
+		m_thread->join();
+		m_thread.reset();
+	}
+
+	if (m_queue_thread)
+	{
+		m_queue_thread->join();
+		m_queue_thread.reset();
+	}
 #ifdef HAVE_LINUX_SPI
     if (m_fd > 0) {
         close(m_fd);
@@ -768,10 +777,10 @@ bool CPiFace::WriteToHardware(const char *pdata, const unsigned char length)
     unsigned char CurrentLatchState;
     unsigned char OutputData;
     unsigned char PortType=' ';
-    
+
     int devId =0;
     int pinnr =0;
-    
+
     if ((SendData->LIGHTING1.packettype == pTypeLighting1) && (SendData->LIGHTING1.subtype == sTypeIMPULS))
     {
         PortType=(SendData->LIGHTING1.housecode);
@@ -782,16 +791,16 @@ bool CPiFace::WriteToHardware(const char *pdata, const unsigned char length)
             //_log.Log(LOG_NORM,"Piface: WriteToHardware housecode %c, packetlength %d", SendData->LIGHTING1.housecode,SendData->LIGHTING1.packetlength );
             CurrentLatchState = Read_MCP23S17_Register(devId, MCP23x17_OLATA);
             //_log.Log(LOG_NORM,"PiFace: Read input state 0x%02X", m_OutputState[devId].Current);
-            
+
             OutputData = CurrentLatchState;
             mask <<= pinnr;
-            
+
             if (SendData->LIGHTING1.cmnd == light1_sOff)
             {
                 OutputData &= ~mask;
             }
             else OutputData |= mask;
-            
+
             Write_MCP23S17_Register(devId, MCP23x17_GPIOA, OutputData);
             //  _log.Log(LOG_NORM,"Piface: WriteToHardware housecode %c, devid %d, output %d, PrevOut 0x%02X, Set 0x%02X",PortType, devId, pinnr, CurrentLatchState,OutputData );
         }
@@ -815,12 +824,8 @@ void CPiFace::Do_Work()
     _log.Log(LOG_STATUS,"PiFace: Worker started...");
     int msec_counter = 0;
     int sec_counter = 0;
-    while (!m_stoprequested)
+    while (!IsStopRequested(PIFACE_WORKER_THREAD_SLEEP_INTERVAL_MS))
     {
-        boost::this_thread::sleep(boost::posix_time::millisec(PIFACE_WORKER_THREAD_SLEEP_INTERVAL_MS));
-        if (m_stoprequested)
-            break;
-        
         msec_counter++;
         if (msec_counter == (1000 / PIFACE_WORKER_THREAD_SLEEP_INTERVAL_MS))
         {
@@ -830,10 +835,10 @@ void CPiFace::Do_Work()
                 m_LastHeartbeat = mytime(NULL);
             }
         }
-        
+
         m_InputSample_waitcntr++;
         m_CounterEdgeSample_waitcntr++;
-        
+
         //sample interrupt states faster for edge detection on count
         if (m_CounterEdgeSample_waitcntr>=PIFACE_COUNTER_COUNTER_INTERVAL)
         {
@@ -843,7 +848,7 @@ void CPiFace::Do_Work()
                 Sample_and_Process_Input_Interrupts(devId);
             }
         }
-        
+
         //sample pin states more slowly to avoid spamming Domoticz
         if (m_InputSample_waitcntr>=PIFACE_INPUT_PROCESS_INTERVAL)
         {
@@ -861,14 +866,11 @@ void CPiFace::Do_Work()
 void CPiFace::Do_Work_Queue()
 {
     std::vector<std::string>::iterator itt;
-    while (!m_stoprequested)
+    while (!m_TaskQueue.IsStopRequested(100))
     {
-        boost::this_thread::sleep(boost::posix_time::millisec(100));
-        if (m_stoprequested)
-            break;
         if (m_send_queue.empty())
             continue;
-        
+
         std::string sendData;
         m_queue_mutex.lock();
         itt = m_send_queue.begin();
@@ -888,7 +890,7 @@ int CPiFace::Init_SPI_Device(int Init)
     unsigned char spiMode  = 0 ;
     unsigned char spiBPW   = 8 ;
     int           speed       = 4000000 ;
-    
+
     _log.Log(LOG_STATUS,"PiFace: Starting PiFace_SPI_Start()");
 #ifdef HAVE_LINUX_SPI
     // Open port for reading and writing
@@ -904,23 +906,23 @@ int CPiFace::Init_SPI_Device(int Init)
                     result=1;
                     //we are successfull
                     _log.Log(LOG_NORM,"PiFace: SPI device opened successfully");
-                    
+
                 }
                 else
                     _log.Log(LOG_NORM,"PiFace: SPI Speed Change failure: %s", strerror (errno)) ;
             }
             else
                 _log.Log(LOG_NORM,"PiFace: SPI BPW Change failure: %s", strerror (errno)) ;
-            
+
         }
         else
             _log.Log(LOG_NORM,"PiFace: SPI Mode Change failure: %s", strerror (errno)) ;
     }
     else
         _log.Log(LOG_NORM,"PiFace: Unable to open SPI device: %s", strerror (errno));
-    
+
 #endif
-    
+
     if (result == -1)
     {
 #ifdef HAVE_LINUX_SPI
@@ -937,21 +939,21 @@ int CPiFace::Detect_PiFace_Hardware(void)
     unsigned char read_iocon;
     unsigned char read_ioconb;
     int devId;
-    
+
     //first write to all possible PiFace addresses.
     for (devId=0; devId<4; devId++)
     {
         Write_MCP23S17_Register (devId, MCP23x17_IOCON,  IOCON_INIT | IOCON_HAEN) ;
         Write_MCP23S17_Register (devId, MCP23x17_IOCONB,  IOCON_INIT | IOCON_HAEN) ;
     }
-    
+
     //now read them back, to determine if they are present
-    
+
     for (devId=0; devId<4; devId++)
     {
         read_iocon=Read_MCP23S17_Register (devId, MCP23x17_IOCON);
         read_ioconb=Read_MCP23S17_Register (devId, MCP23x17_IOCONB);
-        
+
         if ((read_iocon == ( IOCON_INIT | IOCON_HAEN)) && (read_ioconb == ( IOCON_INIT | IOCON_HAEN)))
         {
             //hé someone appears to be home...
@@ -959,7 +961,7 @@ int CPiFace::Detect_PiFace_Hardware(void)
             NrOfFoundBoards++;
         }
     }
-    
+
     if (NrOfFoundBoards)
     {
         _log.Log(LOG_STATUS,"PiFace: Found the following PiFaces:");
@@ -977,10 +979,10 @@ void CPiFace::Init_Hardware(unsigned char devId)
 {
     Write_MCP23S17_Register (devId, MCP23x17_IOCON,  IOCON_INIT | IOCON_HAEN) ;
     Write_MCP23S17_Register (devId, MCP23x17_IOCONB, IOCON_INIT | IOCON_HAEN) ;
-    
+
     //setup PortA as output
     Write_MCP23S17_Register (devId, MCP23x17_IODIRA,  0x00) ;         //set all pins on Port A as output
-    
+
     //lets init the other registers so we always have a clear startup situation
     Write_MCP23S17_Register (devId, MCP23x17_GPINTENA, 0x00);          //The PiFace does not support the interrupt capabilities, so set it to 0, and useless for output.
     Write_MCP23S17_Register (devId, MCP23x17_DEFVALA,     0x00);      //Default compare value register, useless for output
@@ -988,12 +990,12 @@ void CPiFace::Init_Hardware(unsigned char devId)
     //Write_MCP23S17_Register (devId, MCP23x17_INTFA,     0);           // Read only interrupt flag register,not used on output
     //Write_MCP23S17_Register (devId, MCP23x17_INTCAPA, );             // Read only interrupt status register, captures port status on interrupt, not ued on output
     Write_MCP23S17_Register (devId, MCP23x17_OLATA, 0xFF);             // Set the output latches to 1, otherwise the relays are activated
-    
+
     //setup PortB as input
     Write_MCP23S17_Register (devId, MCP23x17_IODIRB, 0xFF) ;         //set all pins on Port B as input
     Write_MCP23S17_Register (devId, MCP23x17_GPPUB,  0xFF) ;          //Enable pullup resistors, so the buttons work immediately
     Write_MCP23S17_Register (devId, MCP23x17_IPOLB,  0xFF) ;          //Invert input pin state, so we will see an active state as as 1.
-    
+
     //lets init the other registers so we always have a clear startup situation
     Write_MCP23S17_Register (devId, MCP23x17_GPINTENB, 0x00);          //The PiFace does not support the interrupt capabilities, so set it to 0.
     //Todo: use the int detection for small pulses
@@ -1002,7 +1004,7 @@ void CPiFace::Init_Hardware(unsigned char devId)
     //Write_MCP23S17_Register (devId, MCP23x17_INTFB,     0);           // Read only interrupt flag register, listed as a reminder
     //Write_MCP23S17_Register (devId, MCP23x17_INTCAPB, );             // Read only interrupt status register, captures port status on interrupt, listed as a reminder
     Write_MCP23S17_Register (devId, MCP23x17_OLATB, 0x00);             // Set the output latches to 0, note: not used.
-    
+
     Write_MCP23S17_Register (devId, MCP23x17_GPIOA,  0x00) ;         //set all pins on Port A as output, and deactivate
 }
 
@@ -1011,7 +1013,7 @@ void CPiFace::GetAndSetInitialDeviceState(int devId)
     unsigned char PortState;
     PortState= Read_MCP23S17_Register (devId, MCP23x17_OLATA) ;
     m_Outputs[devId].Init(true,m_HwdID,devId,'O',PortState);
-    
+
     PortState= Read_MCP23S17_Register (devId, MCP23x17_GPIOB) ;
     m_Inputs[devId].Init(true,m_HwdID,devId,'I',PortState);
 }
@@ -1021,14 +1023,14 @@ int CPiFace::Read_Write_SPI_Byte(unsigned char *data, int len)
 #ifdef HAVE_LINUX_SPI
     struct spi_ioc_transfer spi ;
     memset (&spi, 0, sizeof(spi));
-    
+
     spi.tx_buf        = (unsigned long)data ;
     spi.rx_buf        = (unsigned long)data ;
     spi.len           = len ;
     spi.delay_usecs   = 0 ;
     spi.speed_hz      = 4000000;
     spi.bits_per_word = 8 ;
-    
+
     return ioctl (m_fd, SPI_IOC_MESSAGE(1), &spi) ;
 #else
     return 0;
@@ -1038,30 +1040,30 @@ int CPiFace::Read_Write_SPI_Byte(unsigned char *data, int len)
 int CPiFace::Read_MCP23S17_Register(unsigned char devId, unsigned char reg)
 {
     unsigned char spiData [4] ;
-    
+
     spiData [0] = CMD_READ | ((devId & 7) << 1);
     spiData [1] = reg ;
-    
+
     Read_Write_SPI_Byte(spiData, 3);
-    
+
     return spiData [2];
 }
 
 int CPiFace::Write_MCP23S17_Register(unsigned char devId, unsigned char reg, unsigned char data)
 {
     unsigned char spiData [4] ;
-    
+
     spiData [0] = CMD_WRITE | ((devId & 7) << 1) ;
     spiData [1] = reg ;
     spiData [2] = data ;
-    
+
     return (Read_Write_SPI_Byte( spiData, 3)) ;
 }
 
 void CPiFace::Sample_and_Process_Inputs(unsigned char devId)
 {
     unsigned char PortState;
-    
+
     if ( m_Inputs[devId].IsDevicePresent())
     {
         PortState=Read_MCP23S17_Register (devId, MCP23x17_GPIOB);
@@ -1072,7 +1074,7 @@ void CPiFace::Sample_and_Process_Inputs(unsigned char devId)
 void CPiFace::Sample_and_Process_Outputs(unsigned char devId)
 {
     unsigned char PortState;
-    
+
     if ( m_Outputs[devId].IsDevicePresent())
     {
         PortState=Read_MCP23S17_Register (devId, MCP23x17_GPIOA);
@@ -1084,7 +1086,7 @@ void CPiFace::Sample_and_Process_Input_Interrupts(unsigned char devId)
 {
     unsigned char PortInterruptState;
     unsigned char PortState;
-    
+
     if ( m_Inputs[devId].IsDevicePresent())
     {
         PortInterruptState=Read_MCP23S17_Register(devId, MCP23x17_INTFB);
@@ -1115,9 +1117,9 @@ bool CIOCount::ProcessUpdateInterval(unsigned long PassedTime_ms)
             InitialStateSent=true;
             Update=true;
         }
-        
+
         boost::posix_time::ptime Now = boost::posix_time::microsec_clock::universal_time();
-        
+
         if (
             UpdateIntervalPerc > 0 &&
             Cur_Interval.total_milliseconds() > 0 &&
@@ -1130,7 +1132,7 @@ bool CIOCount::ProcessUpdateInterval(unsigned long PassedTime_ms)
             {
                 Cur_Interval = Now - Cur_Pulse;
             }
-            
+
 			double perc = (100. / Last_Interval.total_milliseconds()) * labs((long)(Cur_Interval.total_milliseconds() - Last_Interval.total_milliseconds()));
             if ( perc > UpdateIntervalPerc )
             {
@@ -1145,11 +1147,11 @@ bool CIOCount::ProcessUpdateInterval(unsigned long PassedTime_ms)
 int CIOCount::Update(unsigned long Counts)
 {
     int result = -1;
-    
+
     Last_Pulse = Cur_Pulse;
     Cur_Pulse = boost::posix_time::microsec_clock::universal_time();
     Cur_Interval = Cur_Pulse - Last_Pulse;
-    
+
     if (
         Enabled && (
             Minimum_Pulse_Period_ms == 0 ||
@@ -1185,16 +1187,16 @@ CIOCount::CIOCount()
 
 CIOCount::~CIOCount()
 {
-    
+
 };
 
 int CIOPinState::Update(bool New)
 {
     int StateChange=-1; //nothing changed
-    
+
     Last=Current;
     Current=New;
-    
+
     if (Enabled)
     {
         switch (Type)
@@ -1240,7 +1242,7 @@ int CIOPinState::Update(bool New)
                 break;
         }
     }
-    
+
     if (Direction == 'O')
     {
         if (((Last ^ Current) == true) && (Current == true))
@@ -1248,14 +1250,14 @@ int CIOPinState::Update(bool New)
             Count.Update(1);
         }
     }
-    
+
     return StateChange;
 }
 
 int CIOPinState::UpdateInterrupt(bool IntFlag,bool PinState)
 {
     int Changed=-1;
-    
+
     if (IntFlag && PinState) //we have a rising edge so count
     {
         Changed=Count.Update(1);
@@ -1267,7 +1269,7 @@ int CIOPinState::UpdateInterrupt(bool IntFlag,bool PinState)
 int CIOPinState::GetInitialState(void)
 {
     int CurState=-1; //Report not enabled
-    
+
     if (Enabled)
     {
         switch (Type)
@@ -1291,7 +1293,7 @@ int CIOPinState::GetInitialState(void)
                 else
                     CurState=0;
                 break;
-                
+
             case TOGGLE_FALLING:
                 if (Toggle)
                     CurState=1;
@@ -1318,7 +1320,7 @@ CIOPinState::CIOPinState()
 
 CIOPinState::~CIOPinState()
 {
-    
+
 };
 
 void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned char housecode, unsigned char initial_state)
@@ -1331,7 +1333,7 @@ void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned ch
     std::vector<std::string> resultparts;
 
     Present = Available;
-    
+
     //set generic packet info for LIGHTING1 packet, so we do not have every packet
     IOPinStatusPacket.LIGHTING1.packetlength = sizeof(IOPinStatusPacket.LIGHTING1) -1;
     IOPinStatusPacket.LIGHTING1.housecode = housecode;//report to user as input
@@ -1339,7 +1341,7 @@ void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned ch
     IOPinStatusPacket.LIGHTING1.subtype = sTypeIMPULS;
     IOPinStatusPacket.LIGHTING1.rssi = 12;
     IOPinStatusPacket.LIGHTING1.seqnbr = 0;
-    
+
     for (PinNr=0;PinNr <= 7;PinNr++)
     {
         found = false;
@@ -1359,7 +1361,7 @@ void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned ch
                     meterid=100+PinNr + (devId*10);
                 else
                     meterid=0+PinNr + (devId*10);
-                
+
                 result = m_sql.safe_query("SELECT sValue FROM DeviceStatus WHERE (HardwareID=%d AND Type=%d AND DeviceID='%d')", hwdId, int(pTypeRFXMeter), meterid);
                 if (result.size() == 1)
                 {
@@ -1367,7 +1369,7 @@ void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned ch
                     found = true;
                 }
                 break;
-      
+
             case COUNT_TYPE_ENERGY:
                 int nodeId;
                 if (housecode == 'I')
@@ -1376,7 +1378,7 @@ void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned ch
                     nodeId=200+devId;
 
                 int dID = (nodeId << 8) | PinNr;
-              
+
                 result = m_sql.safe_query("SELECT sValue FROM DeviceStatus WHERE (HardwareID=%d AND Type=%d AND DeviceID='%08X')", hwdId, int(pTypeGeneral), dID);
                 if (result.size() == 1)
                 {
@@ -1396,12 +1398,12 @@ void CIOPort::Init(bool Available, int hwdId, int devId /* 0 - 4 */, unsigned ch
                 }
                 break;
         }
-    
+
         if ( found ) {
             Pin[PinNr].Count.SetTotal(value);
         }
     }
-    
+
     Last=initial_state;
     Current=initial_state;
     Update(initial_state);
@@ -1412,12 +1414,12 @@ int CIOPort::Update(unsigned char New)
     int mask=0x01;
     int ChangeState=-1;
     bool UpdateCounter=false;
-    
+
     CPiFace *myCallback = reinterpret_cast<CPiFace*>(Callback_pntr);
-    
+
     Last=Current;
     Current=New;
-    
+
     for (int PinNr=0; PinNr<=7 ;PinNr++)
     {
         ChangeState=Pin[PinNr].Update((New&mask)==mask);
@@ -1435,15 +1437,15 @@ int CIOPort::Update(unsigned char New)
             IOPinStatusPacket.LIGHTING1.unitcode = PinNr + (devId*10) ; //report inputs from PiFace X (X0..X9)
             myCallback->CallBackSendEvent((const unsigned char *)&IOPinStatusPacket, sizeof(IOPinStatusPacket.LIGHTING1));
         }
-        
+
         UpdateCounter=Pin[PinNr].Count.ProcessUpdateInterval(PIFACE_INPUT_PROCESS_INTERVAL*PIFACE_WORKER_THREAD_SLEEP_INTERVAL_MS);
         if (UpdateCounter)
         {
             unsigned long Count = Pin[PinNr].Count.GetTotal();
             unsigned long LastCount = Pin[PinNr].Count.GetLastTotal();
-            
+
             Pin[PinNr].Count.Last_Callback = boost::posix_time::microsec_clock::universal_time();
-            
+
             switch( Pin[PinNr].Count.Type )
             {
                 case COUNT_TYPE_GENERIC:
@@ -1453,10 +1455,10 @@ int CIOPort::Update(unsigned char New)
                         meterid=100+PinNr + (devId*10);
                     else
                         meterid=0+PinNr + (devId*10);
-    
+
                     Pin[PinNr].IOPinCounterPacket.RFXMETER.id1=((meterid >>8) & 0xFF);
                     Pin[PinNr].IOPinCounterPacket.RFXMETER.id2= (meterid & 0xFF);
-                    
+
                     if (Count != LastCount)
                     {
                         Pin[PinNr].Count.SetLastTotal(Count);
@@ -1470,18 +1472,18 @@ int CIOPort::Update(unsigned char New)
                         myCallback->CallBackSendEvent((const unsigned char *)&Pin[PinNr].IOPinCounterPacket, sizeof(Pin[PinNr].IOPinCounterPacket.RFXMETER));
                     }
                     break;
-                    
+
                 case COUNT_TYPE_ENERGY:
                     int nodeId;
                     if (IOPinStatusPacket.LIGHTING1.housecode == 'I')
                         nodeId=300+devId;
                     else
                         nodeId=200+devId;
-                    
+
                     // Energy devices are updated *every* time because their current power usage (Watt) drops if no pulses
                     // are registered between the interval.
                     Pin[PinNr].Count.SetLastTotal(Count);
-                    
+
                     if (
                         Pin[PinNr].Count.Cur_Interval.total_milliseconds() > 0 &&
                         Pin[PinNr].Count.GetDivider() > 0
@@ -1497,7 +1499,7 @@ int CIOPort::Update(unsigned char New)
         }
         mask<<=1;
     }
-    
+
     return (ChangeState);
 }
 
@@ -1506,7 +1508,7 @@ int CIOPort::UpdateInterrupt(unsigned char IntFlag,unsigned char PinState)
     int mask=0x01;
     int ChangeState=-1; //nothing changed
     int Changed;
-    
+
     for (int PinNr=0; PinNr<=7 ;PinNr++)
     {
         Changed=Pin[PinNr].UpdateInterrupt(((IntFlag&mask)==mask),((PinState&mask)==mask));
@@ -1520,7 +1522,7 @@ int CIOPort::UpdateInterrupt(unsigned char IntFlag,unsigned char PinState)
 void CIOPort::ConfigureCounter(unsigned char Pinnr,bool Enable)
 {
     CPiFace *myCallback = reinterpret_cast<CPiFace*>(Callback_pntr);
-    
+
     Pin[Pinnr].Count.Enabled=Enable;
     if (Pin[Pinnr].Direction == 'I')
     {
@@ -1540,12 +1542,12 @@ CIOPort::CIOPort()
     {
         Pin[PinNr].Id=PinNr;
     }
-    
+
 };
 
 CIOPort::~CIOPort()
 {
-    
+
 };
 
 //Webserver helpers
@@ -1564,7 +1566,7 @@ namespace http {
             if (idx == "") {
                 return;
             }
-            
+
             m_mainworker.RestartHardware(idx);
         }
     }
