@@ -1,7 +1,7 @@
 #pragma once
 
 /* Handles reading data from Rainforest Automation RAVEn:
- 
+
    http://rainforestautomation.com/wp-content/uploads/2014/02/raven_xml_api_r127.pdf
 
    Sample xml output:
@@ -22,25 +22,23 @@
 
 #include "DomoticzHardware.h"
 #include "ASyncSerial.h"
-#include <iostream>
 
 #define MAX_BUFFER_LEN  10000
 
-class RAVEn : public CDomoticzHardwareBase, 
+class RAVEn : public CDomoticzHardwareBase,
               public AsyncSerial
 {
 public:
 	explicit RAVEn(const int ID, const std::string& devname);
 	~RAVEn(void);
-
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
+private:
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void readCallback(const char *indata, size_t inlen);
 private:
     const std::string device_;
-	boost::shared_ptr<boost::thread> m_thread;
-
-	bool StartHardware();
-	bool StopHardware();
-	void readCallback(const char *indata, size_t inlen);
+	std::shared_ptr<std::thread> m_thread;
 
     char m_buffer[MAX_BUFFER_LEN];
     char* m_wptr;
