@@ -3,14 +3,12 @@
 #include "../main/Logger.h"
 #include "../main/Helper.h"
 #include "../main/localtime_r.h"
-
-#define RFLINK_RETRY_DELAY 30
+#include <boost/exception/diagnostic_information.hpp>
 
 CRFLinkSerial::CRFLinkSerial(const int ID, const std::string& devname) :
 m_szSerialPort(devname)
 {
-	m_HwdID=ID;
-	m_stoprequested=false;
+	m_HwdID = ID;
 	m_retrycntr = RFLINK_RETRY_DELAY * 5;
 }
 
@@ -21,24 +19,25 @@ CRFLinkSerial::~CRFLinkSerial()
 
 bool CRFLinkSerial::StartHardware()
 {
-	m_retrycntr=RFLINK_RETRY_DELAY*5; //will force reconnect first thing
+	RequestStart();
+
+	m_retrycntr = RFLINK_RETRY_DELAY*5; //will force reconnect first thing
 
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CRFLinkSerial::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CRFLinkSerial::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CRFLinkSerial::StopHardware()
 {
-	m_stoprequested=true;
 	if (m_thread)
 	{
+		RequestStop();
 		m_thread->join();
-		// Wait a while. The read thread might be reading. Adding this prevents a pointer error in the async serial class.
-		sleep_milliseconds(10);
+		m_thread.reset();
 	}
-	terminate();
 	m_bIsStarted=false;
 	return true;
 }
@@ -48,12 +47,8 @@ void CRFLinkSerial::Do_Work()
 {
 	int msec_counter = 0;
 	int sec_counter = 0;
-	while (!m_stoprequested)
+	while (!IsStopRequested(200))
 	{
-		sleep_milliseconds(200);
-		if (m_stoprequested)
-			break;
-
 		msec_counter++;
 		if (msec_counter == 5)
 		{
@@ -132,7 +127,7 @@ void CRFLinkSerial::Do_Work()
 		/*
 		if (m_sendqueue.size()>0)
 		{
-			boost::lock_guard<boost::mutex> l(m_sendMutex);
+			std::lock_guard<std::mutex> l(m_sendMutex);
 
 			std::vector<std::string>::iterator itt=m_sendqueue.begin();
 			if (itt!=m_sendqueue.end())
@@ -144,7 +139,9 @@ void CRFLinkSerial::Do_Work()
 		}
 		*/
 	}
-	_log.Log(LOG_STATUS,"RFLink: Serial Worker stopped...");
+	terminate();
+
+	_log.Log(LOG_STATUS,"RFLink: Worker stopped...");
 }
 
 /*
@@ -152,7 +149,7 @@ void CRFLinkSerial::Add2SendQueue(const char* pData, const size_t length)
 {
 	std::string sBytes;
 	sBytes.insert(0,pData,length);
-	boost::lock_guard<boost::mutex> l(m_sendMutex);
+	std::lock_guard<std::mutex> l(m_sendMutex);
 	m_sendqueue.push_back(sBytes);
 }
 */
@@ -192,7 +189,6 @@ bool CRFLinkSerial::OpenSerialDevice()
 
 void CRFLinkSerial::readCallback(const char *data, size_t len)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
 	ParseData(data, len);
 }
 
