@@ -5,16 +5,12 @@
 #include <iostream>
 #include "../main/localtime_r.h"
 
-#define RFLINK_RETRY_DELAY 30
-
 CRFLinkTCP::CRFLinkTCP(const int ID, const std::string &IPAddress, const unsigned short usIPPort):
 	m_szIPAddress(IPAddress)
 {
-	m_HwdID=ID;
-	m_bDoRestart=false;
-	m_stoprequested=false;
-	m_usIPPort=usIPPort;
-	m_retrycntr = RFLINK_RETRY_DELAY;
+	m_HwdID = ID;
+	m_bDoRestart = false;
+	m_usIPPort = usIPPort;
 }
 
 CRFLinkTCP::~CRFLinkTCP(void)
@@ -23,50 +19,35 @@ CRFLinkTCP::~CRFLinkTCP(void)
 
 bool CRFLinkTCP::StartHardware()
 {
-	m_stoprequested=false;
+	RequestStart();
+
 	m_bDoRestart=false;
 
 	//force connect the next first time
-	m_retrycntr=RFLINK_RETRY_DELAY;
+	m_retrycntr = RFLINK_RETRY_DELAY;
 	m_bIsStarted=true;
 
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CRFLinkTCP::Do_Work, this)));
-	return (m_thread!=NULL);
+	m_thread = std::make_shared<std::thread>(&CRFLinkTCP::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
+	return (m_thread != nullptr);
 }
 
 bool CRFLinkTCP::StopHardware()
 {
-	m_stoprequested=true;
-	if (isConnected())
+	if (m_thread)
 	{
-		try {
-			disconnect();
-			close();
-		} catch(...)
-		{
-			//Don't throw from a Stop command
-		}
+		RequestStop();
+		m_thread->join();
+		m_thread.reset();
 	}
-	try {
-		if (m_thread)
-		{
-			m_thread->join();
-			m_thread.reset();
-		}
-	}
-	catch (...)
-	{
-		//Don't throw from a Stop command
-	}
-
 	m_bIsStarted=false;
 	return true;
 }
 
 void CRFLinkTCP::OnConnect()
 {
-	_log.Log(LOG_STATUS,"RFLink: connected to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+	_log.Log(LOG_STATUS,"RFLink: connected to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 	m_bDoRestart=false;
 	m_bIsStarted=true;
 	m_rfbufferpos = 0;
@@ -77,25 +58,27 @@ void CRFLinkTCP::OnConnect()
 
 void CRFLinkTCP::OnDisconnect()
 {
+	// Note: No need to set m_bDoRestart = true here, the connection is automatically reinited by ASyncTCP
 	_log.Log(LOG_STATUS,"RFLink: disconnected");
-	m_bDoRestart = true;
 }
 
 void CRFLinkTCP::Do_Work()
 {
 	bool bFirstTime=true;
 	int sec_counter = 0;
-	while (!m_stoprequested)
+	_log.Log(LOG_STATUS, "RFLink: trying to connect to %s:%d", m_szIPAddress.c_str(), m_usIPPort);
+	connect(m_szIPAddress,m_usIPPort);
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 
 		time_t atime = mytime(NULL);
 		if (sec_counter % 12 == 0) {
-			m_LastHeartbeat= atime;
+			m_LastHeartbeat = mytime(NULL);
 		}
-		if ((sec_counter % 20 == 0) && (mIsConnected))
+		if ((sec_counter % 20 == 0) && (isConnected()))
 		{
+			time_t atime = mytime(NULL);
 			//Send ping (keep alive)
 			if (atime - m_LastReceivedTime > 30)
 			{
@@ -103,69 +86,30 @@ void CRFLinkTCP::Do_Work()
 				_log.Log(LOG_ERROR, "RFLink: Nothing received for more than 30 seconds, restarting...");
 				m_retrycntr = 0;
 				m_LastReceivedTime = atime;
+				//TODO: Add method to ASyncTCP to schedule a reconnect
 				m_bDoRestart = true;
-				try {
-					disconnect();
-					close();
-				}
-				catch (...)
-				{
-					//Don't throw from a Stop command
-				}
 			}
 			else
 				write("10;PING;\n");
 		}
 
-		if (bFirstTime)
+		if ((m_bDoRestart) && (sec_counter % 30 == 0))
 		{
-			bFirstTime=false;
-			if (mIsConnected)
-			{
-				try {
-					disconnect();
-					close();
-				}
-				catch (...)
-				{
-					//Don't throw from a Stop command
-				}
-			}
 			_log.Log(LOG_STATUS, "RFLink: trying to connect to %s:%d", m_szIPAddress.c_str(), m_usIPPort);
-			connect(m_szIPAddress,m_usIPPort);
-		}
-		else
-		{
-			if ((m_bDoRestart) && (sec_counter % 30 == 0))
-			{
-				_log.Log(LOG_STATUS, "RFLink: trying to connect to %s:%d", m_szIPAddress.c_str(), m_usIPPort);
-				try {
-					disconnect();
-					close();
-				}
-				catch (...)
-				{
-					//Don't throw from a Stop command
-				}
-				connect(m_szIPAddress, m_usIPPort);
-			}
-			update();
+			disconnect();
+			connect(m_szIPAddress, m_usIPPort);
 		}
 	}
+	terminate();
+
 	_log.Log(LOG_STATUS,"RFLink: TCP/IP Worker stopped...");
-} 
+}
 
 void CRFLinkTCP::OnData(const unsigned char *pData, size_t length)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
 	ParseData((const char*)pData,length);
 }
 
-void CRFLinkTCP::OnError(const std::exception e)
-{
-	_log.Log(LOG_ERROR,"RFLink: Error: %s",e.what());
-}
-
 void CRFLinkTCP::OnError(const boost::system::error_code& error)
 {
 	if (
@@ -176,7 +120,7 @@ void CRFLinkTCP::OnError(const boost::system::error_code& error)
 		(error == boost::asio::error::timed_out)
 		)
 	{
-		_log.Log(LOG_ERROR, "RFLink: Can not connect to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+		_log.Log(LOG_ERROR, "RFLink: Can not connect to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 	}
 	else if (
 		(error == boost::asio::error::eof) ||
@@ -191,7 +135,7 @@ void CRFLinkTCP::OnError(const boost::system::error_code& error)
 
 bool CRFLinkTCP::WriteInt(const std::string &sendString)
 {
-	if (!mIsConnected)
+	if (!isConnected())
 	{
 		return false;
 	}
