@@ -34,7 +34,7 @@ typedef enum {
 	REGO_TYPE_END
 } RegoRegType;
 
-typedef struct _tRegoRegisters 
+typedef struct _tRegoRegisters
 {
 	char name[25];
 	unsigned short regNum_type1;
@@ -70,7 +70,7 @@ typedef union _tRegoReply
     } data;
 } RegoReply;
 
-RegoRegisters g_allRegisters[] = {  
+RegoRegisters g_allRegisters[] = {
 	{ "GT1 Radiator",           0x0209,	0x020B,	0x020D,	REGO_TYPE_TEMP,         -50.0, -1, 0 },
 	{ "GT2 Out",		        0x020A,	0x020C,	0x020E,	REGO_TYPE_TEMP,         -50.0, -1, 0 },
 	{ "GT3 Hot water",	        0x020B,	0x020D,	0x020F,	REGO_TYPE_TEMP,         -50.0, -1, 0 },
@@ -110,7 +110,6 @@ m_szSerialPort(devname)
     m_regoType = type;
     m_errorcntr = 0;
 
-	m_stoprequested=false;
 	m_retrycntr = 0;
 	m_pollcntr = 0;
 	m_pollDelaycntr = 0;
@@ -134,24 +133,25 @@ CRego6XXSerial::~CRego6XXSerial()
 
 bool CRego6XXSerial::StartHardware()
 {
+	RequestStart();
+
 	m_retrycntr=Rego6XX_RETRY_DELAY; //will force reconnect first thing
 
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CRego6XXSerial::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CRego6XXSerial::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CRego6XXSerial::StopHardware()
 {
-	if (m_thread != NULL)
+	if (m_thread)
 	{
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
-    // Wait a while. The read thread might be reading. Adding this prevents a pointer error in the async serial class.
-    sleep_milliseconds(10);
-    terminate();
 	m_bIsStarted=false;
 	return true;
 }
@@ -160,12 +160,8 @@ bool CRego6XXSerial::StopHardware()
 void CRego6XXSerial::Do_Work()
 {
 	int sec_counter = 0;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
-		if (m_stoprequested)
-			break;
-
 		sec_counter++;
 
 		if (sec_counter % 12 == 0) {
@@ -280,8 +276,10 @@ void CRego6XXSerial::Do_Work()
 			}
 		}
 	}
-	_log.Log(LOG_STATUS,"Rego6XX: Serial Worker stopped...");
-} 
+	terminate();
+
+	_log.Log(LOG_STATUS,"Rego6XX: Worker stopped...");
+}
 
 
 bool CRego6XXSerial::OpenSerialDevice()
@@ -315,8 +313,6 @@ bool CRego6XXSerial::OpenSerialDevice()
 
 void CRego6XXSerial::readCallback(const char *data, size_t len)
 {
-	boost::lock_guard<boost::mutex> l(readQueueMutex);
-
 	if (!m_bEnableReceive)
 		return; //receiving not enabled
 
@@ -365,9 +361,9 @@ bool CRego6XXSerial::ParseData()
 		if(m_readBuffer[tail] == 0x01)
 		{
 			// Check crc
-			if(m_readBuffer[(tail + 4) & Rego6XX_READ_BUFFER_MASK] == 
-				(m_readBuffer[(tail + 1) & Rego6XX_READ_BUFFER_MASK] ^ 
-				 m_readBuffer[(tail + 2) & Rego6XX_READ_BUFFER_MASK] ^ 
+			if(m_readBuffer[(tail + 4) & Rego6XX_READ_BUFFER_MASK] ==
+				(m_readBuffer[(tail + 1) & Rego6XX_READ_BUFFER_MASK] ^
+				 m_readBuffer[(tail + 2) & Rego6XX_READ_BUFFER_MASK] ^
 				 m_readBuffer[(tail + 3) & Rego6XX_READ_BUFFER_MASK]))
 			{
 				// This is a proper message
@@ -375,7 +371,7 @@ bool CRego6XXSerial::ParseData()
 		        time_t atime=mytime(NULL);
                 signed short data = 0;
 				data = (m_readBuffer[(tail + 1) & Rego6XX_READ_BUFFER_MASK] << 14) |
-					   (m_readBuffer[(tail + 2) & Rego6XX_READ_BUFFER_MASK] << 7) | 
+					   (m_readBuffer[(tail + 2) & Rego6XX_READ_BUFFER_MASK] << 7) |
 						m_readBuffer[(tail + 3) & Rego6XX_READ_BUFFER_MASK];
 
 				if(g_allRegisters[m_pollcntr].type == REGO_TYPE_TEMP)
@@ -394,7 +390,7 @@ bool CRego6XXSerial::ParseData()
 				else if(g_allRegisters[m_pollcntr].type == REGO_TYPE_STATUS)
 				{
         			strcpy(m_Rego6XXValue.ID, g_allRegisters[m_pollcntr].name);
-					m_Rego6XXValue.value = data; 
+					m_Rego6XXValue.value = data;
                 	m_Rego6XXValue.subtype=sTypeRego6XXStatus;
                     if((m_Rego6XXValue.value != g_allRegisters[m_pollcntr].lastValue) || // Only send changes.
 			(difftime(atime,g_allRegisters[m_pollcntr].lastSent) >= (3600 * 23))) // Send at least every 23 hours
@@ -407,7 +403,7 @@ bool CRego6XXSerial::ParseData()
 				else if(g_allRegisters[m_pollcntr].type == REGO_TYPE_COUNTER)
 				{
         			strcpy(m_Rego6XXValue.ID, g_allRegisters[m_pollcntr].name);
-					m_Rego6XXValue.value = data; 
+					m_Rego6XXValue.value = data;
                 	m_Rego6XXValue.subtype=sTypeRego6XXCounter;
                     if((m_Rego6XXValue.value != g_allRegisters[m_pollcntr].lastValue) || // Only send changes.
 			(difftime(atime,g_allRegisters[m_pollcntr].lastSent) >= 3000)) // Send at least every 50 minutes
@@ -454,7 +450,7 @@ namespace http {
 			}
 			std::vector<std::vector<std::string> > result;
 			result = m_sql.safe_query("SELECT Mode1, Mode2, Mode3, Mode4, Mode5, Mode6 FROM Hardware WHERE (ID='%q')", idx.c_str());
-			if (result.size() < 1)
+			if (result.empty())
 				return;
 
 			unsigned char currentMode1 = atoi(result[0][0].c_str());
