@@ -27,35 +27,22 @@ typedef struct _tRego6XXStatus {
 class CRego6XXSerial: public AsyncSerial, public CDomoticzHardwareBase
 {
 public:
-    /**
-    * Opens a serial device.
-    * \param devname serial device name, example "/dev/ttyS0" or "COM1"
-    * \param baud_rate serial baud rate
-    * \param opt_parity serial parity, default even
-    * \param opt_csize serial character size, default 7bit
-    * \param opt_flow serial flow control, default none
-    * \param opt_stop serial stop bits, default 1
-    * \throws boost::system::system_error if cannot open the
-    * serial device
-    */
 	CRego6XXSerial(const int ID, const std::string& devname, const int type);
-
     ~CRego6XXSerial();
-	std::string m_szSerialPort;
-
-	Rego6XXTemp	    m_Rego6XXTemp;
-	Rego6XXStatus	m_Rego6XXValue;
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
 	void Init();
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	bool OpenSerialDevice();
 	void Do_Work();
 	bool ParseData();
+private:
+	std::string m_szSerialPort;
+	Rego6XXTemp	    m_Rego6XXTemp;
+	Rego6XXStatus	m_Rego6XXValue;
 
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
+	std::shared_ptr<std::thread> m_thread;
 	int m_retrycntr;
 	int m_pollDelaycntr;
 	unsigned int m_pollcntr;
@@ -66,12 +53,7 @@ private:
     char m_readBuffer[Rego6XX_READ_BUFFER_SIZE];
 	volatile unsigned char m_readBufferHead;
 	volatile unsigned char m_readBufferTail;
-
-	/**
-     * Read callback, stores data in the buffer
-     */
     void readCallback(const char *data, size_t len);
-
 };
 
 #endif //BUFFEREDRego6XXASYNCSERIAL_H
