@@ -8,14 +8,10 @@ class S0MeterSerial: public AsyncSerial, public S0MeterBase
 public:
 	S0MeterSerial(const int ID, const std::string& devname, const unsigned int baud_rate);
     ~S0MeterSerial();
-
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
-	bool StartHardware();
-	bool StopHardware();
-    /**
-     * Read callback, stores data in the buffer
-     */
+	bool StartHardware() override;
+	bool StopHardware() override;
     void readCallback(const char *data, size_t len);
 };
 
