@@ -8,7 +8,7 @@
 #include "../main/mainworker.h"
 #include "../main/WebServer.h"
 #include "../webserver/cWebem.h"
-#include "../json/json.h"
+#include <json/json.h>
 #include "hardwaretypes.h"
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
@@ -18,7 +18,7 @@
 CSBFSpot::CSBFSpot(const int ID, const std::string &SMAConfigFile)
 {
 	std::vector<std::string> results;
-	
+
 	m_HwdID=ID;
 #ifdef WIN32
 	StringSplit(SMAConfigFile, ";", results);
@@ -30,7 +30,6 @@ CSBFSpot::CSBFSpot(const int ID, const std::string &SMAConfigFile)
 	if (results.size() > 1)
 		m_SBFInverter = results[1];
 	m_SBFDataPath="";
-	m_stoprequested=false;
 	Init();
 }
 
@@ -101,21 +100,24 @@ void CSBFSpot::Init()
 
 bool CSBFSpot::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CSBFSpot::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CSBFSpot::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CSBFSpot::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -128,9 +130,8 @@ void CSBFSpot::Do_Work()
 	int LastMinute=-1;
 
 	_log.Log(LOG_STATUS,"SBFSpot: Worker started...");
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		time_t atime=mytime(NULL);
 		struct tm ltime;
 		localtime_r(&atime,&ltime);
@@ -227,21 +228,19 @@ void CSBFSpot::ImportOldMonthData()
 	std::vector<std::vector<std::string> > result;
 	result = m_sql.safe_query("SELECT ID FROM DeviceStatus WHERE (HardwareID==%d) AND (DeviceID=='%q') AND (Type==%d) AND (Subtype==%d)",
 		m_HwdID, "00000001", int(pTypeGeneral), int(sTypeKwh));
-	if (result.size() < 1)
+	if (result.empty())
 	{
 		//Lets create the sensor, and try again
 		SendMeter(0, 1, 0, 0, "SolarMain");
 		result = m_sql.safe_query("SELECT ID FROM DeviceStatus WHERE (HardwareID==%d) AND (DeviceID=='%q') AND (Type==%d) AND (Subtype==%d)",
 			m_HwdID, "00000001", int(pTypeGeneral), int(sTypeKwh));
-		if (result.size() < 1)
+		if (result.empty())
 		{
 			_log.Log(LOG_ERROR, "SBFSpot Import Old Month Data: FAILED - Cannot find sensor in database");
 			return;
 		}
 	}
-	uint64_t ulID;
-	std::stringstream s_str(result[0][0]);
-	s_str >> ulID;
+	uint64_t ulID = std::stoull(result[0][0]);
 
 	//Try actual year, and previous year
 	time_t atime = time(NULL);
@@ -271,9 +270,7 @@ void CSBFSpot::ImportOldMonthData(const uint64_t DevID, const int Year, const in
 	int iInvOff = 1;
 	char szLogFile[256];
 	std::string tmpPath = m_SBFDataPath;
-	std::stringstream sstr;
-	sstr << Year;
-	stdreplace(tmpPath, "%Y", sstr.str());
+	stdreplace(tmpPath, "%Y", std::to_string(Year));
 	sprintf(szLogFile, "%s%s-%04d%02d.csv", tmpPath.c_str(), m_SBFPlantName.c_str(),Year, Month);
 
 	std::ifstream infile;
@@ -326,7 +323,7 @@ void CSBFSpot::ImportOldMonthData(const uint64_t DevID, const int Year, const in
 				sprintf(szDate, "%04d-%02d-%02d", year, month, day);
 
 				result = m_sql.safe_query("SELECT Value FROM Meter_Calendar WHERE (DeviceRowID==%" PRIu64 ") AND (Date=='%q')", DevID, szDate);
-				if (result.size() == 0)
+				if (result.empty())
 				{
 					//Insert value into our database
 					m_sql.safe_query("INSERT INTO Meter_Calendar (DeviceRowID, Value, Date) VALUES ('%" PRIu64 "', '%llu', '%q')", DevID, ulCounter, szDate);
@@ -393,7 +390,7 @@ void CSBFSpot::ImportOldMonthData(const uint64_t DevID, const int Year, const in
 
 						result = m_sql.safe_query("SELECT Value FROM Meter_Calendar WHERE (DeviceRowID==%" PRIu64 ") AND (Date=='%q')",
 							DevID, szDate);
-						if (result.size() == 0)
+						if (result.empty())
 						{
 							//Insert value into our database
 							m_sql.safe_query("INSERT INTO Meter_Calendar (DeviceRowID, Value, Date) VALUES ('%" PRIu64 "', '%llu', '%q')",
@@ -470,7 +467,7 @@ void CSBFSpot::GetMeterDetails()
 	if (ActHourMin - 120 > sunSet)
 		return;
 
-	char szLogFile[256];
+	char szLogFile[400];
 	char szDateStr[50];
 	strcpy(szDateStr, strftime_t("%Y%m%d", atime));
 	sprintf(szLogFile, "%s%s-Spot-%s.csv", strftime_t(m_SBFDataPath.c_str(), atime), m_SBFPlantName.c_str(), szDateStr);
