@@ -6,7 +6,7 @@
 #include "hardwaretypes.h"
 #include "../main/localtime_r.h"
 #include "../httpclient/HTTPClient.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 #include "../main/RFXtrx.h"
 #include "../main/mainworker.h"
 #include <iostream>
@@ -61,7 +61,8 @@ SolarEdgeAPI::SolarEdgeAPI(const int ID, const std::string &APIKey):
 {
 	m_SiteID = 0;
 	m_HwdID = ID;
-	m_stoprequested=false;
+	m_totalActivePower = 0;
+	m_totalEnergy = 0;
 }
 
 SolarEdgeAPI::~SolarEdgeAPI(void)
@@ -70,20 +71,23 @@ SolarEdgeAPI::~SolarEdgeAPI(void)
 
 bool SolarEdgeAPI::StartHardware()
 {
+	RequestStart();
+
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&SolarEdgeAPI::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&SolarEdgeAPI::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool SolarEdgeAPI::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -93,9 +97,8 @@ void SolarEdgeAPI::Do_Work()
 {
 	_log.Log(LOG_STATUS, "SolarEdgeAPI Worker started...");
 	int sec_counter = 295;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 		if (sec_counter % 12 == 0) {
 			m_LastHeartbeat = mytime(NULL);
@@ -103,13 +106,10 @@ void SolarEdgeAPI::Do_Work()
 		if (sec_counter % 300 == 0)
 		{
 			if (m_SiteID == 0)
-				GetSite();
-			if (m_SiteID != 0)
 			{
-				if (m_inverters.empty())
-				{
-					GetInverters();
-				}
+				if (!GetSite())
+					continue;
+				GetInverters();
 			}
 			if (!m_inverters.empty())
 				GetMeterDetails();
@@ -148,7 +148,7 @@ int SolarEdgeAPI::getSunRiseSunSetMinutes(const bool bGetSunRise)
 	return 0;
 }
 
-void SolarEdgeAPI::GetSite()
+bool SolarEdgeAPI::GetSite()
 {
 	m_SiteID = 0;
 	std::string sResult;
@@ -157,13 +157,10 @@ void SolarEdgeAPI::GetSite()
 #else
 	std::stringstream sURL;
 	sURL << "https://monitoringapi.solaredge.com/sites/list?size=1&api_key=" << m_APIKey << "&format=application/json";
-	bool bret;
-	std::string szURL = sURL.str();
-	bret = HTTPClient::GET(szURL, sResult);
-	if (!bret)
+	if (!HTTPClient::GET(sURL.str(), sResult))
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Error getting http data!");
-		return;
+		return false;
 	}
 #ifdef DEBUG_SolarEdgeAPIW
 	SaveString2Disk(sResult, "E:\\SolarEdge_sites.json");
@@ -171,34 +168,34 @@ void SolarEdgeAPI::GetSite()
 #endif
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Invalid data received!");
-		return;
+		return false;
 	}
 	if (root["sites"].empty() == true)
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Invalid data received, or invalid APIKey");
-		return;
+		return false;
 	}
 	if (root["sites"]["count"].empty() == true)
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Invalid data received, or invalid APIKey");
-		return;
+		return false;
 	}
 	int tot_results = root["sites"]["count"].asInt();
 	if (tot_results < 1)
-		return;
+		return false;
 	Json::Value reading = root["sites"]["site"][0];
 
 	if (reading["id"].empty() == true)
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Invalid data received, or invalid APIKey");
-		return;
+		return false;
 	}
 	m_SiteID = reading["id"].asInt();
+	return true;
 }
 
 void SolarEdgeAPI::GetInverters()
@@ -210,10 +207,7 @@ void SolarEdgeAPI::GetInverters()
 #else
 	std::stringstream sURL;
 	sURL << "https://monitoringapi.solaredge.com/equipment/" << m_SiteID << "/list?api_key=" << m_APIKey << "&format=application/json";
-	bool bret;
-	std::string szURL = sURL.str();
-	bret = HTTPClient::GET(szURL, sResult);
-	if (!bret)
+	if (!HTTPClient::GET(sURL.str(), sResult))
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Error getting http data!");
 		return;
@@ -224,8 +218,7 @@ void SolarEdgeAPI::GetInverters()
 #endif
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Invalid data received!");
@@ -311,10 +304,7 @@ void SolarEdgeAPI::GetInverterDetails(const _tInverterSettings *pInverterSetting
 	std::string endDate = CURLEncode::URLEncode(szTmp);
 	std::stringstream sURL;
 	sURL << "https://monitoringapi.solaredge.com/equipment/" << m_SiteID << "/" << pInverterSettings->SN << "/data.json?startTime=" << startDate << "&endTime=" << endDate << "&api_key=" << m_APIKey << "&format=application/json";
-	bool bret;
-	std::string szURL = sURL.str();
-	bret = HTTPClient::GET(szURL, sResult);
-	if (!bret)
+	if (!HTTPClient::GET(sURL.str(), sResult))
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Error getting http data!");
 		return;
@@ -325,8 +315,7 @@ void SolarEdgeAPI::GetInverterDetails(const _tInverterSettings *pInverterSetting
 #endif
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "SolarEdgeAPI: Invalid data received!");
@@ -389,7 +378,7 @@ void SolarEdgeAPI::GetInverterDetails(const _tInverterSettings *pInverterSetting
 		{
 			float acFrequency = reading["L1Data"]["acFrequency"].asFloat();
 			sprintf(szTmp, "Hz %s", pInverterSettings->name.c_str());
-			SendPercentageSensor(1 + iInverterNumber, SE_FREQ, 255, acFrequency, szTmp);
+			SendCustomSensor(1 + iInverterNumber, SE_FREQ, 255, acFrequency, szTmp, "Hz");
 		}
 	}
 }
