@@ -2,7 +2,6 @@
 
 #include "DomoticzHardware.h"
 #include <string>
-#include <vector>
 
 class SolarEdgeAPI : public CDomoticzHardwareBase
 {
@@ -12,12 +11,20 @@ class SolarEdgeAPI : public CDomoticzHardwareBase
 		std::string manufacturer;
 		std::string model;
 		std::string SN;
-		uint16_t connectedOptimizers;
 	};
 public:
 	SolarEdgeAPI(const int ID, const std::string &APIKey);
 	~SolarEdgeAPI(void);
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
+private:
+	bool StartHardware() override;
+	bool StopHardware() override;
+	void Do_Work();
+	bool GetSite();
+	void GetInverters();
+	void GetMeterDetails();
+	void GetInverterDetails(const _tInverterSettings *pInverterSettings, const int iInverterNumber);
+	int getSunRiseSunSetMinutes(const bool bGetSunRise);
 private:
 	int m_SiteID;
 	std::string m_APIKey;
@@ -26,17 +33,6 @@ private:
 	double m_totalActivePower;
 	double m_totalEnergy;
 
-
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
-	bool StartHardware();
-	bool StopHardware();
-	void Do_Work();
-	void GetSite();
-	void GetInverters();
-	void GetMeterDetails();
-	void GetInverterDetails(const _tInverterSettings *pInverterSettings, const int iInverterNumber);
-	int getSunRiseSunSetMinutes(const bool bGetSunRise);
+	std::shared_ptr<std::thread> m_thread;
 };
 
