@@ -21,7 +21,6 @@ m_szIPAddress(IPAddress)
 {
 	m_HwdID=ID;
 	m_bDoRestart=false;
-	m_stoprequested=false;
 	m_usIPPort=usIPPort;
 	m_socket = INVALID_SOCKET;
 	m_bufferpos = 0;
@@ -45,7 +44,7 @@ std::string SolarMaxTCP::MakeRequestString()
 	int DestAddress = SM_DEST_ADDRESS;
 	std::string RequestString = "64:KDY;KT0;PAC;UDC;UL1;IDC;IL1;PIN;PRL;TNF;TKK";
 	char szSendTemp[100];
-	char szSendRequest[100];
+	char szSendRequest[120];
 	sprintf(szSendTemp, "%02X;%02X;%02X|%s|", SourceAddress, DestAddress, (unsigned int)(RequestString.size() + 16), RequestString.c_str());
 	int Chksum = SolarMaxCalcChecksum((const unsigned char*)&szSendTemp, (int)strlen(szSendTemp));
 	sprintf(szSendRequest, "{%s%04X}", szSendTemp, Chksum);
@@ -58,9 +57,9 @@ SolarMaxTCP::~SolarMaxTCP(void)
 
 bool SolarMaxTCP::StartHardware()
 {
-	m_bIsStarted = true;
+	RequestStart();
 
-	m_stoprequested = false;
+	m_bIsStarted = true;
 
 	memset(&m_addr, 0, sizeof(sockaddr_in));
 	m_addr.sin_family = AF_INET;
@@ -96,35 +95,19 @@ bool SolarMaxTCP::StartHardware()
 	m_retrycntr = RETRY_DELAY; //will force reconnect first thing
 
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&SolarMaxTCP::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&SolarMaxTCP::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 
-	return (m_thread != NULL);
+	return (m_thread != nullptr);
 }
 
 bool SolarMaxTCP::StopHardware()
 {
-	if (isConnected())
+	if (m_thread)
 	{
-		try {
-			disconnect();
-		}
-		catch (...)
-		{
-			//Don't throw from a Stop command
-		}
-	}
-	else {
-		try {
-			if (m_thread)
-			{
-				m_stoprequested = true;
-				m_thread->join();
-			}
-		}
-		catch (...)
-		{
-			//Don't throw from a Stop command
-		}
+		RequestStop();
+		m_thread->join();
+		m_thread.reset();
 	}
 	m_bIsStarted = false;
 	return true;
@@ -156,11 +139,11 @@ bool SolarMaxTCP::ConnectInternal()
 	{
 		closesocket(m_socket);
 		m_socket = INVALID_SOCKET;
-		_log.Log(LOG_ERROR, "SolarMax: TCP could not connect to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+		_log.Log(LOG_ERROR, "SolarMax: TCP could not connect to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 		return false;
 	}
 
-	_log.Log(LOG_STATUS, "SolarMax: TCP connected to: %s:%ld", m_szIPAddress.c_str(), m_usIPPort);
+	_log.Log(LOG_STATUS, "SolarMax: TCP connected to: %s:%d", m_szIPAddress.c_str(), m_usIPPort);
 
 	sOnConnected(this);
 	return true;
@@ -168,14 +151,11 @@ bool SolarMaxTCP::ConnectInternal()
 
 void SolarMaxTCP::disconnect()
 {
-	m_stoprequested = true;
 	if (m_socket != INVALID_SOCKET)
 	{
-		closesocket(m_socket);	//will terminate the thread
+		closesocket(m_socket);
 		m_socket = INVALID_SOCKET;
-		sleep_seconds(1);
 	}
-	//m_thread-> join();
 }
 
 void SolarMaxTCP::Do_Work()
@@ -183,22 +163,16 @@ void SolarMaxTCP::Do_Work()
 	char buf[1024];
 	bool bFirstTime = true;
 	int sec_counter = POLL_INTERVAL-5;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 
 		if (sec_counter % 12 == 0) {
 			m_LastHeartbeat=mytime(NULL);
 		}
 
-		if (
-			(m_socket == INVALID_SOCKET) &&
-			(!m_stoprequested)
-			)
+		if (m_socket == INVALID_SOCKET)
 		{
-			if (m_stoprequested)
-				break;
 			m_retrycntr++;
 			if (m_retrycntr >= RETRY_DELAY)
 			{
@@ -223,28 +197,24 @@ void SolarMaxTCP::Do_Work()
 				//so it's no good to-do the heartbeat timing here
 
 				int bread = recv(m_socket, (char*)&buf, sizeof(buf), 0);
-				if (m_stoprequested)
+				if (IsStopRequested(0))
 					break;
 				if (bread <= 0) {
-					_log.Log(LOG_ERROR, "SolarMax: TCP/IP connection closed! %s", m_szIPAddress.c_str());
-					closesocket(m_socket);
-					m_socket = INVALID_SOCKET;
-					if (!m_stoprequested)
-					{
-						_log.Log(LOG_STATUS, "SolarMax: retrying in %d seconds...", RETRY_DELAY);
-						m_retrycntr = 0;
-						continue;
-					}
+					_log.Log(LOG_ERROR, "SolarMax: TCP/IP connection closed! retrying in %d seconds...", RETRY_DELAY);
+					disconnect();
+					m_retrycntr = 0;
+					continue;
 				}
 				else
 				{
-					boost::lock_guard<boost::mutex> l(readQueueMutex);
 					ParseData((const unsigned char *)&buf, bread);
 				}
 			}
 		}
 
 	}
+	disconnect();
+
 	_log.Log(LOG_STATUS, "SolarMax: TCP/IP Worker stopped...");
 }
 
