@@ -154,15 +154,13 @@ master occurs once, 30 seconds after startup.
 #define GPIO_MAX_PATH		64
 #define GPIO_PATH			"/sys/class/gpio/gpio"
 #define GPIO_DEVICE_ID_BASE	0x030E0E00
-using namespace std;
 
-vector<gpio_info> CSysfsGpio::m_saved_state;
+std::vector<gpio_info> CSysfsGpio::m_saved_state;
 int CSysfsGpio::m_sysfs_hwdid;
 int CSysfsGpio::m_sysfs_req_update;
 
 CSysfsGpio::CSysfsGpio(const int ID, const int AutoConfigureDevices, const int Debounce)
 {
-	m_stoprequested = false;
 	m_bIsStarted = false;
 	m_polling_enabled = false;
 	m_interrupts_enabled = false;
@@ -170,6 +168,7 @@ CSysfsGpio::CSysfsGpio(const int ID, const int AutoConfigureDevices, const int D
 	m_sysfs_hwdid = ID;
 	m_auto_configure_devices = AutoConfigureDevices;
 	m_debounce_msec = Debounce;
+	m_maxfd = 0;
 }
 
 CSysfsGpio::~CSysfsGpio(void)
@@ -178,44 +177,47 @@ CSysfsGpio::~CSysfsGpio(void)
 
 bool CSysfsGpio::StartHardware()
 {
+	RequestStart();
+
 	Init();
 
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CSysfsGpio::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CSysfsGpio::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted = true;
 
-	return (m_thread != NULL);
+	return (m_thread != nullptr);
 }
 
 bool CSysfsGpio::StopHardware()
 {
-	m_stoprequested = true;
+	RequestStop();
 
 	try
 	{
 		if (m_thread)
 		{
 			m_thread->join();
+			m_thread.reset();
 		}
 	}
 	catch (...)
 	{
-		_log.Log(LOG_STATUS, "Sysfs GPIO: Worker - error during rundown");
-	}
 
+	}
 	try
 	{
 		if (m_edge_thread)
 		{
 			m_edge_thread->join();
+			m_edge_thread.reset();
 		}
 	}
 	catch (...)
 	{
-		_log.Log(LOG_STATUS, "Sysfs GPIO: Edge detection - error during rundown");
+
 	}
 
 	m_bIsStarted = false;
-
 	return true;
 }
 
@@ -248,9 +250,8 @@ void CSysfsGpio::Do_Work()
 	bool bUpdateMaster = true;
 	int counter = 0;
 
-	while (!m_stoprequested)
+	while (!IsStopRequested(GPIO_POLL_MSEC))
 	{
-		sleep_milliseconds(GPIO_POLL_MSEC);
 		counter++;
 
 		if (counter % HEARTBEAT_COUNT == 0)	/* Heartbeat maintenance */
@@ -258,13 +259,10 @@ void CSysfsGpio::Do_Work()
 			m_LastHeartbeat = mytime(NULL);
 		}
 
-		if (!m_stoprequested)
+		if (m_polling_enabled)
 		{
-			if (m_polling_enabled)
-			{
-				PollGpioInputs(false);
-				UpdateDomoticzInputs(false);
-			}
+			PollGpioInputs(false);
+			UpdateDomoticzInputs(false);
 		}
 
 		if (bUpdateMaster)
@@ -272,10 +270,10 @@ void CSysfsGpio::Do_Work()
 			if (counter == UPDATE_MASTER_COUNT)	/* only executes once, and only if we have a master/slave configuration */
 			{
 				bUpdateMaster = false;
-				vector<vector<string> > result;
+				std::vector<std::vector<std::string> > result;
 				result = m_sql.safe_query("SELECT ID FROM Users WHERE (RemoteSharing==1) AND (Active==1)");
 
-				if (result.size() > 0)
+				if (!result.empty())
 				{
 					_log.Log(LOG_STATUS, "Sysfs GPIO: Update master devices");
 					UpdateDomoticzInputs(true);
@@ -318,12 +316,11 @@ void CSysfsGpio::EdgeDetectThread()
 
 	After one or more GPIO state change interrupts have been detected a poll is done to make
 	sure the domoticz database reflects the actual states of all GPIO pins in all cases. A
-	missed interrupt can occur when a GPIO pin changes state twice within m_debounce_msec. 
+	missed interrupt can occur when a GPIO pin changes state twice within m_debounce_msec.
 	Therefore it is a good practice to set m_debounce_msec to a value not higher then needed
 	depending on the input switch behavior.
 	*/
 
-	int retval = 0;
 	ssize_t nb;
 	fd_set tmp_fds;
 	timeval tv;
@@ -355,19 +352,18 @@ void CSysfsGpio::EdgeDetectThread()
 
 	_log.Log(LOG_STATUS, "Sysfs GPIO: Edge detection started");
 
-	while (!m_stoprequested) /* detect gpio state changes */
+	while (!IsStopRequested(0)) /* detect gpio state changes */
 	{
 		tv.tv_sec = 1;	// Set select timeout to 1 second.
 		tv.tv_usec = 0;
 		memcpy(&tmp_fds, &m_rfds, sizeof(tmp_fds));
-		int value = -1;
 
 		//-------------------------------------------------------------------
 		//	Wait for GPIO interrupt.
 		//
-		retval = select(m_maxfd + 1, NULL, NULL, &tmp_fds, &tv);
+		int retval = select(m_maxfd + 1, NULL, NULL, &tmp_fds, &tv);
 
-		if (m_stoprequested)
+		if (IsStopRequested(0))
 		{
 			break;
 		}
@@ -385,7 +381,7 @@ void CSysfsGpio::EdgeDetectThread()
 					(m_saved_state[i].direction == GPIO_IN) &&
 					(m_saved_state[i].edge != GPIO_EDGE_NONE)))
 				{
-					value = GpioReadFd(m_saved_state[i].edge_fd);
+					int value = GpioReadFd(m_saved_state[i].edge_fd);
 					GpioSaveState(i, value);
 					FD_CLR(m_saved_state[i].edge_fd, &tmp_fds);
 					GpioWrite(m_saved_state[i].edge_fd, 1);
@@ -488,15 +484,16 @@ void CSysfsGpio::Init()
 	UpdateDomoticzInputs(false); /* Make sure database inputs are in sync with actual hardware */
 
 	_log.Log(LOG_STATUS, "Sysfs GPIO: Startup - polling:%s interrupts:%s debounce:%dmsec inputs:%d outputs:%d",
-		m_polling_enabled ? "yes":"no", 
-		m_interrupts_enabled ? "yes":"no", 
-		m_debounce_msec, 
-		input_count, 
+		m_polling_enabled ? "yes":"no",
+		m_interrupts_enabled ? "yes":"no",
+		m_debounce_msec,
+		input_count,
 		output_count);
 
 	if (m_interrupts_enabled)
 	{
-		m_edge_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CSysfsGpio::EdgeDetectThread, this)));
+		m_edge_thread = std::make_shared<std::thread>(&CSysfsGpio::EdgeDetectThread, this);
+		SetThreadName(m_edge_thread->native_handle(), "SysfsGpio_Edge");
 	}
 }
 
@@ -555,8 +552,8 @@ void CSysfsGpio::PollGpioInputs(bool PollOnce)
 
 void CSysfsGpio::CreateDomoticzDevices()
 {
-	vector<vector<string> > result;
-	vector<string> deviceid;
+	std::vector<std::vector<std::string> > result;
+	std::vector<std::string> deviceid;
 
 	for (int i = 0; i < m_saved_state.size(); i++)
 	{
@@ -577,9 +574,9 @@ void CSysfsGpio::CreateDomoticzDevices()
 			}
 			else
 			{
-				if (result.size() > 0) /* found */
+				if (!result.empty()) /* found */
 				{
-					vector<string> sd = result[0];
+					std::vector<std::string> sd = result[0];
 
 					if (sd[1].empty())
 					{
@@ -624,9 +621,9 @@ void CSysfsGpio::CreateDomoticzDevices()
 			}
 			else
 			{
-				if (result.size() > 0) /* found */
+				if (!result.empty()) /* found */
 				{
-					vector<string> sd = result[0];
+					std::vector<std::string> sd = result[0];
 
 					if (sd[1].empty())
 					{
@@ -670,14 +667,13 @@ void CSysfsGpio::UpdateDomoticzDatabase()
 	{
 		if (m_saved_state[i].request_update == 1)
 		{
-			vector<vector<string> > result = m_sql.safe_query("SELECT nValue,Options FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
+			std::vector<std::vector<std::string> > result = m_sql.safe_query("SELECT nValue,Options FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
 				m_HwdID,
 				m_saved_state[i].pin_number);
 
 			if (!result.empty())
 			{
-				vector<string> sd = result[0];
-
+				//std::vector<std::string> sd = result[0];
 				m_saved_state[i].value = GpioRead(m_saved_state[i].pin_number, "value");
 
 				m_sql.safe_query(
@@ -699,8 +695,6 @@ void CSysfsGpio::UpdateDomoticzInputs(bool forceUpdate)
 	{
 		if ((m_saved_state[i].direction == GPIO_IN) && (m_saved_state[i].value != -1))
 		{
-			bool	updateDatabase = false;
-			bool	log_db_change = false;
 			int		state = GPIO_LOW;
 
 			if (m_saved_state[i].active_low)
@@ -720,13 +714,15 @@ void CSysfsGpio::UpdateDomoticzInputs(bool forceUpdate)
 
 			if ((m_saved_state[i].db_state != state) || (forceUpdate))
 			{
-				vector< vector<string> > result = m_sql.safe_query("SELECT nValue,Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
+				bool	updateDatabase = false;
+				bool	log_db_change = false;
+				std::vector< std::vector<std::string> > result = m_sql.safe_query("SELECT nValue,Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
 					m_HwdID,
 					m_saved_state[i].pin_number);
 
 				if ((!result.empty()) && (result.size() > 0))
 				{
-					vector<string> sd = result[0];
+					std::vector<std::string> sd = result[0];
 
 					if (atoi(sd[1].c_str()) == 1) /* Check if device is used */
 					{
@@ -800,25 +796,23 @@ void CSysfsGpio::UpdateDeviceID(int pin)
 	{
 		/* Existing pin was found */
 
-		string sdeviceid;
-		int id1, id2, id3, id4;
-
 		if (m_saved_state[index].id_valid == -1)
 		{
-			vector< vector<string> > result = m_sql.safe_query("SELECT DeviceID FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
+			std::string sdeviceid;
+			std::vector< std::vector<std::string> > result = m_sql.safe_query("SELECT DeviceID FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
 				m_HwdID,
 				pin);
 
 			if ((!result.empty() && (result.size() > 0)))
 			{
 				/* use database device id */
-				vector<string> sd = result[0];
+				std::vector<std::string> sd = result[0];
 				sdeviceid = sd[0];
 			}
 			else
 			{
 				/* use generated device id */
-				vector<string> deviceid = GetGpioDeviceId();
+				std::vector<std::string> deviceid = GetGpioDeviceId();
 				sdeviceid = deviceid[0];
 			}
 
@@ -846,13 +840,13 @@ void CSysfsGpio::UpdateGpioOutputs()
 	{
 		if (m_saved_state[i].direction == GPIO_OUT)
 		{
-			vector<vector<string> > result = m_sql.safe_query("SELECT nValue,Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
+			std::vector<std::vector<std::string> > result = m_sql.safe_query("SELECT nValue,Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
 				m_HwdID,
 				m_saved_state[i].pin_number);
 
 			if ((!result.empty()) && (result.size() > 0))
 			{
-				vector<string> sd = result[0];
+				std::vector<std::string> sd = result[0];
 				m_saved_state[i].db_state = atoi(sd[0].c_str());
 
 				if (atoi(sd[1].c_str()))
@@ -866,9 +860,9 @@ void CSysfsGpio::UpdateGpioOutputs()
 	}
 }
 
-vector<string> CSysfsGpio::GetGpioDeviceId()
+std::vector<std::string> CSysfsGpio::GetGpioDeviceId()
 {
-	vector<string> gpio_deviceid;
+	std::vector<std::string> gpio_deviceid;
 	char szIdx[10];
 	int id = GPIO_DEVICE_ID_BASE + m_sysfs_hwdid;
 
@@ -935,7 +929,6 @@ int CSysfsGpio::GpioRead(int gpio_pin, const char *param)
 	char value_str[GPIO_MAX_VALUE_SIZE];
 	int fd;
 	int bytecount = -1;
-	int retval = -1;
 
 	snprintf(path, GPIO_MAX_PATH, "%s%d/%s", GPIO_PATH, gpio_pin, param);
 	fd = open(path, O_RDONLY);
@@ -960,7 +953,6 @@ int CSysfsGpio::GpioRead(int gpio_pin, const char *param)
 int CSysfsGpio::GpioReadFd(int fd)
 {
 	int bytecount = -1;
-	int retval = -1;
 	char value_str[GPIO_MAX_VALUE_SIZE];
 
 	if (fd == -1)
@@ -1014,13 +1006,13 @@ int CSysfsGpio::GpioOpenRw(int gpio_pin)
 //---------------------------------------------------------------------------
 //	Called by WebServer when devices are manually configured.
 //
-vector<int> CSysfsGpio::GetGpioIds()
+std::vector<int> CSysfsGpio::GetGpioIds()
 {
-	vector<int> gpio_ids;
+	std::vector<int> gpio_ids;
 
 	for (int i = 0; i < m_saved_state.size(); i++)
 	{
-		vector<vector<string> > result = m_sql.safe_query("SELECT ID, Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
+		std::vector<std::vector<std::string> > result = m_sql.safe_query("SELECT ID, Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
 			m_sysfs_hwdid,
 			m_saved_state[i].pin_number);
 
@@ -1034,13 +1026,13 @@ vector<int> CSysfsGpio::GetGpioIds()
 	return gpio_ids;
 }
 
-vector<string> CSysfsGpio::GetGpioNames()
+std::vector<std::string> CSysfsGpio::GetGpioNames()
 {
-	vector<string> gpio_names;
+	std::vector<std::string> gpio_names;
 
 	for (int i = 0; i < m_saved_state.size(); i++)
 	{
-		vector<vector<string> > result = m_sql.safe_query("SELECT ID, Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
+		std::vector<std::vector<std::string> > result = m_sql.safe_query("SELECT ID, Used FROM DeviceStatus WHERE (HardwareID==%d) AND (Unit==%d)",
 			m_sysfs_hwdid,
 			m_saved_state[i].pin_number);
 
