@@ -5,24 +5,21 @@
 #ifdef WITH_LIBUSB
 
 #include "DomoticzHardware.h"
-#include <iostream>
 
 class CTE923 : public CDomoticzHardwareBase
 {
 public:
 	explicit CTE923(const int ID);
 	~CTE923(void);
-
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
 	void Init();
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	void Do_Work();
 	void GetSensorDetails();
+private:
+	std::shared_ptr<std::thread> m_thread;
 };
 
 #endif //WITH_LIBUSB
