@@ -9,7 +9,7 @@
 #include "../main/SQLHelper.h"
 #include "../httpclient/HTTPClient.h"
 #include "../main/mainworker.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 
 #define round(a) ( int ) ( a + .5 )
 
@@ -90,32 +90,32 @@ void CThermosmart::Init()
 {
 	m_AccessToken = "";
 	m_ThermostatID = "";
-	m_stoprequested = false;
 	m_bDoLogin = true;
 }
 
 bool CThermosmart::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	m_LastMinute = -1;
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CThermosmart::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CThermosmart::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CThermosmart::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
-	if (!m_bDoLogin)
-		Logout();
     return true;
 }
 
@@ -125,9 +125,8 @@ void CThermosmart::Do_Work()
 {
 	_log.Log(LOG_STATUS,"Thermosmart: Worker started...");
 	int sec_counter = THERMOSMART_POLL_INTERVAL-5;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 		if (sec_counter % 12 == 0) {
 			m_LastHeartbeat=mytime(NULL);
@@ -138,6 +137,8 @@ void CThermosmart::Do_Work()
 			GetMeterDetails();
 		}
 	}
+	Logout();
+
 	_log.Log(LOG_STATUS,"Thermosmart: Worker stopped...");
 }
 
@@ -146,9 +147,7 @@ bool CThermosmart::GetOutsideTemperatureFromDomoticz(float &tvalue)
 	if (m_OutsideTemperatureIdx == 0)
 		return false;
 	Json::Value tempjson;
-	std::stringstream sstr;
-	sstr << m_OutsideTemperatureIdx;
-	m_webservers.GetJSonDevices(tempjson, "", "temp", "ID", sstr.str(), "", "", true, false, false, 0, "");
+	m_webservers.GetJSonDevices(tempjson, "", "temp", "ID", std::to_string(m_OutsideTemperatureIdx), "", "", true, false, false, 0, "");
 
 	size_t tsize = tempjson.size();
 	if (tsize < 1)
@@ -279,8 +278,7 @@ bool CThermosmart::Login()
 #endif
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "Thermosmart: Invalid/no data received...");
@@ -338,7 +336,7 @@ void CThermosmart::GetMeterDetails()
 	std::string sResult;
 #ifdef DEBUG_ThermosmartThermostat_read
 	sResult = ReadFile("E:\\thermosmart_getdata.txt");
-#else	
+#else
 	if (m_bDoLogin)
 	{
 		if (!Login())
@@ -359,8 +357,7 @@ void CThermosmart::GetMeterDetails()
 #endif
 #endif
 	Json::Value root;
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
 		_log.Log(LOG_ERROR, "Thermosmart: Invalid/no data received...");
