@@ -1,20 +1,14 @@
 #include "stdafx.h"
 #include "ToonThermostat.h"
+#include "hardwaretypes.h"
+#include "../httpclient/HTTPClient.h"
 #include "../main/Helper.h"
 #include "../main/Logger.h"
-#include "hardwaretypes.h"
 #include "../main/localtime_r.h"
+#include "../main/mainworker.h"
 #include "../main/RFXtrx.h"
 #include "../main/SQLHelper.h"
-#include "../httpclient/HTTPClient.h"
-#include "../main/mainworker.h"
-#include "../json/json.h"
-
-#include <boost/uuid/uuid.hpp>            // uuid class
-#include <boost/uuid/uuid_generators.hpp> // generators
-#include <boost/uuid/uuid_io.hpp>         // streaming operators etc.
-
-#define round(a) ( int ) ( a + .5 )
+#include "../main/json_helper.h"
 
 #ifdef _DEBUG
 //	#define DEBUG_ToonThermostat
@@ -112,7 +106,6 @@ m_Agreement(Agreement)
 
 	m_ClientID = "";
 	m_ClientIDChecksum = "";
-	m_stoprequested = false;
 	m_lastSharedSendElectra = 0;
 	m_lastSharedSendGas = 0;
 	m_lastgasusage = 0;
@@ -141,7 +134,6 @@ void CToonThermostat::Init()
 {
 	m_ClientID = "";
 	m_ClientIDChecksum = "";
-	m_stoprequested = false;
 	m_lastSharedSendElectra = 0;
 	m_lastSharedSendGas = 0;
 	m_lastgasusage = 0;
@@ -164,7 +156,7 @@ void CToonThermostat::Init()
 	{
 		unsigned long devID = (unsigned long)atol(result[0][0].c_str());
 		result = m_sql.safe_query("SELECT MAX(Counter1), MAX(Counter2), MAX(Counter3), MAX(Counter4) FROM Multimeter_Calendar WHERE (DeviceRowID==%ld)", devID);
-		if (result.size() > 0)
+		if (!result.empty())
 		{
 			std::vector<std::string> sd = *result.begin();
 			m_OffsetUsage1 = (unsigned long)atol(sd[0].c_str());
@@ -180,25 +172,26 @@ void CToonThermostat::Init()
 
 bool CToonThermostat::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CToonThermostat::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CToonThermostat::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CToonThermostat::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
-	if (!m_bDoLogin)
-		Logout();
     return true;
 }
 
@@ -206,9 +199,8 @@ void CToonThermostat::Do_Work()
 {
 	_log.Log(LOG_STATUS,"ToonThermostat: Worker started...");
 	int sec_counter = 1;
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		m_poll_counter--;
 		sec_counter++;
 		if (sec_counter % 12 == 0) {
@@ -226,6 +218,8 @@ void CToonThermostat::Do_Work()
 			}
 		}
 	}
+	Logout();
+
 	_log.Log(LOG_STATUS,"ToonThermostat: Worker stopped...");
 }
 
@@ -246,7 +240,6 @@ void CToonThermostat::SendSetPointSensor(const unsigned char Idx, const float Te
 
 void CToonThermostat::UpdateSwitch(const unsigned char Idx, const bool bOn, const std::string &defaultname)
 {
-	bool bDeviceExits = true;
 	char szIdx[10];
 	sprintf(szIdx, "%X%02X%02X%02X", 0, 0, 0, Idx);
 	std::vector<std::vector<std::string> > result;
@@ -290,12 +283,9 @@ void CToonThermostat::UpdateSwitch(const unsigned char Idx, const bool bOn, cons
 	sDecodeRXMessage(this, (const unsigned char *)&lcmd.LIGHTING2, defaultname.c_str(), 255);
 }
 
-
 std::string CToonThermostat::GetRandom()
 {
-	boost::uuids::uuid uuid = boost::uuids::random_generator()();
-	std::string suuid = boost::uuids::to_string(uuid);
-	return suuid;
+	return GenerateUUID();
 }
 
 bool CToonThermostat::Login()
@@ -306,7 +296,7 @@ bool CToonThermostat::Login()
 	}
 	m_ClientID = "";
 	std::stringstream sstr;
-	sstr << "username=" << m_UserName << "&password=" << m_Password;
+	sstr << "username=" << m_UserName << "&password=" << CURLEncode::URLEncode(m_Password);
 	std::string szPostdata=sstr.str();
 	std::vector<std::string> ExtraHeaders;
 	std::string sResult;
@@ -326,8 +316,7 @@ bool CToonThermostat::Login()
 #endif
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if (!bRet)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received, or invalid username/password!");
@@ -364,7 +353,7 @@ bool CToonThermostat::Login()
 	agreementIdChecksum = root["agreements"][m_Agreement]["agreementIdChecksum"].asString();
 
 	std::stringstream sstr2;
-	sstr2 << "clientId=" << m_ClientID 
+	sstr2 << "clientId=" << m_ClientID
 		 << "&clientIdChecksum=" << m_ClientIDChecksum
 		 << "&agreementId=" << agreementId
 		 << "&agreementIdChecksum=" << agreementIdChecksum
@@ -386,7 +375,7 @@ bool CToonThermostat::Login()
 #endif
 
 	root.clear();
-	bRet = jReader.parse(sResult, root);
+	bRet = ParseJSon(sResult, root);
 	if (!bRet)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received!");
@@ -397,12 +386,11 @@ bool CToonThermostat::Login()
 		_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received!");
 		return false;
 	}
-	if (root["success"] == true)
+	if (root["success"].asBool() == true)
 	{
 		m_bDoLogin = false;
 		return true;
 	}
-
 	return false;
 }
 
@@ -441,7 +429,7 @@ bool CToonThermostat::GetUUIDIdx(const std::string &UUID, int &idx)
 	std::vector<std::vector<std::string> > result;
 	result = m_sql.safe_query("SELECT [ROWID] FROM ToonDevices WHERE (HardwareID=%d) AND (UUID='%q')",
 		m_HwdID, UUID.c_str());
-	if (result.size() < 1)
+	if (result.empty())
 		return false;
 	std::vector<std::string> sd = result[0];
 	idx = atoi(sd[0].c_str());
@@ -453,7 +441,7 @@ bool CToonThermostat::GetUUIDFromIdx(const int idx, std::string &UUID)
 	std::vector<std::vector<std::string> > result;
 	result = m_sql.safe_query("SELECT [UUID] FROM ToonDevices WHERE (HardwareID=%d) AND (ROWID=%d)",
 		m_HwdID, idx);
-	if (result.size() < 1)
+	if (result.empty())
 		return false;
 	std::vector<std::string> sd = result[0];
 	UUID = sd[0];
@@ -481,8 +469,7 @@ bool CToonThermostat::SwitchLight(const std::string &UUID, const int SwitchState
 	}
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if (!bRet)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received!");
@@ -502,7 +489,8 @@ bool CToonThermostat::SwitchLight(const std::string &UUID, const int SwitchState
 */
 	m_retry_counter = 0;
 	m_poll_counter = TOON_POLL_INTERVAL_SHORT;
-	return (root["success"] == true);
+
+	return (root["success"].asBool() == true);
 }
 
 bool CToonThermostat::SwitchAll(const int SwitchState)
@@ -525,8 +513,7 @@ bool CToonThermostat::SwitchAll(const int SwitchState)
 	}
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if (!bRet)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received!");
@@ -539,7 +526,7 @@ bool CToonThermostat::SwitchAll(const int SwitchState)
 	}
 	m_retry_counter = 0;
 	m_poll_counter = TOON_POLL_INTERVAL_SHORT;
-	return (root["success"] == true);
+	return (root["success"].asBool() == true);
 }
 
 bool CToonThermostat::WriteToHardware(const char *pdata, const unsigned char length)
@@ -624,8 +611,7 @@ void CToonThermostat::GetMeterDetails()
 	SaveString2Disk(sResult, szFileName);
 #endif
 
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if (!bRet)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received!");
@@ -923,8 +909,7 @@ void CToonThermostat::SetSetpoint(const int idx, const float temp)
 		}
 
 		Json::Value root;
-		Json::Reader jReader;
-		bool bRet = jReader.parse(sResult, root);
+		bool bRet = ParseJSon(sResult, root);
 		if (!bRet)
 		{
 			_log.Log(LOG_ERROR, "ToonThermostat: Invalid data received!");
@@ -937,7 +922,7 @@ void CToonThermostat::SetSetpoint(const int idx, const float temp)
 			m_bDoLogin = true;
 			return;
 		}
-		if (root["success"] == false)
+		if (root["success"].asBool() == false)
 		{
 			_log.Log(LOG_ERROR, "ToonThermostat: setPoint request not successful, restarting..!");
 			m_bDoLogin = true;
@@ -981,8 +966,7 @@ void CToonThermostat::SetProgramState(const int newState)
 	}
 
 	Json::Value root;
-	Json::Reader jReader;
-	bool bRet = jReader.parse(sResult, root);
+	bool bRet = ParseJSon(sResult, root);
 	if (!bRet)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: setProgramState request not successful, restarting..!");
@@ -995,7 +979,7 @@ void CToonThermostat::SetProgramState(const int newState)
 		m_bDoLogin = true;
 		return;
 	}
-	if (root["success"] == false)
+	if (root["success"].asBool() == false)
 	{
 		_log.Log(LOG_ERROR, "ToonThermostat: setProgramState request not successful, restarting..!");
 		m_bDoLogin = true;
