@@ -3,7 +3,6 @@
 #ifndef WIN32
 #ifdef WITH_LIBUSB
 #include "DomoticzHardware.h"
-#include <iostream>
 
 class CVolcraftCO20 : public CDomoticzHardwareBase
 {
@@ -11,15 +10,14 @@ public:
 	explicit CVolcraftCO20(const int ID);
 	~CVolcraftCO20(void);
 
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 private:
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
-
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	void Do_Work();
 	void GetSensorDetails();
+private:
+	std::shared_ptr<std::thread> m_thread;
 };
 
 #endif //WITH_LIBUSB
