@@ -6,9 +6,10 @@
 #include "hardwaretypes.h"
 #include "../main/localtime_r.h"
 #include "../httpclient/HTTPClient.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 #include "../main/RFXtrx.h"
 #include "../main/mainworker.h"
+#include "../main/SQLHelper.h"
 
 #define round(a) ( int ) ( a + .5 )
 
@@ -54,7 +55,6 @@ m_bForceSingleStation(false),
 m_bFirstTime(true)
 {
 	m_HwdID = ID;
-	m_stoprequested = false;
 	Init();
 }
 
@@ -69,9 +69,12 @@ void CWunderground::Init()
 
 bool CWunderground::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CWunderground::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CWunderground::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	if (!m_thread)
 		return false;
 	m_bIsStarted=true;
@@ -81,11 +84,11 @@ bool CWunderground::StartHardware()
 
 bool CWunderground::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -93,12 +96,14 @@ bool CWunderground::StopHardware()
 
 void CWunderground::Do_Work()
 {
+#ifdef DEBUG_WUNDERGROUNDR
+	GetMeterDetails();
+#endif
 	int sec_counter = 590;
 	_log.Log(LOG_STATUS, "Wunderground: Worker started...");
 
-	while (!m_stoprequested)
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 		if (sec_counter % 10 == 0) {
 			m_LastHeartbeat=mytime(NULL);
@@ -124,25 +129,97 @@ std::string CWunderground::GetForecastURL()
 {
 	std::stringstream sURL;
 	std::string szLoc = CURLEncode::URLEncode(m_Location);
+	sURL << "https://api.weather.com/v3/location/point?geocode=" << szLoc << "&language=en-US&format=json&apiKey=" << m_APIKey;
 	sURL << "http://www.wunderground.com/cgi-bin/findweather/getForecast?query=" << szLoc;
 	return sURL.str();
 }
 
+std::string CWunderground::GetWeatherStationFromGeo()
+{
+	std::string sValue;
+	int nValue;
+	if (m_sql.GetPreferencesVar("Location", nValue, sValue))
+	{
+		std::vector<std::string> strarray;
+		StringSplit(sValue, ";", strarray);
+
+		if (strarray.size() == 2)
+		{
+			std::string Latitude = strarray[0];
+			std::string Longitude = strarray[1];
+
+			std::string sResult;
+#ifdef DEBUG_WUNDERGROUNDR
+			sResult = ReadFile("E:\\wu_location.json");
+#else
+			std::stringstream sURL;
+			sURL << "https://api.weather.com/v3/location/near?geocode=" << Latitude << "," << Longitude << "&product=pws&format=json&apiKey=" << m_APIKey;
+			if (!HTTPClient::GET(sURL.str(), sResult))
+			{
+				_log.Log(LOG_ERROR, "Wunderground: Error getting location/near result! (Check API key!)");
+				return "";
+			}
+#ifdef DEBUG_WUNDERGROUNDW
+			SaveString2Disk(sResult, "E:\\wu_location.json");
+#endif
+#endif
+			Json::Value root;
+
+			bool ret = ParseJSon(sResult, root);
+			if ((!ret) || (!root.isObject()))
+			{
+				_log.Log(LOG_ERROR, "WUnderground: Problem getting location/near result. Invalid data received! (Check Station ID!)");
+				return "";
+			}
+
+			bool bValid = true;
+			if (root["location"].empty() == true)
+			{
+				bValid = false;
+			}
+			else if (root["location"]["stationId"].empty())
+			{
+				bValid = false;
+			}
+			if (!bValid)
+			{
+				_log.Log(LOG_ERROR, "WUnderground: Problem getting location/near result.Invalid data received, or no data returned!");
+				return "";
+			}
+			if (root["location"]["stationId"].size() > 0)
+			{
+				std::string szFirstStation = root["location"]["stationId"][0].asString();
+				return szFirstStation;
+			}
+			else
+				_log.Log(LOG_ERROR, "WUnderground: Problem getting location/near result. No stations returned!");
+		}
+	}
+	return "";
+}
+
 void CWunderground::GetMeterDetails()
 {
+	if (m_Location.find(",") != std::string::npos)
+	{
+		std::string newLocation = GetWeatherStationFromGeo();
+		if (newLocation.empty())
+			return;
+		m_Location = newLocation;
+	}
+	if (m_Location.empty())
+		return;
+
 	std::string sResult;
 #ifdef DEBUG_WUNDERGROUNDR
 	sResult= ReadFile("E:\\wu.json");
 #else
 	std::stringstream sURL;
 	std::string szLoc = CURLEncode::URLEncode(m_Location);
-	sURL << "http://api.wunderground.com/api/" << m_APIKey << "/conditions/q/" << szLoc << ".json";
-	bool bret;
-	std::string szURL=sURL.str();
-	bret=HTTPClient::GET(szURL,sResult);
-	if (!bret)
+	sURL << "https://api.weather.com/v2/pws/observations/current?stationId=" << szLoc << "&format=json&units=s&numericPrecision=decimal&apiKey=" << m_APIKey;
+	if (!HTTPClient::GET(sURL.str(), sResult))
 	{
-		_log.Log(LOG_ERROR,"Wunderground: Error getting http data!");
+		_log.Log(LOG_ERROR,"Wunderground: Error getting http data! (Check API key!)");
 		return;
 	}
 #ifdef DEBUG_WUNDERGROUNDW
@@ -151,97 +228,62 @@ void CWunderground::GetMeterDetails()
 #endif
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret=jReader.parse(sResult,root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (!root.isObject()))
 	{
-		_log.Log(LOG_ERROR,"WUnderground: Invalid data received!");
+		_log.Log(LOG_ERROR,"WUnderground: Invalid data received! (Check Station ID!)");
 		return;
 	}
 
 	bool bValid = true;
-	if (root["response"].empty() == true)
+	if (root["observations"].empty() == true)
 	{
 		bValid = false;
 	}
-	else if (!root["response"]["error"].empty())
-	{
-		bValid = false;
-		if (!root["response"]["error"]["description"].empty())
-		{
-			_log.Log(LOG_ERROR, "WUnderground: Error: %s", root["response"]["error"]["description"].asString().c_str());
-			return;
-		}
-	}
-	else if (root["current_observation"].empty()==true)
-	{
-		bValid = false;
-		return;
-	}
-	else if (root["current_observation"]["temp_c"].empty() == true)
+	else if (root["observations"][0]["country"].empty())
 	{
 		bValid = false;
 	}
-	else if (m_bForceSingleStation && root["current_observation"]["station_id"].empty())
+	else if (m_Location.find(root["observations"][0]["stationID"].asString()) == std::string::npos)
 	{
 		bValid = false;
 	}
-	else if (m_bForceSingleStation && m_Location.find(root["current_observation"]["station_id"].asString()) == std::string::npos)
-	{
-		bValid = false;
-	}
-	else if (root["current_observation"]["observation_epoch"].empty() == true)
-	{
-		bValid = false;
-	}
-	else if (root["current_observation"]["local_epoch"].empty() == true)
-	{
-		bValid = false;
-	}
-	else
-	{
-		if (!m_bFirstTime)
-		{
-			time_t tlocal = static_cast<time_t>(atoll(root["current_observation"]["local_epoch"].asString().c_str()));
-			time_t tobserver = static_cast<time_t>(atoll(root["current_observation"]["observation_epoch"].asString().c_str()));
-			if (difftime(tlocal, tobserver) >= 1800)
-			{
-				//When we don't get any valid data in 30 minutes, we also stop using the values
-				_log.Log(LOG_ERROR, "WUnderground: Receiving old data from WU! (No new data return for more than 30 minutes)");
-				return;
-			}
-		}
-		m_bFirstTime = false;
-	}
 	if (!bValid)
 	{
 		_log.Log(LOG_ERROR, "WUnderground: Invalid data received, or no data returned!");
 		return;
 	}
 
+	root = root["observations"][0];
+
+	if (!m_bFirstTime)
+	{
+		time_t tlocal = time(NULL);
+		time_t tobserver = (time_t)root["epoch"].asInt();
+		if (difftime(tlocal, tobserver) >= 1800)
+		{
+			//When we don't get any valid data in 30 minutes, we also stop using the values
+			_log.Log(LOG_ERROR, "WUnderground: Receiving old data from WU! (No new data return for more than 30 minutes)");
+			return;
+		}
+	}
+	m_bFirstTime = false;
+
 	std::string tmpstr;
 	float temp;
 	int humidity=0;
 	int barometric=0;
 	int barometric_forcast=baroForecastNoInfo;
 
+	temp=root["metric_si"]["temp"].asFloat();
 
-	temp=root["current_observation"]["temp_c"].asFloat();
-
-	if (root["current_observation"]["relative_humidity"].empty()==false)
+	if (root["humidity"].empty()==false)
 	{
-		tmpstr=root["current_observation"]["relative_humidity"].asString();
-		size_t pos=tmpstr.find("%");
-		if (pos==std::string::npos)
-		{
-			_log.Log(LOG_ERROR,"WUnderground: Invalid data received!");
-			return;
-		}
-		humidity=atoi(tmpstr.substr(0,pos).c_str());
+		humidity = root["humidity"].asInt();
 	}
-	if (root["current_observation"]["pressure_mb"].empty()==false)
+	if (root["metric_si"]["pressure"].empty()==false)
 	{
-		barometric=atoi(root["current_observation"]["pressure_mb"].asString().c_str());
+		barometric=atoi(root["metric_si"]["pressure"].asString().c_str());
 		if (barometric<1000)
 			barometric_forcast=baroForecastRain;
 		else if (barometric<1020)
@@ -250,31 +292,6 @@ void CWunderground::GetMeterDetails()
 			barometric_forcast=baroForecastPartlyCloudy;
 		else
 			barometric_forcast=baroForecastSunny;
-
-		if (root["current_observation"]["icon"].empty()==false)
-		{
-			std::string forcasticon=root["current_observation"]["icon"].asString();
-			if (forcasticon=="partlycloudy")
-			{
-				barometric_forcast=baroForecastPartlyCloudy;
-			}
-			else if (forcasticon=="cloudy")
-			{
-				barometric_forcast=baroForecastCloudy;
-			}
-			else if (forcasticon=="sunny")
-			{
-				barometric_forcast=baroForecastSunny;
-			}
-			else if (forcasticon=="clear")
-			{
-				barometric_forcast=baroForecastSunny;
-			}			
-			else if (forcasticon=="rain")
-			{
-				barometric_forcast=baroForecastRain;
-			}
-		}
 	}
 
 	if (barometric!=0)
@@ -295,8 +312,6 @@ void CWunderground::GetMeterDetails()
 
 	//Wind
 	int wind_degrees=-1;
-	float wind_mph=-1;
-	float wind_gust_mph=-1;
 	float windspeed_ms=0;
 	float windgust_ms=0;
 	float wind_temp=temp;
@@ -304,41 +319,29 @@ void CWunderground::GetMeterDetails()
 	int windgust=1;
 	float windchill=-1;
 
-	if (root["current_observation"]["wind_degrees"].empty()==false)
+	if (root["winddir"].empty()==false)
 	{
-		wind_degrees=atoi(root["current_observation"]["wind_degrees"].asString().c_str());
+		wind_degrees=atoi(root["winddir"].asString().c_str());
 	}
-	if (root["current_observation"]["wind_mph"].empty()==false)
+	if (root["metric_si"]["windSpeed"].empty()==false)
 	{
-		if ((root["current_observation"]["wind_mph"] != "N/A") && (root["current_observation"]["wind_mph"] != "--"))
+		if ((root["metric_si"]["windSpeed"] != "N/A") && (root["metric_si"]["windSpeed"] != "--"))
 		{
-			float temp_wind_mph = static_cast<float>(atof(root["current_observation"]["wind_mph"].asString().c_str()));
-			if (temp_wind_mph!=-9999.00f)
-			{
-				wind_mph=temp_wind_mph;
-				//convert to m/s
-				windspeed_ms=wind_mph*0.44704f;
-			}
+			windspeed_ms = static_cast<float>(atof(root["metric_si"]["windSpeed"].asString().c_str()));
 		}
 	}
-	if (root["current_observation"]["wind_gust_mph"].empty()==false)
+	if (root["metric_si"]["windGust"].empty()==false)
 	{
-		if ((root["current_observation"]["wind_gust_mph"] != "N/A") && (root["current_observation"]["wind_gust_mph"] != "--"))
+		if ((root["metric_si"]["windGust"] != "N/A") && (root["metric_si"]["windGust"] != "--"))
 		{
-			float temp_wind_gust_mph = static_cast<float>(atof(root["current_observation"]["wind_gust_mph"].asString().c_str()));
-			if (temp_wind_gust_mph!=-9999.00f)
-			{
-				wind_gust_mph=temp_wind_gust_mph;
-				//convert to m/s
-				windgust_ms=wind_gust_mph*0.44704f;
-			}
+			windgust_ms = static_cast<float>(atof(root["metric_si"]["windGust"].asString().c_str()));
 		}
 	}
-	if (root["current_observation"]["feelslike_c"].empty()==false)
+	if (root["metric_si"]["windChill"].empty()==false)
 	{
-		if ((root["current_observation"]["feelslike_c"] != "N/A") && (root["current_observation"]["feelslike_c"] != "--"))
+		if ((root["metric_si"]["windChill"] != "N/A") && (root["metric_si"]["windChill"] != "--"))
 		{
-			wind_chill = static_cast<float>(atof(root["current_observation"]["feelslike_c"].asString().c_str()));
+			wind_chill = static_cast<float>(atof(root["metric_si"]["windChill"].asString().c_str()));
 		}
 	}
 	if (wind_degrees!=-1)
@@ -385,7 +388,7 @@ void CWunderground::GetMeterDetails()
 		at10-=(tsen.WIND.temperatureh*256);
 		tsen.WIND.temperaturel=(BYTE)(at10);
 
-		tsen.WIND.chillsign=(wind_temp>=0)?0:1;
+		tsen.WIND.chillsign=(wind_chill>=0)?0:1;
 		at10=round(std::abs(wind_chill*10.0f));
 		tsen.WIND.chillh=(BYTE)(at10/256);
 		at10-=(tsen.WIND.chillh*256);
@@ -395,27 +398,24 @@ void CWunderground::GetMeterDetails()
 	}
 
 	//UV
-	if (root["current_observation"].empty() == false)
+	if (root["uv"].empty() == false)
 	{
-		if (root["current_observation"]["UV"].empty() == false)
+		if ((root["uv"] != "N/A") && (root["uv"] != "--"))
 		{
-			if ((root["current_observation"]["UV"] != "N/A") && (root["current_observation"]["UV"] != "--"))
+			float UV = static_cast<float>(atof(root["uv"].asString().c_str()));
+			if ((UV < 16) && (UV >= 0))
 			{
-				float UV = static_cast<float>(atof(root["current_observation"]["UV"].asString().c_str()));
-				if ((UV < 16) && (UV >= 0))
-				{
-					SendUVSensor(0, 1, 255, UV, "UV");
-				}
+				SendUVSensor(0, 1, 255, UV, "UV");
 			}
 		}
 	}
 
 	//Rain
-	if (root["current_observation"]["precip_today_metric"].empty() == false)
+	if (root["metric_si"]["precipTotal"].empty() == false)
 	{
-		if ((root["current_observation"]["precip_today_metric"] != "N/A") && (root["current_observation"]["precip_today_metric"] != "--"))
+		if ((root["metric_si"]["precipTotal"] != "N/A") && (root["metric_si"]["precipTotal"] != "--"))
 		{
-			float RainCount = static_cast<float>(atof(root["current_observation"]["precip_today_metric"].asString().c_str()));
+			float RainCount = static_cast<float>(atof(root["metric_si"]["precipTotal"].asString().c_str()));
 			if ((RainCount != -9999.00f) && (RainCount >= 0.00f))
 			{
 				RBUF tsen;
@@ -431,11 +431,11 @@ void CWunderground::GetMeterDetails()
 				tsen.RAIN.rainrateh = 0;
 				tsen.RAIN.rainratel = 0;
 
-				if (root["current_observation"]["precip_1hr_metric"].empty() == false)
+				if (root["metric_si"]["precipRate"].empty() == false)
 				{
-					if ((root["current_observation"]["precip_1hr_metric"] != "N/A") && (root["current_observation"]["precip_1hr_metric"] != "--"))
+					if ((root["metric_si"]["precipRate"] != "N/A") && (root["metric_si"]["precipRate"] != "--"))
 					{
-						float rainrateph = static_cast<float>(atof(root["current_observation"]["precip_1hr_metric"].asString().c_str()));
+						float rainrateph = static_cast<float>(atof(root["metric_si"]["precipRate"].asString().c_str()));
 						if (rainrateph != -9999.00f)
 						{
 							int at10 = round(std::abs(rainrateph*100.0f));
@@ -457,12 +457,12 @@ void CWunderground::GetMeterDetails()
 		}
 	}
 
-	//Visibility
-	if (root["current_observation"]["visibility_km"].empty() == false)
+	//Visibility (seems not to exist anymore!?)
+	if (root["visibility"].empty() == false)
 	{
-		if ((root["current_observation"]["visibility_km"] != "N/A") && (root["current_observation"]["visibility_km"] != "--"))
+		if ((root["visibility"] != "N/A") && (root["visibility"] != "--"))
 		{
-			float visibility = static_cast<float>(atof(root["current_observation"]["visibility_km"].asString().c_str()));
+			float visibility = static_cast<float>(atof(root["visibility"].asString().c_str()));
 			if (visibility >= 0)
 			{
 				_tGeneralDevice gdevice;
@@ -474,18 +474,15 @@ void CWunderground::GetMeterDetails()
 	}
 
 	//Solar Radiation
-	if (root["current_observation"]["solarradiation"].empty() == false)
+	if (root["solarRadiation"].empty() == false)
 	{
-		if ((root["current_observation"]["solarradiation"] != "N/A") && (root["current_observation"]["solarradiation"] != "--"))
+		float radiation = static_cast<float>(atof(root["solarRadiation"].asString().c_str()));
+		if (radiation >= 0.0f)
 		{
-			float radiation = static_cast<float>(atof(root["current_observation"]["solarradiation"].asString().c_str()));
-			if (radiation >= 0.0f)
-			{
-				_tGeneralDevice gdevice;
-				gdevice.subtype = sTypeSolarRadiation;
-				gdevice.floatval1 = radiation;
-				sDecodeRXMessage(this, (const unsigned char *)&gdevice, NULL, 255);
-			}
+			_tGeneralDevice gdevice;
+			gdevice.subtype = sTypeSolarRadiation;
+			gdevice.floatval1 = radiation;
+			sDecodeRXMessage(this, (const unsigned char *)&gdevice, NULL, 255);
 		}
 	}
 }
