@@ -7,7 +7,7 @@
 #include "hardwaretypes.h"
 #include "../main/localtime_r.h"
 #include "../main/mainworker.h"
-#include "../json/json.h"
+#include "../main/json_helper.h"
 
 #define YOULESS_POLL_INTERVAL 10
 
@@ -17,7 +17,6 @@ m_Password(CURLEncode::URLEncode(password))
 {
 	m_HwdID=ID;
 	m_usIPPort=usIPPort;
-	m_stoprequested=false;
 	Init();
 }
 
@@ -47,7 +46,7 @@ void CYouLess::Init()
 	m_p1gas.subtype = sTypeP1Gas;
 	m_p1gas.ID = 1;
 
-	m_bHaveP1 = false;
+	m_bHaveP1OrS0 = false;
 	m_bCheckP1 = true;
 	m_lastgasusage = 0;
 	m_lastSharedSendGas = mytime(NULL);
@@ -55,21 +54,24 @@ void CYouLess::Init()
 
 bool CYouLess::StartHardware()
 {
+	RequestStart();
+
 	Init();
 	//Start worker thread
-	m_thread = boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&CYouLess::Do_Work, this)));
+	m_thread = std::make_shared<std::thread>(&CYouLess::Do_Work, this);
+	SetThreadNameInt(m_thread->native_handle());
 	m_bIsStarted=true;
 	sOnConnected(this);
-	return (m_thread!=NULL);
+	return (m_thread != nullptr);
 }
 
 bool CYouLess::StopHardware()
 {
-	if (m_thread!=NULL)
+	if (m_thread)
 	{
-		assert(m_thread);
-		m_stoprequested = true;
+		RequestStop();
 		m_thread->join();
+		m_thread.reset();
 	}
     m_bIsStarted=false;
     return true;
@@ -79,9 +81,9 @@ void CYouLess::Do_Work()
 {
 	int sec_counter = YOULESS_POLL_INTERVAL - 2;
 
-	while (!m_stoprequested)
+	_log.Log(LOG_STATUS, "YouLess: Worker started...");
+	while (!IsStopRequested(1000))
 	{
-		sleep_seconds(1);
 		sec_counter++;
 
 		if (sec_counter % 12 == 0) {
@@ -105,27 +107,20 @@ bool CYouLess::GetP1Details()
 	m_bCheckP1 = false;
 
 	std::string sResult;
+	std::stringstream szURL;
 
-	char szURL[200];
+	szURL << "http://" << m_szIPAddress << ":" << m_usIPPort << "/e";
+	if (!m_Password.empty())
+		szURL << "&w=" << m_Password;
 
-	if (m_Password.size() == 0)
+	if (!HTTPClient::GET(szURL.str(), sResult))
 	{
-		sprintf(szURL, "http://%s:%d/e", m_szIPAddress.c_str(), m_usIPPort);
-	}
-	else
-	{
-		sprintf(szURL, "http://%s:%d/e&w=%s", m_szIPAddress.c_str(), m_usIPPort, m_Password.c_str());
-	}
-
-	if (!HTTPClient::GET(szURL, sResult))
-	{
-		_log.Log(LOG_ERROR, "YouLess: Error getting meter details!");
+		_log.Log(LOG_ERROR, "YouLess: Error getting meter details from %s !", m_szIPAddress.c_str() );
 		return false;
 	}
 	Json::Value root;
 
-	Json::Reader jReader;
-	bool ret = jReader.parse(sResult, root);
+	bool ret = ParseJSon(sResult, root);
 	if ((!ret) || (root.empty()))
 	{
 		return false;
@@ -133,79 +128,73 @@ bool CYouLess::GetP1Details()
 	if (root.size() < 1)
 		return false;
 	root = root[0];
-	if (root["p1"].empty() == true)
-		return false;
 
-	int Pwr = root["pwr"].asInt();
-	
-	unsigned long temp_usage;
-	temp_usage= (unsigned long)(root["p1"].asDouble() * 1000);
-	m_p1power.powerusage1 = temp_usage;
-	temp_usage = (unsigned long)(root["p2"].asDouble() * 1000);
-	m_p1power.powerusage2 = temp_usage;
-	temp_usage = (unsigned long)(root["n1"].asDouble() * 1000);
-	m_p1power.powerdeliv1 = temp_usage;
-	temp_usage = (unsigned long)(root["n2"].asDouble() * 1000);
-	m_p1power.powerdeliv2 = temp_usage;
 
-	if (Pwr >= 0)
+	if (!root["p1"].empty())
 	{
-		m_p1power.usagecurrent = Pwr;
-		m_p1power.delivcurrent = 0;
-	}
-	else
-	{
-		m_p1power.delivcurrent = -Pwr;
-		m_p1power.usagecurrent = 0;
-	}
-	sDecodeRXMessage(this, (const unsigned char *)&m_p1power, "Power", 255);
+		int Pwr = root["pwr"].asInt();
 
-	temp_usage = (unsigned long)(root["gas"].asDouble() * 1000);
-	m_p1gas.gasusage = temp_usage;
-	time_t atime = mytime(NULL);
-	if (
-		(m_p1gas.gasusage != m_lastgasusage) ||
-		(difftime(atime, m_lastSharedSendGas) >= 300)
-		)
-	{
-		m_lastgasusage = temp_usage;
-		m_lastSharedSendGas = atime;
-		sDecodeRXMessage(this, (const unsigned char *)&m_p1gas, "Gas", 255);
+		m_p1power.powerusage1 = (unsigned long)(root["p1"].asDouble() * 1000);
+		m_p1power.powerusage2 = (unsigned long)(root["p2"].asDouble() * 1000);
+		m_p1power.powerdeliv1 = (unsigned long)(root["n1"].asDouble() * 1000);
+		m_p1power.powerdeliv2 = (unsigned long)(root["n2"].asDouble() * 1000);
+
+		if (Pwr >= 0)
+		{
+			m_p1power.usagecurrent = Pwr;
+			m_p1power.delivcurrent = 0;
+		}
+		else
+		{
+			m_p1power.delivcurrent = -Pwr;
+			m_p1power.usagecurrent = 0;
+		}
+		sDecodeRXMessage(this, (const unsigned char *)&m_p1power, "Power", 255);
+
+		m_p1gas.gasusage = (unsigned long)(root["gas"].asDouble() * 1000);
+		time_t atime = mytime(NULL);
+		if (
+			(m_p1gas.gasusage != m_lastgasusage) ||
+			(difftime(atime, m_lastSharedSendGas) >= 300)
+			)
+		{
+			m_lastgasusage = m_p1gas.gasusage;
+			m_lastSharedSendGas = atime;
+			sDecodeRXMessage(this, (const unsigned char *)&m_p1gas, "Gas", 255);
+		}
+		m_bHaveP1OrS0 = true;
 	}
-	m_bHaveP1 = true;
-/*
-	//Old Meter
-	m_meter.powerusage = (unsigned long)(root["net"].asDouble()*1000);
-	m_meter.usagecurrent = Pwr;
-	sDecodeRXMessage(this, (const unsigned char *)&m_meter, NULL, 255);
-*/
-	return true;
+
+	if (!root["cs0"].empty())
+	{
+		//S0 Meter
+		double mcntr = root["cs0"].asDouble();
+		double musage = root["ps0"].asDouble();
+		if (mcntr != 0)
+		{
+			SendKwhMeter(m_HwdID, 1, 255, musage, mcntr, "S0");
+			m_bHaveP1OrS0 = true;
+		}
+	}
+	return m_bHaveP1OrS0;
 }
 
 void CYouLess::GetMeterDetails()
 {
-	if (m_bCheckP1 || m_bHaveP1)
+	if (m_bCheckP1 || m_bHaveP1OrS0)
 	{
 		if (GetP1Details())
 			return;
-		if (m_bHaveP1)
-			return;
 	}
 
 	std::string sResult;
+	std::stringstream szURL;
 
-	char szURL[200];
+	szURL << "http://" << m_szIPAddress << ":" << m_usIPPort << "/a";
+	if (!m_Password.empty())
+		szURL << "&w=" << m_Password;
 
-	if(m_Password.size() == 0)
-	{
-		sprintf(szURL,"http://%s:%d/a",m_szIPAddress.c_str(), m_usIPPort);
-	}
-	else
-	{
-		sprintf(szURL,"http://%s:%d/a&w=%s",m_szIPAddress.c_str(), m_usIPPort, m_Password.c_str());
-	}
-
-	if (!HTTPClient::GET(szURL,sResult))
+	if (!HTTPClient::GET(szURL.str(), sResult))
 	{
 		_log.Log(LOG_ERROR,"YouLess: Error connecting to: %s", m_szIPAddress.c_str());
 		return;
@@ -231,7 +220,12 @@ void CYouLess::GetMeterDetails()
 		pcurrent=pcurrent.substr(0,fpos);
 	stdreplace(pcurrent,",","");
 
-	m_meter.powerusage=atol(pusage.c_str());
-	m_meter.usagecurrent=atol(pcurrent.c_str());
-	sDecodeRXMessage(this, (const unsigned char *)&m_meter, NULL, 255);
+	unsigned long lpusage = atol(pusage.c_str());
+	unsigned long lpcurrent = atol(pcurrent.c_str());
+	if (lpusage)
+	{
+		m_meter.powerusage = lpusage;
+		m_meter.usagecurrent = lpcurrent;
+		sDecodeRXMessage(this, (const unsigned char*)&m_meter, NULL, 255);
+	}
 }
