@@ -1,6 +1,5 @@
 #pragma once
 
-#include <map>
 #include <time.h>
 #include "DomoticzHardware.h"
 
@@ -32,6 +31,7 @@ class ZWaveBase : public CDomoticzHardwareBase
 		ZDTYPE_SENSOR_THERMOSTAT_CLOCK,
 		ZDTYPE_SENSOR_THERMOSTAT_FAN_MODE,
 		ZDTYPE_SENSOR_THERMOSTAT_MODE,
+		ZDTYPE_SENSOR_THERMOSTAT_OPERATING_STATE,
 
 		ZDTYPE_SENSOR_VELOCITY,
 		ZDTYPE_SENSOR_BAROMETER,
@@ -41,42 +41,37 @@ class ZWaveBase : public CDomoticzHardwareBase
 		ZDTYPE_SENSOR_WATER,
 		ZDTYPE_SENSOR_MOISTURE,
 		ZDTYPE_SENSOR_TANK_CAPACITY,
+		ZDTYPE_SENSOR_LOUDNESS,
 
 		ZDTYPE_ALARM,
 		ZDTYPE_CENTRAL_SCENE,
 
 		ZDTYPE_SENSOR_CUSTOM,
-
 	};
 	struct _tZWaveDevice
 	{
-		int nodeID;
-		int commandClassID;
-		int instanceID;
-		int indexID;
-		int orgInstanceID;
-		int orgIndexID;
-		_eZWaveDeviceType devType;
-		int scaleID;
-		int scaleMultiply;
-		int basicType;
-		int genericType;
-		int specificType;
-		bool isListening;
-		bool sensor250;
-		bool sensor1000;
-		bool isFLiRS;
-		bool hasWakeup;
-		bool hasBattery;
+		uint8_t nodeID;
+		uint8_t instanceID;
+		uint16_t indexID;
+		uint8_t orgInstanceID;
+		uint16_t orgIndexID;
+		uint8_t commandClassID;
 
-		int Manufacturer_id;
-		int Product_id;
-		int Product_type;
+		_eZWaveDeviceType devType;
+		float scaleMultiply;
+		int basicType;
+		bool isListening;
+		bool hasWakeup;
+
+		uint16_t Manufacturer_id;
+		uint16_t Product_id;
+		uint16_t Product_type;
 
 		//values
+		bool bValidValue;
+
 		float floatValue;
 		int intvalue;
-		bool bValidValue;
 
 		//battery
 		int batValue;
@@ -94,18 +89,14 @@ class ZWaveBase : public CDomoticzHardwareBase
 		int Alarm_Type;
 
 		_tZWaveDevice() :
-			label("Unknown")
+			label("Unknown"),
+			lastreceived(0)
 		{
 			sequence_number=1;
-			nodeID=-1;
-			scaleID=1;
-			scaleMultiply=1;
+			nodeID=(uint8_t)-1;
+			scaleMultiply=1.0f;
 			isListening=false;
-			sensor250=false;
-			sensor1000=false;
-			isFLiRS=false;
 			hasWakeup=false;
-			hasBattery=false;
 			batValue = 255;
 			floatValue=0;
 			intvalue=0;
@@ -117,12 +108,9 @@ class ZWaveBase : public CDomoticzHardwareBase
 			orgIndexID=0;
 			devType = ZDTYPE_SWITCH_NORMAL;
 			basicType=0;
-			genericType=0;
-			specificType=0;
-			Manufacturer_id = -1;
-			Product_id = -1;
-			Product_type = -1;
-			lastreceived = 0;
+			Manufacturer_id = (uint16_t)-1;
+			Product_id = (uint16_t)-1;
+			Product_type = (uint16_t)-1;
 			Alarm_Type = -1;
 		}
 	};
@@ -132,45 +120,41 @@ public:
 
 	virtual bool GetInitialDevices()=0;
 	virtual bool GetUpdates()=0;
-	bool StartHardware();
-	bool StopHardware();
-	bool WriteToHardware(const char *pdata, const unsigned char length);
+	bool StartHardware() override;
+	bool StopHardware() override;
+	bool WriteToHardware(const char *pdata, const unsigned char length) override;
 public:
 	int m_LastIncludedNode;
 	std::string m_LastIncludedNodeType;
 	bool m_bHaveLastIncludedNodeInfo;
 	int m_LastRemovedNode;
-	boost::mutex m_NotificationMutex;
+	std::mutex m_NotificationMutex;
 private:
 	void Do_Work();
 	void SendDevice2Domoticz(const _tZWaveDevice *pDevice);
 	void SendSwitchIfNotExists(const _tZWaveDevice *pDevice);
-	
-	_tZWaveDevice* FindDevice(const int nodeID, const int instanceID, const int indexID);
-	_tZWaveDevice* FindDevice(const int nodeID, const int instanceID, const int indexID, const _eZWaveDeviceType devType);
-	_tZWaveDevice* FindDevice(const int nodeID, const int instanceID, const int indexID, const int CommandClassID, const _eZWaveDeviceType devType);
-	_tZWaveDevice* FindDeviceEx(const int nodeID, const int instanceID, const _eZWaveDeviceType devType);
 
-	void ForceUpdateForNodeDevices(const unsigned int homeID, const int nodeID);
-	bool IsNodeRGBW(const unsigned int homeID, const int nodeID);
+	_tZWaveDevice* FindDevice(const uint8_t nodeID, const int instanceID, const int indexID);
+	_tZWaveDevice* FindDevice(const uint8_t nodeID, const int instanceID, const int indexID, const _eZWaveDeviceType devType);
+	_tZWaveDevice* FindDevice(const uint8_t nodeID, const int instanceID, const int indexID, const int CommandClassID, const _eZWaveDeviceType devType);
+	_tZWaveDevice* FindDeviceEx(const uint8_t nodeID, const int instanceID, const _eZWaveDeviceType devType);
 
 	std::string GenerateDeviceStringID(const _tZWaveDevice *pDevice);
 	void InsertDevice(_tZWaveDevice device);
-	void UpdateDeviceBatteryStatus(const int nodeID, const int value);
 	unsigned char Convert_Battery_To_PercInt(const unsigned char level);
-	virtual bool SwitchLight(const int nodeID, const int instanceID, const int commandClass, const int value)=0;
-	virtual bool SwitchColor(const int nodeID, const int instanceID, const int commandClass, const std::string &ColorStr) = 0;
-	virtual void SetThermostatSetPoint(const int nodeID, const int instanceID, const int commandClass, const float value)=0;
-	virtual void SetClock(const int nodeID, const int instanceID, const int commandClass, const int day, const int hour, const int minute)=0;
-	virtual void SetThermostatMode(const int nodeID, const int instanceID, const int commandClass, const int tMode) = 0;
-	virtual void SetThermostatFanMode(const int nodeID, const int instanceID, const int commandClass, const int fMode) = 0;
+	virtual bool SwitchLight(_tZWaveDevice* pDevice, const int instanceID, const int value)=0;
+	virtual bool SwitchColor(const uint8_t nodeID, const uint8_t instanceID, const std::string &ColorStr) = 0;
+	virtual void SetThermostatSetPoint(const uint8_t nodeID, const uint8_t instanceID, const uint8_t commandClass, const float value)=0;
+	virtual void SetClock(const uint8_t nodeID, const uint8_t instanceID, const uint8_t commandClass, const uint8_t day, const uint8_t hour, const uint8_t minute)=0;
+	virtual void SetThermostatMode(const uint8_t nodeID, const uint8_t instanceID, const uint8_t commandClass, const int tMode) = 0;
+	virtual void SetThermostatFanMode(const uint8_t nodeID, const uint8_t instanceID, const uint8_t commandClass, const int fMode) = 0;
 	virtual std::string GetSupportedThermostatFanModes(const unsigned long ID) = 0;
 	virtual void StopHardwareIntern() = 0;
 	virtual bool IncludeDevice(const bool bSecure) = 0;
-	virtual bool ExcludeDevice(const int nodeID)=0;
-	virtual bool RemoveFailedDevice(const int nodeID)=0;
+	virtual bool ExcludeDevice(const uint8_t nodeID)=0;
+	virtual bool RemoveFailedDevice(const uint8_t nodeID)=0;
 	virtual bool CancelControllerCommand(const bool bForce = false)=0;
-	virtual bool HasNodeFailed(const int nodeID) = 0;
+	virtual bool HasNodeFailed(const uint8_t nodeID) = 0;
 	virtual bool IsNodeIncluded() = 0;
 	virtual bool IsNodeExcluded() = 0;
 
@@ -180,8 +164,7 @@ private:
 	time_t m_updateTime;
 	bool m_bInitState;
 	std::map<std::string,_tZWaveDevice> m_devices;
-	boost::shared_ptr<boost::thread> m_thread;
-	bool m_stoprequested;
+	std::shared_ptr<std::thread> m_thread;
 };
 
 
