@@ -1,6 +1,5 @@
 #pragma once
 
-#include <vector>
 #include "ASyncSerial.h"
 #include "ZiBlueBase.h"
 
@@ -11,14 +10,13 @@ public:
     ~CZiBlueSerial();
 private:
 	void Init();
-	bool StartHardware();
-	bool StopHardware();
+	bool StartHardware() override;
+	bool StopHardware() override;
 	bool OpenSerialDevice();
 	void Do_Work();
-	bool WriteInt(const std::string &sendString);
-	bool WriteInt(const uint8_t *pData, const size_t length);
-	boost::shared_ptr<boost::thread> m_thread;
-	volatile bool m_stoprequested;
+	bool WriteInt(const std::string &sendString) override;
+	bool WriteInt(const uint8_t *pData, const size_t length) override;
+	std::shared_ptr<std::thread> m_thread;
 	std::string m_szSerialPort;
     void readCallback(const char *data, size_t len);
 };
