@@ -1,23 +1,40 @@
 #pragma once
 
+#include <cereal/cereal.hpp>
+//#include "../cereal/types/string.hpp"
+//#include "../cereal/types/utility.hpp"
+//#include "../cereal/types/memory.hpp"
+//#include "../cereal/types/complex.hpp"
+//#include "../cereal/types/base_class.hpp"
+//#include "../cereal/types/array.hpp"
+//#include "../cereal/types/vector.hpp"
+//#include "../cereal/types/map.hpp"
+
+#include "ColorSwitch.h"
+
+#define sTypeTHBFloat 0x10   //Weather Station
+#define sTypeWINDNoTemp 0x30    //Weather Station
+#define sTypeWINDNoTempNoChill 0x31    //Weather Station
+
 #define sTypeDomoticzSecurity 0x83
 #define sTypeSmartwaresSwitchRadiator 0x84
 
 #define sTypeRAINWU 0x70	 //Weather Underground (Total rain reported, no counter)
+#define sTypeRAINByRate 0x71	 //DarkSky for example (Only rate, no total, no counter) rate in mm/hour x 10000, so all decimals will fit
 
-#define sTypeTHBFloat 0x10   //Weather Station
-#define sTypeWINDNoTemp 0x30    //Weather Station
+#define sTypeTH_LC_TC 0xA0			//La Cross Temp_Hum combined
+#define sTypeTEMP_SYSTEM 0xA0		//Internal sensor
 
-#define wsbaroforcast_heavy_snow 0x01
-#define wsbaroforcast_snow 0x01
-#define wsbaroforcast_heavy_rain 0x02
-#define wsbaroforcast_rain 0x03
-#define wsbaroforcast_cloudy 0x04
-#define wsbaroforcast_some_clouds 0x05
-#define wsbaroforcast_sunny 0x06
-#define wsbaroforcast_unknown 0x07
-#define wsbaroforcast_unstable 0x08
-#define wsbaroforcast_stable 0x09
+#define wsbaroforecast_heavy_snow 0x01
+#define wsbaroforecast_snow 0x01
+#define wsbaroforecast_heavy_rain 0x02
+#define wsbaroforecast_rain 0x03
+#define wsbaroforecast_cloudy 0x04
+#define wsbaroforecast_some_clouds 0x05
+#define wsbaroforecast_sunny 0x06
+#define wsbaroforecast_unknown 0x07
+#define wsbaroforecast_unstable 0x08
+#define wsbaroforecast_stable 0x09
 
 #define bmpbaroforecast_stable			0x00
 #define bmpbaroforecast_sunny			0x01
@@ -27,12 +44,6 @@
 #define bmpbaroforecast_unknown			0x05
 #define bmpbaroforecast_rain			0x06 //when forecast was cloudy and pressure is below 1010 we have 50%+ change of rain
 
-#define pTypeLimitlessLights	0xF1
-#define sTypeLimitlessRGBW		0x01
-#define sTypeLimitlessRGB		0x02
-#define sTypeLimitlessWhite		0x03
-#define sTypeLimitlessRGBWW		0x04
-
 #define pTypeThermostat			0xF2
 #define sTypeThermSetpoint		0x01
 #define sTypeThermTemperature	0x02
@@ -65,6 +76,8 @@
 #define sTypeWaterflow				0x1E
 #define sTypeCustom					0x1F
 #define sTypeZWaveAlarm				0x20
+#define sTypeManagedCounter			0x21
+#define sTypeZWaveThermostatOperatingState	0x23
 
 //General Switch
 #define pTypeGeneralSwitch			0xF4
@@ -180,6 +193,10 @@
 #define sSwitchTypeYW_Sensor		0x73
 #define sSwitchTypeLegrandcad		0x74
 #define sSwitchTypeSysfsGpio		0x75
+#define sSwitchTypeHager			0x76
+#define sSwitchTypeFaber			0x77
+#define sSwitchTypeDrayton			0x78
+#define sSwitchTypeV2Phoenix		0x79
 
 //Switch commands
 #define gswitch_sOff				0x00
@@ -280,24 +297,45 @@ typedef struct _tThermostat {
 	uint8_t utemp2;
 	uint8_t utemp3;
 
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("id3", id3);
+		ar & cereal::make_nvp("id4", id4);
+		ar & cereal::make_nvp("dunit", dunit);
+		ar & cereal::make_nvp("battery_level", battery_level);
+		ar & cereal::make_nvp("temp", temp);
+		ar & cereal::make_nvp("temp1", temp1);
+		ar & cereal::make_nvp("temp2", temp2);
+		ar & cereal::make_nvp("temp3", temp3);
+		ar & cereal::make_nvp("utemp1", utemp1);
+		ar & cereal::make_nvp("utemp2", utemp2);
+		ar & cereal::make_nvp("utemp3", utemp3);
+	}
+
 	_tThermostat()
 	{
-		len=sizeof(_tThermostat)-1;
-		type=pTypeThermostat;
-		subtype=sTypeThermTemperature;
-		battery_level=255;
-		id1=1;
-		id2=0;
-		id3=0;
-		id4=0;
-		dunit=0;
-		temp=0;
-		temp1=0;
-		temp2=0;
-		temp3=0;
-		utemp1=0;
-		utemp2=0;
-		utemp3=0;
+		len = sizeof(_tThermostat) - 1;
+		type = pTypeThermostat;
+		subtype = sTypeThermTemperature;
+		battery_level = 255;
+		id1 = 1;
+		id2 = 0;
+		id3 = 0;
+		id4 = 0;
+		dunit = 0;
+		temp = 0;
+		temp1 = 0;
+		temp2 = 0;
+		temp3 = 0;
+		utemp1 = 0;
+		utemp2 = 0;
+		utemp3 = 0;
 	}
 } tThermostat;
 
@@ -311,16 +349,29 @@ typedef struct _tTempBaro {
 	float altitude;
 	uint8_t forecast;
 
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("temp", temp);
+		ar & cereal::make_nvp("baro", baro);
+		ar & cereal::make_nvp("altitude", altitude);
+		ar & cereal::make_nvp("forecast", forecast);
+	}
+
 	_tTempBaro()
 	{
-		len=sizeof(_tTempBaro)-1;
-		type=pTypeTEMP_BARO;
-		subtype=sTypeBMP085;
-		id1=1;
-		temp=0;
-		baro=0;
-		altitude=0;
-		forecast=0;
+		len = sizeof(_tTempBaro) - 1;
+		type = pTypeTEMP_BARO;
+		subtype = sTypeBMP085;
+		id1 = 1;
+		temp = 0;
+		baro = 0;
+		altitude = 0;
+		forecast = 0;
 	}
 } _tTempBaro;
 
@@ -331,11 +382,23 @@ typedef struct _tAirQualityMeter {
 	uint8_t	id1;
 	uint8_t	id2;
 	int32_t airquality;
+
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("airquality", airquality);
+	}
+
 	_tAirQualityMeter()
 	{
-		len=sizeof(_tAirQualityMeter)-1;
-		type=pTypeAirQuality;
-		subtype=sTypeVoltcraft;
+		len = sizeof(_tAirQualityMeter) - 1;
+		type = pTypeAirQuality;
+		subtype = sTypeVoltcraft;
 		id1 = 0;
 		id2 = 1;
 		airquality = 0;
@@ -353,17 +416,31 @@ typedef struct _tUsageMeter {
 	uint8_t dunit;
 	float fusage;
 
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("id3", id3);
+		ar & cereal::make_nvp("id4", id4);
+		ar & cereal::make_nvp("dunit", dunit);
+		ar & cereal::make_nvp("fusage", fusage);
+	}
+
 	_tUsageMeter()
 	{
-		len=sizeof(_tUsageMeter)-1;
-		type=pTypeUsage;
-		subtype=sTypeElectric;
-		id1=0;
-		id2=0;
-		id3=0;
-		id4=0;
-		dunit=0;
-		fusage=0;
+		len = sizeof(_tUsageMeter) - 1;
+		type = pTypeUsage;
+		subtype = sTypeElectric;
+		id1 = 0;
+		id2 = 0;
+		id3 = 0;
+		id4 = 0;
+		dunit = 0;
+		fusage = 0;
 	}
 } UsageMeter;
 
@@ -379,18 +456,33 @@ typedef struct _tLightMeter {
 	uint8_t battery_level;
 	float fLux;
 
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("id3", id3);
+		ar & cereal::make_nvp("id4", id4);
+		ar & cereal::make_nvp("dunit", dunit);
+		ar & cereal::make_nvp("battery_level", battery_level);
+		ar & cereal::make_nvp("fLux", fLux);
+	}
+
 	_tLightMeter()
 	{
-		len=sizeof(_tLightMeter)-1;
-		type=pTypeLux;
-		subtype=sTypeLux;
-		id1=0;
-		id2=0;
-		id3=0;
-		id4=0;
-		battery_level=255;
-		dunit=0;
-		fLux=0;
+		len = sizeof(_tLightMeter) - 1;
+		type = pTypeLux;
+		subtype = sTypeLux;
+		id1 = 0;
+		id2 = 0;
+		id3 = 0;
+		id4 = 0;
+		battery_level = 255;
+		dunit = 0;
+		fLux = 0;
 	}
 } LightMeter;
 
@@ -404,16 +496,31 @@ typedef struct _tGeneralDevice {
 	int32_t intval1;
 	int32_t intval2;
 	char text[64];
+
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id", id);
+		ar & cereal::make_nvp("floatval1", floatval1);
+		ar & cereal::make_nvp("floatval2", floatval2);
+		ar & cereal::make_nvp("intval1", intval1);
+		ar & cereal::make_nvp("intval2", intval2);
+		ar & cereal::make_nvp("text", text);
+	}
+
 	_tGeneralDevice()
 	{
-		len=sizeof(_tGeneralDevice)-1;
-		type=pTypeGeneral;
-		subtype=sTypeVisibility;
-		id=0;
-		floatval1=0;
-		floatval2=0;
-		intval1=0;
-		intval2=0;
+		len = sizeof(_tGeneralDevice) - 1;
+		type = pTypeGeneral;
+		subtype = sTypeVisibility;
+		id = 0;
+		floatval1 = 0;
+		floatval2 = 0;
+		intval1 = 0;
+		intval2 = 0;
 		text[0] = 0;
 	}
 } GeneralDevice;
@@ -431,6 +538,24 @@ typedef struct _tGeneralSwitch {
 	uint8_t seqnbr;
 	int32_t reserved_int;
 	float	reserved_float;
+
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id", id);
+		ar & cereal::make_nvp("unitcode", unitcode);
+		ar & cereal::make_nvp("cmnd", cmnd);
+		ar & cereal::make_nvp("level", level);
+		ar & cereal::make_nvp("rssi", rssi);
+		ar & cereal::make_nvp("battery_level", battery_level);
+		ar & cereal::make_nvp("seqnbr", seqnbr);
+		ar & cereal::make_nvp("reserved_int", reserved_int);
+		ar & cereal::make_nvp("reserved_float", reserved_float);
+	}
+
 	_tGeneralSwitch()
 	{
 		len = sizeof(_tGeneralSwitch) - 1;
@@ -444,156 +569,158 @@ typedef struct _tGeneralSwitch {
 		level = 0;
 		battery_level = 255;
 		rssi = 12;
+		cmnd = 0;
 	}
 } GeneralSwitch;
 
 typedef struct _tP1Power {
-	uint8_t len;
-	uint8_t type;
-	uint8_t subtype;
-	uint32_t powerusage1;
-	uint32_t powerusage2;
-	uint32_t powerdeliv1;
-	uint32_t powerdeliv2;
-	uint32_t usagecurrent;
-	uint32_t delivcurrent;
-	int32_t ID;
-	_tP1Power()
+	uint8_t len = sizeof(_tP1Power) - 1;
+	uint8_t type = pTypeP1Power;
+	uint8_t subtype = sTypeP1Power;
+	uint32_t powerusage1 = 0;
+	uint32_t powerusage2 = 0;
+	uint32_t powerdeliv1 = 0;
+	uint32_t powerdeliv2 = 0;
+	uint32_t usagecurrent = 0;
+	uint32_t delivcurrent = 0;
+	int32_t ID = 1;
+
+	template <class Archive>
+	void serialize(Archive & ar)
 	{
-		len = sizeof(_tP1Power) - 1;
-		type = pTypeP1Power;
-		subtype = sTypeP1Power;
-		ID = 1;
-		powerusage1 = 0;
-		powerusage2 = 0;
-		powerdeliv1 = 0;
-		powerdeliv2 = 0;
-		usagecurrent = 0;
-		delivcurrent = 0;
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("powerusage1", powerusage1);
+		ar & cereal::make_nvp("powerusage2", powerusage2);
+		ar & cereal::make_nvp("powerdeliv1", powerdeliv1);
+		ar & cereal::make_nvp("powerdeliv2", powerdeliv2);
+		ar & cereal::make_nvp("usagecurrent", usagecurrent);
+		ar & cereal::make_nvp("delivcurrent", delivcurrent);
+		ar & cereal::make_nvp("ID", ID);
 	}
 } P1Power;
 
 typedef struct _tP1Gas {
-	uint8_t len;
-	uint8_t type;
-	uint8_t subtype;
-	uint32_t gasusage;
-	int32_t ID;
-	_tP1Gas()
+	uint8_t len = sizeof(_tP1Gas) - 1;
+	uint8_t type = pTypeP1Gas;
+	uint8_t subtype = sTypeP1Gas;
+	uint32_t gasusage = 0;
+	int32_t ID = 1;
+
+	template <class Archive>
+	void serialize(Archive & ar)
 	{
-		len = sizeof(_tP1Gas) - 1;
-		type = pTypeP1Gas;
-		subtype = sTypeP1Gas;
-		ID = 1;
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("gasusage", gasusage);
+		ar & cereal::make_nvp("ID", ID);
 	}
 } P1Gas;
 
-typedef struct _tLimitlessLights {
+typedef struct _tEVOHOME1 {
 	uint8_t len;
 	uint8_t type;
 	uint8_t subtype;
-	uint32_t id;
-	uint8_t dunit; //0=All, 1=Group1,2=Group2,3=Group3,4=Group4, 5=IboxLed
-	uint8_t command;
-	uint8_t value;
-	_tLimitlessLights()
+	uint8_t	id1;
+	uint8_t	id2;
+	uint8_t	id3;
+	uint8_t	status;
+	uint8_t	mode;
+	uint16_t year;
+	uint8_t	month;
+	uint8_t	day;
+	uint8_t	hrs;
+	uint8_t	mins;
+	uint8_t	action;
+
+	template <class Archive>
+	void serialize(Archive & ar)
 	{
-		id = 1;
-		dunit = 1;
-		len=sizeof(_tLimitlessLights)-1;
-		type=pTypeLimitlessLights;
-		subtype=sTypeLimitlessRGBW;
-		command=0;
-		value=0;
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("id3", id3);
+		ar & cereal::make_nvp("status", status);
+		ar & cereal::make_nvp("mode", mode);
+		ar & cereal::make_nvp("year", year);
+		ar & cereal::make_nvp("month", month);
+		ar & cereal::make_nvp("day", day);
+		ar & cereal::make_nvp("hrs", hrs);
+		ar & cereal::make_nvp("mins", mins);
+		ar & cereal::make_nvp("action", action);
 	}
-} _tLimitlessLights;
+} EVOHOME1;
 
-#define Limitless_LedOff 0
-#define Limitless_LedOn 1
-#define Limitless_LedNight 2
-#define Limitless_LedFull 3
-#define Limitless_BrightnessUp 4
-#define Limitless_BrightnessDown 5
-#define Limitless_ColorTempUp 6
-#define Limitless_ColorTempDown 7
-#define Limitless_RGBDiscoNext 8
-#define Limitless_RGBDiscoPrevious 9
-#define Limitless_SetRGBColour 10
-#define Limitless_DiscoSpeedSlower 11
-#define Limitless_DiscoSpeedFaster 12
-#define Limitless_DiscoMode 13
-#define Limitless_SetColorToWhite 14
-#define Limitless_SetBrightnessLevel 15
-#define Limitless_SetBrightUp 16
-#define Limitless_SetBrightDown 17
-#define Limitless_WarmWhiteIncrease 18
-#define Limitless_CoolWhiteIncrease 19
-#define Limitless_NightMode 20
-#define Limitless_FullBrightness 21
-#define Limitless_DiscoSpeedFasterLong 22 //exclude RGB
-#define Limitless_SetHEXColour 23
-#define Limitless_DiscoMode_1 24
-#define Limitless_DiscoMode_2 25
-#define Limitless_DiscoMode_3 26
-#define Limitless_DiscoMode_4 27
-#define Limitless_DiscoMode_5 28
-#define Limitless_DiscoMode_6 29
-#define Limitless_DiscoMode_7 30
-#define Limitless_DiscoMode_8 31
-#define Limitless_DiscoMode_9 32
-#define Limitless_SetKelvinLevel 33
-#define Limitless_DiscoSpeedMinimal 34
-#define Limitless_DiscoSpeedMaximal 35
+typedef struct _tEVOHOME2 {
+	uint8_t len;
+	uint8_t type;
+	uint8_t subtype;
+	uint8_t	id1;
+	uint8_t	id2;
+	uint8_t	id3;
+	uint8_t	zone;
+	uint8_t	updatetype;
+	int16_t	temperature;
+	uint8_t	mode;
+	uint8_t	controllermode;
+	uint16_t year;
+	uint8_t	month;
+	uint8_t	day;
+	uint8_t	hrs;
+	uint8_t	mins;
+	uint8_t	battery_level;
 
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("id3", id3);
+		ar & cereal::make_nvp("zone", zone);
+		ar & cereal::make_nvp("updatetype", updatetype);
+		ar & cereal::make_nvp("temperature", temperature);
+		ar & cereal::make_nvp("mode", mode);
+		ar & cereal::make_nvp("controllermode", controllermode);
+		ar & cereal::make_nvp("year", year);
+		ar & cereal::make_nvp("month", month);
+		ar & cereal::make_nvp("day", day);
+		ar & cereal::make_nvp("hrs", hrs);
+		ar & cereal::make_nvp("mins", mins);
+		ar & cereal::make_nvp("battery_level", battery_level);
+	}
+} EVOHOME2;
 
-typedef union tREVOBUF {
-	struct _tEVOHOME1 {
-		uint8_t len;
-		uint8_t type;
-		uint8_t subtype;
-		uint8_t	id1;
-		uint8_t	id2;
-		uint8_t	id3;
-		uint8_t	status;
-		uint8_t	mode;
-		uint16_t year;
-		uint8_t	month;
-		uint8_t	day;
-		uint8_t	hrs;
-		uint8_t	mins;
-		uint8_t	action;
-	} EVOHOME1;
+typedef struct _tEVOHOME3 {
+	uint8_t len;
+	uint8_t type;
+	uint8_t subtype;
+	uint8_t	id1;
+	uint8_t	id2;
+	uint8_t	id3;
+	uint8_t	devno;
+	uint8_t	demand;
+	uint8_t	updatetype;
+	uint8_t	battery_level;
 
-	struct _tEVOHOME2 {
-		uint8_t len;
-		uint8_t type;
-		uint8_t subtype;
-		uint8_t	id1;
-		uint8_t	id2;
-		uint8_t	id3;
-		uint8_t	zone;
-		uint8_t	updatetype;
-		int16_t	temperature;
-		uint8_t	mode;
-		uint8_t	controllermode;
-		uint16_t year;
-		uint8_t	month;
-		uint8_t	day;
-		uint8_t	hrs;
-		uint8_t	mins;
-		uint8_t	battery_level;
-	} EVOHOME2;
-
-	struct _tEVOHOME3 {
-		uint8_t len;
-		uint8_t type;
-		uint8_t subtype;
-		uint8_t	id1;
-		uint8_t	id2;
-		uint8_t	id3;
-		uint8_t	devno;
-		uint8_t	demand;
-		uint8_t	updatetype;
-		uint8_t	battery_level;
-	} EVOHOME3;
-} REVOBUF;
+	template <class Archive>
+	void serialize(Archive & ar)
+	{
+		ar & cereal::make_nvp("len", len);
+		ar & cereal::make_nvp("type", type);
+		ar & cereal::make_nvp("subtype", subtype);
+		ar & cereal::make_nvp("id1", id1);
+		ar & cereal::make_nvp("id2", id2);
+		ar & cereal::make_nvp("id3", id3);
+		ar & cereal::make_nvp("devno", devno);
+		ar & cereal::make_nvp("demand", demand);
+		ar & cereal::make_nvp("updatetype", updatetype);
+		ar & cereal::make_nvp("battery_level", battery_level);
+	}
+} EVOHOME3;
