@@ -5,7 +5,7 @@
 //
 #ifdef ENABLE_PYTHON
 
-#include "../main/Helper.h"
+#include "../../main/Helper.h"
 #include "DelayedLink.h"
 
 namespace Plugins {
