@@ -1,5 +1,7 @@
 #pragma once
 
+#include "../../main/StoppableTask.h"
+
 //
 //	Domoticz Plugin System - Dnpwwo, 2016
 //
@@ -8,7 +10,7 @@ class CDomoticzHardwareBase;
 
 namespace Plugins {
 
-	class CPluginSystem
+	class CPluginSystem : public StoppableTask
 	{
 	private:
 		bool	m_bEnabled;
@@ -20,12 +22,10 @@ namespace Plugins {
 		static	std::map<int, CDomoticzHardwareBase*>	m_pPlugins;
 		static	std::map<std::string, std::string>		m_PluginXml;
 
-		boost::thread* m_thread;
-		volatile bool m_stoprequested;
-		boost::mutex m_mutex;
+		std::shared_ptr<std::thread> m_thread;
+		std::mutex m_mutex;
 
 		void Do_Work();
-
 	public:
 		CPluginSystem();
 		~CPluginSystem(void);
@@ -36,9 +36,11 @@ namespace Plugins {
 		std::map<int, CDomoticzHardwareBase*>* GetHardware() { return &m_pPlugins; };
 		CDomoticzHardwareBase* RegisterPlugin(const int HwdID, const std::string &Name, const std::string &PluginKey);
 		void	 DeregisterPlugin(const int HwdID);
-		bool StopPluginSystem();
-		void AllPluginsStarted() { m_bAllPluginsStarted = true; };
+		bool	StopPluginSystem();
+		void	AllPluginsStarted() { m_bAllPluginsStarted = true; };
 		static void LoadSettings();
+		void	DeviceModified(uint64_t ID);
+		void*	PythonThread() { return m_InitialPythonThread; };
 	};
 };
 
