@@ -1,24 +1,19 @@
 #pragma once
 
 #include <string>
-#include <vector>
+#include <boost/thread/shared_mutex.hpp>
 
-extern "C" {
-#ifdef WITH_EXTERNAL_LUA
-#include <lua.h>
-#include <lualib.h>
-#include <lauxlib.h>
-#else
-#include "../lua/src/lua.h"
-#include "../lua/src/lualib.h"
-#include "../lua/src/lauxlib.h"
-#endif
-}
+#include "../httpclient/HTTPClient.h"
 
 #include "LuaCommon.h"
+#include "concurrent_queue.h"
+#include "StoppableTask.h"
+#include "NotificationObserver.h"
 
-class CEventSystem : public CLuaCommon
+class CEventSystem : public CLuaCommon, StoppableTask, CNotificationObserver
 {
+	friend class CdzVents;
+	friend class CLuaHandler;
 	typedef struct lua_State lua_State;
 
 	struct _tEventItem
@@ -37,14 +32,25 @@ class CEventSystem : public CLuaCommon
 	struct _tActionParseResults
 	{
 		std::string sCommand;
-		float fForSec;
-		float fAfterSec;
-		float fRandomSec;
-		int iRepeat;
-		float fRepeatSec;
+		float fForSec = 0;
+		float fAfterSec = 0;
+		float fRandomSec = 0;
+		int iRepeat = 1;
+		float fRepeatSec = 0;
+		bool bEventTrigger = false;
+	};
+public:
+	enum _eReason
+	{
+		REASON_DEVICE,			// 0
+		REASON_SCENEGROUP,		// 1
+		REASON_USERVARIABLE,		// 2
+		REASON_TIME,			// 3
+		REASON_SECURITY,		// 4
+		REASON_URL,			// 5
+		REASON_NOTIFICATION		// 6
 	};
 
-public:
 	struct _tDeviceStatus
 	{
 		uint64_t ID;
@@ -60,6 +66,7 @@ public:
 		std::string description;
 		std::string deviceID;
 		int batteryLevel;
+		int protection;
 		int signalLevel;
 		int unit;
 		int hardwareID;
@@ -84,7 +91,9 @@ public:
 		std::string scenesgroupName;
 		std::string scenesgroupValue;
 		int scenesgroupType;
+		int protection;
 		std::string lastUpdate;
+		std::vector<uint64_t> memberID;
 	};
 
 	struct _tHardwareListInt {
@@ -101,73 +110,130 @@ public:
 	void StopEventSystem();
 
 	void LoadEvents();
-	void ProcessUserVariable(const uint64_t varId);
-	void ProcessDevice(const int HardwareID, const uint64_t ulDevID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const unsigned char signallevel, const unsigned char batterylevel, const int nValue, const char* sValue, const std::string &devname, const int varId);
-	void RemoveSingleState(int ulDevID);
-	void WWWUpdateSingleState(const uint64_t ulDevID, const std::string &devname);
+	void ProcessDevice(const int HardwareID, const uint64_t ulDevID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const unsigned char signallevel, const unsigned char batterylevel, const int nValue, const char* sValue, const std::string &devname);
+	void UpdateBatteryLevel(const uint64_t ulDevID, const unsigned char batteryLevel);
+
+	void RemoveSingleState(const uint64_t ulDevID, const _eReason reason);
+	void WWWUpdateSingleState(const uint64_t ulDevID, const std::string &devname, const _eReason reason);
 	void WWWUpdateSecurityState(int securityStatus);
 	void WWWGetItemStates(std::vector<_tDeviceStatus> &iStates);
 	void SetEnabled(const bool bEnabled);
 	void GetCurrentStates();
 	void GetCurrentScenesGroups();
 	void GetCurrentUserVariables();
-	void UpdateScenesGroups(const uint64_t ulDevID, const int nValue, const std::string &lastUpdate);
-	void UpdateUserVariable(const uint64_t ulDevID, const std::string &varName, const std::string varValue, const int varType, const std::string &lastUpdate);
-	void ExportDeviceStatesToLua(lua_State *lua_state);
+	bool UpdateSceneGroup(const uint64_t ulDevID, const int nValue, const std::string &lastUpdate);
+	void UpdateUserVariable(const uint64_t ulDevID, const std::string &varValue, const std::string &lastUpdate);
 	bool PythonScheduleEvent(std::string ID, const std::string &Action, const std::string &eventName);
+	bool GetEventTrigger(const uint64_t ulDevID, const _eReason reason, const bool bEventTrigger);
+	void SetEventTrigger(const uint64_t ulDevID, const _eReason reason, const float fDelayTime);
+	bool CustomCommand(const uint64_t idx, const std::string &sCommand);
+
+	void TriggerURL(const std::string &result, const std::vector<std::string> &headerData, const std::string &callback);
 
 private:
-	//lua_State	*m_pLUA;
+	enum _eJsonType
+	{
+		JTYPE_STRING = 0,	// 0
+		JTYPE_FLOAT,		// 1
+		JTYPE_INT,			// 2
+		JTYPE_BOOL			// 3
+	};
+
+	struct _tJsonMap
+	{
+		const char* szOriginal;
+		const char* szNew;
+		_eJsonType eType;
+	};
+
+	struct _tEventTrigger
+	{
+		uint64_t ID;
+		_eReason reason;
+		time_t timestamp;
+	};
+
+	struct _tEventQueue
+	{
+		_eReason reason;
+		uint64_t id;
+		std::string devname;
+		int nValue;
+		std::string sValue;
+		std::string nValueWording;
+		std::string lastUpdate;
+		uint8_t lastLevel;
+		std::vector<std::string> vData;
+		std::map<uint8_t, int> JsonMapInt;
+		std::map<uint8_t, float> JsonMapFloat;
+		std::map<uint8_t, bool> JsonMapBool;
+		std::map<uint8_t, std::string> JsonMapString;
+		queue_element_trigger* trigger = nullptr;
+	};
+	concurrent_queue<_tEventQueue> m_eventqueue;
+
+	std::vector<_tEventTrigger> m_eventtrigger;
 	bool m_bEnabled;
-	bool m_bdzVentsExist;
 	boost::shared_mutex m_devicestatesMutex;
 	boost::shared_mutex m_eventsMutex;
 	boost::shared_mutex m_uservariablesMutex;
 	boost::shared_mutex m_scenesgroupsMutex;
-	boost::mutex m_measurementStatesMutex;
-	boost::mutex luaMutex;
-	volatile bool m_stoprequested;
-	boost::shared_ptr<boost::thread> m_thread;
+	boost::shared_mutex m_eventtriggerMutex;
+	std::mutex m_measurementStatesMutex;
+	std::mutex luaMutex;
+	std::shared_ptr<std::thread> m_thread;
+	std::shared_ptr<std::thread> m_eventqueuethread;
+	StoppableTask m_TaskQueue;
 	int m_SecStatus;
 	std::string m_lua_Dir;
-	std::string m_dzv_Dir;
+	std::string m_szStartTime;
+
+	static const std::string m_szReason[], m_szSecStatus[];
+	static const _tJsonMap JsonMap[];
 
 	//our thread
 	void Do_Work();
 	void ProcessMinute();
 	void GetCurrentMeasurementStates();
-	std::string UpdateSingleState(const uint64_t ulDevID, const std::string &devname, const int nValue, const char* sValue, const unsigned char devType, const unsigned char subType, const _eSwitchType switchType, const std::string &lastUpdate, const unsigned char lastLevel, const std::map<std::string, std::string> & options);
-	void EvaluateEvent(const std::string &reason);
-	void EvaluateEvent(const std::string &reason, const uint64_t varId);
-	void EvaluateEvent(const std::string &reason, const uint64_t DeviceID, const std::string &devname, const int nValue, const char* sValue, std::string nValueWording, const uint64_t varId);
-	void EvaluateBlockly(const std::string &reason, const uint64_t DeviceID, const std::string &devname, const int nValue, const char* sValue, std::string nValueWording, const uint64_t varId);
-	bool parseBlocklyActions(const std::string &Actions, const std::string &eventName, const uint64_t eventID);
+	std::string UpdateSingleState(
+		const uint64_t ulDevID, 
+		const std::string &devname, 
+		const int nValue, const std::string &sValue, 
+		const unsigned char devType, const unsigned char subType, 
+		const _eSwitchType switchType, 
+		const std::string &lastUpdate, 
+		const unsigned char lastLevel, 
+		const unsigned char batteryLevel,
+		const std::map<std::string, std::string> & options
+	);
+	void EvaluateEvent(const std::vector<_tEventQueue> &items);
+	void EvaluateDatabaseEvents(const _tEventQueue &item);
+	lua_State *ParseBlocklyLua(lua_State *lua_state, const _tEventItem &item);
+	bool parseBlocklyActions(const _tEventItem &item);
 	std::string ProcessVariableArgument(const std::string &Argument);
 #ifdef ENABLE_PYTHON
 	std::string m_python_Dir;
-	void EvaluatePython(const std::string &reason, const std::string &filename, const std::string &PyString, const uint64_t varId);
-	void EvaluatePython(const std::string &reason, const std::string &filename, const std::string &PyString);
-	void EvaluatePython(const std::string &reason, const std::string &filename, const std::string &PyString, const uint64_t DeviceID, const std::string &devname, const int nValue, const char* sValue, std::string nValueWording, const uint64_t varId);
+	void EvaluatePython(const _tEventQueue &item, const std::string &filename, const std::string &PyString);
 #endif
-	void EvaluateLua(const std::string &reason, const std::string &filename, const std::string &LuaString, const uint64_t varId);
-	void EvaluateLua(const std::string &reason, const std::string &filename, const std::string &LuaString);
-	void ExportDomoticzDataToLua(lua_State *lua_state, uint64_t deviceID, uint64_t varID);
-	void EvaluateLua(const std::string &reason, const std::string &filename, const std::string &LuaString, const uint64_t DeviceID, const std::string &devname, const int nValue, const char* sValue, std::string nValueWording, const uint64_t varId);
+	void EvaluateLua(const _tEventQueue &item, const std::string &filename, const std::string &LuaString);
+	void EvaluateLua(const std::vector<_tEventQueue> &items, const std::string &filename, const std::string &LuaString);
 	void luaThread(lua_State *lua_state, const std::string &filename);
 	static void luaStop(lua_State *L, lua_Debug *ar);
 	std::string nValueToWording(const uint8_t dType, const uint8_t dSubType, const _eSwitchType switchtype, const int nValue, const std::string &sValue, const std::map<std::string, std::string> & options);
 	static int l_domoticz_print(lua_State* lua_state);
-	void OpenURL(const std::string &URL);
+	void OpenURL(const float delay, const std::string &URL);
 	void WriteToLog(const std::string &devNameNoQuotes, const std::string &doWhat);
-	bool ScheduleEvent(int deviceID, std::string Action, bool isScene, const std::string &eventName, int sceneType);
+	bool ScheduleEvent(int deviceID, const std::string &Action, bool isScene, const std::string &eventName, int sceneType);
 	bool ScheduleEvent(std::string ID, const std::string &Action, const std::string &eventName);
-	void UpdateDevice(const std::string &DevParams);
-	void UpdateLastUpdate(const uint64_t ulDevID, const std::string &lastUpdate, const uint8_t lastLevel);
 	lua_State *CreateBlocklyLuaState();
 
 	std::string ParseBlocklyString(const std::string &oString);
 	void ParseActionString( const std::string &oAction_, _tActionParseResults &oResults_ );
 	void UpdateJsonMap(_tDeviceStatus &item, const uint64_t ulDevID);
+	void EventQueueThread();
+	void UnlockEventQueueThread();
+	void ExportDeviceStatesToLua(lua_State *lua_state, const _tEventQueue &item);
+	void EvaluateLuaClassic(lua_State *lua_state, const _tEventQueue &item, const int secStatus);
 
 	//std::string reciprocalAction (std::string Action);
 	std::vector<_tEventItem> m_events;
@@ -204,14 +270,16 @@ private:
 	std::map<uint64_t, float> m_windgustValuesByID;
 	std::map<uint64_t, int> m_zwaveAlarmValuesByID;
 
-	void reportMissingDevice(const int deviceID, const std::string &EventName, const uint64_t eventID);
+	void reportMissingDevice(const int deviceID, const _tEventItem &item);
 	int getSunRiseSunSetMinutes(const std::string &what);
 	bool isEventscheduled(const std::string &eventName);
 	bool iterateLuaTable(lua_State *lua_state, const int tIndex, const std::string &filename);
 	bool processLuaCommand(lua_State *lua_state, const std::string &filename);
 	void report_errors(lua_State *L, int status, std::string filename);
-	unsigned char calculateDimLevel(int deviceID, int percentageLevel);
+	int calculateDimLevel(int deviceID, int percentageLevel);
 	void StripQuotes(std::string &sString);
 	std::string SpaceToUnderscore(std::string sResult);
 	std::string LowerCase(std::string sResult);
+
+	bool Update(const Notification::_eType type, const Notification::_eStatus status, const std::string &eventdata) override;
 };
