@@ -12,11 +12,11 @@
     namespace Plugins {
         #define GETSTATE(m) ((struct eventModule_state*)PyModule_GetState(m))
 
-		extern boost::mutex PythonMutex;		// only used during startup when multiple threads could use Python
+		extern std::mutex PythonMutex;		// only used during startup when multiple threads could use Python
 
 		void*   m_PyInterpreter;
         bool ModuleInitalized = false;
-        
+
         struct eventModule_state {
             PyObject*	error;
         };
@@ -51,7 +51,7 @@
             else
             {
                 std::string	message = msg;
-                _log.Log((_eLogLevel)LOG_NORM, message.c_str());
+                _log.Log((_eLogLevel)LOG_NORM, message);
             }
 
             Py_INCREF(Py_None);
@@ -73,8 +73,8 @@
             }
             else
             {
-                std::string	dev = device;
-                std::string act = action;
+                //std::string	dev = device;
+                //std::string act = action;
                 // _log.Log((_eLogLevel)LOG_NORM, "Python EventSystem - Command: Target: %s Command: %s", dev.c_str(), act.c_str());
                 m_mainworker.m_eventsystem.PythonScheduleEvent(device, action, "Test");
             }
@@ -105,64 +105,69 @@
             PyObject* pModule = PyModule_Create2(&DomoticzEventsModuleDef, PYTHON_API_VERSION);
             return pModule;
         }
-        
+
         int PythonEventsInitalized = 0;
 
-        bool PythonEventsInitialize(std::string szUserDataFolder) {
-            
+        bool PythonEventsInitialize(const std::string &szUserDataFolder) {
+
             if (!Plugins::Py_LoadLibrary())
             {
                 _log.Log(LOG_STATUS, "EventSystem - Python: Failed dynamic library load, install the latest libpython3.x library that is available for your platform.");
                 return false;
             }
-            
+
             if (!Plugins::Py_IsInitialized()) {
                 _log.Log(LOG_STATUS, "EventSystem - Python: Failed dynamic library load, install the latest libpython3.x library that is available for your platform.");
                 return false;
             }
-            
-			boost::lock_guard<boost::mutex> l(PythonMutex);
+
+			std::lock_guard<std::mutex> l(PythonMutex);
+			PyEval_RestoreThread((PyThreadState*)m_mainworker.m_pluginsystem.PythonThread());
 			m_PyInterpreter = Py_NewInterpreter();
             if (!m_PyInterpreter)
             {
                 _log.Log(LOG_ERROR, "EventSystem - Python: Failed to create interpreter.");
                 return false;
             }
-            
+
             std::string ssPath;
 #ifdef WIN32
             ssPath  = szUserDataFolder + "scripts\\python\\;";
 #else
             ssPath  = szUserDataFolder + "scripts/python/:";
 #endif
-            
+
             std::wstring sPath = std::wstring(ssPath.begin(), ssPath.end());
-            
+
             sPath += Plugins::Py_GetPath();
             Plugins::PySys_SetPath((wchar_t*)sPath.c_str());
-            
+
             PythonEventsInitalized = 1;
-            
+
             PyObject* pModule = Plugins::PythonEventsGetModule();
-            if (!pModule) {
+			PyEval_SaveThread();
+			if (!pModule) {
                 _log.Log(LOG_ERROR, "EventSystem - Python: Failed to initialize module.");
                 return false;
             }
             ModuleInitalized = true;
             return true;
         }
-        
+
         bool PythonEventsStop() {
             if (m_PyInterpreter) {
-                PyEval_RestoreThread((PyThreadState*)m_PyInterpreter);
+				std::lock_guard<std::mutex> l(PythonMutex);
+				PyEval_RestoreThread((PyThreadState*)m_PyInterpreter);
 				if (Plugins::Py_IsInitialized())
 					Py_EndInterpreter((PyThreadState*)m_PyInterpreter);
-                _log.Log(LOG_STATUS, "EventSystem - Python stopped...");
+				m_PyInterpreter = NULL;
+				PyEval_ReleaseLock();
+				_log.Log(LOG_STATUS, "EventSystem - Python stopped...");
                 return true;
             } else
                 return false;
         }
-        
+
         PyObject* PythonEventsGetModule (void) {
             PyObject* pModule = PyState_FindModule(&DomoticzEventsModuleDef);
 
@@ -170,7 +175,6 @@
                 // _log.Log(LOG_STATUS, "Python Event System: Module found");
                 return pModule;
             } else {
-                _log.Log(LOG_STATUS, "Python EventSystem: Module not found - Trying to initialize.");
                 Plugins::PyRun_SimpleStringFlags("import DomoticzEvents", NULL);
                 pModule = PyState_FindModule(&DomoticzEventsModuleDef);
 
@@ -185,29 +189,30 @@
         }
 
         // main_namespace["otherdevices_temperature"] = toPythonDict(m_tempValuesByName);
-        
-        PyObject* mapToPythonDict(std::map<std::string, float> floatMap) {
-            
+
+        PyObject* mapToPythonDict(const std::map<std::string, float> &floatMap) {
+
             return Py_None;
         }
-       
+
 
         void PythonEventsProcessPython(const std::string &reason, const std::string &filename, const std::string &PyString, const uint64_t DeviceID, std::map<uint64_t, CEventSystem::_tDeviceStatus> m_devicestates, std::map<uint64_t, CEventSystem::_tUserVariable> m_uservariables, int intSunRise, int intSunSet) {
 
-       
+
             if (!ModuleInitalized) {
                 return;
             }
-            
+
 
            if (Plugins::Py_IsInitialized()) {
-               
-               if (m_PyInterpreter) PyEval_RestoreThread((PyThreadState*)m_PyInterpreter);
-               
+
+			   std::lock_guard<std::mutex> l(PythonMutex);
+			   if (m_PyInterpreter) PyEval_RestoreThread((PyThreadState*)m_PyInterpreter);
+
                /*{
                    _log.Log(LOG_ERROR, "EventSystem - Python: Failed to attach to interpreter");
                }*/
-            
+
                PyObject* pModule = Plugins::PythonEventsGetModule();
                if (pModule) {
 
@@ -215,7 +220,8 @@
 
                    if (!pModuleDict) {
                        _log.Log(LOG_ERROR, "Python EventSystem: Failed to open module dictionary.");
-                       return;
+					   PyEval_SaveThread();
+					   return;
                    }
 
                    if (Plugins::PyDict_SetItemString(pModuleDict, "changed_device_name", Plugins::PyUnicode_FromString(m_devicestates[DeviceID].deviceName.c_str())) == -1) {
@@ -228,22 +234,24 @@
                    if (Plugins::PyDict_SetItemString(pModuleDict, "Devices", (PyObject*)m_DeviceDict) == -1)
                    {
                        _log.Log(LOG_ERROR, "Python EventSystem: Failed to add Device dictionary.");
-                       return;
+					   PyEval_SaveThread();
+					   return;
                    }
                    Py_DECREF(m_DeviceDict);
 
                    if (Plugins::PyType_Ready(&Plugins::PDeviceType) < 0) {
                        _log.Log(LOG_ERROR, "Python EventSystem: Unable to ready DeviceType Object.");
-                       return;
+					   PyEval_SaveThread();
+					   return;
                    }
 
                    // Mutex
                    // boost::shared_lock<boost::shared_mutex> devicestatesMutexLock1(m_devicestatesMutex);
 
-                   typedef std::map<uint64_t, CEventSystem::_tDeviceStatus>::iterator it_type;
-                   for (it_type iterator = m_devicestates.begin(); iterator != m_devicestates.end(); ++iterator)
+                   std::map<uint64_t, CEventSystem::_tDeviceStatus>::const_iterator it_type;
+                   for (it_type = m_devicestates.begin(); it_type != m_devicestates.end(); ++it_type)
                    {
-                       CEventSystem::_tDeviceStatus sitem = iterator->second;
+                       CEventSystem::_tDeviceStatus sitem = it_type->second;
                        // object deviceStatus = domoticz_module.attr("Device")(sitem.ID, sitem.deviceName, sitem.devType, sitem.subType, sitem.switchtype, sitem.nValue, sitem.nValueWording, sitem.sValue, sitem.lastUpdate);
                        // devices[sitem.deviceName] = deviceStatus;
 
@@ -330,38 +338,80 @@
                    if (Plugins::PyDict_SetItemString(pModuleDict, "user_variables", (PyObject*)m_uservariablesDict) == -1)
                    {
                        _log.Log(LOG_ERROR, "Python EventSystem: Failed to add uservariables dictionary.");
-                       return;
+					   PyEval_SaveThread();
+					   return;
                    }
                    Py_DECREF(m_uservariablesDict);
 
                    // This doesn't work
                    // boost::unique_lock<boost::shared_mutex> uservariablesMutexLock2 (m_uservariablesMutex);
 
-                   typedef std::map<uint64_t, CEventSystem::_tUserVariable>::iterator it_var;
-                   for (it_var iterator = m_uservariables.begin(); iterator != m_uservariables.end(); ++iterator) {
-                       CEventSystem::_tUserVariable uvitem = iterator->second;
+                   std::map<uint64_t, CEventSystem::_tUserVariable>::const_iterator it_var;
+                   for (it_var = m_uservariables.begin(); it_var != m_uservariables.end(); ++it_var) {
+                       CEventSystem::_tUserVariable uvitem = it_var->second;
                        Plugins::PyDict_SetItemString(m_uservariablesDict, uvitem.variableName.c_str(), Plugins::PyUnicode_FromString(uvitem.variableValue.c_str()));
                    }
 
                    // uservariablesMutexLock2.unlock();
-                   
+
+                   // Add __main__ module
+                   PyObject *pModule = Plugins::PyImport_AddModule("__main__");
+                   Py_INCREF(pModule);
+
+                   // Override sys.stderr
+                   Plugins::PyRun_SimpleStringFlags("import sys\nclass StdErrRedirect:\n    def __init__(self):\n        self.buffer = ''\n    def write(self, msg):\n        self.buffer += msg\nstdErrRedirect = StdErrRedirect()\nsys.stderr = stdErrRedirect\n", NULL);
 
                    if(PyString.length() > 0) {
                        // Python-string from WebEditor
                        Plugins::PyRun_SimpleStringFlags(PyString.c_str(), NULL);
-                       
                    } else {
                        // Script-file
                        FILE* PythonScriptFile = fopen(filename.c_str(), "r");
                        Plugins::PyRun_SimpleFileExFlags(PythonScriptFile, filename.c_str(), 0, NULL);
-                       
+
                        if (PythonScriptFile!=NULL)
                            fclose(PythonScriptFile);
                    }
+
+                   // Get message from stderr redirect
+                   PyObject *stdErrRedirect = NULL, *logBuffer = NULL, *logBytes = NULL;
+                   std::string logString;
+                   if ((stdErrRedirect = Plugins::PyObject_GetAttrString(pModule, "stdErrRedirect")) == NULL) goto free_module;
+                   if ((logBuffer = Plugins::PyObject_GetAttrString(stdErrRedirect, "buffer")) == NULL) goto free_stderrredirect;
+                   if ((logBytes = PyUnicode_AsUTF8String(logBuffer)) == NULL) goto free_logbuffer;
+                   logString.append(PyBytes_AsString(logBytes));
+
+                   // Check if there were some errors written to stderr
+                   if (logString.length() > 0) {
+                       // Print error source
+                       _log.Log(LOG_ERROR, "EventSystem: Failed to execute python event script \"%s\"", filename.c_str());
+
+                       // Loop over all lines of the error message
+                       std::size_t lineBreakPos;
+                       while ((lineBreakPos = logString.find('\n')) != std::string::npos) {
+                           // Print line
+                           _log.Log(LOG_ERROR, "EventSystem: %s", logString.substr(0, lineBreakPos).c_str());
+
+                           // Remove line from buffer
+                           logString = logString.substr(lineBreakPos + 1);
+                       }
+                   }
+
+                   // Cleanup
+                   Py_DECREF(logBytes);
+free_logbuffer:
+                   Py_DECREF(logBuffer);
+free_stderrredirect:
+                   Py_DECREF(stdErrRedirect);
+free_module:
+                   Py_DECREF(pModule);
                 } else {
                     _log.Log(LOG_ERROR, "Python EventSystem: Module not available to events");
                 }
-            } else {
+
+				PyEval_SaveThread();
+
+			} else {
                 _log.Log(LOG_ERROR, "EventSystem: Python not initalized");
             }
 
