@@ -1,11 +1,12 @@
 #pragma once
 
-#include <vector>
 #include <string>
 #include "RFXNames.h"
+#include "../hardware/hardwaretypes.h"
 #include "Helper.h"
 #include "../httpclient/UrlEncode.h"
-#include <map>
+#include "../httpclient/HTTPClient.h"
+#include "StoppableTask.h"
 
 #define timer_resolution_hz 25
 
@@ -32,6 +33,15 @@ enum _eWeightUnit
     WEIGHTUNIT_LB,
 };
 
+enum _eUsrVariableType
+{
+	USERVARTYPE_INTEGER = 0,
+	USERVARTYPE_FLOAT, //1
+	USERVARTYPE_STRING, //2
+	USERVARTYPE_DATE, //3 Date in format DD/MM/YYYY
+	USERVARTYPE_TIME, //4 Time in 24 hr format HH:MM
+};
+
 enum _eTaskItemType
 {
 	TITEM_SWITCHCMD=0,
@@ -46,6 +56,10 @@ enum _eTaskItemType
 	TITEM_SEND_SMS,
 	TITEM_SEND_NOTIFICATION,
 	TITEM_SET_SETPOINT,
+	TITEM_SEND_IFTTT_TRIGGER,
+	TITEM_UPDATEDEVICE,
+	TITEM_CUSTOM_COMMAND,
+	TITEM_CUSTOM_EVENT,
 };
 
 struct _tTaskItem
@@ -64,14 +78,26 @@ struct _tTaskItem
 	int _nValue;
 	std::string _sValue;
 	std::string _command;
-	unsigned char _level;
-	int _Hue;
+	std::string _sUntil;
+	std::string _sUser;
+	int _level;
+	_tColor _Color;
 	std::string _relatedEvent;
 	timeval _DelayTimeBegin;
 
-	_tTaskItem()
+	static _tTaskItem UpdateDevice(const float DelayTime, const uint64_t idx, const int nValue, const std::string &sValue, const int Protected, const bool bEventTrigger)
 	{
-
+		_tTaskItem tItem;
+		tItem._ItemType = TITEM_UPDATEDEVICE;
+		tItem._DelayTime = DelayTime;
+		tItem._idx = idx;
+		tItem._nValue = nValue;
+		tItem._sValue = sValue;
+		tItem._HardwareID = Protected;
+		tItem._switchtype = bEventTrigger ? 1 : 0;
+		if (DelayTime)
+			getclock(&tItem._DelayTimeBegin);
+		return tItem;
 	}
 
 	static _tTaskItem SwitchLight(const float DelayTime, const uint64_t idx, const int HardwareID, const char* ID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const int switchtype, const unsigned char signallevel, const unsigned char batterylevel, const int nValue, const char* sValue)
@@ -149,7 +175,18 @@ struct _tTaskItem
 			getclock(&tItem._DelayTimeBegin);
 		return tItem;
 	}
-	static _tTaskItem SwitchLightEvent(const float DelayTime, const uint64_t idx, const std::string &Command, const unsigned char Level, const int Hue, const std::string &eventName)
+	static _tTaskItem SendIFTTTTrigger(const float DelayTime, const std::string &EventID, const std::string &Value1, const std::string &Value2, const std::string &Value3)
+	{
+		_tTaskItem tItem;
+		tItem._ItemType = TITEM_SEND_IFTTT_TRIGGER;
+		tItem._DelayTime = DelayTime;
+		tItem._ID = EventID;
+		tItem._command = Value1 + "!#" + Value2 + "!#" + Value3;
+		if (DelayTime)
+			getclock(&tItem._DelayTimeBegin);
+		return tItem;
+	}
+	static _tTaskItem SwitchLightEvent(const float DelayTime, const uint64_t idx, const std::string &Command, const int Level, const _tColor Color, const std::string &eventName, const std::string& User)
 	{
 		_tTaskItem tItem;
 		tItem._ItemType=TITEM_SWITCHCMD_EVENT;
@@ -157,13 +194,14 @@ struct _tTaskItem
 		tItem._idx=idx;
 		tItem._command= Command;
 		tItem._level= Level;
-		tItem._Hue=Hue;
+		tItem._Color=Color;
 		tItem._relatedEvent = eventName;
+		tItem._sUser = User;
 		if (DelayTime)
 			getclock(&tItem._DelayTimeBegin);
 		return tItem;
 	}
-	static _tTaskItem SwitchSceneEvent(const float DelayTime, const uint64_t idx, const std::string &Command, const std::string &eventName)
+	static _tTaskItem SwitchSceneEvent(const float DelayTime, const uint64_t idx, const std::string &Command, const std::string &eventName, const std::string& User)
 	{
 		_tTaskItem tItem;
 		tItem._ItemType=TITEM_SWITCHCMD_SCENE;
@@ -171,17 +209,25 @@ struct _tTaskItem
 		tItem._idx=idx;
 		tItem._command= Command;
 		tItem._relatedEvent = eventName;
+		tItem._sUser = User;
 		if (DelayTime)
 			getclock(&tItem._DelayTimeBegin);
 		return tItem;
 	}
-	static _tTaskItem GetHTTPPage(const float DelayTime, const std::string &URL, const std::string &eventName)
+	static _tTaskItem GetHTTPPage(const float DelayTime, const std::string &URL, const std::string &/*eventName*/)
+	{
+		return GetHTTPPage(DelayTime, URL, "", HTTPClient::HTTP_METHOD_GET, "", "");
+	}
+	static _tTaskItem GetHTTPPage(const float DelayTime, const std::string &URL, const std::string &extraHeaders, const HTTPClient::_eHTTPmethod method, const std::string &postData, const std::string &trigger)
 	{
 		_tTaskItem tItem;
-		tItem._ItemType=TITEM_GETURL;
-		tItem._DelayTime=DelayTime;
-		tItem._sValue= URL;
-		tItem._relatedEvent = eventName;
+		tItem._ItemType = TITEM_GETURL;
+		tItem._DelayTime = DelayTime;
+		tItem._sValue = URL;
+		tItem._switchtype = method;
+		tItem._relatedEvent = extraHeaders;
+		tItem._command = postData;
+		tItem._ID = trigger;
 		if (DelayTime)
 			getclock(&tItem._DelayTimeBegin);
 		return tItem;
@@ -204,23 +250,43 @@ struct _tTaskItem
 		tItem._ItemType = TITEM_SEND_NOTIFICATION;
 		tItem._DelayTime = DelayTime;
 		tItem._idx = Priority;
-		std::string tSubject((!Subject.empty()) ? Subject : " ");
-		std::string tBody((!Body.empty()) ? Body : " ");
-		std::string tExtraData((!ExtraData.empty()) ? ExtraData : " ");
-		std::string tSound((!Sound.empty()) ? Sound : " ");
-		std::string tSubSystem((!SubSystem.empty()) ? SubSystem : " ");
-		tItem._command = tSubject + "!#" + tBody + "!#" + tExtraData + "!#" + tSound + "!#" + tSubSystem ;
+		tItem._command = Subject + "!#" + Body + "!#" + ExtraData + "!#" + Sound + "!#" + SubSystem ;
 		if (DelayTime)
 			getclock(&tItem._DelayTimeBegin);
 		return tItem;
 	}
-	static _tTaskItem SetSetPoint(const float DelayTime, const uint64_t idx, const std::string &varvalue)
+	static _tTaskItem SetSetPoint(const float DelayTime, const uint64_t idx, const std::string &varvalue, const std::string &mode = std::string(), const std::string &until = std::string())
 	{
 		_tTaskItem tItem;
 		tItem._ItemType = TITEM_SET_SETPOINT;
 		tItem._DelayTime = DelayTime;
 		tItem._idx = idx;
 		tItem._sValue = varvalue;
+		tItem._command = mode;
+		tItem._sUntil = until;
+
+		if (DelayTime)
+			getclock(&tItem._DelayTimeBegin);
+		return tItem;
+	}
+	static _tTaskItem CustomCommand(const float DelayTime, const uint64_t idx, const std::string &cmdstr)
+	{
+		_tTaskItem tItem;
+		tItem._ItemType = TITEM_CUSTOM_COMMAND;
+		tItem._DelayTime = DelayTime;
+		tItem._idx = idx;
+		tItem._command = cmdstr;
+		if (DelayTime)
+			getclock(&tItem._DelayTimeBegin);
+		return tItem;
+	}
+	static _tTaskItem CustomEvent(const float DelayTime, const std::string& namestr, const std::string parameterstr)
+	{
+		_tTaskItem tItem;
+		tItem._ItemType = TITEM_CUSTOM_EVENT;
+		tItem._DelayTime = DelayTime;
+		tItem._ID = namestr;
+		tItem._sValue = parameterstr;
 		if (DelayTime)
 			getclock(&tItem._DelayTimeBegin);
 		return tItem;
@@ -233,17 +299,18 @@ typedef   std::vector<std::string> TSqlRowQuery;
 // result for an sql query : Vector of TSqlRowQuery
 typedef   std::vector<TSqlRowQuery> TSqlQueryResult;
 
-class CSQLHelper
+class CSQLHelper : public StoppableTask
 {
 public:
 	CSQLHelper(void);
 	~CSQLHelper(void);
 
-	bool OpenDatabase();
 	void SetDatabaseName(const std::string &DBName);
 
-	bool BackupDatabase(const std::string &OutputFile);
+	bool OpenDatabase();
+	void CloseDatabase();
 
+	bool BackupDatabase(const std::string &OutputFile);
 	bool RestoreDatabase(const std::string &dbase);
 
 	//Returns DeviceRowID
@@ -253,6 +320,10 @@ public:
 	uint64_t UpdateValueLighting2GroupCmd(const int HardwareID, const char* ID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const unsigned char signallevel, const unsigned char batterylevel, const int nValue, const char* sValue, std::string &devname, const bool bUseOnOffAction = true);
 	uint64_t UpdateValueHomeConfortGroupCmd(const int HardwareID, const char* ID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const unsigned char signallevel, const unsigned char batterylevel, const int nValue, const char* sValue, std::string &devname, const bool bUseOnOffAction = true);
 
+	bool DoesDeviceExist(const int HardwareID, const char* ID, const unsigned char unit, const unsigned char devType, const unsigned char subType);
+
+	uint64_t InsertDevice(const int HardwareID, const char* ID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const int switchType, const int nValue, const char* sValue, const std::string &devname, const unsigned char signallevel = 12, const unsigned char batterylevel = 255, const int used = 0);
+
 	bool GetLastValue(const int HardwareID, const char* DeviceID, const unsigned char unit, const unsigned char devType, const unsigned char subType, int &nvalue, std::string &sValue, struct tm &LastUpdateTime);
 
 	void Lighting2GroupCmd(const std::string &ID, const unsigned char subType, const unsigned char GroupCmd);
@@ -303,13 +374,14 @@ public:
 
 	void DeleteEvent(const std::string &idx);
 
-	void DeleteDevices(const std::string &idx);
+	void DeleteDevices(const std::string& idx);
+	void DeleteScenes(const std::string& idx);
 
 	void TransferDevice(const std::string &oldidx, const std::string &newidx);
 
 	bool DoesSceneByNameExits(const std::string &SceneName);
 
-	void AddTaskItem(const _tTaskItem &tItem);
+	void AddTaskItem(const _tTaskItem &tItem, const bool cancelItem = false);
 
 	void EventsGetTaskItems(std::vector<_tTaskItem> &currentTasks);
 
@@ -323,25 +395,24 @@ public:
 	std::vector<std::vector<std::string> > safe_query(const char *fmt, ...);
 	std::vector<std::vector<std::string> > safe_queryBlob(const char *fmt, ...);
 	void safe_exec_no_return(const char *fmt, ...);
+	bool safe_UpdateBlobInTableWithID(const std::string &Table, const std::string &Column, const std::string &sID, const std::string &BlobData);
 	bool DoesColumnExistsInTable(const std::string &columnname, const std::string &tablename);
-	std::string DeleteUserVariable(const std::string &idx);
-	std::string SaveUserVariable(const std::string &varname, const std::string &vartype, const std::string &varvalue);
-	std::string UpdateUserVariable(const std::string &idx, const std::string &varname, const std::string &vartype, const std::string &varvalue, const bool eventtrigger);
-	bool SetUserVariable(const uint64_t idx, const std::string &varvalue, const bool eventtrigger);
-	std::vector<std::vector<std::string> > GetUserVariables();
 
-	void UpdateDeviceValue(const char * FieldName , std::string &Value , std::string &Idx );
-	void UpdateDeviceValue(const char * FieldName , int Value , std::string &Idx )   ;
-	void UpdateDeviceValue(const char * FieldName , float Value , std::string &Idx ) ;
-	double ConvertTemperature(double tempcelcius);
-	double ConvertTemperatureUnit(double tempcelcius);
-	std::string GetDeviceValue(const char * FieldName , const char *Idx );
+	bool AddUserVariable(const std::string &varname, const _eUsrVariableType eVartype, const std::string &varvalue, std::string &errorMessage);
+	bool UpdateUserVariable(const std::string &idx, const std::string &varname, const _eUsrVariableType eVartype, const std::string &varvalue, const bool eventtrigger, std::string &errorMessage);
+	void DeleteUserVariable(const std::string &idx);
+	bool CheckUserVariable(const _eUsrVariableType eVartype, const std::string &varvalue, std::string &errorMessage);
 
-	float getTemperatureFromSValue(const char * sValue);
+	uint64_t CreateDevice(const int HardwareID, const int SensorType, const int SensorSubType, std::string &devname, const unsigned long nid, const std::string &soptions);
+
+	void UpdateDeviceValue(const char * FieldName , const std::string &Value , const std::string &Idx );
+	void UpdateDeviceValue(const char * FieldName , const int Value , const std::string &Idx )   ;
+	void UpdateDeviceValue(const char* FieldName, const float Value, const std::string& Idx);
+	void UpdateDeviceName(const std::string& Idx, const std::string &Name);
 
 	bool GetPreferencesVar(const std::string &Key, double &Value);
 	void UpdatePreferencesVar(const std::string &Key, const double Value);
-	void DeletePreferencesVar(const std::string Key );
+	void DeletePreferencesVar(const std::string &Key);
 	void AllowNewHardwareTimer(const int iTotMinutes);
 
 	bool InsertCustomIconFromZip(const std::string &szZip, std::string &ErrorMessage);
@@ -349,7 +420,10 @@ public:
 
 	std::map<std::string, std::string> BuildDeviceOptions(const std::string & options, const bool decode = true);
 	std::map<std::string, std::string> GetDeviceOptions(const std::string & idx);
+	std::string FormatDeviceOptions(const std::map<std::string, std::string> & optionsMap);
 	bool SetDeviceOptions(const uint64_t idx, const std::map<std::string, std::string> & options);
+
+	float GetCounterDivider(const int metertype, const int dType, const float DefaultValue);
 public:
 	std::string m_LastSwitchID;	//for learning command
 	uint64_t m_LastSwitchRowID;
@@ -365,12 +439,13 @@ public:
 	bool		m_bAcceptNewHardware;
 	bool		m_bAllowWidgetOrdering;
 	int			m_ActiveTimerPlan;
-	bool		m_bDisableEventSystem;
+	bool		m_bEnableEventSystem;
+	bool		m_bEnableEventSystemFullURLLog;
 	int			m_ShortLogInterval;
 	bool		m_bLogEventScriptTrigger;
 	bool		m_bDisableDzVentsSystem;
 private:
-	boost::mutex	m_sqlQueryMutex;
+	std::mutex		m_sqlQueryMutex;
 	sqlite3			*m_dbase;
 	std::string		m_dbase_name;
 	unsigned char	m_sensortimeoutcounter;
@@ -381,14 +456,14 @@ private:
 	bool			m_bPreviousAcceptNewHardware;
 
 	std::vector<_tTaskItem> m_background_task_queue;
-	boost::shared_ptr<boost::thread> m_background_task_thread;
-	boost::mutex m_background_task_mutex;
-	bool m_stoprequested;
+	std::shared_ptr<std::thread> m_thread;
+	std::mutex m_background_task_mutex;
 	bool StartThread();
+	void StopThread();
 	void Do_Work();
 
-	bool SwitchLightFromTasker(const std::string &idx, const std::string &switchcmd, const std::string &level, const std::string &hue);
-	bool SwitchLightFromTasker(uint64_t idx, const std::string &switchcmd, int level, int hue);
+	bool SwitchLightFromTasker(const std::string &idx, const std::string &switchcmd, const std::string &level, const std::string &color, const std::string& User);
+	bool SwitchLightFromTasker(uint64_t idx, const std::string &switchcmd, int level, _tColor color, const std::string& User);
 
 	void FixDaylightSavingTableSimple(const std::string &TableName);
 	void FixDaylightSaving();
@@ -396,6 +471,26 @@ private:
 	//Returns DeviceRowID
 	uint64_t UpdateValueInt(const int HardwareID, const char* ID, const unsigned char unit, const unsigned char devType, const unsigned char subType, const unsigned char signallevel, const unsigned char batterylevel, const int nValue, const char* sValue, std::string &devname, const bool bUseOnOffAction);
 
+	bool UpdateCalendarMeter(
+		const int HardwareID,
+		const char* DeviceID,
+		const unsigned char unit,
+		const unsigned char devType,
+		const unsigned char subType,
+		const bool shortLog,
+		const bool multiMeter,
+		const char* date,
+		const long long value1 = 0,
+		const long long value2 = 0,
+		const long long value3 = 0,
+		const long long value4 = 0,
+		const long long value5 = 0,
+		const long long value6 = 0,
+		const long long counter1 = 0,
+		const long long counter2 = 0,
+		const long long counter3 = 0,
+		const long long counter4 = 0);
+
 	void CheckAndUpdateDeviceOrder();
 	void CheckAndUpdateSceneDeviceOrder();
 
@@ -418,14 +513,13 @@ private:
 	void AddCalendarUpdatePercentage();
 	void AddCalendarUpdateFan();
 	void CleanupShortLog();
-	std::string CheckUserVariable(const int vartype, const std::string &varvalue);
-	std::string CheckUserVariableName(const std::string &varname);
 	bool CheckDate(const std::string &sDate, int &d, int &m, int &y);
+	bool CheckDateSQL(const std::string &sDate);
+	bool CheckDateTimeSQL(const std::string &sDateTime);
 	bool CheckTime(const std::string &sTime);
 
 	std::vector<std::vector<std::string> > query(const std::string &szQuery);
 	std::vector<std::vector<std::string> > queryBlob(const std::string &szQuery);
-	void LogQueryResult(TSqlQueryResult &result);
 };
 
 extern CSQLHelper m_sql;
