@@ -1,13 +1,13 @@
 #include "stdafx.h"
 #include "WebServerHelper.h"
+#include "../main/Logger.h"
 #include "../main/SQLHelper.h"
 
 namespace http {
 	namespace server {
 
-		typedef std::vector<boost::shared_ptr<CWebServer> >::iterator server_iterator;
+		typedef std::vector<std::shared_ptr<CWebServer> >::iterator server_iterator;
 #ifndef NOCLOUD
-		typedef std::vector<boost::shared_ptr<CProxyManager> >::iterator proxy_iterator;
 		extern CProxySharedData sharedData;
 #endif
 
@@ -37,7 +37,9 @@ namespace http {
 			our_listener_port = web_settings.listening_port;
 #ifdef WWW_ENABLE_SSL
 			if (secure_web_settings.is_enabled()) {
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(LIBRESSL_VERSION_NUMBER)
 				SSL_library_init();
+#endif
 				secureServer_.reset(new CWebServer());
 				bRet |= secureServer_->StartServer(secure_web_settings, serverpath, bIgnoreUsernamePassword);
 				serverCollection.push_back(secureServer_);
@@ -64,38 +66,31 @@ namespace http {
 #endif
 
 #ifndef NOCLOUD
-			for (proxy_iterator it = proxymanagerCollection.begin(); it != proxymanagerCollection.end(); ++it) {
-				(*it)->Stop();
-			}
-			proxymanagerCollection.clear();
+			proxymanager.Stop();
 #endif
 		}
 
 #ifndef NOCLOUD
 		void CWebServerHelper::RestartProxy() {
 			sharedData.StopTCPClients();
-			for (proxy_iterator it = proxymanagerCollection.begin(); it != proxymanagerCollection.end(); ++it) {
-				(*it)->Stop();
+			proxymanager.Stop();
+			// restart
+#ifdef WWW_ENABLE_SSL
+			cWebem *my_pWebEm = (plainServer_ != NULL ? plainServer_->m_pWebEm : (secureServer_ != NULL ? secureServer_->m_pWebEm : NULL));
+#else
+			cWebem* my_pWebEm = plainServer_ != NULL ? plainServer_->m_pWebEm : NULL;
+#endif
+			if (my_pWebEm == NULL) {
+				_log.Log(LOG_ERROR, "No servers are configured. Hence mydomoticz will not be started either (if configured)");
+				return;
 			}
-
-			// restart threads
-			const unsigned int connections = GetNrMyDomoticzThreads();
-			proxymanagerCollection.clear();
-			for (unsigned int i = 0; i < connections; i++) {
-				proxymanagerCollection.push_back(boost::shared_ptr<CProxyManager>(new CProxyManager(our_serverpath, plainServer_->m_pWebEm, m_pDomServ)));
-				proxymanagerCollection[i]->Start(i == 0);
+			if (proxymanager.Start(my_pWebEm, m_pDomServ)) {
+				_log.Log(LOG_STATUS, "Proxymanager started.");
 			}
-			_log.Log(LOG_STATUS, "Proxymanager started.");
 		}
 
-		boost::shared_ptr<CProxyClient> CWebServerHelper::GetProxyForMaster(DomoticzTCP *master) {
-			if (proxymanagerCollection.size() > 0) {
-				// todo: make this a random connection?
-				return proxymanagerCollection[0]->GetProxyForMaster(master);
-			}
-			// we are not connected yet. save this master and connect later.
-			sharedData.AddTCPClient(master);
-			return boost::shared_ptr<CProxyClient>();
+		CProxyClient *CWebServerHelper::GetProxyForMaster(DomoticzTCP *master) {
+			return proxymanager.GetProxyForMaster(master);
 		}
 
 		void CWebServerHelper::RemoveMaster(DomoticzTCP *master) {
@@ -103,7 +98,14 @@ namespace http {
 		}
 #endif
 
-		void CWebServerHelper::SetAuthenticationMethod(int amethod)
+		void CWebServerHelper::SetWebCompressionMode(const _eWebCompressionMode gzmode)
+		{
+			for (server_iterator it = serverCollection.begin(); it != serverCollection.end(); ++it) {
+				(*it)->SetWebCompressionMode(gzmode);
+			 }
+		}
+
+		void CWebServerHelper::SetAuthenticationMethod(const _eAuthenticationMethod amethod)
 		{
 			for (server_iterator it = serverCollection.begin(); it != serverCollection.end(); ++it) {
 				(*it)->SetAuthenticationMethod(amethod);
@@ -122,6 +124,7 @@ namespace http {
 			for (server_iterator it = serverCollection.begin(); it != serverCollection.end(); ++it) {
 				(*it)->SetWebRoot(webRoot);
 			 }
+			proxymanager.SetWebRoot(webRoot);
 		}
 
 		void CWebServerHelper::ClearUserPasswords()
@@ -163,15 +166,6 @@ namespace http {
 				(*it)->ReloadCustomSwitchIcons();
 			 }
 		}
-
-#ifndef NOCLOUD
-		int CWebServerHelper::GetNrMyDomoticzThreads()
-		{
-			int nrThreads = 1; // default value
-			m_sql.GetPreferencesVar("MyDomoticzNrThreads", nrThreads);
-			return nrThreads;
-		}
-#endif
 	}
 
 }
