@@ -11,8 +11,8 @@
 time_t mytime(time_t * _Time);
 
 // DST safe SQL datetime string parser
-bool ParseSQLdatetime(time_t &time, struct tm &result, const std::string szSQLdate);
-bool ParseSQLdatetime(time_t &time, struct tm &result, const std::string szSQLdate, int isdst);
+bool ParseSQLdatetime(time_t &time, struct tm &result, const std::string &szSQLdate);
+bool ParseSQLdatetime(time_t &time, struct tm &result, const std::string &szSQLdate, int isdst);
 
 
 // DST safe datetime constructors
