@@ -1,28 +1,23 @@
 #pragma once
 
-#include "../appversion.h"
+#include "../main/appversion.h"
  
 #define STRINGIZE2(s) #s
 #define STRINGIZE(s) STRINGIZE2(s)
  
-#define VERSION_MAJOR               2
-#define VERSION_MINOR               0
-#define VERSION_REVISION            0
-#define VERSION_BUILD               APPVERSION
- 
-#define VER_FILE_DESCRIPTION_STR    "Domoticz binary"
+#define VER_FILE_DESCRIPTION_STR    "Domoticz Home Automation System"
 #define VER_FILE_VERSION            VERSION_MAJOR, VERSION_MINOR, VERSION_REVISION, VERSION_BUILD
 #define VER_FILE_VERSION_STR        STRINGIZE(VERSION_MAJOR)        \
                                     "." STRINGIZE(VERSION_MINOR)    \
                                     "." STRINGIZE(VERSION_REVISION) \
                                     "." STRINGIZE(VERSION_BUILD)    \
  
-#define VER_PRODUCTNAME_STR         "Domoticz"
+#define VER_PRODUCTNAME_STR         "Domoticz Home Automation System"
 #define VER_PRODUCT_VERSION         VER_FILE_VERSION
 #define VER_PRODUCT_VERSION_STR     VER_FILE_VERSION_STR
 #define VER_ORIGINAL_FILENAME_STR   VER_PRODUCTNAME_STR ".exe"
 #define VER_INTERNAL_NAME_STR       VER_ORIGINAL_FILENAME_STR
-#define VER_COPYRIGHT_STR           "Copyright (C) 2014"
+#define VER_COPYRIGHT_STR           "Copyright (C) 2014-" STRINGIZE(VERSION_MINOR)
  
 #ifdef _DEBUG
   #define VER_VER_DEBUG             VS_FF_DEBUG
