@@ -9,8 +9,6 @@
 #define OPTIONS_HTML_BODY 8
 #define OPTIONS_URL_PARAMS 16
 
-using namespace http::server;
-
 class CNotificationBase {
 	friend class CNotificationHelper;
 protected:
@@ -38,7 +36,7 @@ protected:
 	bool IsInConfigString(const std::string &Key);
 	bool IsInConfigInt(const std::string &Key);
 	bool IsInConfigBase64(const std::string &Key);
-	void ConfigFromGetvars(const request& req, const bool save);
+	void ConfigFromGetvars(const http::server::request& req, const bool save);
 	virtual bool IsConfigured() = 0;
 	void SetupConfig(const std::string &Key, std::string& Value);
 	void SetupConfig(const std::string &Key, int *Value);
@@ -56,7 +54,7 @@ protected:
 
 	int m_IsEnabled;
 private:
-	boost::mutex SendMessageExMutex;
+	std::mutex SendMessageExMutex;
 	std::string _subsystemid;
 	std::map<std::string, std::string* > _configValues;
 	std::map<std::string, std::string* > _configValuesBase64;
