@@ -3,33 +3,36 @@
 #include "../main/Helper.h"
 #include "../main/SQLHelper.h"
 #include "../main/localtime_r.h"
+#include "../main/RFXtrx.h"
+#include "../main/mainworker.h"
+#include "../hardware/DomoticzHardware.h"
 #include "../hardware/hardwaretypes.h"
 #include "NotificationHelper.h"
 #include "NotificationProwl.h"
-#include "NotificationNma.h"
 #include "NotificationPushbullet.h"
 #include "NotificationPushover.h"
 #include "NotificationPushsafer.h"
 #include "NotificationPushalot.h"
 #include "NotificationEmail.h"
+#include "NotificationTelegram.h"
 #include "NotificationSMS.h"
 #include "NotificationHTTP.h"
 #include "NotificationKodi.h"
 #include "NotificationLogitechMediaServer.h"
-#include "NotificationGCM.h"
+#include "NotificationFCM.h"
 
+#include "NotificationBrowser.h"
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 
-#include <boost/lexical_cast.hpp>
-#include <map>
-
 #if defined WIN32
 	#include "../msbuild/WindowsHelper.h"
 #endif
 
 typedef std::map<std::string, CNotificationBase*>::iterator it_noti_type;
 
+using namespace http::server;
+
 CNotificationHelper::CNotificationHelper()
 {
 	m_NotificationSwitchInterval = 0;
@@ -38,8 +41,8 @@ CNotificationHelper::CNotificationHelper()
 	/* more notifiers can be added here */
 
 	AddNotifier(new CNotificationProwl());
-	AddNotifier(new CNotificationNma());
 	AddNotifier(new CNotificationPushbullet());
+	AddNotifier(new CNotificationTelegram());
 	AddNotifier(new CNotificationPushover());
 	AddNotifier(new CNotificationPushsafer());
 	AddNotifier(new CNotificationPushalot());
@@ -48,7 +51,8 @@ CNotificationHelper::CNotificationHelper()
 	AddNotifier(new CNotificationHTTP());
 	AddNotifier(new CNotificationKodi());
 	AddNotifier(new CNotificationLogitechMediaServer());
-	AddNotifier(new CNotificationGCM());
+	AddNotifier(new CNotificationFCM());
+	AddNotifier(new CNotificationBrowser());
 }
 
 CNotificationHelper::~CNotificationHelper()
@@ -89,18 +93,20 @@ bool CNotificationHelper::SendMessage(
 
 bool CNotificationHelper::SendMessageEx(
 	const uint64_t Idx,
-	const std::string &Name,
-	const std::string &Subsystems,
-	const std::string &Subject,
-	const std::string &Text,
-	const std::string &ExtraData,
+	const std::string& Name,
+	const std::string& Subsystems,
+	const std::string& Subject,
+	const std::string& Text,
+	const std::string& ExtraData,
 	int Priority,
-	const std::string &Sound,
+	const std::string& Sound,
 	const bool bFromNotification)
 {
 	bool bRet = false;
 	bool bThread = true;
 
+	bool bIsTestMessage = (Subject == "Domoticz test") && (Text == "Domoticz test message!");
+
 	if (Priority == -100)
 	{
 		Priority = 0;
@@ -112,6 +118,8 @@ bool CNotificationHelper::SendMessageEx(
 	//Make a system tray message
 	ShowSystemTrayNotification(Subject.c_str());
 #endif
+	_log.Log(LOG_STATUS, "Notification: %s", Subject.c_str());
+
 	std::vector<std::string> sResult;
 	StringSplit(Subsystems, ";", sResult);
 
@@ -128,10 +136,16 @@ bool CNotificationHelper::SendMessageEx(
 		std::map<std::string, int>::const_iterator ittSystem = ActiveSystems.find(iter->first);
 		if ((ActiveSystems.empty() || ittSystem != ActiveSystems.end()) && iter->second->IsConfigured())
 		{
-			if (bThread)
-				boost::thread SendMessageEx(boost::bind(&CNotificationBase::SendMessageEx, iter->second, Idx, Name, Subject, Text, ExtraData, Priority, Sound, bFromNotification));
-			else
-				bRet |= iter->second->SendMessageEx(Idx, Name, Subject, Text, ExtraData, Priority, Sound, bFromNotification);
+			if ((iter->second->m_IsEnabled) || bIsTestMessage)
+			{
+				if (bThread)
+				{
+					boost::thread SendMessageEx(boost::bind(&CNotificationBase::SendMessageEx, iter->second, Idx, Name, Subject, Text, ExtraData, Priority, Sound, bFromNotification));
+					SetThreadName(SendMessageEx.native_handle(), "SendMessageEx");
+				}
+				else
+					bRet |= iter->second->SendMessageEx(Idx, Name, Subject, Text, ExtraData, Priority, Sound, bFromNotification);
+			}
 		}
 	}
 	return bRet;
@@ -181,7 +195,7 @@ void CNotificationHelper::LoadConfig()
 		}
 	}
 	logline << " (" << active << "/" << tot << ")";
-	_log.Log(LOG_NORM, std::string(logline.str()).c_str());
+	_log.Log(LOG_NORM, logline.str());
 }
 
 std::string CNotificationHelper::ParseCustomMessage(const std::string &cMessage, const std::string &sName, const std::string &sValue)
@@ -192,7 +206,7 @@ std::string CNotificationHelper::ParseCustomMessage(const std::string &cMessage,
 	return ret;
 }
 
-bool CNotificationHelper::ApplyRule(std::string rule, bool equal, bool less)
+bool CNotificationHelper::ApplyRule(const std::string &rule, const bool equal, const bool less)
 {
 	if (((rule == ">") || (rule == ">=")) && (!less) && (!equal))
 		return true;
@@ -205,6 +219,263 @@ bool CNotificationHelper::ApplyRule(std::string rule, bool equal, bool less)
 	return false;
 }
 
+bool CNotificationHelper::CheckAndHandleNotification(const uint64_t DevRowIdx, const int HardwareID, const std::string &ID, const std::string &sName, const unsigned char unit, const unsigned char cType, const unsigned char cSubType, const int nValue) {
+	return CheckAndHandleNotification(DevRowIdx, HardwareID, ID, sName, unit, cType, cSubType, nValue, "", 0.0f);
+}
+
+bool CNotificationHelper::CheckAndHandleNotification(const uint64_t DevRowIdx, const int HardwareID, const std::string &ID, const std::string &sName, const unsigned char unit, const unsigned char cType, const unsigned char cSubType, const float fValue) {
+	return CheckAndHandleNotification(DevRowIdx, HardwareID, ID, sName, unit, cType, cSubType, 0, "", fValue);
+}
+
+bool CNotificationHelper::CheckAndHandleNotification(const uint64_t DevRowIdx, const int HardwareID, const std::string &ID, const std::string &sName, const unsigned char unit, const unsigned char cType, const unsigned char cSubType, const std::string &sValue) {
+	return CheckAndHandleNotification(DevRowIdx, HardwareID, ID, sName, unit, cType, cSubType, 0, sValue, static_cast<float>(atof(sValue.c_str())));
+}
+
+bool CNotificationHelper::CheckAndHandleNotification(const uint64_t DevRowIdx, const int HardwareID, const std::string &ID, const std::string &sName, const unsigned char unit, const unsigned char cType, const unsigned char cSubType, const int nValue, const std::string &sValue) {
+	return CheckAndHandleNotification(DevRowIdx, HardwareID, ID, sName, unit, cType, cSubType, nValue, sValue, static_cast<float>(atof(sValue.c_str())));
+}
+
+bool CNotificationHelper::CheckAndHandleNotification(const uint64_t DevRowIdx, const int HardwareID, const std::string &ID, const std::string &sName, const unsigned char unit, const unsigned char cType, const unsigned char cSubType, const int nValue, const std::string &sValue, const float fValue) {
+	float fValue2;
+	bool r1, r2, r3;
+	int nsize;
+	int nexpected = 0;
+
+	// Don't send notification for devices not in db
+	// Notifications for switches are handled by CheckAndHandleSwitchNotification in UpdateValue() of SQLHelper
+	if ((DevRowIdx == -1) || IsLightOrSwitch(cType, cSubType)) {
+		return false;
+	}
+
+	int meterType = 0;
+	std::vector<std::string> strarray;
+	StringSplit(sValue, ";", strarray);
+	nsize = strarray.size();
+	switch(cType) {
+		case pTypeP1Power:
+			nexpected = 5;
+			if (nsize >= nexpected) {
+				return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, (float)atof(strarray[4].c_str()));
+			}
+			break;
+		case pTypeRFXSensor:
+			switch(cSubType) {
+				case sTypeRFXSensorTemp:
+					return CheckAndHandleTempHumidityNotification(DevRowIdx, sName, fValue, 0, true, false);
+				case sTypeRFXSensorAD:
+				case sTypeRFXSensorVolt:
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue);
+				default:
+					break;
+			}
+			break;
+		case pTypeThermostat:
+			switch(cSubType) {
+				case sTypeThermSetpoint:
+					return CheckAndHandleTempHumidityNotification(DevRowIdx, sName, fValue, 0, true, false);
+				default:
+					break;
+			}
+			break;
+		case pTypeTEMP:
+			return CheckAndHandleTempHumidityNotification(DevRowIdx, sName, fValue, 0, true, false);
+		case pTypeHUM:
+			return CheckAndHandleTempHumidityNotification(DevRowIdx, sName, 0.0, nValue, false, true);
+		case pTypeTEMP_HUM:
+			nexpected = 2;
+			if (nsize >= nexpected) {
+				float Temp = (float)atof(strarray[0].c_str());
+				int Hum = atoi(strarray[1].c_str());
+				float dewpoint = (float)CalculateDewPoint(Temp, Hum);
+				r1 = CheckAndHandleTempHumidityNotification(DevRowIdx, sName, Temp, Hum, true, true);
+				r2 = CheckAndHandleDewPointNotification(DevRowIdx, sName, Temp, dewpoint);
+				return r1 && r2;
+			}
+			break;
+		case pTypeTEMP_HUM_BARO:
+			nexpected = 4;
+			if (nsize >= nexpected) {
+				float Temp = (float)atof(strarray[0].c_str());
+				int Hum = atoi(strarray[1].c_str());
+				float dewpoint = (float)CalculateDewPoint(Temp, Hum);
+				r1 = CheckAndHandleTempHumidityNotification(DevRowIdx, sName, Temp, Hum, true, true);
+				r2 = CheckAndHandleDewPointNotification(DevRowIdx, sName, Temp, dewpoint);
+				r3 = CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_BARO, (float)atof(strarray[3].c_str()));
+				return r1 && r2 && r3;
+			}
+			break;
+		case pTypeRAIN:
+			nexpected = 2;
+			if (nsize >= nexpected) {
+				fValue2 = (float)atof(strarray[1].c_str());
+				return CheckAndHandleRainNotification(DevRowIdx, sName, cType, cSubType, NTYPE_RAIN, fValue2);
+			}
+			break;
+		case pTypeTEMP_BARO:
+			nexpected = 2;
+			if (nsize >= nexpected) {
+				float Temp = (float)atof(strarray[0].c_str());
+				float Baro = (float)atof(strarray[1].c_str());
+				r1 = CheckAndHandleTempHumidityNotification(DevRowIdx, sName, Temp, 0, true, false);
+				r2 = CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_BARO, Baro);
+				return r1 && r2;
+			}
+			break;
+		case pTypeUV:
+			nexpected = 2;
+			if (nsize >= nexpected) {
+				float Level = (float)atof(strarray[0].c_str());
+				float Temp = (float)atof(strarray[1].c_str());
+				if (cSubType == sTypeUV3)
+				{
+					r1 = CheckAndHandleTempHumidityNotification(DevRowIdx, sName, Temp, 0, true, false);
+				}
+				else
+					r1 = true;
+				r2 = CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_UV, Level);
+				return r1 && r2;
+			}
+			break;
+		case pTypeCURRENT:
+			nexpected = 3;
+			if (nsize >= nexpected) {
+				float CurrentChannel1 = (float)atof(strarray[0].c_str());
+				float CurrentChannel2 = (float)atof(strarray[1].c_str());
+				float CurrentChannel3 = (float)atof(strarray[2].c_str());
+				return CheckAndHandleAmpere123Notification(DevRowIdx, sName, CurrentChannel1, CurrentChannel2, CurrentChannel3);
+			}
+			break;
+		case pTypeCURRENTENERGY:
+			nexpected = 3;
+			if (nsize >= nexpected) {
+				float CurrentChannel1 = (float)atof(strarray[0].c_str());
+				float CurrentChannel2 = (float)atof(strarray[1].c_str());
+				float CurrentChannel3 = (float)atof(strarray[2].c_str());
+				return CheckAndHandleAmpere123Notification(DevRowIdx, sName, CurrentChannel1, CurrentChannel2, CurrentChannel3);
+			}
+			break;
+		case pTypeWIND:
+			nexpected = 5;
+			if (nsize >= nexpected) {
+				float wspeedms = (float)(atof(strarray[2].c_str()) / 10.0f);
+				float temp = (float)atof(strarray[4].c_str());
+				r1 = CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_WIND, wspeedms);
+				r2 = CheckAndHandleTempHumidityNotification(DevRowIdx, sName, temp, 0, true, false);
+				return r1 && r2;
+			}
+			break;
+		case pTypeYouLess:
+			nexpected = 2;
+			if (nsize >= nexpected) {
+				float usagecurrent = (float)atof(strarray[1].c_str());
+				return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, usagecurrent);
+			}
+			break;
+		case pTypeAirQuality:
+			return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, (float)nValue);
+		case pTypeWEIGHT:
+		case pTypeLux:
+			return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue);
+		case pTypeRego6XXTemp:
+			return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_TEMPERATURE, fValue);
+		case pTypePOWER:
+			nexpected = 1;
+			if (nsize >= nexpected) {
+				fValue2 = (float)atof(strarray[0].c_str());
+				return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue2);
+			}
+			break;
+		case pTypeRFXMeter:
+			switch(cSubType) {
+				case sTypeRFXMeterCount:
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_TODAYCOUNTER, fValue);
+				default:
+					break;
+			}
+			break;
+		case pTypeUsage:
+			return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue);
+			break;
+		case pTypeP1Gas:
+			// ignore, notification is done day by day in SQLHelper
+			return false;
+		case pTypeGeneral:
+			switch(cSubType) {
+				case sTypeVisibility:
+					m_sql.GetMeterType(HardwareID, ID.c_str(), unit, cType, cSubType, meterType);
+					fValue2 = fValue;
+					if (meterType == 1) {
+						//miles
+						fValue2 *= 0.6214f;
+					}
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue2);
+				case sTypeDistance:
+					m_sql.GetMeterType(HardwareID, ID.c_str(), unit, cType, cSubType, meterType);
+					fValue2 = fValue;
+					if (meterType == 1) {
+						//inches
+						fValue2 *= 0.393701f;
+					}
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue2);
+				case sTypeBaro:
+				case sTypeKwh:
+					nexpected = 1;
+					if (nsize >= nexpected) {
+						fValue2 = (float)atof(strarray[0].c_str());
+						return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue2);
+					}
+					break;
+				case sTypeZWaveAlarm:
+					return CheckAndHandleValueNotification(DevRowIdx, sName, nValue);
+				case sTypePercentage:
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_PERCENTAGE, fValue);
+				case sTypeSoilMoisture:
+				case sTypeLeafWetness:
+				case sTypeAlert:
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, (float)nValue);
+				case sTypeFan:
+				case sTypeSoundLevel:
+				case sTypeSolarRadiation:
+				case sTypeVoltage:
+				case sTypeCurrent:
+				case sTypePressure:
+				case sTypeWaterflow:
+				case sTypeCustom:
+					return CheckAndHandleNotification(DevRowIdx, sName, cType, cSubType, NTYPE_USAGE, fValue);
+				default:
+					// silently ignore other general devices
+					return false;
+			}
+			break;
+		case pTypeEvohome:
+		case pTypeEvohomeRelay:
+		case pTypeEvohomeWater:
+		case pTypeEvohomeZone:
+			//not handled
+			return false;
+			break;
+		default:
+			break;
+	}
+
+	std::string hName;
+	CDomoticzHardwareBase *pHardware = m_mainworker.GetHardware(HardwareID);
+	if (pHardware == NULL) {
+		hName = "";
+	}
+	else {
+		hName = pHardware->m_Name;
+	}
+
+	if (nexpected > 0) {
+		_log.Log(LOG_STATUS, "Warning: Expecting svalue with at least %d elements separated by semicolon, %d elements received (\"%s\"), notification not sent (Hardware: %d - %s, ID: %s, Unit: %d, Type: %02X - %s, SubType: %d - %s)", nexpected, nsize, sValue.c_str(), HardwareID, hName.c_str(), ID.c_str(), unit, cType, RFX_Type_Desc(cType, 1), cSubType, RFX_Type_SubType_Desc(cType, cSubType));
+	}
+	else {
+		_log.Log(LOG_STATUS, "Warning: Notification NOT handled (Hardware: %d - %s, ID: %s, Unit: %d, Type: %02X - %s, SubType: %d - %s), please report on GitHub!", HardwareID, hName.c_str(), ID.c_str(), unit, cType, RFX_Type_Desc(cType, 1), cSubType, RFX_Type_SubType_Desc(cType, cSubType));
+	}
+
+	return false;
+}
 
 bool CNotificationHelper::CheckAndHandleTempHumidityNotification(
 	const uint64_t Idx,
@@ -231,7 +502,7 @@ bool CNotificationHelper::CheckAndHandleTempHumidityNotification(
 
 	std::string msg = "";
 
-	std::string ltemp = Notification_Type_Label(NTYPE_TEMPERATURE);
+	std::string label = Notification_Type_Label(NTYPE_TEMPERATURE);
 	std::string signtemp = Notification_Type_Desc(NTYPE_TEMPERATURE, 1);
 	std::string signhum = Notification_Type_Desc(NTYPE_HUMIDITY, 1);
 
@@ -278,7 +549,7 @@ bool CNotificationHelper::CheckAndHandleTempHumidityNotification(
 				bSendNotification = ApplyRule(splitresults[1], (temp == svalue), (temp < svalue));
 				if (bSendNotification && (!bRecoveryMessage || itt->SendAlways))
 				{
-					sprintf(szTmp, "%s temperature is %.1f %s [%s %.1f %s]", devicename.c_str(), temp, ltemp.c_str(), splitresults[1].c_str(), svalue, ltemp.c_str());
+					sprintf(szTmp, "%s Temperature is %.1f %s [%s %.1f %s]", devicename.c_str(), temp, label.c_str(), splitresults[1].c_str(), svalue, label.c_str());
 					msg = szTmp;
 					sprintf(szTmp, "%.1f", temp);
 					notValue = szTmp;
@@ -496,7 +767,7 @@ bool CNotificationHelper::CheckAndHandleAmpere123Notification(
 			std::string custommsg;
 			std::string ltype;
 			float svalue = static_cast<float>(atof(splitresults[2].c_str()));
-			float ampere;
+			float ampere = 0.0f;
 			bool bSendNotification = false;
 			bool bCustomMessage = false;
 			bCustomMessage = CustomRecoveryMessage(itt->ID, custommsg, false);
@@ -563,7 +834,7 @@ bool CNotificationHelper::CheckAndHandleNotification(
 
 	std::vector<std::vector<std::string> > result;
 	result = m_sql.safe_query("SELECT SwitchType, CustomImage FROM DeviceStatus WHERE (ID=%" PRIu64 ")", Idx);
-	if (result.size() == 0)
+	if (result.empty())
 		return false;
 
 	std::string szExtraData = "|Name=" + devicename + "|SwitchType=" + result[0][0] + "|CustomImage=" + result[0][1] + "|";
@@ -624,7 +895,7 @@ bool CNotificationHelper::CheckAndHandleNotification(
 
 	std::vector<std::vector<std::string> > result;
 	result = m_sql.safe_query("SELECT SwitchType FROM DeviceStatus WHERE (ID=%" PRIu64 ")", Idx);
-	if (result.size() == 0)
+	if (result.empty())
 		return false;
 	std::string szExtraData = "|Name=" + devicename + "|SwitchType=" + result[0][0] + "|";
 
@@ -712,7 +983,7 @@ bool CNotificationHelper::CheckAndHandleSwitchNotification(
 
 	result = m_sql.safe_query("SELECT SwitchType, CustomImage FROM DeviceStatus WHERE (ID=%" PRIu64 ")",
 		Idx);
-	if (result.size() == 0)
+	if (result.empty())
 		return false;
 	_eSwitchType switchtype = (_eSwitchType)atoi(result[0][0].c_str());
 	std::string szExtraData = "|Name=" + devicename + "|SwitchType=" + result[0][0] + "|CustomImage=" + result[0][1] + "|";
@@ -752,15 +1023,19 @@ bool CNotificationHelper::CheckAndHandleSwitchNotification(
 						break;
 					case STYPE_Contact:
 						notValue = "Open";
-						szExtraData += "Image=contact48_open|";
+						szExtraData += "Image=Contact48_On|";
 						break;
 					case STYPE_DoorContact:
 						notValue = "Open";
-						szExtraData += "Image=door48open|";
+						szExtraData += "Image=Door48_On|";
 						break;
 					case STYPE_DoorLock:
 						notValue = "Locked";
-						szExtraData += "Image=door48|";
+						szExtraData += "Image=Door48_Off|";
+						break;
+					case STYPE_DoorLockInverted:
+						notValue = "Unlocked";
+						szExtraData += "Image=Door48_On|";
 						break;
 					case STYPE_Motion:
 						notValue = "movement detected";
@@ -783,7 +1058,11 @@ bool CNotificationHelper::CheckAndHandleSwitchNotification(
 						break;
 					case STYPE_DoorLock:
 						notValue = "Unlocked";
-						szExtraData += "Image=door48open|";
+						szExtraData += "Image=Door48_On|";
+						break;
+					case STYPE_DoorLockInverted:
+						notValue = "Locked";
+						szExtraData += "Image=Door48_Off|";
 						break;
 					default:
 						notValue = ">> OFF";
@@ -817,7 +1096,7 @@ bool CNotificationHelper::CheckAndHandleSwitchNotification(
 
 	result = m_sql.safe_query("SELECT SwitchType, CustomImage, Options FROM DeviceStatus WHERE (ID=%" PRIu64 ")",
 		Idx);
-	if (result.size() == 0)
+	if (result.empty())
 		return false;
 	_eSwitchType switchtype = (_eSwitchType)atoi(result[0][0].c_str());
 	std::string szExtraData = "|Name=" + devicename + "|SwitchType=" + result[0][0] + "|CustomImage=" + result[0][1] + "|";
@@ -859,7 +1138,7 @@ bool CNotificationHelper::CheckAndHandleSwitchNotification(
 					if (llevel == iLevel)
 					{
 						bSendNotification = true;
-						std::string sLevel = boost::lexical_cast<std::string>(llevel);
+						std::string sLevel = std::to_string(llevel);
 						szExtraData += "Status=Level " + sLevel + "|";
 
 						if (switchtype == STYPE_Selector)
@@ -910,7 +1189,7 @@ bool CNotificationHelper::CheckAndHandleRainNotification(
 
 	result = m_sql.safe_query("SELECT AddjValue,AddjMulti FROM DeviceStatus WHERE (ID=%" PRIu64 ")",
 		Idx);
-	if (result.size() == 0)
+	if (result.empty())
 		return false;
 	//double AddjValue = atof(result[0][0].c_str());
 	double AddjMulti = atof(result[0][1].c_str());
@@ -931,11 +1210,18 @@ bool CNotificationHelper::CheckAndHandleRainNotification(
 	ltime.tm_mday = tm1.tm_mday;
 	sprintf(szDateEnd, "%04d-%02d-%02d", ltime.tm_year + 1900, ltime.tm_mon + 1, ltime.tm_mday);
 
-	if (subType != sTypeRAINWU)
+	if (subType == sTypeRAINWU || subType == sTypeRAINByRate)
+	{
+		//value is already total rain
+		double total_real = mvalue;
+		total_real *= AddjMulti;
+		CheckAndHandleNotification(Idx, devicename, devType, subType, NTYPE_RAIN, (float)total_real);
+	}
+	else
 	{
 		result = m_sql.safe_query("SELECT MIN(Total) FROM Rain WHERE (DeviceRowID=%" PRIu64 " AND Date>='%q')",
 			Idx, szDateEnd);
-		if (result.size() > 0)
+		if (!result.empty())
 		{
 			std::vector<std::string> sd = result[0];
 
@@ -946,13 +1232,6 @@ bool CNotificationHelper::CheckAndHandleRainNotification(
 			CheckAndHandleNotification(Idx, devicename, devType, subType, NTYPE_RAIN, (float)total_real);
 		}
 	}
-	else
-	{
-		//value is already total rain
-		double total_real = mvalue;
-		total_real *= AddjMulti;
-		CheckAndHandleNotification(Idx, devicename, devType, subType, NTYPE_RAIN, (float)total_real);
-	}
 	return false;
 }
 
@@ -991,18 +1270,20 @@ void CNotificationHelper::CheckAndHandleLastUpdateNotification()
 					std::string szExtraData;
 					std::string custommsg;
 					uint64_t Idx = itt->first;
-					int SensorTimeOut = atoi(splitresults[2].c_str());  // minutes
-					int diff = (int)round(difftime(btime, itt2->LastUpdate));
-					bool bStartTime = (difftime(btime, m_StartTime) < SensorTimeOut*60);
-					bool bSendNotification = ApplyRule(splitresults[1], (diff == SensorTimeOut*60), (diff < SensorTimeOut*60));
+					uint32_t SensorTimeOut = static_cast<uint32_t>(atoi(splitresults[2].c_str()));  // minutes
+					uint32_t diff = static_cast<uint32_t>(round(difftime(btime, itt2->LastUpdate)));
+					bool bStartTime = (difftime(btime, m_StartTime) < SensorTimeOut * 60);
+					bool bSendNotification = ApplyRule(splitresults[1], (diff == SensorTimeOut * 60), (diff < SensorTimeOut * 60));
 					bool bCustomMessage = false;
 					bCustomMessage = CustomRecoveryMessage(itt2->ID, custommsg, false);
 
 					if (bSendNotification && !bStartTime && (!bRecoveryMessage || itt2->SendAlways))
 					{
+						if (SystemUptime() < SensorTimeOut * 60 && (!bRecoveryMessage || itt2->SendAlways))
+							continue;
 						std::vector<std::vector<std::string> > result;
 						result = m_sql.safe_query("SELECT SwitchType FROM DeviceStatus WHERE (ID=%" PRIu64 ")", Idx);
-						if (result.size() == 0)
+						if (result.empty())
 							continue;
 						szExtraData = "|Name=" + itt2->DeviceName + "|SwitchType=" + result[0][0] + "|";
 						std::string ltype = Notification_Type_Desc(NTYPE_LASTUPDATE, 0);
@@ -1019,25 +1300,20 @@ void CNotificationHelper::CheckAndHandleLastUpdateNotification()
 					}
 					else if (!bSendNotification && bRecoveryMessage)
 					{
-						bSendNotification = true;
 						msg = recoverymsg;
 						std::string clearstr = "!";
 						CustomRecoveryMessage(itt2->ID, clearstr, true);
 					}
 					else
+						continue;
+
+					if (bCustomMessage && !bRecoveryMessage)
+						msg = ParseCustomMessage(custommsg, itt2->DeviceName, "");
+					SendMessageEx(Idx, itt2->DeviceName, itt2->ActiveSystems, msg, msg, szExtraData, itt2->Priority, std::string(""), true);
+					if (!bRecoveryMessage)
 					{
-						bSendNotification = false;
-					}
-					if (bSendNotification)
-					{
-						if (bCustomMessage && !bRecoveryMessage)
-							msg = ParseCustomMessage(custommsg, itt2->DeviceName, "");
-						SendMessageEx(Idx, itt2->DeviceName, itt2->ActiveSystems, msg, msg, szExtraData, itt2->Priority, std::string(""), true);
-						if (!bRecoveryMessage)
-						{
-							TouchNotification(itt2->ID);
-							CustomRecoveryMessage(itt2->ID, msg, true);
-						}
+						TouchNotification(itt2->ID);
+						CustomRecoveryMessage(itt2->ID, msg, true);
 					}
 				}
 			}
@@ -1058,7 +1334,7 @@ void CNotificationHelper::TouchNotification(const uint64_t ID)
 		szDate, ID);
 
 	//Also touch it internally
-	boost::lock_guard<boost::mutex> l(m_mutex);
+	std::lock_guard<std::mutex> l(m_mutex);
 
 	std::map<uint64_t, std::vector<_tNotification> >::iterator itt;
 	for (itt = m_notifications.begin(); itt != m_notifications.end(); ++itt)
@@ -1078,7 +1354,7 @@ void CNotificationHelper::TouchNotification(const uint64_t ID)
 void CNotificationHelper::TouchLastUpdate(const uint64_t ID)
 {
 	time_t atime = mytime(NULL);
-	boost::lock_guard<boost::mutex> l(m_mutex);
+	std::lock_guard<std::mutex> l(m_mutex);
 
 	std::map<uint64_t, std::vector<_tNotification> >::iterator itt;
 	for (itt = m_notifications.begin(); itt != m_notifications.end(); ++itt)
@@ -1097,7 +1373,7 @@ void CNotificationHelper::TouchLastUpdate(const uint64_t ID)
 
 bool CNotificationHelper::CustomRecoveryMessage(const uint64_t ID, std::string &msg, const bool isRecovery)
 {
-	boost::lock_guard<boost::mutex> l(m_mutex);
+	std::lock_guard<std::mutex> l(m_mutex);
 
 	std::map<uint64_t, std::vector<_tNotification> >::iterator itt;
 	for (itt = m_notifications.begin(); itt != m_notifications.end(); ++itt)
@@ -1156,7 +1432,7 @@ bool CNotificationHelper::CustomRecoveryMessage(const uint64_t ID, std::string &
 				}
 				std::vector<std::vector<std::string> > result;
 				result = m_sql.safe_query("SELECT ID FROM Notifications WHERE (ID=='%" PRIu64 "') AND (Params=='%q')", itt2->ID, itt2->Params.c_str());
-				if (result.size() == 0)
+				if (result.empty())
 					return false;
 
 				m_sql.safe_query("UPDATE Notifications SET CustomMessage='%q' WHERE ID=='%" PRIu64 "'", szTmp.c_str(), itt2->ID);
@@ -1182,7 +1458,7 @@ bool CNotificationHelper::AddNotification(
 	//First check for duplicate, because we do not want this
 	result = m_sql.safe_query("SELECT ROWID FROM Notifications WHERE (DeviceRowID=='%q') AND (Params=='%q')",
 		DevIdx.c_str(), Param.c_str());
-	if (result.size() > 0)
+	if (!result.empty())
 		return false;//already there!
 
 	int iSendAlways = (SendAlways == true) ? 1 : 0;
@@ -1218,7 +1494,7 @@ std::vector<_tNotification> CNotificationHelper::GetNotifications(const std::str
 
 std::vector<_tNotification> CNotificationHelper::GetNotifications(const uint64_t DevIdx)
 {
-	boost::lock_guard<boost::mutex> l(m_mutex);
+	std::lock_guard<std::mutex> l(m_mutex);
 	std::vector<_tNotification> ret;
 	std::map<uint64_t, std::vector<_tNotification> >::const_iterator itt = m_notifications.find(DevIdx);
 	if (itt != m_notifications.end())
@@ -1238,14 +1514,14 @@ bool CNotificationHelper::HasNotifications(const std::string &DevIdx)
 
 bool CNotificationHelper::HasNotifications(const uint64_t DevIdx)
 {
-	boost::lock_guard<boost::mutex> l(m_mutex);
+	std::lock_guard<std::mutex> l(m_mutex);
 	return (m_notifications.find(DevIdx) != m_notifications.end());
 }
 
 //Re(Loads) all notifications stored in the database, so we do not have to query this all the time
 void CNotificationHelper::ReloadNotifications()
 {
-	boost::lock_guard<boost::mutex> l(m_mutex);
+	std::lock_guard<std::mutex> l(m_mutex);
 	m_notifications.clear();
 	std::vector<std::vector<std::string> > result;
 
@@ -1253,7 +1529,7 @@ void CNotificationHelper::ReloadNotifications()
 	m_sql.GetPreferencesVar("NotificationSwitchInterval", m_NotificationSwitchInterval);
 
 	result = m_sql.safe_query("SELECT ID, DeviceRowID, Params, CustomMessage, ActiveSystems, Priority, SendAlways, LastSend FROM Notifications ORDER BY DeviceRowID");
-	if (result.size() == 0)
+	if (result.empty())
 		return;
 
 	time_t mtime = mytime(NULL);
