@@ -2,7 +2,6 @@
 #include "NotificationBase.h"
 #include "../webserver/cWebem.h"
 
-#include <vector>
 #include <string>
 
 #define NOTIFYALL std::string("")
@@ -46,7 +45,7 @@ public:
 		const std::string &Sound,
 		const bool bFromNotification);
 	void LoadConfig();
-	void ConfigFromGetvars(const request& req, const bool save);
+	void ConfigFromGetvars(const http::server::request& req, const bool save);
 	bool IsInConfig(const std::string &Key);
 
 	//notification functions
@@ -69,6 +68,85 @@ public:
 	bool HasNotifications(const uint64_t DevIdx);
 	bool HasNotifications(const std::string &DevIdx);
 
+	bool CheckAndHandleNotification(
+		const uint64_t DevRowIdx,
+		const int HardwareID,
+		const std::string &ID,
+		const std::string &sName,
+		const unsigned char unit,
+		const unsigned char cType,
+		const unsigned char cSubType,
+		const int nValue);
+	bool CheckAndHandleNotification(
+		const uint64_t DevRowIdx,
+		const int HardwareID,
+		const std::string &ID,
+		const std::string &sName,
+		const unsigned char unit,
+		const unsigned char cType,
+		const unsigned char cSubType,
+		const float fValue);
+	bool CheckAndHandleNotification(
+		const uint64_t DevRowIdx,
+		const int HardwareID,
+		const std::string &ID,
+		const std::string &sName,
+		const unsigned char unit,
+		const unsigned char cType,
+		const unsigned char cSubType,
+		const std::string &sValue);
+	bool CheckAndHandleNotification(
+		const uint64_t DevRowIdx,
+		const int HardwareID,
+		const std::string &ID,
+		const std::string &sName,
+		const unsigned char unit,
+		const unsigned char cType,
+		const unsigned char cSubType,
+		const int nValue,
+		const std::string &sValue);
+
+	bool CheckAndHandleNotification(
+		const uint64_t Idx,
+		const std::string &DeviceName,
+		const unsigned char devType,
+		const unsigned char subType,
+		const _eNotificationTypes ntype,
+		const float mvalue);
+	bool CheckAndHandleNotification(
+		const uint64_t Idx,
+		const std::string &DeviceName,
+		const _eNotificationTypes ntype,
+		const std::string &message);
+
+	bool CheckAndHandleSwitchNotification(
+		const uint64_t Idx,
+		const std::string &DeviceName,
+		const _eNotificationTypes ntype);
+	bool CheckAndHandleSwitchNotification(
+		const uint64_t Idx,
+		const std::string & DeviceName,
+		const _eNotificationTypes ntype,
+		const int llevel);
+
+	std::map<std::string, CNotificationBase*> m_notifiers;
+	void AddNotifier(CNotificationBase *notifier);
+	void RemoveNotifier(CNotificationBase * notifier);
+protected:
+	void SetConfigValue(const std::string &key, const std::string &value);
+private:
+	bool CheckAndHandleNotification(
+		const uint64_t DevRowIdx,
+		const int HardwareID,
+		const std::string &ID,
+		const std::string &sName,
+		const unsigned char unit,
+		const unsigned char cType,
+		const unsigned char cSubType,
+		const int nValue,
+		const std::string &sValue,
+		const float fValue);
+
 	bool CheckAndHandleTempHumidityNotification(
 		const uint64_t Idx,
 		const std::string &DeviceName,
@@ -85,27 +163,6 @@ public:
 		const uint64_t Idx,
 		const std::string &DeviceName,
 		const int value);
-	bool CheckAndHandleNotification(
-		const uint64_t Idx,
-		const std::string &DeviceName,
-		const unsigned char devType,
-		const unsigned char subType,
-		const _eNotificationTypes ntype,
-		const float mvalue);
-	bool CheckAndHandleNotification(
-		const uint64_t Idx,
-		const std::string &DeviceName,
-		const _eNotificationTypes ntype,
-		const std::string &message);
-	bool CheckAndHandleSwitchNotification(
-		const uint64_t Idx,
-		const std::string &DeviceName,
-		const _eNotificationTypes ntype);
-	bool CheckAndHandleSwitchNotification(
-		const uint64_t Idx,
-		const std::string & DeviceName,
-		const _eNotificationTypes ntype,
-		const int llevel);
 	bool CheckAndHandleRainNotification(
 		const uint64_t Idx,
 		const std::string &DeviceName,
@@ -121,15 +178,9 @@ public:
 		const float Ampere3
 		);
 
-	std::map<std::string, CNotificationBase*> m_notifiers;
-	void AddNotifier(CNotificationBase *notifier);
-	void RemoveNotifier(CNotificationBase * notifier);
-protected:
-	void SetConfigValue(const std::string &key, const std::string &value);
-private:
 	std::string ParseCustomMessage(const std::string &cMessage, const std::string &sName, const std::string &sValue);
-	bool ApplyRule(std::string rule, bool equal, bool less);
-	boost::mutex m_mutex;
+	bool ApplyRule(const std::string &rule, const bool equal, const bool less);
+	std::mutex m_mutex;
 	std::map<uint64_t, std::vector<_tNotification> > m_notifications;
 	int m_NotificationSensorInterval;
 	int m_NotificationSwitchInterval;
