@@ -9,23 +9,15 @@ public:
 protected:
 	virtual bool SendMessageImplementation(
 		const uint64_t Idx,
-		const std::string &Name,
-		const std::string &Subject,
-		const std::string &Text,
-		const std::string &ExtraData,
+		const std::string& Name,
+		const std::string& Subject,
+		const std::string& Text,
+		const std::string& ExtraData,
 		const int Priority,
-		const std::string &Sound,
+		const std::string& Sound,
 		const bool bFromNotification);
 private:
 	/* config vars */
 	std::string _clickatellApi;
-	std::string _clickatellUser;
-	std::string _clickatellPassword;
-	std::string _clickatellFrom;
 	std::string _clickatellTo;
-	/* default vars */
-	int _maxCredits;
-	int _escalate;
-	int _features;
-	int _concat;
 };
