@@ -1,7 +1,7 @@
 #include "stdafx.h"
 #include "BasePush.h"
 #include "../hardware/hardwaretypes.h"
-#include "../json/json.h"
+#include <json/json.h>
 #include "../main/Helper.h"
 #include "../main/Logger.h"
 #include "../main/RFXtrx.h"
@@ -67,7 +67,11 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeRAIN, sTypeRAIN4, "Rain rate,Total rain" },
 		{ pTypeRAIN, sTypeRAIN5, "Rain rate,Total rain" },
 		{ pTypeRAIN, sTypeRAIN6, "Rain rate,Total rain" },
+		{ pTypeRAIN, sTypeRAIN7, "Rain rate,Total rain" },
+		{ pTypeRAIN, sTypeRAIN8, "Rain rate,Total rain" },
+		{ pTypeRAIN, sTypeRAIN9, "Rain rate,Total rain" },
 		{ pTypeRAIN, sTypeRAINWU, "Rain rate,Total rain" },
+		{ pTypeRAIN, sTypeRAINByRate, "Rain rate,Total rain" },
 
 		{ pTypeWIND, sTypeWIND1, "Direction,Direction string,Speed,Gust,Temperature,Chill" },
 		{ pTypeWIND, sTypeWIND2, "Direction,Direction string,Speed,Gust,Temperature,Chill" },
@@ -76,7 +80,8 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeWIND, sTypeWIND5, "Direction,Direction string,Speed,Gust,Temperature,Chill" },
 		{ pTypeWIND, sTypeWIND6, "Direction,Direction string,Speed,Gust,Temperature,Chill" },
 		{ pTypeWIND, sTypeWIND7, "Direction,Direction string,Speed,Gust,Temperature,Chill" },
-		{ pTypeWIND, sTypeWINDNoTemp, "Direction,Direction string,Speed,Gust,Temperature,Chill" },
+		{ pTypeWIND, sTypeWINDNoTemp, "Direction,Direction string,Speed,Gust,Chill" },
+		{ pTypeWIND, sTypeWINDNoTempNoChill, "Direction,Direction string,Speed,Gust" },
 
 		{ pTypeUV, sTypeUV1, "UV,Temperature" },
 		{ pTypeUV, sTypeUV2, "UV,Temperature" },
@@ -94,6 +99,7 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeLighting1, sTypeEnergenie5, "Status" },
 		{ pTypeLighting1, sTypeGDR2, "Status" },
 		{ pTypeLighting1, sTypeHQ, "Status" },
+		{ pTypeLighting1, sTypeOase, "Status" },
 
 		{ pTypeLighting2, sTypeAC, "Status" },
 		{ pTypeLighting2, sTypeHEU, "Status" },
@@ -114,7 +120,7 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeLighting5, sTypeTRC02_2, "Status" },
 		{ pTypeLighting5, sTypeAoke, "Status" },
 		{ pTypeLighting5, sTypeEurodomest, "Status" },
-		{ pTypeLighting5, sTypeLivoloAppliance, "Status" },
+		{ pTypeLighting5, sTypeLivolo1to10, "Status" },
 		{ pTypeLighting5, sTypeRGB432W, "Status" },
 		{ pTypeLighting5, sTypeMDREMOTE107, "Status" },
 		{ pTypeLighting5, sTypeLegrandCAD, "Status" },
@@ -125,6 +131,10 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 
 		{ pTypeLighting6, sTypeBlyss, "Status" },
 
+		{ pTypeFS20, sTypeFS20, "Status" },
+		{ pTypeFS20, sTypeFHT8V, "Status" },
+		{ pTypeFS20, sTypeFHT80, "Status" },
+
 		{ pTypeHomeConfort, sTypeHomeConfortTEL010, "Status" },
 
 		{ pTypeCurtain, sTypeHarrison, "Status" },
@@ -143,6 +153,7 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeBlinds, sTypeBlindsT11, "Status" },
 		{ pTypeBlinds, sTypeBlindsT12, "Status" },
 		{ pTypeBlinds, sTypeBlindsT13, "Status" },
+		{ pTypeBlinds, sTypeBlindsT14, "Status" },
 
 		{ pTypeSecurity1, sTypeSecX10, "Status" },
 		{ pTypeSecurity1, sTypeSecX10M, "Status" },
@@ -154,12 +165,17 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeSecurity1, sTypePowercodeAux, "Status" },
 		{ pTypeSecurity1, sTypeMeiantech, "Status" },
 		{ pTypeSecurity1, sTypeSA30, "Status" },
+		{ pTypeSecurity1, sTypeRM174RF, "Status" },
 		{ pTypeSecurity1, sTypeDomoticzSecurity, "Status" },
 
 		{ pTypeSecurity2, sTypeSec2Classic, "Status" },
 
 		{ pTypeCamera, sTypeNinja, "Not implemented" },
 
+		{ pTypeFS20, sTypeFS20, "FS20" },
+		{ pTypeFS20, sTypeFHT8V, "FHT8V" },
+		{ pTypeFS20, sTypeFHT80, "FHT80" },
+
 		{ pTypeRemote, sTypeATI, "Status" },
 		{ pTypeRemote, sTypeATIplus, "Status" },
 		{ pTypeRemote, sTypeMedion, "Status" },
@@ -235,6 +251,7 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeGeneral, sTypeTextStatus, "Text" },
 		{ pTypeGeneral, sTypeZWaveThermostatMode, "Thermostat Mode" },
 		{ pTypeGeneral, sTypeZWaveThermostatFanMode, "Thermostat Fan Mode" },
+		{ pTypeGeneral, sTypeZWaveThermostatOperatingState, "Thermostat Operating State" },
 		{ pTypeGeneral, sTypeAlert, "Alert" },
 		{ pTypeGeneral, sTypeSoundLevel, "Sound Level" },
 		{ pTypeGeneral, sTypeUV, "UV,Temperature" },
@@ -244,6 +261,7 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeGeneral, sTypeWaterflow, "Percentage" },
 		{ pTypeGeneral, sTypeCustom, "Percentage" },
 		{ pTypeGeneral, sTypeZWaveAlarm, "Status" },
+		{ pTypeGeneral, sTypeManagedCounter, "Counter" },
 
 		{ pTypeThermostat, sTypeThermSetpoint, "Temperature" },
 		{ pTypeThermostat, sTypeThermTemperature, "Temperature" },
@@ -251,8 +269,9 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeChime, sTypeByronSX, "Status" },
 		{ pTypeChime, sTypeByronMP001, "Status" },
 		{ pTypeChime, sTypeSelectPlus, "Status" },
-		{ pTypeChime, sTypeSelectPlus3, "Status" },
+		{ pTypeChime, sTypeByronBY, "Status" },
 		{ pTypeChime, sTypeEnvivo, "Status" },
+		{ pTypeChime, sTypeAlfawise, "Status" },
 
 		{ pTypeTEMP_RAIN, sTypeTR1, "Temperature,Total rain" },
 
@@ -260,10 +279,13 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 
 		{ pTypePOWER, sTypeELEC5, "Instant,Usage" },
 
-		{ pTypeLimitlessLights, sTypeLimitlessRGBW, "Status" },
-		{ pTypeLimitlessLights, sTypeLimitlessRGB, "Status" },
-		{ pTypeLimitlessLights, sTypeLimitlessWhite, "Status" },
-		{ pTypeLimitlessLights, sTypeLimitlessRGBWW, "Status" },
+		{ pTypeColorSwitch, sTypeColor_RGB_W, "Status" },
+		{ pTypeColorSwitch, sTypeColor_RGB, "Status" },
+		{ pTypeColorSwitch, sTypeColor_White, "Status" },
+		{ pTypeColorSwitch, sTypeColor_RGB_CW_WW, "Status" },
+		{ pTypeColorSwitch, sTypeColor_RGB_W_Z, "Status" },
+		{ pTypeColorSwitch, sTypeColor_RGB_CW_WW_Z, "Status" },
+		{ pTypeColorSwitch, sTypeColor_CW_WW, "Status" },
 
 		{ pTypeRFY, sTypeRFY, "Status" },
 		{ pTypeRFY, sTypeRFY2, "Status" },
@@ -272,9 +294,7 @@ const char *RFX_Type_SubType_Values(const unsigned char dType, const unsigned ch
 		{ pTypeEvohome, sTypeEvohome, "Status" },
 		{ pTypeEvohomeZone, sTypeEvohomeZone, "Temperature,Set point,Status" },
 		{ pTypeEvohomeWater, sTypeEvohomeWater, "Temperature,State,Status" },
-		{ pTypeEvohomeRelay, sTypeEvohomeRelay, "Status" },
-
-		{ pTypeGeneralSwitch, sSwitchTypeX10, "Status" },
+		{ pTypeEvohomeRelay, sTypeEvohomeRelay, "Status,Value" },
 
 		{ pTypeGeneralSwitch, sSwitchTypeX10, "Status" },
 		{ pTypeGeneralSwitch, sSwitchTypeARC, "Status" },
@@ -422,20 +442,20 @@ std::vector<std::string> CBasePush::DropdownOptions(const uint64_t DeviceRowIdxI
 	std::vector<std::string> dropdownOptions;
 
 	std::vector<std::vector<std::string> > result;
-	result=m_sql.safe_query("SELECT Type, SubType FROM DeviceStatus WHERE (ID== %" PRIu64 ")", DeviceRowIdxIn);
-	if (result.size()>0)
+	result = m_sql.safe_query("SELECT Type, SubType FROM DeviceStatus WHERE (ID== %" PRIu64 ")", DeviceRowIdxIn);
+	if (!result.empty())
 	{
-		int dType=atoi(result[0][0].c_str());
-		int dSubType=atoi(result[0][1].c_str());
+		int dType = atoi(result[0][0].c_str());
+		int dSubType = atoi(result[0][1].c_str());
 
-		std::string sOptions = RFX_Type_SubType_Values(dType,dSubType);
-		std::vector<std::string> tmpV; 
-		StringSplit(sOptions, ",", tmpV); 
-		for (int i = 0; i < (int) tmpV.size(); ++i) { 
-			dropdownOptions.push_back(tmpV[i]); 
+		std::string sOptions = RFX_Type_SubType_Values(dType, dSubType);
+		std::vector<std::string> tmpV;
+		StringSplit(sOptions, ",", tmpV);
+		for (int i = 0; i < (int)tmpV.size(); ++i) {
+			dropdownOptions.push_back(tmpV[i]);
 		}
 	}
-   	else 
+	else
 	{
 		dropdownOptions.push_back("Not implemented");
 	}
@@ -443,23 +463,23 @@ std::vector<std::string> CBasePush::DropdownOptions(const uint64_t DeviceRowIdxI
 }
 
 std::string CBasePush::DropdownOptionsValue(const uint64_t DeviceRowIdxIn, const int pos)
-{	
+{
 	std::string wording = "???";
-	int getpos = pos-1; // 0 pos is always nvalue/status, 1 and higher goes to svalues
+	int getpos = pos - 1; // 0 pos is always nvalue/status, 1 and higher goes to svalues
 	std::vector<std::vector<std::string> > result;
-	
-	result=m_sql.safe_query("SELECT Type, SubType FROM DeviceStatus WHERE (ID== %" PRIu64 ")", DeviceRowIdxIn);
-	if (result.size()>0)
-	{
-		int dType=atoi(result[0][0].c_str());
-		int dSubType=atoi(result[0][1].c_str());
 
-		std::string sOptions = RFX_Type_SubType_Values(dType,dSubType);
-		std::vector<std::string> tmpV; 
+	result = m_sql.safe_query("SELECT Type, SubType FROM DeviceStatus WHERE (ID== %" PRIu64 ")", DeviceRowIdxIn);
+	if (!result.empty())
+	{
+		int dType = atoi(result[0][0].c_str());
+		int dSubType = atoi(result[0][1].c_str());
+
+		std::string sOptions = RFX_Type_SubType_Values(dType, dSubType);
+		std::vector<std::string> tmpV;
 		StringSplit(sOptions, ",", tmpV);
 		if (tmpV.size() > 1)
 		{
-			if ( (int) tmpV.size() >= pos && getpos >= 0) {
+			if ((int)tmpV.size() >= pos && getpos >= 0) {
 				wording = tmpV[getpos];
 			}
 		}
@@ -471,232 +491,246 @@ std::string CBasePush::DropdownOptionsValue(const uint64_t DeviceRowIdxIn, const
 	return wording;
 }
 
-std::string CBasePush::ProcessSendValue(const std::string &rawsendValue, const int delpos, const int nValue, const int includeUnit, const int metertypein)
+std::string CBasePush::ProcessSendValue(const std::string &rawsendValue, const int delpos, const int nValue, const int includeUnit, const int devType, const int devSubType, const int metertypein)
 {
-	std::string vType = DropdownOptionsValue(m_DeviceRowIdx,delpos);
-	unsigned char tempsign=m_sql.m_tempsign[0];
-	_eMeterType metertype = (_eMeterType) metertypein;
-	char szData[100]= "";
+	char szData[100];
+	szData[0] = 0;
+	try
+	{
+		std::string vType = DropdownOptionsValue(m_DeviceRowIdx, delpos);
+		unsigned char tempsign = m_sql.m_tempsign[0];
+		_eMeterType metertype = (_eMeterType)metertypein;
 
-	std::string unit = getUnit(delpos, metertypein);
-
-	if ((vType=="Temperature") || (vType=="Temperature 1") || (vType=="Temperature 2")|| (vType == "Set point"))
-	{
-		double tvalue=ConvertTemperature(atof(rawsendValue.c_str()),tempsign);
-		sprintf(szData,"%.1f", tvalue);
-	}
-	else if (vType == "Concentration")
-	{
-		sprintf(szData,"%d", nValue);
-	}
-	else if (vType == "Humidity")
-	{
-		sprintf(szData,"%d", atoi(rawsendValue.c_str()));
-	}
-	else if (vType == "Humidity Status")
-	{
-		sprintf(szData,"%s", RFX_Humidity_Status_Desc(atoi(rawsendValue.c_str())));	
-	}
-	else if (vType == "Barometer")
-	{
-		sprintf(szData,"%.1f", atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Forecast")
-	{
-		int forecast=atoi(rawsendValue.c_str());
-		if (forecast!=baroForecastNoInfo)
+		if ((vType == "Temperature") || (vType == "Temperature 1") || (vType == "Temperature 2") || (vType == "Set point"))
 		{
-			sprintf(szData,"%s", RFX_Forecast_Desc(forecast));
+			sprintf(szData, "%g", ConvertTemperature(std::stod(rawsendValue), tempsign));
 		}
-		else {
-			sprintf(szData,"%d", forecast);
-		}
-	}
-	else if (vType == "Altitude")
-	{
-		sprintf(szData, "%.1f", atof(rawsendValue.c_str()));
-	}
-	else if (vType == "UV")
-	{
-		float UVI = static_cast<float>(atof(rawsendValue.c_str()));
-		sprintf(szData,"%.1f",UVI);
-	}
-	else if (vType == "Direction")
-	{
-		float Direction = static_cast<float>(atof(rawsendValue.c_str()));
-		sprintf(szData,"%.1f",Direction); 
-	}
-	else if (vType == "Direction string")
-	{
-		sprintf(szData,"%s",rawsendValue.c_str());
-	}
-	else if (vType == "Speed")
-	{
-		int intSpeed = atoi(rawsendValue.c_str());
-		if (m_sql.m_windunit != WINDUNIT_Beaufort)
+		else if (vType == "Concentration")
 		{
-			sprintf(szData, "%.1f", float(intSpeed) * m_sql.m_windscale);
+			sprintf(szData, "%d", nValue);
 		}
-		else
+		else if (vType == "Humidity")
 		{
-			float speedms = float(intSpeed)*0.1f;
-			sprintf(szData, "%d", MStoBeaufort(speedms));
+			if (devType == pTypeHUM)
+				sprintf(szData, "%d", nValue);
+			else
+				sprintf(szData, "%g", std::stof(rawsendValue));
 		}
-	}
-	else if (vType == "Gust")
-	{
-		int intGust=atoi(rawsendValue.c_str());
-		if (m_sql.m_windunit != WINDUNIT_Beaufort)
+		else if (vType == "Humidity Status")
 		{
-			sprintf(szData, "%.1f", float(intGust) *m_sql.m_windscale);
+			sprintf(szData, "%s", RFX_Humidity_Status_Desc(std::stoi(rawsendValue)));
 		}
-		else
+		else if (vType == "Barometer")
 		{
-			float gustms = float(intGust)*0.1f;
-			sprintf(szData, "%d", MStoBeaufort(gustms));
+			sprintf(szData, "%g", std::stof(rawsendValue));
 		}
-	}
-	else if (vType == "Chill")
-	{
-		double tvalue=ConvertTemperature(atof(rawsendValue.c_str()),tempsign);
-		sprintf(szData,"%.1f", tvalue);
-	}
-	else if (vType == "Rain rate")
-	{
-		sprintf(szData, "%.1f", atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Total rain")
-	{
-		sprintf(szData, "%.1f", atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Counter")
-	{
-		strcpy(szData, rawsendValue.c_str());
-	}	
-	else if (vType == "Mode")
-	{
-		sprintf(szData,"Not supported yet");
-	}
-	else if (vType == "Sound Level")
-	{
-		strcpy(szData, rawsendValue.c_str());
-	}
-	
-	else if (vType == "Status")
-	{
-		sprintf(szData, "%d", nValue);
-	}	
-	else if ((vType == "Current 1") || (vType == "Current 2") || (vType == "Current 3"))
-	{
-		sprintf(szData, "%.3f", atof(rawsendValue.c_str()));
-	}	
-	else if (vType == "Instant")
-	{
-		sprintf(szData, "%.3f", atof(rawsendValue.c_str()));
-	}
-	else if ((vType == "Usage") || (vType == "Usage 1") || (vType == "Usage 2") )
-	{
-		strcpy(szData,rawsendValue.c_str());
-	}	
-	else if ((vType == "Delivery") || (vType == "Delivery 1") || (vType == "Delivery 2") )
-	{
-		strcpy(szData, rawsendValue.c_str());
-	}
-	else if (vType == "Usage current")
-	{
-		sprintf(szData,"%.1f",atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Delivery current")
-	{
-		sprintf(szData,"%.1f",atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Gas usage")
-	{
-		sprintf(szData, "%.3f", atof(rawsendValue.c_str()) / 1000.0f);
-	}
-	else if (vType == "Weight")
-	{
-		sprintf(szData,"%.1f",atof(rawsendValue.c_str()));
-	}	
-	else if (vType == "Voltage")
-	{
-		sprintf(szData,"%.3f",atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Value")
-	{
-		sprintf(szData,"%d", atoi(rawsendValue.c_str())); //??
-	}
-	else if (vType == "Visibility")
-	{
-		float vis = static_cast<float>(atof(rawsendValue.c_str()));
-		if (metertype==0)
+		else if (vType == "Forecast")
 		{
-			//km
-			sprintf(szData,"%.1f",vis);
+			int forecast = std::stoi(rawsendValue);
+			if (forecast != baroForecastNoInfo)
+			{
+				sprintf(szData, "%s", RFX_Forecast_Desc(forecast));
+			}
+			else {
+				sprintf(szData, "%d", forecast);
+			}
 		}
-		else
+		else if (vType == "Altitude")
 		{
-			//miles
-			sprintf(szData,"%.1f",vis*0.6214f);
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "UV")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Direction")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Direction string")
+		{
+			strcpy(szData, rawsendValue.c_str());
+		}
+		else if (vType == "Speed")
+		{
+			int intSpeed = std::stoi(rawsendValue);
+			if (m_sql.m_windunit != WINDUNIT_Beaufort)
+			{
+				sprintf(szData, "%g", float(intSpeed) * m_sql.m_windscale);
+			}
+			else
+			{
+				float speedms = float(intSpeed) * 0.1f;
+				sprintf(szData, "%d", MStoBeaufort(speedms));
+			}
+		}
+		else if (vType == "Gust")
+		{
+			int intGust = std::stoi(rawsendValue);
+			if (m_sql.m_windunit != WINDUNIT_Beaufort)
+			{
+				sprintf(szData, "%g", float(intGust) * m_sql.m_windscale);
+			}
+			else
+			{
+				float gustms = float(intGust) * 0.1f;
+				sprintf(szData, "%d", MStoBeaufort(gustms));
+			}
+		}
+		else if (vType == "Chill")
+		{
+			sprintf(szData, "%g", ConvertTemperature(std::stof(rawsendValue), tempsign));
+		}
+		else if (vType == "Rain rate")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Total rain")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Counter")
+		{
+			strcpy(szData, rawsendValue.c_str());
+		}
+		else if (vType == "Mode")
+		{
+			sprintf(szData, "Not supported yet");
+		}
+		else if (vType == "Sound Level")
+		{
+			strcpy(szData, rawsendValue.c_str());
+		}
+		else if (vType == "Distance")
+		{
+			strcpy(szData, rawsendValue.c_str());
+		}
+		else if (vType == "Status")
+		{
+			sprintf(szData, "%d", nValue);
+		}
+		else if ((vType == "Current 1") || (vType == "Current 2") || (vType == "Current 3"))
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Instant")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if ((vType == "Usage") || (vType == "Usage 1") || (vType == "Usage 2"))
+		{
+			strcpy(szData, rawsendValue.c_str());
+		}
+		else if ((vType == "Delivery") || (vType == "Delivery 1") || (vType == "Delivery 2"))
+		{
+			strcpy(szData, rawsendValue.c_str());
+		}
+		else if (vType == "Usage current")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Delivery current")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Gas usage")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue) / 1000.0f);
+		}
+		else if (vType == "Weight")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Voltage")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Value")
+		{
+			sprintf(szData, "%d", std::stoi(rawsendValue));
+		}
+		else if (vType == "Visibility")
+		{
+			float vis = std::stof(rawsendValue);
+			if (metertype == 0)
+			{
+				//km
+				sprintf(szData, "%g", vis);
+			}
+			else
+			{
+				//miles
+				sprintf(szData, "%g", vis * 0.6214f);
+			}
+		}
+		else if (vType == "Solar Radiation")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Soil Moisture")
+		{
+			sprintf(szData, "%d", nValue);
+		}
+		else if (vType == "Leaf Wetness")
+		{
+			sprintf(szData, "%d", nValue);
+		}
+		else if (vType == "Percentage")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Fanspeed")
+		{
+			sprintf(szData, "%d", std::stoi(rawsendValue));
+		}
+		else if (vType == "Pressure")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
+		}
+		else if (vType == "Lux")
+		{
+			sprintf(szData, "%g", std::stof(rawsendValue));
 		}
 	}
-	else if (vType == "Solar Radiation")
+	catch (...)
 	{
-		float radiation = static_cast<float>(atof(rawsendValue.c_str()));
-		sprintf(szData,"%.1f",radiation);
+		_log.Log(LOG_ERROR, "BasePush: Problem with conversion!");
 	}
-	else if (vType == "Soil Moisture")
+	if (szData[0] != 0)
 	{
-		sprintf(szData,"%d",nValue);
-	}
-	else if (vType == "Leaf Wetness")
-	{
-		sprintf(szData,"%d",nValue);
-	}
-	else if (vType == "Percentage")
-	{
-		sprintf(szData,"%.2f",atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Fanspeed")
-	{
-		sprintf(szData,"%d",atoi(rawsendValue.c_str()));
-	}
-	else if (vType == "Pressure")
-	{
-		sprintf(szData,"%.1f",atof(rawsendValue.c_str()));
-	}
-	else if (vType == "Lux")
-	{
-		sprintf(szData,"%.0f",atof(rawsendValue.c_str()));
-	}
-	if (szData[0] != '\0') { 
 		std::string sendValue(szData);
-		if (includeUnit) {
-			sendValue+=" ";
-			sendValue+=unit;
+		if (includeUnit)
+		{
+			std::string unit = getUnit(delpos, metertypein);
+			if (!unit.empty())
+			{
+				sendValue += " ";
+				sendValue += unit;
+			}
 		}
 		return sendValue;
 	}
 	else {
-		_log.Log(LOG_ERROR,"Could not determine data push value");
+		_log.Log(LOG_ERROR, "BasePush: Could not determine data push value");
 		return "";
 	}
 }
 
 std::string CBasePush::getUnit(const int delpos, const int metertypein)
 {
-	std::string vType = DropdownOptionsValue(m_DeviceRowIdx,delpos);
-	unsigned char tempsign=m_sql.m_tempsign[0];
-	_eMeterType metertype = (_eMeterType) metertypein;
-	char szData[100]= "";
+	std::string vType = DropdownOptionsValue(m_DeviceRowIdx, delpos);
+	unsigned char tempsign = m_sql.m_tempsign[0];
+	_eMeterType metertype = (_eMeterType)metertypein;
+	char szData[100];
+	szData[0] = 0;
 
-	if ((vType=="Temperature") || (vType=="Temperature 1") || (vType=="Temperature 2")|| (vType == "Set point"))
+	if ((vType == "Temperature") || (vType == "Temperature 1") || (vType == "Temperature 2") || (vType == "Set point"))
 	{
-		sprintf(szData,"%c", tempsign);
+		sprintf(szData, "%c", tempsign);
 	}
 	else if (vType == "Humidity")
 	{
-		strcpy(szData,"%%");
+		strcpy(szData, "%%");
 	}
 	else if (vType == "Humidity Status")
 	{
@@ -740,16 +774,16 @@ std::string CBasePush::getUnit(const int delpos, const int metertypein)
 	}
 	else if (vType == "Rain rate")
 	{
-		strcpy(szData,"");
+		strcpy(szData, "");
 	}
 	else if (vType == "Total rain")
 	{
 		strcpy(szData, "");
-	}	
+	}
 	else if (vType == "Counter")
 	{
 		strcpy(szData, "");
-	}	
+	}
 	else if (vType == "Mode")
 	{
 		strcpy(szData, "");
@@ -761,20 +795,20 @@ std::string CBasePush::getUnit(const int delpos, const int metertypein)
 	else if (vType == "Status")
 	{
 		strcpy(szData, "");
-	}	
+	}
 	else if ((vType == "Current 1") || (vType == "Current 2") || (vType == "Current 3"))
 	{
 		strcpy(szData, "");
-	}	
+	}
 	else if (vType == "Instant")
 	{
 		strcpy(szData, "");
-	}	
-	else if ((vType == "Usage") || (vType == "Usage 1") || (vType == "Usage 2") )
+	}
+	else if ((vType == "Usage") || (vType == "Usage 1") || (vType == "Usage 2"))
 	{
 		strcpy(szData, "Watt");
-	}	
-	else if ((vType == "Delivery") || (vType == "Delivery 1") || (vType == "Delivery 2") )
+	}
+	else if ((vType == "Delivery") || (vType == "Delivery 1") || (vType == "Delivery 2"))
 	{
 		strcpy(szData, "Watt");
 	}
@@ -793,7 +827,7 @@ std::string CBasePush::getUnit(const int delpos, const int metertypein)
 	else if (vType == "Weight")
 	{
 		strcpy(szData, m_sql.m_weightsign.c_str());
-	}	
+	}
 	else if (vType == "Voltage")
 	{
 		strcpy(szData, "V");
@@ -804,7 +838,7 @@ std::string CBasePush::getUnit(const int delpos, const int metertypein)
 	}
 	else if (vType == "Visibility")
 	{
-		if (metertype==0)
+		if (metertype == 0)
 		{
 			//km
 			strcpy(szData, "km");
@@ -847,7 +881,7 @@ std::string CBasePush::getUnit(const int delpos, const int metertypein)
 	{
 		strcpy(szData, "ppm");
 	}
-	if (szData[0] != '\0') { 
+	if (szData[0] != 0) {
 		std::string sendValue(szData);
 		return sendValue;
 	}
@@ -862,12 +896,12 @@ namespace http {
 		{
 			root["status"] = "OK";
 			root["title"] = "GetDevicesListOnOff";
-			int ii = 0;
 			std::vector<std::vector<std::string> > result;
 			result = m_sql.safe_query("SELECT ID, Name, Type, SubType FROM DeviceStatus WHERE (Used == 1) ORDER BY Name");
-			if (result.size() > 0)
+			if (!result.empty())
 			{
 				std::vector<std::vector<std::string> >::const_iterator itt;
+				int ii = 0;
 				for (itt = result.begin(); itt != result.end(); ++itt)
 				{
 					std::vector<std::string> sd = *itt;
