@@ -11,12 +11,13 @@
 #include "../main/WebServer.h"
 #include "../webserver/cWebem.h"
 #include "../main/mainworker.h"
-#include "../json/json.h"
+#include <json/json.h>
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 
 CHttpPush::CHttpPush()
 {
+	m_PushType = PushType::PUSHTYPE_HTTP;
 	m_bLinkActive = false;
 }
 
@@ -35,7 +36,7 @@ void CHttpPush::Stop()
 
 void CHttpPush::UpdateActive()
 {
-	int fActive;
+	int fActive = 0;
 	m_sql.GetPreferencesVar("HttpActive", fActive);
 	m_bLinkActive = (fActive == 1);
 }
@@ -50,13 +51,12 @@ void CHttpPush::OnDeviceReceived(const int m_HwdID, const uint64_t DeviceRowIdx,
 }
 
 void CHttpPush::DoHttpPush()
-{			
+{
 	std::string httpUrl = "";
 	std::string httpData = "";
-	std::string httpHeaders= "";
+	std::string httpHeaders = "";
 	int httpMethodInt = 0;
 	int httpAuthInt = 0;
-	std::string httpAuth = "";
 	std::string httpAuthBasicLogin = "";
 	std::string httpAuthBasicPassword = "";
 	m_sql.GetPreferencesVar("HttpMethod", httpMethodInt);
@@ -64,22 +64,23 @@ void CHttpPush::DoHttpPush()
 	m_sql.GetPreferencesVar("HttpAuthBasicLogin", httpAuthBasicLogin);
 	m_sql.GetPreferencesVar("HttpAuthBasicPassword", httpAuthBasicPassword);
 
-	int httpDebugActiveInt;
+	int httpDebugActiveInt = 0;
 	bool httpDebugActive = false;
 	m_sql.GetPreferencesVar("HttpDebug", httpDebugActiveInt);
 	if (httpDebugActiveInt == 1) {
 		httpDebugActive = true;
 	}
 	std::vector<std::vector<std::string> > result;
-	result=m_sql.safe_query(
-		"SELECT A.DeviceID, A.DelimitedValue, B.ID, B.Type, B.SubType, B.nValue, B.sValue, A.TargetType, A.TargetVariable, A.TargetDeviceID, A.TargetProperty, A.IncludeUnit, B.SwitchType, strftime('%%s', B.LastUpdate), B.Name FROM HttpLink as A, DeviceStatus as B "
-		"WHERE (A.DeviceID == '%" PRIu64 "' AND A.Enabled = '1' AND A.DeviceID==B.ID)",
+	result = m_sql.safe_query(
+		"SELECT A.DeviceRowID, A.DelimitedValue, B.ID, B.Type, B.SubType, B.nValue, B.sValue, A.TargetType, A.TargetVariable, A.TargetDeviceID, A.TargetProperty, A.IncludeUnit, B.SwitchType, strftime('%%s', B.LastUpdate), B.Name FROM PushLink as A, DeviceStatus as B "
+		"WHERE (A.PushType==%d AND A.DeviceRowID == '%" PRIu64 "' AND A.Enabled = '1' AND A.DeviceRowID==B.ID)",
+		PushType::PUSHTYPE_HTTP,
 		m_DeviceRowIdx);
-	if (result.size()>0)
+	if (!result.empty())
 	{
 		std::string sendValue;
 		std::vector<std::vector<std::string> >::const_iterator itt;
-		for (itt=result.begin(); itt!=result.end(); ++itt)
+		for (itt = result.begin(); itt != result.end(); ++itt)
 		{
 			m_sql.GetPreferencesVar("HttpUrl", httpUrl);
 			m_sql.GetPreferencesVar("HttpData", httpData);
@@ -87,8 +88,8 @@ void CHttpPush::DoHttpPush()
 			if (httpUrl == "")
 				return;
 
-			std::vector<std::string> sd=*itt;
-			unsigned int deviceId = atoi(sd[0].c_str());
+			std::vector<std::string> sd = *itt;
+			//unsigned int deviceId = atoi(sd[0].c_str());
 			std::string sdeviceId = sd[0].c_str();
 			std::string ldelpos = sd[1].c_str();
 			int delpos = atoi(sd[1].c_str());
@@ -96,10 +97,10 @@ void CHttpPush::DoHttpPush()
 			int dSubType = atoi(sd[4].c_str());
 			int nValue = atoi(sd[5].c_str());
 			std::string sValue = sd[6].c_str();
-			int targetType = atoi(sd[7].c_str());
+			//int targetType = atoi(sd[7].c_str());
 			std::string targetVariable = sd[8].c_str();
-			int targetDeviceID = atoi(sd[9].c_str());
-			std::string targetProperty = sd[10].c_str();
+			//int targetDeviceID = atoi(sd[9].c_str());
+			//std::string targetProperty = sd[10].c_str();
 			int includeUnit = atoi(sd[11].c_str());
 			int metertype = atoi(sd[12].c_str());
 			int lastUpdate = atoi(sd[13].c_str());
@@ -118,13 +119,13 @@ void CHttpPush::DoHttpPush()
 			unsigned long long int localTimeUtc = lastUpdate - tzoffset;
 #endif
 
-			char szLocalTime[16];
+			char szLocalTime[21];
 			sprintf(szLocalTime, "%llu", localTime);
-			char szLocalTimeUtc[16];
+			char szLocalTimeUtc[21];
 			sprintf(szLocalTimeUtc, "%llu", localTimeUtc);
-			char szLocalTimeMs[16];
-			sprintf(szLocalTimeMs, "%llu", localTime*1000);
-			char szLocalTimeUtcMs[16];
+			char szLocalTimeMs[21];
+			sprintf(szLocalTimeMs, "%llu", localTime * 1000);
+			char szLocalTimeUtcMs[21];
 			sprintf(szLocalTimeUtcMs, "%llu", localTimeUtc * 1000);
 
 			std::string llastUpdate = get_lastUpdate(localTimeUtc);
@@ -148,28 +149,28 @@ void CHttpPush::DoHttpPush()
 			*/
 
 			std::string lunit = getUnit(delpos, metertype);
-			std::string lType = RFX_Type_Desc(dType,1);
-			std::string lSubType = RFX_Type_SubType_Desc(dType,dSubType);
-			
+			std::string lType = RFX_Type_Desc(dType, 1);
+			std::string lSubType = RFX_Type_SubType_Desc(dType, dSubType);
+
 			char hostname[256];
 			gethostname(hostname, sizeof(hostname));
 
 			std::vector<std::string> strarray;
-			if (sendValue.find(";")!=std::string::npos)
+			if (sendValue.find(";") != std::string::npos)
 			{
 				StringSplit(sendValue, ";", strarray);
-				if (int(strarray.size())>=delpos && delpos > 0)
+				if (int(strarray.size()) >= delpos && delpos > 0)
 				{
-					std::string rawsendValue = strarray[delpos-1].c_str();
-					sendValue = ProcessSendValue(rawsendValue,delpos,nValue,false,metertype);
+					std::string rawsendValue = strarray[delpos - 1].c_str();
+					sendValue = ProcessSendValue(rawsendValue, delpos, nValue, false, dType, dSubType, metertype);
 				}
 			}
 			else
 			{
-				sendValue = ProcessSendValue(sendValue,delpos,nValue,false,metertype);
+				sendValue = ProcessSendValue(sendValue, delpos, nValue, false, dType, dSubType, metertype);
 			}
-			ltargetDeviceId+="_";
-			ltargetDeviceId+=ldelpos;
+			ltargetDeviceId += "_";
+			ltargetDeviceId += ldelpos;
 
 			replaceAll(httpUrl, "%v", sendValue);
 			replaceAll(httpUrl, "%u", includeUnit ? lunit : "");
@@ -207,7 +208,7 @@ void CHttpPush::DoHttpPush()
 				if (httpAuthInt == 1) {			// BASIC authentication
 					std::stringstream sstr;
 					sstr << httpAuthBasicLogin << ":" << httpAuthBasicPassword;
-					std::string m_AccessToken = base64_encode((const unsigned char *)(sstr.str().c_str()), strlen(sstr.str().c_str()));
+					std::string m_AccessToken = base64_encode(sstr.str());
 					ExtraHeaders.push_back("Authorization:Basic " + m_AccessToken);
 				}
 				sendValue = CURLEncode::URLEncode(sendValue);
@@ -228,7 +229,7 @@ void CHttpPush::DoHttpPush()
 					{
 						// Add additional headers
 						std::vector<std::string> ExtraHeaders2;
-						StringSplit(httpHeaders,"\r\n", ExtraHeaders2);
+						StringSplit(httpHeaders, "\r\n", ExtraHeaders2);
 						for (size_t i = 0; i < ExtraHeaders2.size(); i++)
 						{
 							ExtraHeaders.push_back(ExtraHeaders2[i]);
@@ -239,7 +240,7 @@ void CHttpPush::DoHttpPush()
 						_log.Log(LOG_ERROR, "HttpLink: Error sending data to http with POST!");
 					}
 				}
-				else if(httpMethodInt == 2) {		// PUT
+				else if (httpMethodInt == 2) {		// PUT
 					if (!HTTPClient::PUT(httpUrl, httpData, ExtraHeaders, sResult, true))
 					{
 						_log.Log(LOG_ERROR, "HttpLink: Error sending data to http with PUT!");
@@ -248,7 +249,7 @@ void CHttpPush::DoHttpPush()
 
 				// debug
 				if (httpDebugActive) {
-					_log.Log(LOG_NORM, "HttpLink: response ", sResult.c_str());
+					_log.Log(LOG_NORM, "HttpLink: response %s", sResult.c_str());
 				}
 			}
 		}
@@ -368,8 +369,8 @@ namespace http {
 				return; //Only admin user allowed
 			}
 			std::vector<std::vector<std::string> > result;
-			result = m_sql.safe_query("SELECT A.ID,A.DeviceID,A.Delimitedvalue,A.TargetType,A.TargetVariable,A.TargetDeviceID,A.TargetProperty,A.Enabled, B.Name, A.IncludeUnit FROM HttpLink as A, DeviceStatus as B WHERE (A.DeviceID==B.ID)");
-			if (result.size() > 0)
+			result = m_sql.safe_query("SELECT A.ID,A.DeviceRowID,A.Delimitedvalue,A.TargetType,A.TargetVariable,A.TargetDeviceID,A.TargetProperty,A.Enabled, B.Name, A.IncludeUnit FROM PushLink as A, DeviceStatus as B WHERE (A.PushType==%d AND A.DeviceRowID==B.ID)", CBasePush::PushType::PUSHTYPE_HTTP);
+			if (!result.empty())
 			{
 				std::vector<std::vector<std::string> >::const_iterator itt;
 				int ii = 0;
@@ -419,7 +420,8 @@ namespace http {
 				return;
 			if (idx == "0") {
 				m_sql.safe_query(
-					"INSERT INTO HttpLink (DeviceID,DelimitedValue,TargetType,TargetVariable,TargetDeviceID,TargetProperty,IncludeUnit,Enabled) VALUES ('%d','%d','%d','%q','%d','%q','%d','%d')",
+					"INSERT INTO PushLink (PushType, DeviceRowID,DelimitedValue,TargetType,TargetVariable,TargetDeviceID,TargetProperty,IncludeUnit,Enabled) VALUES ('%d','%d','%d','%d','%q','%d','%q','%d','%d')",
+					CBasePush::PushType::PUSHTYPE_HTTP,
 					deviceidi,
 					atoi(valuetosend.c_str()),
 					targettypei,
@@ -428,11 +430,11 @@ namespace http {
 					targetproperty.c_str(),
 					atoi(includeunit.c_str()),
 					atoi(linkactive.c_str())
-					);
+				);
 			}
 			else {
 				m_sql.safe_query(
-					"UPDATE HttpLink SET DeviceID='%d', DelimitedValue=%d, TargetType=%d, TargetVariable='%q', TargetDeviceID=%d, TargetProperty='%q', IncludeUnit='%d', Enabled='%d' WHERE (ID == '%q')",
+					"UPDATE PushLink SET DeviceRowID='%d', DelimitedValue=%d, TargetType=%d, TargetVariable='%q', TargetDeviceID=%d, TargetProperty='%q', IncludeUnit='%d', Enabled='%d' WHERE (ID == '%q')",
 					deviceidi,
 					atoi(valuetosend.c_str()),
 					targettypei,
@@ -442,7 +444,7 @@ namespace http {
 					atoi(includeunit.c_str()),
 					atoi(linkactive.c_str()),
 					idx.c_str()
-					);
+				);
 			}
 			root["status"] = "OK";
 			root["title"] = "SaveHttpLink";
@@ -459,7 +461,7 @@ namespace http {
 			std::string idx = request::findValue(&req, "idx");
 			if (idx == "")
 				return;
-			m_sql.safe_query("DELETE FROM HttpLink WHERE (ID=='%q')", idx.c_str());
+			m_sql.safe_query("DELETE FROM PushLink WHERE (ID=='%q')", idx.c_str());
 			root["status"] = "OK";
 			root["title"] = "DeleteHttpLink";
 		}
