@@ -1,10 +1,5 @@
 #pragma once
-
-#include <boost/signals2.hpp>
-
 #include "BasePush.h"
-#include <map>
-#include <string>
 
 class CInfluxPush : public CBasePush
 {
@@ -16,18 +11,15 @@ class CInfluxPush : public CBasePush
 	};
 public:
 	CInfluxPush();
-	void Start();
+	bool Start();
 	void Stop();
 	void UpdateSettings();
 private:
 	void OnDeviceReceived(const int m_HwdID, const uint64_t DeviceRowIdx, const std::string &DeviceName, const unsigned char *pRXCommand);
 	void DoInfluxPush();
 
-	boost::shared_ptr<boost::thread> m_background_task_thread;
-	boost::mutex m_background_task_mutex;
-	bool m_stoprequested;
-	bool StartThread();
-	void StopThread();
+	std::shared_ptr<std::thread> m_thread;
+	std::mutex m_background_task_mutex;
 	void Do_Work();
 
 	std::map<std::string,_tPushItem> m_PushedItems;
@@ -35,7 +27,10 @@ private:
 	std::string m_szURL;
 	std::string m_InfluxIP;
 	int m_InfluxPort;
+	std::string m_InfluxPath;
 	std::string m_InfluxDatabase;
+	std::string m_InfluxUsername;
+	std::string m_InfluxPassword;
 	bool m_bInfluxDebugActive;
 };
 extern CInfluxPush m_influxpush;
