@@ -9,6 +9,9 @@
 #include "../main/Logger.h"
 #include "../main/Helper.h"
 #include "../main/localtime_r.h"
+#include "../main/mainworker.h"
+
+extern bool g_bIsWSL;
 
 namespace http {
 namespace server {
@@ -21,7 +24,8 @@ server_base::server_base(const server_settings & settings, request_handler & use
 		request_handler_(user_request_handler),
 		timeout_(20), // default read timeout in seconds
 		is_running(false),
-		is_stop_complete(false) {
+		is_stop_complete(false),
+		m_heartbeat_timer(io_service_) {
 	if (!settings.is_enabled()) {
 		throw std::invalid_argument("cannot initialize a disabled server (listening port cannot be empty or 0)");
 	}
@@ -61,6 +65,8 @@ void server_base::run() {
 	// for new incoming connections.
 	try {
 		is_running = true;
+		// Don't enable heartbeat in WSL due to https://github.com/Microsoft/WSL/issues/3091 (Fixed in Windows 10 1809 / build 17686)
+		if (!g_bIsWSL) heart_beat(boost::system::error_code());
 		io_service_.run();
 		is_running = false;
 	} catch (std::exception& e) {
@@ -84,6 +90,9 @@ void server_base::run() {
 void server_base::stop() {
 	if (is_running) {
 		// Post a call to the stop function so that server_base::stop() is safe to call from any thread.
+		// Rene, set is_running to false, because the following is an io_service call, which makes is_running
+		// never set to false whilst in the call itself
+		is_running = false;
 		io_service_.post(boost::bind(&server_base::handle_stop, this));
 	} else {
 		// if io_service is not running then the post call will not be performed
@@ -103,6 +112,10 @@ void server_base::stop() {
 		}
 		sleep_milliseconds(500);
 	}
+	io_service_.stop();
+
+	// Deregister heartbeat
+	m_mainworker.HeartbeatRemove(std::string("WebServer:") + settings_.listening_port);
 }
 
 void server_base::handle_stop() {
@@ -116,11 +129,20 @@ void server_base::handle_stop() {
 	is_stop_complete = true;
 }
 
+void server_base::heart_beat(const boost::system::error_code& error)
+{
+	if (!error) {
+		// Heartbeat
+		m_mainworker.HeartbeatUpdate(std::string("WebServer:") + settings_.listening_port);
+
+		// Schedule next heartbeat
+		m_heartbeat_timer.expires_from_now(std::chrono::seconds(4));
+		m_heartbeat_timer.async_wait(boost::bind(&server_base::heart_beat, this, boost::asio::placeholders::error));
+	}
+}
+
 server::server(const server_settings & settings, request_handler & user_request_handler) :
 		server_base(settings, user_request_handler) {
-#ifdef DEBUG_WWW
-	_log.Log(LOG_STATUS, "[web:%s] create server using settings : %s", settings.listening_port.c_str(), settings.to_string().c_str());
-#endif
 	init(boost::bind(&server::init_connection, this),
 			boost::bind(&server::handle_accept, this, _1));
 }
@@ -148,11 +170,8 @@ void server::handle_accept(const boost::system::error_code& e) {
 ssl_server::ssl_server(const ssl_server_settings & ssl_settings, request_handler & user_request_handler) :
 		server_base(ssl_settings, user_request_handler),
 		settings_(ssl_settings),
-		context_(io_service_, ssl_settings.get_ssl_method())
+		context_(ssl_settings.get_ssl_method())
 {
-#ifdef DEBUG_WWW
-	_log.Log(LOG_STATUS, "[web:%s] create ssl_server using ssl_server_settings : %s", ssl_settings.listening_port.c_str(), ssl_settings.to_string().c_str());
-#endif
 	init(boost::bind(&ssl_server::init_connection, this),
 			boost::bind(&ssl_server::handle_accept, this, _1));
 }
@@ -161,10 +180,7 @@ ssl_server::ssl_server(const ssl_server_settings & ssl_settings, request_handler
 ssl_server::ssl_server(const server_settings & settings, request_handler & user_request_handler) :
 		server_base(settings, user_request_handler),
 		settings_(dynamic_cast<ssl_server_settings const &>(settings)),
-		context_(io_service_, dynamic_cast<ssl_server_settings const &>(settings).get_ssl_method()) {
-#ifdef DEBUG_WWW
-	_log.Log(LOG_STATUS, "[web:%s] create ssl_server using server_settings : %s", settings.listening_port.c_str(), settings.to_string().c_str());
-#endif
+		context_(dynamic_cast<ssl_server_settings const &>(settings).get_ssl_method()) {
 	init(boost::bind(&ssl_server::init_connection, this),
 			boost::bind(&ssl_server::handle_accept, this, _1));
 }
@@ -176,7 +192,7 @@ void ssl_server::init_connection() {
 	// the following line gets the passphrase for protected private server keys
 	context_.set_password_callback(boost::bind(&ssl_server::get_passphrase, this));
 
-	if (settings_.options.empty()) {
+	if (settings_.ssl_options.empty()) {
 		_log.Log(LOG_ERROR, "[web:%s] missing SSL options parameter !", settings_.listening_port.c_str());
 	} else {
 		context_.set_options(settings_.get_ssl_options());
@@ -184,20 +200,37 @@ void ssl_server::init_connection() {
 	char cipher_list[] = "ECDH+AESGCM:DH+AESGCM:ECDH+AES256:DH+AES256:ECDH+AES128:DH+AES:RSA+AESGCM:RSA+AES:!aNULL:!MD5:!DSS";
 	SSL_CTX_set_cipher_list(context_.native_handle(), cipher_list);
 
+	struct stat st;
 	if (settings_.certificate_chain_file_path.empty()) {
 		_log.Log(LOG_ERROR, "[web:%s] missing SSL certificate chain file parameter !", settings_.listening_port.c_str());
-	} else {
+	} else if (!stat(settings_.certificate_chain_file_path.c_str(), &st)) {
+		cert_chain_tm_ = st.st_mtime;
 		context_.use_certificate_chain_file(settings_.certificate_chain_file_path);
+	} else {
+		_log.Log(LOG_ERROR, "[web:%s] missing SSL certificate chain file %s!", settings_.listening_port.c_str(), settings_.certificate_chain_file_path.c_str());
 	}
+
 	if (settings_.cert_file_path.empty()) {
 		_log.Log(LOG_ERROR, "[web:%s] missing SSL certificate file parameter !", settings_.listening_port.c_str());
-	} else {
+	} else if (!stat(settings_.cert_file_path.c_str(), &st)) {
+		cert_tm_ = st.st_mtime;
 		context_.use_certificate_file(settings_.cert_file_path, boost::asio::ssl::context::pem);
+	} else {
+		_log.Log(LOG_ERROR, "[web:%s] missing SSL certificate file %s!", settings_.listening_port.c_str(), settings_.cert_file_path.c_str());
 	}
+
+
 	if (settings_.private_key_file_path.empty()) {
 		_log.Log(LOG_ERROR, "[web:%s] missing SSL private key file parameter !", settings_.listening_port.c_str());
-	} else {
+	} else if (!stat(settings_.private_key_file_path.c_str(), &st)) {
+		// We don't actually bother to track the mtime of the private
+		// key file as it can't sanely change without changing the
+		// certificate too. And may in fact change *before* the
+		// certificate does, while the cert is being issued. We
+		// don't want to update until the *cert* file changes.
 		context_.use_private_key_file(settings_.private_key_file_path, boost::asio::ssl::context::pem);
+	} else {
+		_log.Log(LOG_ERROR, "[web:%s] missing SSL private key file %s!", settings_.listening_port.c_str(), settings_.private_key_file_path.c_str());
 	}
 
 	// Do not work with mobile devices at this time (2016/02)
@@ -219,15 +252,56 @@ void ssl_server::init_connection() {
 	// Load DH parameters
 	if (settings_.tmp_dh_file_path.empty()) {
 		_log.Log(LOG_ERROR, "[web:%s] missing SSL DH file parameter", settings_.listening_port.c_str());
-	} else {
+	} else if (!stat(settings_.tmp_dh_file_path.c_str(), &st)) {
+		dhparam_tm_ = st.st_mtime;
+
 		std::ifstream ifs(settings_.tmp_dh_file_path.c_str());
 		std::string content((std::istreambuf_iterator<char>(ifs)),
 				(std::istreambuf_iterator<char>()));
 		if (content.find("BEGIN DH PARAMETERS") != std::string::npos) {
 			context_.use_tmp_dh_file(settings_.tmp_dh_file_path);
-#ifdef DEBUG_WWW
-			_log.Log(LOG_STATUS, "[web:%s] 'BEGIN DH PARAMETERS' found in file %s", settings_.listening_port.c_str(), settings_.tmp_dh_file_path.c_str());
-#endif
+			//_log.DEBUG(DEBUG_WEBSERVER, "[web:%s] 'BEGIN DH PARAMETERS' found in file %s", settings_.listening_port.c_str(), settings_.tmp_dh_file_path.c_str());
+		} else {
+			_log.Log(LOG_ERROR, "[web:%s] missing SSL DH parameters from file %s", settings_.listening_port.c_str(), settings_.tmp_dh_file_path.c_str());
+		}
+	} else {
+		_log.Log(LOG_ERROR, "[web:%s] missing SSL DH parameters file %s!", settings_.listening_port.c_str(), settings_.certificate_chain_file_path.c_str());
+	}
+}
+
+void ssl_server::reinit_connection()
+{
+	new_connection_.reset(new connection(io_service_, connection_manager_, request_handler_, timeout_, context_));
+
+	struct stat st;
+
+	if ((!settings_.certificate_chain_file_path.empty() &&
+	     !stat(settings_.certificate_chain_file_path.c_str(), &st) &&
+	     st.st_mtime != cert_chain_tm_)) {
+		cert_chain_tm_ = st.st_mtime;
+		_log.Log(LOG_STATUS, "[web:%s] Reloading SSL certificate chain file", settings_.listening_port.c_str());
+		context_.use_certificate_chain_file(settings_.certificate_chain_file_path);
+	}
+
+	if (!settings_.cert_file_path.empty() &&
+	    !stat(settings_.cert_file_path.c_str(), &st) &&
+	    st.st_mtime != cert_tm_) {
+		cert_tm_ = st.st_mtime;
+		_log.Log(LOG_STATUS, "[web:%s] Reloading SSL certificate and private key", settings_.listening_port.c_str());
+		context_.use_certificate_file(settings_.cert_file_path, boost::asio::ssl::context::pem);
+		context_.use_private_key_file(settings_.private_key_file_path, boost::asio::ssl::context::pem);
+	}
+
+	if (!settings_.tmp_dh_file_path.empty() &&
+	    !stat(settings_.tmp_dh_file_path.c_str(), &st) &&
+	    st.st_mtime != dhparam_tm_) {
+		dhparam_tm_ = st.st_mtime;
+		std::ifstream ifs(settings_.tmp_dh_file_path.c_str());
+		std::string content((std::istreambuf_iterator<char>(ifs)),
+				(std::istreambuf_iterator<char>()));
+		if (content.find("BEGIN DH PARAMETERS") != std::string::npos) {
+			_log.Log(LOG_STATUS, "[web:%s] Reloading SSL DH parameters", settings_.listening_port.c_str());
+			context_.use_tmp_dh_file(settings_.tmp_dh_file_path);
 		} else {
 			_log.Log(LOG_ERROR, "[web:%s] missing SSL DH parameters from file %s", settings_.listening_port.c_str(), settings_.tmp_dh_file_path.c_str());
 		}
@@ -240,8 +314,7 @@ void ssl_server::init_connection() {
 void ssl_server::handle_accept(const boost::system::error_code& e) {
 	if (!e) {
 		connection_manager_.start(new_connection_);
-		new_connection_.reset(new connection(io_service_,
-				connection_manager_, request_handler_, timeout_, context_));
+		reinit_connection();
 		// listen for a subsequent request
 		acceptor_.async_accept(new_connection_->socket(),
 				boost::bind(&ssl_server::handle_accept, this,
@@ -254,18 +327,18 @@ std::string ssl_server::get_passphrase() const {
 }
 #endif
 
-boost::shared_ptr<server_base> server_factory::create(const server_settings & settings, request_handler & user_request_handler) {
+std::shared_ptr<server_base> server_factory::create(const server_settings & settings, request_handler & user_request_handler) {
 #ifdef WWW_ENABLE_SSL
 		if (settings.is_secure()) {
 			return create(dynamic_cast<ssl_server_settings const &>(settings), user_request_handler);
 		}
 #endif
-		return boost::shared_ptr<server_base>(new server(settings, user_request_handler));
+		return std::shared_ptr<server_base>(new server(settings, user_request_handler));
 	}
 
 #ifdef WWW_ENABLE_SSL
-boost::shared_ptr<server_base> server_factory::create(const ssl_server_settings & ssl_settings, request_handler & user_request_handler) {
-		return boost::shared_ptr<server_base>(new ssl_server(ssl_settings, user_request_handler));
+std::shared_ptr<server_base> server_factory::create(const ssl_server_settings & ssl_settings, request_handler & user_request_handler) {
+		return std::shared_ptr<server_base>(new ssl_server(ssl_settings, user_request_handler));
 	}
 #endif
 
